//
//  CARL.hpp - Main CARL header.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/28 10:40:28 $
//  $Revision: 1.31 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  The master include file for C++ Advanced Runtime Library.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
#define __CARL_hpp_included__

/**
 *  The root namespace for all C++ Advanced Runtime Library facilities.
 *
 *  @since 1.0
 */
namespace carl
{
    /**
    *  The namespace for standard hash functions.
    *
    *   This namespace encapsulates predefined hash functions for all CARL
    *   hashable entities. It is set up as a namespace instead of utility class
    *   because multiple CARL subsystems have to contribute to it.
    *
    *  @since 1.0
    */
    namespace StandardHashFunctions
    {
    }
}

//////////
//  Infer platform properties
#include "CARL_Platform.hpp"

//////////
//  CARL linkage and calling conventions
#include "CARL_Linkage.hpp"

//////////
//  Set up CARL environment
#if defined(CARL_TOOLCHAIN_MSVC6) && defined(CARL_OS_WIN32)
    //  Ensure member alignment is independent of command-line options
    #pragma pack(8)
#elif defined(CARL_TOOLCHAIN_MSVCNET) && defined(CARL_OS_WIN32)
    //  Ensure member alignment is independent of command-line options
    #pragma pack(push, __CARL_hpp__)
    #pragma pack(8)
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #pragma pack(push, __CARL_hpp__, 8)
#else
    #error Unsupported platform
#endif

//////////
//  CARL components
#include "CARL_BasicTypes.hpp"
#include "CARL_ByteOrder.hpp"
#include "CARL_Require.hpp"
#include "CARL_System.hpp"
#include "CARL_Memory.hpp"
#include "CARL_String.hpp"
#include "CARL_Exceptions.hpp"
#include "CARL_Locale.hpp"
#include "CARL_Iterator.hpp"
#include "CARL_Comparator.hpp"
#include "CARL_Array.hpp"
#include "CARL_DateTime.hpp"
#include "CARL_Sync.hpp"
#include "CARL_Heap.hpp"
#include "CARL_Hash.hpp"
#include "CARL_Pair.hpp"
#include "CARL_Search.hpp"
#include "CARL_Set.hpp"
#include "CARL_Map.hpp"
#include "CARL_Stack.hpp"
#include "CARL_Queue.hpp"
#include "CARL_Iso10646.hpp"
#include "CARL_Iso639.hpp"
#include "CARL_Iso3166.hpp"
#include "CARL_Charset.hpp"
#include "CARL_FileSystem.hpp"
#include "CARL_Math.hpp"
#include "CARL_Format.hpp"
#include "CARL_Parse.hpp"
#include "CARL_IO.hpp"
#include "CARL_Resource.hpp"
#include "CARL_Library.hpp"
#include "CARL_Environment.hpp"
#include "CARL_Process.hpp"
#include "CARL_Thread.hpp"
#include "CARL_Settings.hpp"
#include "CARL_Application.hpp"
#include "CARL_Random.hpp"
#include "CARL_Compression.hpp"
#include "CARL_Encryption.hpp"
#include "CARL_Extension.hpp"
#include "CARL_Plugin.hpp"
#include "CARL_Sort.hpp"
#include "CARL_Collator.hpp"

//////////
//  Restore changes to environment
#if defined(CARL_TOOLCHAIN_MSVC6) && defined(CARL_OS_WIN32)
    #pragma pack()
#elif defined(CARL_TOOLCHAIN_MSVCNET) && defined(CARL_OS_WIN32)
    #pragma pack(pop, __CARL_hpp__)
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #pragma pack(pop, __CARL_hpp__)
#else
    #error Unsupported platform
#endif

#endif  //  __CARL_hpp_included__
//  End of CARL.hpp
