//
//  CARL_ByteOrder.hpp - Provides byte order - related types and services.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/25 12:11:26 $
//  $Revision: 1.14 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines supported byte orders.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_ByteOrder.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_ByteOrder_hpp_included__
#define __CARL_ByteOrder_hpp_included__

namespace carl
{
    /**
     *  Byte orders supported by target platforms.
     *
     *  @since 1.0
     */
    enum ByteOrder
    {
        /**
         *  Little-endian byte order (least significant byte first).
         *
         *  @hideinitializer
         *  @since 1.0
         */
        ByteOrder_BigEndian     = 1,

        /**
         *  Big-endian byte order (least significant byte first, also
         *  known as network byte order).
         *
         *  @hideinitializer
         *  @since 1.0
         */
        ByteOrder_LittleEndian  = 2
    };
}

#endif  //  __CARL_ByteOrder_hpp_included__
//  End of CARL_ByteOrder.hpp
