//
//  CARL_Hash.hpp - Provides hash functions for primitive types.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/25 12:11:26 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines hash functions for basic CARL types.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_Hash.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_Hash_hpp_included__
#define __CARL_Hash_hpp_included__

namespace carl
{
    namespace StandardHashFunctions
    {
        //////////
        //  Hashing platform-specific types
        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashPlainChar(const char & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashSignedChar(const signed char & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUnsignedChar(const unsigned char & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashSignedShort(const signed short & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUnsignedShort(const unsigned short & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashSignedInt(const signed int & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUnsignedInt(const unsigned int & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashSignedLong(const signed long & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUnsignedLong(const unsigned long & key);

        //////////
        //  Hashing platform-independent types
        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashChar(const Char & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashInt8(const Int8 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashInt16(const Int16 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashInt32(const Int32 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashInt64(const Int64 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUInt8(const UInt8 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUInt16(const UInt16 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUInt32(const UInt32 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUInt64(const UInt64 & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUIntSize(const UIntSize & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashUIntPtr(const UIntPtr & key);

        /**
         *  Returns a hash code for the specified value.
         *
         *  @param key      The value to calculate a hash code for.
         *  @return         The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashIntPtrDiff(const IntPtrDiff & key);
    }
}

#endif  //  __CARL_Hash_hpp_included__
//  End of CARL_Hash.hpp
