//
//  CARL_Linkage.hpp - Defines linkage & calling convention symbols.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:48 $
//  $Revision: 1.18 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines CARL linkage conventions.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_Linkage.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_Linkage_hpp_included__
#define __CARL_Linkage_hpp_included__

//////////
//  Ensure the CARL client is build in a mode supported by CARL
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #if !defined(_MT)
        #error CARL clients must be build in a multi-threaded configuration
    #endif
    #if !defined(_CPPUNWIND)
        #error CARL clients must be build with exception handling
    #endif
    #if !defined(_CPPRTTI)
        #error CARL clients must be build with RTTI
    #endif
    #if defined(_MANAGED)
        #error CARL does not support managed C++
    #endif
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #if !defined(__EXCEPTIONS)
        #error CARL clients must be build with exception handling
    #endif
#else
    #error Unsupported platform
#endif

//////////
//  Define toolchain-independent symbols for linking conventions
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #define CARL_FORCE_IMPORT   __declspec(dllimport)
    #define CARL_FORCE_EXPORT   __declspec(dllexport)
    #define CARL_FORCE_CALLBACK __cdecl
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)
    #define CARL_FORCE_IMPORT   __attribute__((dllimport))
    #define CARL_FORCE_EXPORT   __attribute__((dllexport))
    #define CARL_FORCE_CALLBACK
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC)
    #define CARL_FORCE_IMPORT
    #define CARL_FORCE_EXPORT
    #define CARL_FORCE_CALLBACK
#else
    #error Unsupported platform
#endif

//////////
//  Define visibility/linkage symbols for CARL API entities
#if defined(__CARL_BUILD__)
    //  Building CARL
    #if defined(CARL_LINKAGE_DYNAMIC)
        //  Building CARL DLL
        #define CARL_PUBLIC     CARL_FORCE_EXPORT
        #define CARL_EXPORT     CARL_FORCE_EXPORT
        #define CARL_IMPORT     CARL_FORCE_IMPORT
    #else
        //  Building static CARL library
        #define CARL_PUBLIC
        #define CARL_EXPORT
        #define CARL_IMPORT
    #endif
#else
    //  Building CARL client
    #if defined(CARL_LINKAGE_DYNAMIC)
        //  CARL client is dynamically linked
        #define CARL_PUBLIC     CARL_FORCE_IMPORT
        #define CARL_EXPORT     CARL_FORCE_EXPORT
        #define CARL_IMPORT     CARL_FORCE_IMPORT
    #else
        //  CARL client is statically linked
        #define CARL_PUBLIC
        #define CARL_EXPORT
        #define CARL_IMPORT
    #endif
#endif
#define CARL_CALLBACK   CARL_FORCE_CALLBACK
#define CARL_API        CARL_FORCE_CALLBACK

//////////
//  Pull in the proper library
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    //  Some Win32 DLLs musl be pulled in
    #pragma comment(lib,"psapi.lib")
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
#else
    #error Unsupported platform
#endif

#endif  //  __CARL_Linkage_hpp_included__
//  End of CARL_Linkage.hpp
