//
//  CARL_Platform.hpp - Infers platform properties and defines corresponding preprocessor symbols.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/28 10:40:28 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines CARL platform identification features.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_Platform.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_Platform_hpp_included__
#define __CARL_Platform_hpp_included__

//////////
//  Toolchain identification
//
//  The following preprocessor symbol hierarchy is defined:
//
//  CARL_TOOLCHAIN_MSVC                 - Generic MS Visual C++
//      CARL_TOOLCHAIN_MSVC6            - MS Visual C++ 6.0
//      CARL_TOOLCHAIN_MSVCNET          - Generic MS Visual C++ .NET
//          CARL_TOOLCHAIN_MSVCNET2003  - MS Visual C++ .NET 2003
//          CARL_TOOLCHAIN_MSVCNET2005  - MS Visual C++ .NET 2005
//  CARL_TOOLCHAIN_GCC                  - Generic GCC
//
#if defined(_MSC_VER)
    #define CARL_TOOLCHAIN_MSVC
    #if _MSC_VER >= 1400 && _MSC_VER < 1500
        #define CARL_TOOLCHAIN_MSVCNET
        #define CARL_TOOLCHAIN_MSVCNET2005
    #elif _MSC_VER >= 1300 && _MSC_VER < 1400
        #define CARL_TOOLCHAIN_MSVCNET
        #define CARL_TOOLCHAIN_MSVCNET2003
    #elif _MSC_VER >= 1200
        #define CARL_TOOLCHAIN_MSVC6
    #else
        #error Unsupported MS Visual C++ Version
    #endif
#elif defined(__GNUC__)
    #define CARL_TOOLCHAIN_GCC
#else
    #error Unsupported platform
#endif

//////////
//  CPU identification
//
//  The following CPU symbol hierarchy is defined:
//
//  CARL_CPU_IA32                   - Generic 32-bit Intel
//
#if defined(CARL_TOOLCHAIN_MSVC)
    #if defined(_M_IX86)
        #define CARL_CPU_IA32
    #else
        #error Unsupported CPU
    #endif
#elif defined(CARL_TOOLCHAIN_GCC)
    #if defined(__i386__)
        #define CARL_CPU_IA32
    #else
        #error Unsupported CPU
    #endif
#else
    #error Unsupported platform
#endif

//////////
//  OS identification
//
//  The following CPU symbol hierarchy is defined:
//
//  CARL_OS_WINDOWS             - Generic Windows
//      CARL_OS_WIN32           - Generic 32-bit Windows
//  CARL_OS_CYGWIN              - Generic cygwin
//      CARL_OS_CYGWIN32        - Generic 32-bit cygwin
//  CARL_OS_LINUX               - Generic linux
//      CARL_OS_LINUX32         - Generic 32-bit linux
//
#if defined(CARL_TOOLCHAIN_MSVC)
    #if defined(_WIN32) && !defined(_WIN64)
        #define CARL_OS_WINDOWS
        #define CARL_OS_WIN32
    #else
        #error Unsupported platform
    #endif
#elif defined(CARL_TOOLCHAIN_GCC)
    #if defined(__CYGWIN32__)
        #define CARL_OS_CYGWIN
        #define CARL_OS_CYGWIN32
    #elif defined(__linux__) && defined(__i386__)
        #define CARL_OS_LINUX
        #define CARL_OS_LINUX32
    #else
        #error Unsupported platform
    #endif
#else
    #error Unsupported platform
#endif

//////////
//  Mode
//
//  The following mode symbol hierarchy is defined:
//
//  CARL_MODE_DEBUG         - Debug mode
//  CARL_MODE_RELEASE       - Release mode
//
#if defined(CARL_TOOLCHAIN_MSVC)
    #if defined(_DEBUG)
        #define CARL_MODE_DEBUG
    #else
        #define CARL_MODE_RELEASE
    #endif
#elif defined(CARL_TOOLCHAIN_GCC)
    #if defined(NDEBUG)
        #define CARL_MODE_RELEASE
    #else
        #define CARL_MODE_DEBUG
    #endif
#else
    #error Unsupported platform
#endif

//////////
//  Linkage
//
//  The following linkage symbol hierarchy is defined:
//
//  CARL_LINKAGE_STATIC     - Client statically links to CARL library
//  CARL_LINKAGE_DYNAMIC    - Client dynamically links to CARL DLL/SO
//
#if defined(CARL_TOOLCHAIN_MSVC)
    #if defined(_DLL)
        #define CARL_LINKAGE_DYNAMIC
    #else
        #define CARL_LINKAGE_STATIC
    #endif
#elif defined(CARL_TOOLCHAIN_GCC)
    #if defined(_DLL)
        #define CARL_LINKAGE_DYNAMIC
    #else
        #define CARL_LINKAGE_STATIC
    #endif
#else
    #error Unsupported platform
#endif

//////////
//  Disable unwanted warnings
#if defined(CARL_TOOLCHAIN_MSVC)
    #pragma warning(disable:4251)
    #pragma warning(disable:4290)
#elif defined(CARL_TOOLCHAIN_GCC)
#else
    #error Unsupported platform
#endif

#endif  //  __CARL_Platform_hpp_included__
//  End of CARL_Platform.hpp
