//
//  CARL_Require.hpp - Defines the "require" macro
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:48 $
//  $Revision: 1.3 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines the "require(condition) macro.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_Require.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_Require_hpp_included__
#define __CARL_Require_hpp_included__

namespace carl
{
    namespace _impl
    {
        //  The helper used by require() macro to handle errors
        void CARL_PUBLIC _require(const char * pAssertionText, const char * pSourceFileName, UIntSize sourceLineNumber);
    }

    /**
     *  Checks if the specified condition holds.
     *
     *  In debug builds, an AssertionError is thrown if the specified
     *  condition evaluates to false.
     *
     *  In release builds, the check is removed by the preprocessor.
     *
     *  @hideinitializer
     *  @since 1.0
     */
    #if defined(_DEBUG)
        #define require(x)      (void)((x) || (::carl::_impl::_require(#x, __FILE__, __LINE__), 0))
    #else
        #define require(x)      ((void)(x))
    #endif
}

#endif  //  __CARL_Require_hpp_included__
//  End of CARL_Require.hpp
