//
//  CARL_String.hpp - Variable-length character string.
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/29 11:19:44 $
//  $Revision: 1.19 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
/**
 *  @file
 *
 *  Defines variable-length ISO10646 strings.
 *
 *  Do not include this file directly. Use the master header CARL.hpp instead.
 *
 *  @since 1.0
 */
#ifndef __CARL_hpp_included__
    #error The file CARL_String.hpp shall not be included directly. Use CARL.hpp instead
#endif  //  __CARL_hpp_included__

#ifndef __CARL_String_hpp_included__
#define __CARL_String_hpp_included__

namespace carl
{
    class CARL_PUBLIC PathName;
    class CARL_PUBLIC FileInputStream;
    class CARL_PUBLIC FileOutputStream;
    class CARL_PUBLIC RandomAccessFile;
    class CARL_PUBLIC Library;
    class CARL_PUBLIC FileSystem;
    class CARL_PUBLIC String;

    namespace _impl
    {
        //////////
        //  String implementation
        class CARL_PUBLIC _StringImpl
        {
            friend class ::carl::String;

            //////////
            //  Construction/destruction/assignment
        public:
            _StringImpl();
            explicit _StringImpl(char ch);
            explicit _StringImpl(Char ch);
            _StringImpl(char ch, UIntSize repeatCount);
            _StringImpl(Char ch, UIntSize repeatCount);
            explicit _StringImpl(const char * pStr);
    #if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
            //  Win32 MSVC++ has wide-char library support
            explicit _StringImpl(const wchar_t * pStr);
    #endif
            explicit _StringImpl(const Char * pStr);
            _StringImpl(const char * pChars, UIntSize charCount);
            _StringImpl(const Char * pChars, UIntSize charCount);
            _StringImpl(const _StringImpl & str);

            ~_StringImpl();

            void                    operator = (char ch);
            void                    operator = (Char ch);
            void                    operator = (const char * pStr);
            void                    operator = (const Char * pStr);

            //////////
            //  Operations
        public:
            bool                    isEmpty() const { return _used == 0; }
            UIntSize                length() const { return _used; }
            void                    setLength(UIntSize length);

            int                     compare(char ch) const;
            int                     compare(Char ch) const;
            int                     compare(const char * pStr) const;
            int                     compare(const Char * pStr) const;
            int                     compare(const _StringImpl & str) const;
            int                     compareIgnoreCase(char ch) const;
            int                     compareIgnoreCase(Char ch) const;
            int                     compareIgnoreCase(const char * pStr) const;
            int                     compareIgnoreCase(const Char * pStr) const;
            int                     compareIgnoreCase(const _StringImpl & str) const;

            bool                    startsWith(char ch) const;
            bool                    startsWith(Char ch) const;
            bool                    startsWith(const char * pStr) const;
            bool                    startsWith(const Char * pStr) const;
            bool                    startsWith(const _StringImpl & str) const;
            bool                    startsWithIgnoreCase(char ch) const;
            bool                    startsWithIgnoreCase(Char ch) const;
            bool                    startsWithIgnoreCase(const char * pStr) const;
            bool                    startsWithIgnoreCase(const Char * pStr) const;
            bool                    startsWithIgnoreCase(const _StringImpl & str) const;

            bool                    endsWith(char ch) const;
            bool                    endsWith(Char ch) const;
            bool                    endsWith(const char * pStr) const;
            bool                    endsWith(const Char * pStr) const;
            bool                    endsWith(const _StringImpl & str) const;
            bool                    endsWithIgnoreCase(char ch) const;
            bool                    endsWithIgnoreCase(Char ch) const;
            bool                    endsWithIgnoreCase(const char * pStr) const;
            bool                    endsWithIgnoreCase(const Char * pStr) const;
            bool                    endsWithIgnoreCase(const _StringImpl & str) const;

            bool                    findFirst(char ch, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirst(Char ch, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirst(const char * pStr, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirst(const Char * pStr, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirst(const _StringImpl & str, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirstIgnoreCase(char ch, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirstIgnoreCase(Char ch, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirstIgnoreCase(const char * pStr, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirstIgnoreCase(const Char * pStr, UIntSize & position, UIntSize startIndex = 0) const;
            bool                    findFirstIgnoreCase(const _StringImpl & str, UIntSize & position, UIntSize startIndex = 0) const;

            bool                    findLast(char ch, UIntSize & position) const;
            bool                    findLast(Char ch, UIntSize & position) const;
            bool                    findLast(const char * pStr, UIntSize & position) const;
            bool                    findLast(const Char * pStr, UIntSize & position) const;
            bool                    findLast(const _StringImpl & str, UIntSize & position) const;
            bool                    findLastIgnoreCase(char ch, UIntSize & position) const;
            bool                    findLastIgnoreCase(Char ch, UIntSize & position) const;
            bool                    findLastIgnoreCase(const char * pStr, UIntSize & position) const;
            bool                    findLastIgnoreCase(const Char * pStr, UIntSize & position) const;
            bool                    findLastIgnoreCase(const _StringImpl & str, UIntSize & position) const;

            Char &                  operator [] (UIntSize index) { require(index < _used); return _pChars[index]; }

            void                    operator += (char ch);
            void                    operator += (Char ch);
            void                    operator += (const char * pStr);
            void                    operator += (const Char * pStr);
            void                    operator += (const _StringImpl & str);

            void                    insert(char ch, UIntSize atIndex);
            void                    insert(Char ch, UIntSize atIndex);
            void                    insert(const char * pStr, UIntSize atIndex);
            void                    insert(const Char * pStr, UIntSize atIndex);
            void                    insert(const _StringImpl & str, UIntSize atIndex);

            void                    remove(UIntSize startIndex, UIntSize charCount);

            void                    replace(UIntSize startIndex, UIntSize charCount, char ch);
            void                    replace(UIntSize startIndex, UIntSize charCount, Char ch);
            void                    replace(UIntSize startIndex, UIntSize charCount, const char * pStr);
            void                    replace(UIntSize startIndex, UIntSize charCount, const Char * pStr);
            void                    replace(UIntSize startIndex, UIntSize charCount, const char * pChars, UIntSize replacementCharCount);
            void                    replace(UIntSize startIndex, UIntSize charCount, const Char * pChars, UIntSize replacementCharCount);
            void                    replace(UIntSize startIndex, UIntSize charCount, const _StringImpl & str);

            _StringImpl *           left(UIntSize charCount);
            _StringImpl *           right(UIntSize charCount);
            _StringImpl *           mid(UIntSize startIndex);
            _StringImpl *           mid(UIntSize startIndex, UIntSize charCount);

            void                    trimLeft();
            void                    trimRight();
            void                    trim() { trimLeft(); trimRight(); }

            void                    upperCase();
            void                    lowerCase();
            void                    titleCase();

            UIntSize                getReferenceCount() const { return _referenceCount; }
            void                    incrementReferenceCount() { _referenceCount++; require(_referenceCount != 0); }
            void                    decrementReferenceCount() { require(_referenceCount != 0); _referenceCount--; }

            //////////
            //  Implementation
        private:
            Char *                  _pChars;
            UIntSize                _allocated;
            UIntSize                _used;
            UIntSize                _referenceCount;

            //  Helpers
            static UIntSize         _strlen(const Char * pStr);
            void                    _ensureAllocation(UIntSize charCount);
    #if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
            //  Win32 MSVC++ has wide-char library support
            bool                    _store(wchar_t * pBuffer, UIntSize bufferSize) const;
    #endif
            bool                    _store(char * pBuffer, UIntSize bufferSize) const;
        };
    }

    /**
     *  @class String CARL.hpp
     *  A variable-length string of ISO-10646 characters.
     *
     *  CARL strings use implicit sharing to reduce the copy construction and 
     *  assignment overhead. This means that it is feasible to both pass
     *  strings to functions as value parameters and return strings from functions.
     *
     *  @since 1.0
     */
    class CARL_PUBLIC String
    {
        friend class PathName;
        friend class FileInputStream;
        friend class FileOutputStream;
        friend class RandomAccessFile;
        friend class Library;
        friend class FileSystem;

        //////////
        //  Construction/destruction/assignment
    private:
        String(_impl::_StringImpl * pImpl);
    public:
        /**
         *  The default constructor.
         *
         *  Initializes the newly created String instance to an empty string.
         *
         *  @since 1.0
         */
        String();

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the string
         *  of length 1, containing the specified character.
         *
         *  @param ch           The character to initialize the string with.
         *  @since 1.0
         */
        explicit String(char ch);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the string
         *  of length 1, containing the specified character.
         *
         *  @param ch           The character to initialize the string with.
         *  @since 1.0
         */
        explicit String(Char ch);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the string
         *  of specified length, containing the repeated copied of the 
         *  specified character.
         *
         *  @param ch           The character to initialize the string with.
         *  @param repeatCount  The number of times the character shall be repeated.
         *  @since 1.0
         */
        String(char ch, UIntSize repeatCount);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the string
         *  of specified length, containing the repeated copied of the 
         *  specified character.
         *
         *  @param ch           The character to initialize the string with.
         *  @param repeatCount  The number of times the character shall be repeated.
         *  @since 1.0
         */
        String(Char ch, UIntSize repeatCount);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the specified string.
         *
         *  @param pStr         A pointer to the nul-terminated ASCII string
         *                      to initialize the new String instance with.
         *  @since 1.0
         */
        explicit String(const char * pStr);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        //  Win32 MSVC++ has wide-char library support
        explicit String(const wchar_t * pStr);
#endif
        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the specified string.
         *
         *  @param pStr         A pointer to the nul-terminated ISO-10646 string
         *                      to initialize the new String instance with.
         *  @since 1.0
         */
        explicit String(const Char * pStr);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the specified
         *  sequence of characters.
         *
         *  @param pChars       A pointer to the sequence of ASCII characters
         *                      to initialize the newly created String with.
         *                      The sequence must be at least \a charCount
         *                      characters long; if it is longer then only
         *                      the first \a charCount characters are used.
         *  @param charCount    The number of characters to initialize a
         *                      newly created String with.
         *  @since 1.0
         */
        String(const char * pChars, UIntSize charCount);

        /**
         *  The class constructor.
         *
         *  Initializes the newly created String instance to the specified
         *  sequence of characters.
         *
         *  @param pChars       A pointer to the sequence of ISO-10646 characters
         *                      to initialize the newly created String with.
         *                      The sequence must be at least \a charCount
         *                      characters long; if it is longer then only
         *                      the first \a charCount characters are used.
         *  @param charCount    The number of characters to initialize a
         *                      newly created String with.
         *  @since 1.0
         */
        String(const Char * pChars, UIntSize charCount);

        /**
         *  The copy constructor.
         *
         *  Initializes a newly created string to be a copy of the specified
         *  string. string copy construction uses lazy initialization.
         *
         *  @param str          The existing string that shall be copied into
         *                      the newly constructed string.
         *  @since 1.0
         */
        String(const String & str);

        /**
         *  The class destructor.
         *
         *  Releases all memory used by the string.
         *
         *  @since 1.0
         */
        ~String();

        /**
         *  The assignment operator.
         *
         *  Assigns the specified character to an existing String instance.
         *
         *  @param ch           The ASCII character to assign to the String instance.
         *  @return             A reference to the assigned-to string.
         *  @since 1.0
         */
        String &                operator = (char ch);

        /**
         *  The assignment operator.
         *
         *  Assigns the specified character to an existing String instance.
         *
         *  @param ch           The ISO-10646 character to assign to the String instance.
         *  @return             A reference to the assigned-to string.
         *  @since 1.0
         */
        String &                operator = (Char ch);

        /**
         *  The assignment operator.
         *
         *  Assigns the specified ASCII string to an existing String instance.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to assign to the String instance.
         *  @return             A reference to the assigned-to string.
         *  @since 1.0
         */
        String &                operator = (const char * pStr);

        /**
         *  The assignment operator.
         *
         *  Assigns the specified ISO-10646 string to an existing String instance.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to assign to the String instance.
         *  @return             A reference to the assigned-to string.
         *  @since 1.0
         */
        String &                operator = (const Char * pStr);

        /**
         *  The assignment operator.
         *
         *  Makes the assigned-to string an exact copy of the assigned-from
         *  string. String assignment uses lazy initialization.
         *
         *  @param str          The existing string that shall be assigned 
         *                      to another string.
         *  @return             A reference to the assigned-to string.
         *  @since 1.0
         */
        String &                operator = (const String & str);

        //////////
        //  Operations
    public:
        /**
         *  Checks if a string is empty.
         *
         *  @return             True if this string is empty; false otherwise.
         *  @since 1.0
         */
        bool                    isEmpty() const { return _pImpl->isEmpty(); }

        /**
         *  Queries the length of the string.
         *
         *  @return             The number of characters in this string.
         *  @since 1.0
         */
        UIntSize                length() const { return _pImpl->length(); }

        /**
         *  Sets the length of the string.
         *
         *  If the new length is smaller than the current length, the string is
         *  truncated at the right end to the required length. If the new length
         *  is larger than the current length, the string is padded with spaces
         *  at the right end to the required length.
         *
         *  @param length       The new length of the string.
         *  @since 1.0
         */
        void                    setLength(UIntSize length);

        /**
         *  Compares the string with the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param ch           The ASCII character to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a ch.
         *                      -   >0 if this string is lexicographically greater than \a ch.
         *  @since 1.0
         */
        int                     compare(char ch) const { return _pImpl->compare(ch); }

        /**
         *  Compares the string with the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param ch           The ISO-10646 character to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a ch.
         *                      -   >0 if this string is lexicographically greater than \a ch.
         *  @since 1.0
         */
        int                     compare(Char ch) const { return _pImpl->compare(ch); }

        /**
         *  Compares two strings.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a pStr.
         *                      -   >0 if this string is lexicographically greater than \a pStr.
         *  @since 1.0
         */
        int                     compare(const char * pStr) const { return _pImpl->compare(pStr); }

        /**
         *  Compares two strings.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a pStr.
         *                      -   >0 if this string is lexicographically greater than \a pStr.
         *  @since 1.0
         */
        int                     compare(const Char * pStr) const { return _pImpl->compare(pStr); }

        /**
         *  Compares two strings.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param str          The string to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a str.
         *                      -   >0 if this string is lexicographically greater than \a str.
         *  @since 1.0
         */
        int                     compare(const String & str) const { return _pImpl->compare(*str._pImpl); }

        /**
         *  Compares the string with the specified character, ignoring letter case.
         *
         *  The character is treated as if it was a string of length 1.
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param ch           The ASCII character to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a ch.
         *                      -   >0 if this string is lexicographically greater than \a ch.
         *  @since 1.0
         */
        int                     compareIgnoreCase(char ch) const { return _pImpl->compareIgnoreCase(ch); }


        /**
         *  Compares the string with the specified character, ignoring letter case.
         *
         *  The character is treated as if it was a string of length 1.
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param ch           The ISO-10646 character to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a ch.
         *                      -   >0 if this string is lexicographically greater than \a ch.
         *  @since 1.0
         */
        int                     compareIgnoreCase(Char ch) const { return _pImpl->compareIgnoreCase(ch); }

        /**
         *  Compares two strings, ignoring letter case.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a pStr.
         *                      -   >0 if this string is lexicographically greater than \a pStr.
         *  @since 1.0
         */
        int                     compareIgnoreCase(const char * pStr) const { return _pImpl->compareIgnoreCase(pStr); }

        /**
         *  Compares two strings, ignoring letter case.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a pStr.
         *                      -   >0 if this string is lexicographically greater than \a pStr.
         *  @since 1.0
         */
        int                     compareIgnoreCase(const Char * pStr) const { return _pImpl->compareIgnoreCase(pStr); }

        /**
         *  Compares two strings, ignoring letter case.
         *
         *  Comparision is performed lexicographically, based on ISO-10646
         *  character codes.
         *
         *  @param str          The string to compare this string with.
         *  @return             
         *                      -   0 if the two strings are equal,
         *                      -   <0 if this string is lexicographically smaller than \a str.
         *                      -   >0 if this string is lexicographically greater than \a str.
         *  @since 1.0
         */
        int                     compareIgnoreCase(const String & str) const { return _pImpl->compareIgnoreCase(*str._pImpl); }

        /**
         *  Checks if the string starts with the specified character.
         *
         *  @param ch           The ASCII character to check against.
         *  @return             True if this string starts with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    startsWith(char ch) const { return _pImpl->startsWith(ch); }

        /**
         *  Checks if the string starts with the specified character.
         *
         *  @param ch           The ISO-10646 character to check against.
         *  @return             True if this string starts with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    startsWith(Char ch) const { return _pImpl->startsWith(ch); }

        /**
         *  Checks if the string starts with the specified substring.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWith(const char * pStr) const { return _pImpl->startsWith(pStr); }

        /**
         *  Checks if the string starts with the specified substring.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWith(const Char * pStr) const { return _pImpl->startsWith(pStr); }

        /**
         *  Checks if the string starts with the specified substring.
         *
         *  @param str          The string to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWith(const String & str) const { return _pImpl->startsWith(*str._pImpl); }

        /**
         *  Checks if the string starts with the specified character,
         *  ignoring letter case.
         *
         *  @param ch           The ASCII character to check against.
         *  @return             True if this string starts with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    startsWithIgnoreCase(char ch) const { return _pImpl->startsWithIgnoreCase(ch); }

        /**
         *  Checks if the string starts with the specified character,
         *  ignoring letter case.
         *
         *  @param ch           The ISO-10646 character to check against.
         *  @return             True if this string starts with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    startsWithIgnoreCase(Char ch) const { return _pImpl->startsWithIgnoreCase(ch); }

        /**
         *  Checks if the string starts with the specified substring,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWithIgnoreCase(const char * pStr) const { return _pImpl->startsWithIgnoreCase(pStr); }

        /**
         *  Checks if the string starts with the specified substring,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWithIgnoreCase(const Char * pStr) const { return _pImpl->startsWithIgnoreCase(pStr); }

        /**
         *  Checks if the string starts with the specified substring,
         *  ignoring letter case.
         *
         *  @param str          The string to check against.
         *  @return             True if this string starts with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    startsWithIgnoreCase(const String & str) const { return _pImpl->startsWithIgnoreCase(*str._pImpl); }

        /**
         *  Checks if the string ends with the specified character.
         *
         *  @param ch           The ASCII character to check against.
         *  @return             True if this string ends with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    endsWith(char ch) const { return _pImpl->endsWith(ch); }

        /**
         *  Checks if the string ends with the specified character.
         *
         *  @param ch           The ISO-10646 character to check against.
         *  @return             True if this string ends with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    endsWith(Char ch) const { return _pImpl->endsWith(ch); }

        /**
         *  Checks if the string ends with the specified substring.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWith(const char * pStr) const { return _pImpl->endsWith(pStr); }

        /**
         *  Checks if the string ends with the specified substring.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWith(const Char * pStr) const { return _pImpl->endsWith(pStr); }

        /**
         *  Checks if the string ends with the specified substring.
         *
         *  @param str          The string to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWith(const String & str) const { return _pImpl->endsWith(*str._pImpl); }

        /**
         *  Checks if the string ends with the specified character,
         *  ignoring letter case.
         *
         *  @param ch           The ASCII character to check against.
         *  @return             True if this string ends with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    endsWithIgnoreCase(char ch) const { return _pImpl->endsWithIgnoreCase(ch); }

        /**
         *  Checks if the string ends with the specified character,
         *  ignoring letter case.
         *
         *  @param ch           The ISO-10646 character to check against.
         *  @return             True if this string ends with the specified
         *                      character; false otherwise.
         *  @since 1.0
         */
        bool                    endsWithIgnoreCase(Char ch) const { return _pImpl->endsWithIgnoreCase(ch); }

        /**
         *  Checks if the string ends with the specified substring,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWithIgnoreCase(const char * pStr) const { return _pImpl->endsWithIgnoreCase(pStr); }

        /**
         *  Checks if the string ends with the specified substring,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWithIgnoreCase(const Char * pStr) const { return _pImpl->endsWithIgnoreCase(pStr); }

        /**
         *  Checks if the string ends with the specified substring,
         *  ignoring letter case.
         *
         *  @param str          The string to check against.
         *  @return             True if this string ends with the specified
         *                      substring; false otherwise.
         *  @since 1.0
         */
        bool                    endsWithIgnoreCase(const String & str) const { return _pImpl->endsWithIgnoreCase(*str._pImpl); }

        /**
         *  Finds the first occurrence of a character within a string at or
         *  after the specified position.
         *
         *  @param ch           The ASCII character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirst(char ch, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirst(ch, position, startIndex); }

        /**
         *  Finds the first occurrence of a character within a string at or
         *  after the specified position.
         *
         *  @param ch           The ISO-10646 character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirst(Char ch, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirst(ch, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirst(const char * pStr, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirst(pStr, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirst(const Char * pStr, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirst(pStr, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position.
         *
         *  @param str          The string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirst(const String & str, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirst(*str._pImpl, position, startIndex); }

        /**
         *  Finds the first occurrence of a character within a string at or
         *  after the specified position, ignoring letter case.
         *
         *  @param ch           The ASCII character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirstIgnoreCase(char ch, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirstIgnoreCase(ch, position, startIndex); }

        /**
         *  Finds the first occurrence of a character within a string at or
         *  after the specified position, ignoring letter case.
         *
         *  @param ch           The ISO-10646 character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirstIgnoreCase(Char ch, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirstIgnoreCase(ch, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position, ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirstIgnoreCase(const char * pStr, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirstIgnoreCase(pStr, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position, ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirstIgnoreCase(const Char * pStr, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirstIgnoreCase(pStr, position, startIndex); }

        /**
         *  Finds the first occurrence of a substring within a string at or
         *  after the specified position, ignoring letter case.
         *
         *  @param str          The string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @param startIndex   The position to start searching from.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findFirstIgnoreCase(const String & str, UIntSize & position, UIntSize startIndex = 0) const { return _pImpl->findFirstIgnoreCase(*str._pImpl, position, startIndex); }

        /**
         *  Finds the last occurrence of a character within a string.
         *
         *  @param ch           The ASCII character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLast(char ch, UIntSize & position) const { return _pImpl->findLast(ch, position); }

        /**
         *  Finds the last occurrence of a character within a string.
         *
         *  @param ch           The ISO-10646 character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLast(Char ch, UIntSize & position) const { return _pImpl->findLast(ch, position); }

        /**
         *  Finds the last occurrence of a substring within a string.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLast(const char * pStr, UIntSize & position) const { return _pImpl->findLast(pStr, position); }

        /**
         *  Finds the last occurrence of a substring within a string.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLast(const Char * pStr, UIntSize & position) const { return _pImpl->findLast(pStr, position); }

        /**
         *  Finds the last occurrence of a substring within a string.
         *
         *  @param str          The string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLast(const String & str, UIntSize & position) const { return _pImpl->findLast(*str._pImpl, position); }

        /**
         *  Finds the last occurrence of a character within a string,
         *  ignoring letter case.
         *
         *  @param ch           The ASCII character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLastIgnoreCase(char ch, UIntSize & position) const { return _pImpl->findLastIgnoreCase(ch, position); }

        /**
         *  Finds the last occurrence of a character within a string,
         *  ignoring letter case.
         *
         *  @param ch           The ISO-10646 character to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLastIgnoreCase(Char ch, UIntSize & position) const { return _pImpl->findLastIgnoreCase(ch, position); }

        /**
         *  Finds the last occurrence of a substring within a string,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLastIgnoreCase(const char * pStr, UIntSize & position) const { return _pImpl->findLastIgnoreCase(pStr, position); }

        /**
         *  Finds the last occurrence of a substring within a string,
         *  ignoring letter case.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLastIgnoreCase(const Char * pStr, UIntSize & position) const { return _pImpl->findLastIgnoreCase(pStr, position); }

        /**
         *  Finds the last occurrence of a substring within a string,
         *  ignoring letter case.
         *
         *  @param str          The string to search for.
         *  @param position     Upon successful search, this parameter is assigned
         *                      a position within the string where the match was found.
         *  @return             True if the search was successful; false otherwise.
         *  @since 1.0
         */
        bool                    findLastIgnoreCase(const String & str, UIntSize & position) const { return _pImpl->findLastIgnoreCase(*str._pImpl, position); }

        /**
         *  Checks if the string is equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             True if this string is equal to \a ch;
         *                      false otherwise.
         *  @since 1.0
         */
        bool                    operator == (char ch) const { return _pImpl->compare(ch) == 0; }

        /**
         *  Checks if the string is equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             True if this string is equal to \a ch;
         *                      false otherwise.
         *  @since 1.0
         */
        bool                    operator == (Char ch) const { return _pImpl->compare(ch) == 0; }

        /**
         *  Checks if the string is equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string 
         *                      to compare this string with.
         *  @return             True if this string is equal to \a pStr;
         *                      false otherwise.
         *  @since 1.0
         */
        bool                    operator == (const char * pStr) const { return _pImpl->compare(pStr) == 0;}

        /**
         *  Checks if the string is equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string 
         *                      to compare this string with.
         *  @return             True if this string is equal to \a pStr;
         *                      false otherwise.
         *  @since 1.0
         */
        bool                    operator == (const Char * pStr) const { return _pImpl->compare(pStr) == 0; }

        /**
         *  Checks if the string is equal to the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             True if this string is equal to \a str;
         *                      false otherwise.
         *  @since 1.0
         */
        bool                    operator == (const String & str) const { return (_pImpl == str._pImpl) || (_pImpl->compare(*str._pImpl) == 0); }

        /**
         *  Checks if the string is not equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             False if this string is equal to \a ch;
         *                      true otherwise.
         *  @since 1.0
         */
        bool                    operator != (char ch) const { return _pImpl->compare(ch) != 0; }

        /**
         *  Checks if the string is not equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             False if this string is equal to \a ch;
         *                      true otherwise.
         *  @since 1.0
         */
        bool                    operator != (Char ch) const { return _pImpl->compare(ch) != 0; }

        /**
         *  Checks if the string is not equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string 
         *                      to compare this string with.
         *  @return             False if this string is equal to \a pStr;
         *                      true otherwise.
         *  @since 1.0
         */
        bool                    operator != (const char * pStr) const { return _pImpl->compare(pStr) != 0; }

        /**
         *  Checks if the string is not equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string 
         *                      to compare this string with.
         *  @return             False if this string is equal to \a pStr;
         *                      true otherwise.
         *  @since 1.0
         */
        bool                    operator != (const Char * pStr) const { return _pImpl->compare(pStr) != 0; }

        /**
         *  Checks if the string is not equal to the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             False if this string is equal to \a str;
         *                      true otherwise.
         *  @since 1.0
         */
        bool                    operator != (const String & str) const { return _pImpl->compare(*str._pImpl) != 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator < (char ch) const { return _pImpl->compare(ch) < 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator < (Char ch) const { return _pImpl->compare(ch) < 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator < (const char * pStr) const { return _pImpl->compare(pStr) < 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator < (const Char * pStr) const { return _pImpl->compare(pStr) < 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than \a str; false otherwise.
         *  @since 1.0
         */
        bool                    operator < (const String & str) const { return _pImpl->compare(*str._pImpl) < 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  or equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than or equal to \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator <= (char ch) const { return _pImpl->compare(ch) <= 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  or equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than or equal to \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator <= (Char ch) const { return _pImpl->compare(ch) <= 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  or equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than or equal to \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator <= (const char * pStr) const { return _pImpl->compare(pStr) <= 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  or equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than or equal to \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator <= (const Char * pStr) const { return _pImpl->compare(pStr) <= 0; }

        /**
         *  Checks if the string is lexicographically smaller than 
         *  or equal to the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             True if this string is lexicographically smaller
         *                      than or equal to \a str; false otherwise.
         *  @since 1.0
         */
        bool                    operator <= (const String & str) const { return _pImpl->compare(*str._pImpl) <= 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator > (char ch) const { return _pImpl->compare(ch) > 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator > (Char ch) const { return _pImpl->compare(ch) > 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator > (const char * pStr) const { return _pImpl->compare(pStr) > 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator > (const Char * pStr) const { return _pImpl->compare(pStr) > 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than \a str; false otherwise.
         *  @since 1.0
         */
        bool                    operator > (const String & str) const { return _pImpl->compare(*str._pImpl) > 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  or equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ASCII character to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than or equal to \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator >= (char ch) const { return _pImpl->compare(ch) >= 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  or equal to the specified character.
         *
         *  The character is treated as if it was a string of length 1.
         *
         *  @param ch           An ISO-10646 character to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than or equal to \a ch; false otherwise.
         *  @since 1.0
         */
        bool                    operator >= (Char ch) const { return _pImpl->compare(ch) >= 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  or equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ASCII string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than or equal to \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator >= (const char * pStr) const { return _pImpl->compare(pStr) >= 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  or equal to the specified string.
         *
         *  @param pStr         A pointer to a nul-terminated ISO-10646 string
         *                      to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than or equal to \a pStr; false otherwise.
         *  @since 1.0
         */
        bool                    operator >= (const Char * pStr) const { return _pImpl->compare(pStr) >= 0; }

        /**
         *  Checks if the string is lexicographically greater than 
         *  or equal to the specified string.
         *
         *  @param str          A string to compare this string with.
         *  @return             True if this string is lexicographically greater
         *                      than or equal to \a str; false otherwise.
         *  @since 1.0
         */
        bool                    operator >= (const String & str) const { return _pImpl->compare(*str._pImpl) >= 0; }

        /**
         *  Returns a character at the specified position.
         *
         *  @param index        The position of the character within the string.
         *  @return             The character at the specified position.
         *  @since 1.0
         */
        Char                    operator[] (UIntSize index) const { return (*_pImpl)[index]; }

        /**
         *  Returns a reference to a character at the specified position.
         *
         *  @param index        The position of the character within the string.
         *  @return             The modifyable reference to a character at the
         *                      specified position.
         *  @since 1.0
         */
        Char &                  operator[] (UIntSize index) { return (*_pImpl)[index]; }

        /**
         *  Concatenates a string with a character.
         *
         *  @param ch           The ASCII character to concatenate with.
         *  @return             The concatenation result.
         *  @since 1.0
         */
        String                  operator + (char ch) const;

        /**
         *  Concatenates a string with a character.
         *
         *  @param ch           The ISO-10646 character to concatenate with.
         *  @return             The concatenation result.
         *  @since 1.0
         */
        String                  operator + (Char ch) const;

        /**
         *  Concatenates two strings.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to concatenate with.
         *  @return             The concatenation result.
         *  @since 1.0
         */
        String                  operator + (const char * pStr) const;

        /**
         *  Concatenates two strings.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to concatenate with.
         *  @return             The concatenation result.
         *  @since 1.0
         */
        String                  operator + (const Char * pStr) const;

        /**
         *  Concatenates two strings.
         *
         *  @param str          The string to concatenate with.
         *  @return             The concatenation result.
         *  @since 1.0
         */
        String                  operator + (const String & str) const;

        /**
         *  Appends a character to the end of the string.
         *
         *  @param ch           The ASCII character to append.
         *  @return             A reference to an appended-to string.
         *  @since 1.0
         */
        String &                operator += (char ch);

        /**
         *  Appends a character to the end of the string.
         *
         *  @param ch           The ISO-10646 character to append.
         *  @return             A reference to an appended-to string.
         *  @since 1.0
         */
        String &                operator += (Char ch);

        /**
         *  Appends a string to the end of the string.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string to append.
         *  @return             A reference to an appended-to string.
         *  @since 1.0
         */
        String &                operator += (const char * pStr);

        /**
         *  Appends a string to the end of the string.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string to append.
         *  @return             A reference to an appended-to string.
         *  @since 1.0
         */
        String &                operator += (const Char * pStr);

        /**
         *  Appends a string to the end of the string.
         *
         *  @param str          The string to append.
         *  @return             A reference to an appended-to string.
         *  @since 1.0
         */
        String &                operator += (const String & str);

        /**
         *  Inserts a character into the string at the specified position.
         *
         *  @param ch           The ASCII character to insert.
         *  @param atIndex      The position at which to insert.
         *  @since 1.0
         */
        void                    insert(char ch, UIntSize atIndex);

        /**
         *  Inserts a character into the string at the specified position.
         *
         *  @param ch           The ISO-10646 character to insert.
         *  @param atIndex      The position at which to insert.
         *  @since 1.0
         */
        void                    insert(Char ch, UIntSize atIndex);

        /**
         *  Inserts a substring into the string at the specified position.
         *
         *  @param pStr         The pointer to a nul-terminated ASCII string
         *                      to insert.
         *  @param atIndex      The position at which to insert.
         *  @since 1.0
         */
        void                    insert(const char * pStr, UIntSize atIndex);

        /**
         *  Inserts a substring into the string at the specified position.
         *
         *  @param pStr         The pointer to a nul-terminated ISO-10646 string
         *                      to insert.
         *  @param atIndex      The position at which to insert.
         *  @since 1.0
         */
        void                    insert(const Char * pStr, UIntSize atIndex);

        /**
         *  Inserts a substring into the string at the specified position.
         *
         *  @param str          The string to insert.
         *  @param atIndex      The position at which to insert.
         *  @since 1.0
         */
        void                    insert(const String & str, UIntSize atIndex);

        /**
         *  Removes a substring from a string.
         *
         *  @param startIndex   The position where the removed substring starts.
         *  @param charCount    The number of characters to remove.
         *  @since 1.0
         */
        void                    remove(UIntSize startIndex, UIntSize charCount);

        /**
         *  Replaces a substring within this string with the specified character.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param ch           The ASCII character to replace the substring with.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, char ch);

        /**
         *  Replaces a substring within this string with the specified character.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param ch           The ISO-10646 character to replace the substring with.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, Char ch);

        /**
         *  Replaces a substring within this string with the specified string.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param pStr         The pointer to the nul-terminated ASCII string
         *                      to replace the substring with.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, const char * pStr);

        /**
         *  Replaces a substring within this string with the specified string.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param pStr         The pointer to the nul-terminated ISO-10646 string
         *                      to replace the substring with.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, const Char * pStr);

        /**
         *  Replaces a substring within this string with the specified character sequence.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param pChars       The pointer to the ASCII character sequence to replace
         *                      the substring with. The sequence must be at least
         *                      \a replacementCharCount characters long; if it is 
         *                      longer then only the first \a replacementCharCount
         *                      characters are used.
         *  @param replacementCharCount The length of the replacement character sequence.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, const char * pChars, UIntSize replacementCharCount);

        /**
         *  Replaces a substring within this string with the specified character sequence.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param pChars       The pointer to the ISO-10646 character sequence to replace
         *                      the substring with. The sequence must be at least
         *                      \a replacementCharCount characters long; if it is 
         *                      longer then only the first \a replacementCharCount
         *                      characters are used.
         *  @param replacementCharCount The length of the replacement character sequence.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, const Char * pChars, UIntSize replacementCharCount);

        /**
         *  Replaces a substring within this string with the specified string.
         *
         *  @param startIndex   The position where the replaced substring starts.
         *  @param charCount    The number of characters to replace.
         *  @param str          The string to replace the substring with.
         *  @since 1.0
         */
        void                    replace(UIntSize startIndex, UIntSize charCount, const String & str);

        /**
         *  Returns a left substring of this string.
         *
         *  The returned value contains the \a charCount leftmost characters
         *  of this string. If \a charCount is greater than the length of
         *  this string, the entire string is returned.
         *
         *  @param charCount    The length of the required substring.
         *  @return             The left substring of this string.
         *  @since 1.0
         */
        String                  left(UIntSize charCount) const { return String(_pImpl->left(charCount)); }

        /**
         *  Returns a right substring of this string.
         *
         *  The returned value contains the \a charCount righttmost characters
         *  of this string. If \a charCount is greater than the length of
         *  this string, the entire string is returned.
         *
         *  @param charCount    The length of the required substring.
         *  @return             The right substring of this string.
         *  @since 1.0
         */
        String                  right(UIntSize charCount) const { return String(_pImpl->right(charCount)); }

        /**
         *  Returns a substring of this string.
         *
         *  The extracted substring starts at position \a startIndex and
         *  continues to the end of the string.
         *
         *  @param startIndex   The position where the required substring starts.
         *  @return             The extracted substring.
         *  @since 1.0
         */
        String                  mid(UIntSize startIndex) const { return String(_pImpl->mid(startIndex)); }

        /**
         *  Returns a substring of this string.
         *
         *  The extracted substring starts at position \a startIndex and
         *  is \a charCount characters long.
         *
         *  @param startIndex   The position where the required substring starts.
         *  @param charCount    The number of characters in the required substring.
         *  @return             The extracted substring.
         *  @since 1.0
         */
        String                  mid(UIntSize startIndex, UIntSize charCount) const { return String(_pImpl->mid(startIndex, charCount)); }

        /**
         *  Removes all leading spaces from this string.
         *
         *  @since 1.0
         */
        void                    trimLeft();

        /**
         *  Removes all tailing spaces from this string.
         *
         *  @since 1.0
         */
        void                    trimRight();

        /**
         *  Removes all leading and tailing spaces from this string.
         *
         *  @since 1.0
         */
        void                    trim();

        /**
         *  Returns this string with all leading spaces removed.
         *
         *  @return             This string with all leading spaces removed.
         *  @since 1.0
         */
        String                  trimmedLeft() const;

        /**
         *  Returns this string with all twiling spaces removed.
         *
         *  @return             This string with all tailing spaces removed.
         *  @since 1.0
         */
        String                  trimmedRight() const;

        /**
         *  Returns this string with all leading and tailing spaces removed.
         *
         *  @return             This string with all leading and tailing spaces removed.
         *  @since 1.0
         */
        String                  trimmed() const;

        /**
         *  Converts this string to uppercase.
         *
         *  @since 1.0
         */
        void                    upperCase();

        /**
         *  Converts this string to lowercase.
         *
         *  @since 1.0
         */
        void                    lowerCase();

        /**
         *  Converts this string to titlecase.
         *
         *  @since 1.0
         */
        void                    titleCase();

        /**
         *  Returns this string converted to uppercase.
         *
         *  @return             This string converted to uppercase.
         *  @since 1.0
         */
        String                  upperCased() const;

        /**
         *  Returns this string converted to lowercase.
         *
         *  @return             This string converted to lowercase.
         *  @since 1.0
         */
        String                  lowerCased() const;

        /**
         *  Returns this string converted to titlecase.
         *
         *  @return             This string converted to titlecase.
         *  @since 1.0
         */
        String                  titleCased() const;

        //////////
        //  Implementation
    private:
        _impl::_StringImpl *    _pImpl;
        void                    _ensureNonsharedImplementation();
    #if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        bool                    _store(wchar_t * pBuffer, UIntSize bufferSize) const;
    #endif
        bool                    _store(char * pBuffer, UIntSize bufferSize) const;
    };

    //////////
    //  Additional string operators
    /**
     *  Concatenates a character with a string.
     *
     *  @param ch           The ASCII character to concatenate.
     *  @param str          The string to concatenate.
     *  @return             The concatenation result.
     *  @since 1.0
     */
    inline String CARL_API operator + (char ch, const String & str)
    {
        return String(ch) + str;
    }

    /**
     *  Concatenates a character with a string.
     *
     *  @param ch           The ISO-10646 character to concatenate.
     *  @param str          The string to concatenate.
     *  @return             The concatenation result.
     *  @since 1.0
     */
    inline String CARL_API operator + (Char ch, const String & str)
    {
        return String(ch) + str;
    }

    /**
     *  Concatenates a string with another string.
     *
     *  @param pStr         The pointer to a nul-terminated ASCII string
     *                      to concatenate.
     *  @param str          The string to concatenate.
     *  @return             The concatenation result.
     *  @since 1.0
     */
    inline String CARL_API operator + (const char * pStr, const String & str)
    {
        return String(pStr) + str;
    }

    /**
     *  Concatenates a string with another string.
     *
     *  @param pStr         The pointer to a nul-terminated ISO-10646 string
     *                      to concatenate.
     *  @param str          The string to concatenate.
     *  @return             The concatenation result.
     *  @since 1.0
     */
    inline String CARL_API operator + (const Char * pStr, const String & str)
    {
        return String(pStr) + str;
    }

    //////////
    //  Hashing strings
    namespace StandardHashFunctions
    {
        /**
         *  Calculates a case-sensitive hash code for a String instance.
         *
         *  @param key              The String instance to hash.
         *  @return                 The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashString(const String & key);

        /**
         *  Calculates a case-insensitive hash code for a String instance.
         *
         *  @param key              The String instance to hash.
         *  @return                 The hash code.
         *  @since 1.0
         */
        CARL_PUBLIC UIntSize CARL_CALLBACK hashStringIgnoreCase(const String & key);
    }
}

#endif  //  __CARL_String_hpp_included__
//  End of CARL_String.hpp
