//
//  Application.cpp - Implementation of the carl::Application class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/28 10:40:28 $
//  $Revision: 1.25 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #define WinMain WinMainOld
    #include <windows.h>
    #undef WinMain
    #include <fcntl.h>
    #include <stdlib.h>
#endif
#include <string.h>

#include <stdio.h>

namespace
{
    void initializeApplication()
    {
        //  Initialize the heap manager
        HeapManager::getInstance();

        //  Make sure the application is compatible with CARL
        if (strcmp(CARL_BUILD_KEY, System::getBuildKey()) != 0)
        {
            throw VersionCompatibilityError();
        }

        //  Load installed extensions automatically
#if defined(CARL_LINKAGE_DYNAMIC)
        ExtensionLoader::loadInstalledExtensions();
#endif  //  defined(CARL_LINKAGE_DYNAMIC)
    }
}

//
Application * Application::_pApplication = NULL;
Int64 Application::_startTimeNanosecondsFromReferencePoint = TimeSpan::_InvalidValue;

//////////
//  Program entry points
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
extern "C"
CARL_PUBLIC int main(int argc, char ** argv)
{
#if defined(CARL_TOOLCHAIN_MSVC6) || defined(CARL_TOOLCHAIN_MSVCNET2003)
    _fmode = _O_BINARY;
#else
    _set_fmode(_O_BINARY);
#endif
    initializeApplication();
    Array<String> args;
    for (int i = 0; i < argc; i++)
    {
        args.append(String(argv[i]));
    }
    try
    {
        return Application::getApplication()->run(args);
    }
    catch (Throwable & e)
    {
        StandardStreams::err.writeLn(e.toString());
        ExitProcess(1);
    }
}

extern "C"
CARL_PUBLIC int CALLBACK WinMain(HINSTANCE /*hInstance*/, HINSTANCE /*hPrevInstance*/, LPSTR /*lpCmdLine*/, int /*nCmdShow*/)
{
#if defined(CARL_TOOLCHAIN_MSVC6) || defined(CARL_TOOLCHAIN_MSVCNET2003)
    _fmode = _O_BINARY;
#else
    _set_fmode(_O_BINARY);
#endif
    initializeApplication();
    Array<String> args;
    int argc = 0;
    LPWSTR* argv = CommandLineToArgvW(GetCommandLineW(), &argc);
    require(argv != NULL);
    for (int i = 0; i < argc; i++)
    {
        args.append(String((const wchar_t*)argv[i]));
    }
    GlobalFree(argv);
    try
    {
        return Application::getApplication()->run(args);
    }
    catch (Throwable & e)
    {
        StandardStreams::err.writeLn(e.toString());
        ExitProcess(1);
    }
}

#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
const char * _CARL_pArgv0 = NULL;

extern "C"
CARL_PUBLIC int main(int argc, char ** argv)
{
    initializeApplication();

    Array<String> args;
    for (int i = 0; i < argc; i++)
    {
        args.append(String(argv[i]));
    }
    _CARL_pArgv0 = argv[0];
    try
    {
        return Application::getApplication()->run(args);
    }
    catch (Throwable & e)
    {
        StandardStreams::err.writeLn(e.toString());
        throw;
    }
}
#else
    #error Unsupported platform
#endif

//  End of Application.cpp
