//
//  AsciiCharacterSet.cpp - Implementation of the carl::_impl::_AsciiCharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Singleton
_impl::_AsciiCharacterSet::_AsciiCharacterSet()
{
    _names.append(String("ANSI_X3.4-1968"));
    _names.append(String("iso-ir-6"));
    _names.append(String("ANSI_X3.4-1986"));
    _names.append(String("ISO_646.irv:1991"));
    _names.append(String("ASCII"));
    _names.append(String("ISO646-US"));
    _names.append(String("US-ASCII"));
    _names.append(String("us"));
    _names.append(String("IBM367"));
    _names.append(String("cp367"));
    _names.append(String("csASCII"));
}

_impl::_AsciiCharacterSet::~_AsciiCharacterSet()
{
}

_impl::_AsciiCharacterSet * _impl::_AsciiCharacterSet::getInstance()
{
    static _AsciiCharacterSet instance;
    return &instance;
}

//////////
//  Operations
CharacterSetFamily * _impl::_AsciiCharacterSet::getFamily() const
{
    return _MiscellaneousCharacterSetFamily::getInstance();
}

Array<String> _impl::_AsciiCharacterSet::getAllNames() const
{
    return _names;
}

String _impl::_AsciiCharacterSet::getCanonicalName() const
{
    return String("ANSI_X3.4-1968");
}

String _impl::_AsciiCharacterSet::getPreferredName() const
{
    return String("US-ASCII");
}

String _impl::_AsciiCharacterSet::getDisplayName() const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), Locale::getDefault());
}

String _impl::_AsciiCharacterSet::getDisplayName(const Locale & forLocale) const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), forLocale);
}

int _impl::_AsciiCharacterSet::getMaximumEncodingLength() const
{
    return 1;
}

int _impl::_AsciiCharacterSet::getEncodingLengthForCharacter(Char c) const
{
    if ((c & 0xFFFFFF80) == 0)
    {
        return 1;
    }
    return -1;
}

int _impl::_AsciiCharacterSet::encodeCharacter(Char c, void * buffer) const
{
    if ((c & 0xFFFFFF80) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(c);
        return 1;
    }
    return -1;
}

int _impl::_AsciiCharacterSet::getEncodingLengthForLeadByte(UInt8 leadByte) const
{
    if ((leadByte & 0x80) == 0)
    {
        return 1;
    }
    return -1;
}

int _impl::_AsciiCharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
    if ((pBytes[0] & 0x80) == 0 && bytesAvailable >= 1)
    {
        c = pBytes[0];
        return 1;
    }
    return -1;
}

Char _impl::_AsciiCharacterSet::getDefaultCharacter() const
{
    return '?';
}

Array<UInt8> _impl::_AsciiCharacterSet::getSignature() const
{
    return Array<UInt8>();
}

//  End of AsciiCharacterSet.cpp
