//
//  AssertionError.cpp - Implementation of the carl::AssertionError class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "ExceptionImpl.hpp"

//////////
//  Construction/destruction/assignment
AssertionError::AssertionError(const char * pAssertionText, const char * pSourceFileName, UIntSize sourceLineNumber)
    :   _pAssertionText(pAssertionText),
        _pSourceFileName(pSourceFileName),
        _sourceLineNumber(sourceLineNumber)
{
}

AssertionError::AssertionError(const AssertionError & src)
    :   _pAssertionText(src._pAssertionText),
        _pSourceFileName(src._pSourceFileName),
        _sourceLineNumber(src._sourceLineNumber)
{
}

AssertionError::~AssertionError()
{
}

AssertionError & AssertionError::operator = (const AssertionError & src)
{
    if (this != (&src))
    {
        _pAssertionText = src._pAssertionText;
        _pSourceFileName = src._pSourceFileName;
        _sourceLineNumber = src._sourceLineNumber;
    }
    return *this;
}

//////////
//  Operations
const char * AssertionError::getAssertionText() const
{
    return _pAssertionText;
}

const char * AssertionError::getSourceFileName() const
{
    return _pSourceFileName;
}

UIntSize AssertionError::getSourceLineNumber() const
{
    return _sourceLineNumber;
}

String AssertionError::toString() const
{
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("AssertionError", String(_pAssertionText), String(_pSourceFileName), NumberFormat::getInstance()->format(_sourceLineNumber));
}

String AssertionError::toString(const Locale & forLocale) const
{
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("AssertionError", String(_pAssertionText), String(_pSourceFileName), NumberFormat::getInstance()->format(_sourceLineNumber), forLocale);
}

//  End of AssertionError.cpp
