//
//  BlowfishCipher.cpp - Implementation of the carl::_impl::_BlowfishCipher class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Singleton
_impl::_BlowfishCipher::_BlowfishCipher()
{
}

_impl::_BlowfishCipher::~_BlowfishCipher()
{
}

_impl::_BlowfishCipher * _impl::_BlowfishCipher::getInstance()
{
    static _BlowfishCipher instance;
    return &instance;
}

//////////
//  BlockCipher
Array<UIntSize> _impl::_BlowfishCipher::getSupportedBlockLenghts() const
{
    Array<UIntSize> result;
    result.append(64);
    return result;
}

//////////
//  SymmetricCipher
Array<UIntSize> _impl::_BlowfishCipher::getSupportedKeyLenghts() const
{
    Array<UIntSize> result;
    for (UIntSize i = 32; i <= 448; i += 8)
    {
        result.append(i);
    }
    return result;
}

SecretKey _impl::_BlowfishCipher::generateKey(UInt64 seed, UIntSize keyLengthBits) const
{
    require(System::BitsPerByte == 8);

    if (!getSupportedKeyLenghts().contains(keyLengthBits))
    {
        throw InvalidKeyLengthError();
    }
    Random r(static_cast<UInt32>(seed) ^ static_cast<UInt32>(seed >> 32));
    Array<UInt8> keyBytes;
    for (UIntSize i = 0; i < keyLengthBits / 8; i++)
    {
        keyBytes.append(r.nextUInt8());
    }
    return SecretKey(keyBytes.getData(), keyLengthBits);
}

SecretKey _impl::_BlowfishCipher::generateRandomKey(UIntSize keyLengthBits) const
{
    Random r;
    return generateKey(r.nextUInt64(), keyLengthBits);
}

//////////
//  Cipher
Array<String> _impl::_BlowfishCipher::getAllNames() const
{
    Array<String> result;
    result.append(getCanonicalName());
    return result;
}

String _impl::_BlowfishCipher::getCanonicalName() const
{
    return String("Blowfish");
}

String _impl::_BlowfishCipher::getPreferredName() const
{
    return getCanonicalName();
}

String _impl::_BlowfishCipher::getDisplayName() const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName());
}

String _impl::_BlowfishCipher::getDisplayName(const Locale & forLocale) const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName(), forLocale);
}

Encryptor * _impl::_BlowfishCipher::createEncryptor(const CryptoKey & key) const throw(KeyCorruptException)
{
    if (!getSupportedKeyLenghts().contains(key.getKeyLengthBits()))
    {
        throw InvalidKeyLengthError();
    }
    return new _BlowfishEncryptor(key);
}

Decryptor * _impl::_BlowfishCipher::createDecryptor(const CryptoKey & key) const throw(KeyCorruptException)
{
    if (!getSupportedKeyLenghts().contains(key.getKeyLengthBits()))
    {
        throw InvalidKeyLengthError();
    }
    return new _BlowfishDecryptor(key);
}

//  End of BlowfishCipher.cpp
