//
//  BlowfishDecryptor.cpp - Implementation of the carl::_impl::_BlowfishDecryptor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Construction/destruction
_impl::_BlowfishDecryptor::_BlowfishDecryptor(const CryptoKey & key)
    :   _BlowfishTransformer(key)
{
}

_impl::_BlowfishDecryptor::~_BlowfishDecryptor()
{
}

//////////
//  Operations
Cipher * _impl::_BlowfishDecryptor::getCipher() const
{
    return _BlowfishCipher::getInstance();
}

UIntSize _impl::_BlowfishDecryptor::getCleartextSizeForCiphertext(const void * /*pCiphertextBits*/, UIntSize ciphertextBitsCount)
{
    require(System::BitsPerByte == 8);

    if (ciphertextBitsCount != 64)
    {
        throw InvalidCiphertextBlockSizeError();
    }
    return 64;
}

UIntSize _impl::_BlowfishDecryptor::decryptBlock(const void * pCiphertextBits, UIntSize ciphertextBitsCount, void * pCleartextBits) throw(EncryptedDataCorruptException)
{
    require(pCleartextBits != NULL);
    require(System::BitsPerByte == 8);

    if (ciphertextBitsCount != 64)
    {
        throw InvalidCiphertextBlockSizeError();
    }
    const UInt8 * pCiphertextBytes = static_cast<const UInt8 *>(pCiphertextBits);
    UInt8 * pCleartextBytes = static_cast<UInt8 *>(pCleartextBits);
    UInt32 xl = Memory::loadUInt32(pCiphertextBytes, ByteOrder_BigEndian);
    UInt32 xr = Memory::loadUInt32(pCiphertextBytes + 4, ByteOrder_BigEndian);
    _decrypt(xl, xr);
    Memory::storeUInt32(xl, pCleartextBytes, ByteOrder_BigEndian);
    Memory::storeUInt32(xr, pCleartextBytes + 4, ByteOrder_BigEndian);
    return 64;
}

//  End of BlowfishDecryptor.cpp
