//
//  BufferedInputStream.cpp - Implementation of the carl::BufferedInputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
BufferedInputStream::BufferedInputStream(IByteInput * pSource)
{
    require(pSource != NULL);

    _pSource = pSource;
    _bufferSize = _DefaultBufferSize;
    _pBuffer = new UInt8[_bufferSize];
    _bytesInBuffer = 0;
    _currentPosition = 0;
}

BufferedInputStream::BufferedInputStream(IByteInput * pSource, UIntSize bufferSize)
{
    require(pSource != NULL);
    require(bufferSize != 0);

    _pSource = pSource;
    _bufferSize = bufferSize;
    _pBuffer = new UInt8[_bufferSize];
    _bytesInBuffer = 0;
    _currentPosition = 0;
}

//////////
//  Operations
void BufferedInputStream::close()
{
    delete [] _pBuffer;
    _pBuffer = NULL;
}

//////////
//  IByteInput
UIntSize BufferedInputStream::available() throw(IOException)
{
    require(isOpen());

    UIntSize avail1 = _bytesInBuffer - _currentPosition;
    UIntSize avail2 = _pSource->available();
    UIntSize avail = avail1 + avail2;
    if (avail < avail2)
    {
        return (avail1 > avail2) ? (avail1) : (avail2);
    }
    return avail;
}

int BufferedInputStream::read() throw(IOException)
{
    require(isOpen());

    if (_currentPosition == _bytesInBuffer)
    {
        _refillBuffer();
        if (_currentPosition == _bytesInBuffer)
        {
            return -1;
        }
    }
    return _pBuffer[_currentPosition++];
}

UIntSize BufferedInputStream::read(void * pBuffer, UIntSize bytesToRead) throw(IOException)
{
    UInt8 * pBytes = static_cast<UInt8*>(pBuffer);
    for (UIntSize i = 0; i < bytesToRead; i++)
    {
        int bb = read();
        if (bb == -1)
        {
            return i;
        }
        pBytes[i] = static_cast<UInt8>(bb);
    }
    return bytesToRead;
}

UIntSize BufferedInputStream::skip(UIntSize bytesToSkip) throw(IOException)
{
    for (UIntSize i = 0; i < bytesToSkip; i++)
    {
        if (read() == -1)
        {
            return i;
        }
    }
    return bytesToSkip;
}

//////////
//  Implementation helpers
void BufferedInputStream::_refillBuffer()
{
    _bytesInBuffer = _pSource->read(_pBuffer, _bufferSize);
    _currentPosition = 0;
}

//  End of BufferedInputStream.cpp
