//
//  BufferedOutputStream.cpp - Implementation of the carl::BufferedOutputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
BufferedOutputStream::BufferedOutputStream(IByteOutput * pTarget)
{
    require(pTarget != NULL);

    _pTarget = pTarget;
    _bufferSize = _DefaultBufferSize;
    _pBuffer = new UInt8[_bufferSize];
    _bytesInBuffer = 0;
}

BufferedOutputStream::BufferedOutputStream(IByteOutput * pTarget, UIntSize bufferSize)
{
    require(pTarget != NULL);
    require(bufferSize != 0);

    _pTarget = pTarget;
    _bufferSize = bufferSize;
    _pBuffer = new UInt8[_bufferSize];
    _bytesInBuffer = 0;
}

//////////
//  Operations
void BufferedOutputStream::close()
{
    if (_pBuffer != NULL)
    {
        _flushBuffer();
        delete [] _pBuffer;
        _pBuffer = NULL;
    }
}

//////////
//  IByteOutput
void BufferedOutputStream::flush() throw(IOException)
{
    require(isOpen());

    _flushBuffer();
    _pTarget->flush();
}

void BufferedOutputStream::write(UInt8 byte) throw(IOException)
{
    write(&byte, 1);
}

void BufferedOutputStream::write(const void * pBuffer, UIntSize bytesToWrite) throw(IOException)
{
    require(isOpen());

    const UInt8 * pBytes = static_cast<const UInt8*>(pBuffer);
    UIntSize bytesFreeInBuffer = _bufferSize - _bytesInBuffer;
    if (bytesToWrite <= bytesFreeInBuffer)
    {   //  Can write everything to buffer
        Memory::copy(_pBuffer + _bytesInBuffer, pBytes, bytesToWrite);
        _bytesInBuffer += bytesToWrite;
    }
    else if (bytesToWrite <= _bufferSize)
    {   //  There will be enough space in buffer once we flush it
        _flushBuffer();
        Memory::copy(_pBuffer + _bytesInBuffer, pBytes, bytesToWrite);
        _bytesInBuffer += bytesToWrite;
    }
    else
    {   //  We are writing more bytes that can fit in buffer. It's more
        //  efficient to write directly to the target stream.
        _flushBuffer();
        _pTarget->write(pBuffer, bytesToWrite);
    }
}

//////////
//  Implementation helpers
void BufferedOutputStream::_flushBuffer()
{
    if (_bytesInBuffer != 0)
    {
        _pTarget->write(_pBuffer, _bytesInBuffer);
        _bytesInBuffer = 0;
    }
}

//  End of BufferedOutputStream.cpp
