//
//  Calendar.cpp - Implementation of the carl::Calendar class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeImpl.hpp"

//////////
//  Date helpers
int Calendar::daysInMonth(int year, int month)
{
    require(month >= 1 && month <= 12);
    require(year != 0);

    switch (month)
    {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:
            return 31;
        case 4:
        case 6:
        case 9:
        case 11:
            return 30;
        case 2:
            if (isLeapYear(year))
            {
                return 29;
            }
            else
            {
                return 28;
            }
        default:
            return 0;
    }
}

DayOfWeek Calendar::getDayOfWeek(int year, int month, int day)
{
    require(year != 0);
    require(month >= 1 && month <= 12);
    require(day >= 1 && day <= Calendar::daysInMonth(year, month));

    UInt32 daysSinceBeginningOfTime = day - 1;
    while (month != 1)
    {
        month--;
        daysSinceBeginningOfTime += daysInMonth(year, month);
    }
    while (year != 1)
    {
        year --;
        daysSinceBeginningOfTime += daysInYear(year);
    }
    return static_cast<DayOfWeek>((daysSinceBeginningOfTime) % 7);
}

int Calendar::getDayOfYear(int year, int month, int day)
{
    require(year != 0);
    require(month >= 1 && month <= 12);
    require(day >= 1 && day <= Calendar::daysInMonth(year, month));

    int result = day;
    for (; month != 1; month--)
    {
        int daysInPreviousMonth = daysInMonth(month - 1, year);
        result += daysInPreviousMonth;

    }
    return result;
}

//////////
//  Display names
String Calendar::getDayOfWeekShortDisplayName(int dayOfWeek, const Locale & forLocale)
{
    require(dayOfWeek >= 0 && dayOfWeek <= 6);

    switch (dayOfWeek)
    {
        case DayOfWeek_Monday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MondayShortDisplayName, forLocale);
        case DayOfWeek_Tuesday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::TuesdayShortDisplayName, forLocale);
        case DayOfWeek_Wednesday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::WednesdayShortDisplayName, forLocale);
        case DayOfWeek_Thursday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::ThursdayShortDisplayName, forLocale);
        case DayOfWeek_Friday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::FridayShortDisplayName, forLocale);
        case DayOfWeek_Saturday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SaturdayShortDisplayName, forLocale);
        case DayOfWeek_Sunday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SundayShortDisplayName, forLocale);
        default:
            return String("");
    }
}

String Calendar::getDayOfWeekFullDisplayName(int dayOfWeek, const Locale & forLocale)
{
    require(dayOfWeek >= 0 && dayOfWeek <= 6);

    switch (dayOfWeek)
    {
        case DayOfWeek_Monday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MondayLongDisplayName, forLocale);
        case DayOfWeek_Tuesday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::TuesdayLongDisplayName, forLocale);
        case DayOfWeek_Wednesday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::WednesdayLongDisplayName, forLocale);
        case DayOfWeek_Thursday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::ThursdayLongDisplayName, forLocale);
        case DayOfWeek_Friday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::FridayLongDisplayName, forLocale);
        case DayOfWeek_Saturday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SaturdayLongDisplayName, forLocale);
        case DayOfWeek_Sunday:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SundayLongDisplayName, forLocale);
        default:
            return String("");
    }
}

String Calendar::getMonthShortDisplayName(int month, const Locale & forLocale)
{
    require(month >= 1 && month <= 12);

    switch (month)
    {
        case Month_January:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JanuaryShortDisplayName, forLocale);
        case Month_February:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::FebruaryShortDisplayName, forLocale);
        case Month_March:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MarchShortDisplayName, forLocale);
        case Month_April:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::AprilShortDisplayName, forLocale);
        case Month_May:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MayShortDisplayName, forLocale);
        case Month_June:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JuneShortDisplayName, forLocale);
        case Month_July:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JulyShortDisplayName, forLocale);
        case Month_August:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::AugustShortDisplayName, forLocale);
        case Month_September:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SeptemberShortDisplayName, forLocale);
        case Month_October:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::OctoberShortDisplayName, forLocale);
        case Month_November:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::NovemberShortDisplayName, forLocale);
        case Month_December:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::DecemberShortDisplayName, forLocale);
        default:
            return String("");
    }
}

String Calendar::getMonthFullDisplayName(int month, const Locale & forLocale)
{
    require(month >= 1 && month <= 12);

    switch (month)
    {
        case Month_January:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JanuaryLongDisplayName, forLocale);
        case Month_February:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::FebruaryLongDisplayName, forLocale);
        case Month_March:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MarchLongDisplayName, forLocale);
        case Month_April:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::AprilLongDisplayName, forLocale);
        case Month_May:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::MayLongDisplayName, forLocale);
        case Month_June:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JuneLongDisplayName, forLocale);
        case Month_July:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::JulyLongDisplayName, forLocale);
        case Month_August:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::AugustLongDisplayName, forLocale);
        case Month_September:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::SeptemberLongDisplayName, forLocale);
        case Month_October:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::OctoberLongDisplayName, forLocale);
        case Month_November:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::NovemberLongDisplayName, forLocale);
        case Month_December:
            return _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_DateTimeManagerImpl::DecemberLongDisplayName, forLocale);
        default:
            return String("");
    }
}

//  End of Calendar.cpp
