//
//  CharacterSet.cpp - Implementation of the carl::CharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"
#include <stdlib.h>

//////////
//  Operations
void CharacterSet::registerInstance(CharacterSet * pCharacterSet) throw (InconsistentCharacterSetException, DuplicateCharacterSetException)
{
    require(pCharacterSet != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->registerCharacterSet(pCharacterSet);
}

Array<CharacterSet*> CharacterSet::getAllInstances()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->getAllCharacterSets();
}

CharacterSet * CharacterSet::getInstance(const char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

CharacterSet * CharacterSet::getInstance(const Char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

CharacterSet * CharacterSet::getInstance(const String & name)
{
    CriticalSectionLock lock(*_getStaticGuard());
    CharacterSet * pCharacterSet = _impl::_CharsetManagerImpl::getInstance()->findCharacterSet(name);
    if (pCharacterSet != NULL)
    {
        return pCharacterSet;
    }
    throw NoSuchElementError();
}

CharacterSet * CharacterSet::getSystem()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->getSystem();
}

CharacterSet * CharacterSet::getDefault()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->getDefault();
}

void CharacterSet::setDefault(CharacterSet * pCharacterSet)
{
    require(pCharacterSet != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_CharsetManagerImpl::getInstance()->setDefault(pCharacterSet);
}

//////////
//  Implementation
CriticalSection * CharacterSet::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of CharacterSet.cpp
