//
//  CharacterSetFamily.cpp - Implementation of the carl::CharacterSetFamily class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"
#include <stdio.h>

//////////
//  Construction/destruction - from derived classes only
CharacterSetFamily::CharacterSetFamily()
{
}

CharacterSetFamily::~CharacterSetFamily()
{
}

//////////
//  Operations
void CharacterSetFamily::registerInstance(CharacterSetFamily * pCharacterSetFamily) throw (InconsistentCharacterSetFamilyException, DuplicateCharacterSetFamilyException)
{
    require(pCharacterSetFamily != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->registerCharacterSetFamily(pCharacterSetFamily);
}

Array<CharacterSetFamily*> CharacterSetFamily::getAllInstances()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CharsetManagerImpl::getInstance()->getAllCharacterSetFamilies();
}

CharacterSetFamily * CharacterSetFamily::getInstance(const char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

CharacterSetFamily * CharacterSetFamily::getInstance(const Char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

CharacterSetFamily * CharacterSetFamily::getInstance(const String & name)
{
    CriticalSectionLock lock(*_getStaticGuard());
    CharacterSetFamily * pCharacterSetFamily = _impl::_CharsetManagerImpl::getInstance()->findCharacterSetFamily(name);
    if (pCharacterSetFamily != NULL)
    {
        return pCharacterSetFamily;
    }
    throw NoSuchElementError();
}

Array<CharacterSet*> CharacterSetFamily::getAllMembers() const
{
    Array<CharacterSet*> allCharacterSets = CharacterSet::getAllInstances();
    Array<CharacterSet*> result;
    for (UIntSize i = 0; i < allCharacterSets.size(); i++)
    {
        if (allCharacterSets[i]->getFamily() == this)
        {
            result.append(allCharacterSets[i]);
        }
    }
    return result;
}

CharacterSet * CharacterSetFamily::getMember(const char * pName) const
{
    return getMember(String(pName));
}

CharacterSet * CharacterSetFamily::getMember(const Char * pName) const
{
    return getMember(String(pName));
}

CharacterSet * CharacterSetFamily::getMember(const String & name) const
{
    CharacterSet * pCharacterSet = CharacterSet::getInstance(name);
    if (pCharacterSet->getFamily() != this)
    {
        throw NoSuchElementError();
    }
    return pCharacterSet;
}

//////////
//  Implementation
CriticalSection * CharacterSetFamily::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of CharacterSetFamily.cpp
