//
//  CharsetImpl.hpp - carl::_impl::_CharsetImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/07 19:25:06 $
//  $Revision: 1.17 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __CharsetImpl_hpp_included__
#define __CharsetImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Character set resources
        //  Character set display names use resource keys CS:<canonical charset name>
        //  Character set family display names use resource keys CSF:<canonical charset family name>
        class CARL_PUBLIC _CharsetResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _CharsetResourceMap_en(const _CharsetResourceMap_en &);
            void                        operator = (const _CharsetResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _CharsetResourceMap_en();
            virtual ~_CharsetResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String              getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>     _mapKeysToValues;
        };

        //////////
        //  A generic SBCS
        class CARL_PUBLIC _SingleByteCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _SingleByteCharacterSet(const _SingleByteCharacterSet &);
            void                        operator = (const _SingleByteCharacterSet &);

            //////////
            //  Construction/destruction - from derived classes only
        protected:
            explicit _SingleByteCharacterSet(const Char * pCharTable256);
            virtual ~_SingleByteCharacterSet();

            //////////
            //  Operations
        public:
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;

            //////////
            //  Implementation
        private:
            const Char *                _pCharTable256;
            mutable bool                _initialized;
            mutable HashMap<Char, UInt8>_mapCharsToBytes;
            //  Helpers
            void                        _initialize() const;
        };

        //////////
        //  A generic MBCS with maximum character length 2 bytes
        class CARL_PUBLIC _MultiByte2CharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MultiByte2CharacterSet(const _MultiByte2CharacterSet &);
            void                        operator = (const _MultiByte2CharacterSet &);

            //////////
            //  Construction/destruction - from derived classes only
        protected:
            _MultiByte2CharacterSet(const Char * pSingleByteCharTable256, const Char *const *ppDoubleByteCharTables256);
            virtual ~_MultiByte2CharacterSet();

            //////////
            //  Operations
        public:
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;

            //////////
            //  Implementation
        private:
            const Char *                _pSingleByteCharTable256;
            const Char *const *         _ppDoubleByteCharTables256;
            mutable bool                _initialized;
            mutable HashMap<Char, UInt8>_mapSingleByteCharsToBytes;
            mutable HashMap<Char, UInt16>   _mapDoubleByteCharsToBytePairs;
            //  Helpers
            void                        _initialize() const;
        };

        //////////
        //  ISO-8859 character sets
        class CARL_PUBLIC _Iso8859CharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso8859CharacterSetFamily(const _Iso8859CharacterSetFamily &);
            void                        operator = (const _Iso8859CharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _Iso8859CharacterSetFamily();
            virtual ~_Iso8859CharacterSetFamily();
        public:
            static _Iso8859CharacterSetFamily * getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Iso88591CharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88591CharacterSet(const _Iso88591CharacterSet &);
            void                        operator = (const _Iso88591CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88591CharacterSet();
            virtual ~_Iso88591CharacterSet();
        public:
            static _Iso88591CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88592CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88592CharacterSet(const _Iso88592CharacterSet &);
            void                        operator = (const _Iso88592CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88592CharacterSet();
            virtual ~_Iso88592CharacterSet();
        public:
            static _Iso88592CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88593CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88593CharacterSet(const _Iso88593CharacterSet &);
            void                        operator = (const _Iso88593CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88593CharacterSet();
            virtual ~_Iso88593CharacterSet();
        public:
            static _Iso88593CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88594CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88594CharacterSet(const _Iso88594CharacterSet &);
            void                        operator = (const _Iso88594CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88594CharacterSet();
            virtual ~_Iso88594CharacterSet();
        public:
            static _Iso88594CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88595CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88595CharacterSet(const _Iso88595CharacterSet &);
            void                        operator = (const _Iso88595CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88595CharacterSet();
            virtual ~_Iso88595CharacterSet();
        public:
            static _Iso88595CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88596CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88596CharacterSet(const _Iso88596CharacterSet &);
            void                        operator = (const _Iso88596CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88596CharacterSet();
            virtual ~_Iso88596CharacterSet();
        public:
            static _Iso88596CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88597CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88597CharacterSet(const _Iso88597CharacterSet &);
            void                        operator = (const _Iso88597CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88597CharacterSet();
            virtual ~_Iso88597CharacterSet();
        public:
            static _Iso88597CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88598CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88598CharacterSet(const _Iso88598CharacterSet &);
            void                        operator = (const _Iso88598CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88598CharacterSet();
            virtual ~_Iso88598CharacterSet();
        public:
            static _Iso88598CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso88599CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso88599CharacterSet(const _Iso88599CharacterSet &);
            void                        operator = (const _Iso88599CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso88599CharacterSet();
            virtual ~_Iso88599CharacterSet();
        public:
            static _Iso88599CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso885910CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso885910CharacterSet(const _Iso885910CharacterSet &);
            void                        operator = (const _Iso885910CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso885910CharacterSet();
            virtual ~_Iso885910CharacterSet();
        public:
            static _Iso885910CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso885911CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso885911CharacterSet(const _Iso885911CharacterSet &);
            void                        operator = (const _Iso885911CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso885911CharacterSet();
            virtual ~_Iso885911CharacterSet();
        public:
            static _Iso885911CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso885913CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso885913CharacterSet(const _Iso885913CharacterSet &);
            void                        operator = (const _Iso885913CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso885913CharacterSet();
            virtual ~_Iso885913CharacterSet();
        public:
            static _Iso885913CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso885914CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso885914CharacterSet(const _Iso885914CharacterSet &);
            void                        operator = (const _Iso885914CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso885914CharacterSet();
            virtual ~_Iso885914CharacterSet();
        public:
            static _Iso885914CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Iso885915CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso885915CharacterSet(const _Iso885915CharacterSet &);
            void                        operator = (const _Iso885915CharacterSet &);

            //////////
            //  Singleton
        private:
            _Iso885915CharacterSet();
            virtual ~_Iso885915CharacterSet();
        public:
            static _Iso885915CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  ISO10646 character sets
        class CARL_PUBLIC _Iso10646CharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso10646CharacterSetFamily(const _Iso10646CharacterSetFamily &);
            void                        operator = (const _Iso10646CharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _Iso10646CharacterSetFamily();
            virtual ~_Iso10646CharacterSetFamily();
        public:
            static _Iso10646CharacterSetFamily *   getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Utf32beCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Utf32beCharacterSet(const _Utf32beCharacterSet &);
            void                        operator = (const _Utf32beCharacterSet &);

            //////////
            //  Singleton
        private:
            _Utf32beCharacterSet();
            virtual ~_Utf32beCharacterSet();
        public:
            static _Utf32beCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            Array<UInt8>                _signature;
        };

        class CARL_PUBLIC _Utf32leCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Utf32leCharacterSet(const _Utf32leCharacterSet &);
            void                        operator = (const _Utf32leCharacterSet &);

            //////////
            //  Singleton
        private:
            _Utf32leCharacterSet();
            virtual ~_Utf32leCharacterSet();
        public:
            static _Utf32leCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            Array<UInt8>                _signature;
        };

        class CARL_PUBLIC _Utf16beCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Utf16beCharacterSet(const _Utf16beCharacterSet &);
            void                        operator = (const _Utf16beCharacterSet &);

            //////////
            //  Singleton
        private:
            _Utf16beCharacterSet();
            virtual ~_Utf16beCharacterSet();
        public:
            static _Utf16beCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            Array<UInt8>                _signature;
        };

        class CARL_PUBLIC _Utf16leCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Utf16leCharacterSet(const _Utf16leCharacterSet &);
            void                        operator = (const _Utf16leCharacterSet &);

            //////////
            //  Singleton
        private:
            _Utf16leCharacterSet();
            virtual ~_Utf16leCharacterSet();
        public:
            static _Utf16leCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            Array<UInt8>                _signature;
        };

        class CARL_PUBLIC _Utf8CharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Utf8CharacterSet(const _Utf8CharacterSet &);
            void                        operator = (const _Utf8CharacterSet &);

            //////////
            //  Singleton
        private:
            _Utf8CharacterSet();
            virtual ~_Utf8CharacterSet();
        public:
            static _Utf8CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            Array<UInt8>                _signature;
        };

        //////////
        //  Windows character sets
        class CARL_PUBLIC _WindowsCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _WindowsCharacterSetFamily(const _WindowsCharacterSetFamily &);
            void                        operator = (const _WindowsCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _WindowsCharacterSetFamily();
            virtual ~_WindowsCharacterSetFamily();
        public:
            static _WindowsCharacterSetFamily *   getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Windows1250CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1250CharacterSet(const _Windows1250CharacterSet &);
            void                        operator = (const _Windows1250CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1250CharacterSet();
            virtual ~_Windows1250CharacterSet();
        public:
            static _Windows1250CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1251CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1251CharacterSet(const _Windows1251CharacterSet &);
            void                        operator = (const _Windows1251CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1251CharacterSet();
            virtual ~_Windows1251CharacterSet();
        public:
            static _Windows1251CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1252CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1252CharacterSet(const _Windows1252CharacterSet &);
            void                        operator = (const _Windows1252CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1252CharacterSet();
            virtual ~_Windows1252CharacterSet();
        public:
            static _Windows1252CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1253CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1253CharacterSet(const _Windows1253CharacterSet &);
            void                        operator = (const _Windows1253CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1253CharacterSet();
            virtual ~_Windows1253CharacterSet();
        public:
            static _Windows1253CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1254CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1254CharacterSet(const _Windows1254CharacterSet &);
            void                        operator = (const _Windows1254CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1254CharacterSet();
            virtual ~_Windows1254CharacterSet();
        public:
            static _Windows1254CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1255CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1255CharacterSet(const _Windows1255CharacterSet &);
            void                        operator = (const _Windows1255CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1255CharacterSet();
            virtual ~_Windows1255CharacterSet();
        public:
            static _Windows1255CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1256CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1256CharacterSet(const _Windows1256CharacterSet &);
            void                        operator = (const _Windows1256CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1256CharacterSet();
            virtual ~_Windows1256CharacterSet();
        public:
            static _Windows1256CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1257CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1257CharacterSet(const _Windows1257CharacterSet &);
            void                        operator = (const _Windows1257CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1257CharacterSet();
            virtual ~_Windows1257CharacterSet();
        public:
            static _Windows1257CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Windows1258CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Windows1258CharacterSet(const _Windows1258CharacterSet &);
            void                        operator = (const _Windows1258CharacterSet &);

            //////////
            //  Singleton
        private:
            _Windows1258CharacterSet();
            virtual ~_Windows1258CharacterSet();
        public:
            static _Windows1258CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp874CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp874CharacterSet(const _Cp874CharacterSet &);
            void                        operator = (const _Cp874CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp874CharacterSet();
            virtual ~_Cp874CharacterSet();
        public:
            static _Cp874CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp932CharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp932CharacterSet(const _Cp932CharacterSet &);
            void                        operator = (const _Cp932CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp932CharacterSet();
            virtual ~_Cp932CharacterSet();
        public:
            static _Cp932CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Cp936CharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp936CharacterSet(const _Cp936CharacterSet &);
            void                        operator = (const _Cp936CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp936CharacterSet();
            virtual ~_Cp936CharacterSet();
        public:
            static _Cp936CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Cp949CharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp949CharacterSet(const _Cp949CharacterSet &);
            void                        operator = (const _Cp949CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp949CharacterSet();
            virtual ~_Cp949CharacterSet();
        public:
            static _Cp949CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Cp950CharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp950CharacterSet(const _Cp950CharacterSet &);
            void                        operator = (const _Cp950CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp950CharacterSet();
            virtual ~_Cp950CharacterSet();
        public:
            static _Cp950CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        //////////
        //  PC character sets
        class CARL_PUBLIC _PcCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _PcCharacterSetFamily(const _PcCharacterSetFamily &);
            void                        operator = (const _PcCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _PcCharacterSetFamily();
            virtual ~_PcCharacterSetFamily();
        public:
            static _PcCharacterSetFamily *   getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Cp437CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp437CharacterSet(const _Cp437CharacterSet &);
            void                        operator = (const _Cp437CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp437CharacterSet();
            virtual ~_Cp437CharacterSet();
        public:
            static _Cp437CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp737CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp737CharacterSet(const _Cp737CharacterSet &);
            void                        operator = (const _Cp737CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp737CharacterSet();
            virtual ~_Cp737CharacterSet();
        public:
            static _Cp737CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp775CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp775CharacterSet(const _Cp775CharacterSet &);
            void                        operator = (const _Cp775CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp775CharacterSet();
            virtual ~_Cp775CharacterSet();
        public:
            static _Cp775CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp850CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp850CharacterSet(const _Cp850CharacterSet &);
            void                        operator = (const _Cp850CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp850CharacterSet();
            virtual ~_Cp850CharacterSet();
        public:
            static _Cp850CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp852CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp852CharacterSet(const _Cp852CharacterSet &);
            void                        operator = (const _Cp852CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp852CharacterSet();
            virtual ~_Cp852CharacterSet();
        public:
            static _Cp852CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp855CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp855CharacterSet(const _Cp855CharacterSet &);
            void                        operator = (const _Cp855CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp855CharacterSet();
            virtual ~_Cp855CharacterSet();
        public:
            static _Cp855CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp856CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp856CharacterSet(const _Cp856CharacterSet &);
            void                        operator = (const _Cp856CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp856CharacterSet();
            virtual ~_Cp856CharacterSet();
        public:
            static _Cp856CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp857CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp857CharacterSet(const _Cp857CharacterSet &);
            void                        operator = (const _Cp857CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp857CharacterSet();
            virtual ~_Cp857CharacterSet();
        public:
            static _Cp857CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp860CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp860CharacterSet(const _Cp860CharacterSet &);
            void                        operator = (const _Cp860CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp860CharacterSet();
            virtual ~_Cp860CharacterSet();
        public:
            static _Cp860CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp861CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp861CharacterSet(const _Cp861CharacterSet &);
            void                        operator = (const _Cp861CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp861CharacterSet();
            virtual ~_Cp861CharacterSet();
        public:
            static _Cp861CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp862CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp862CharacterSet(const _Cp862CharacterSet &);
            void                        operator = (const _Cp862CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp862CharacterSet();
            virtual ~_Cp862CharacterSet();
        public:
            static _Cp862CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp863CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp863CharacterSet(const _Cp863CharacterSet &);
            void                        operator = (const _Cp863CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp863CharacterSet();
            virtual ~_Cp863CharacterSet();
        public:
            static _Cp863CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp864CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp864CharacterSet(const _Cp864CharacterSet &);
            void                        operator = (const _Cp864CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp864CharacterSet();
            virtual ~_Cp864CharacterSet();
        public:
            static _Cp864CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp865CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp865CharacterSet(const _Cp865CharacterSet &);
            void                        operator = (const _Cp865CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp865CharacterSet();
            virtual ~_Cp865CharacterSet();
        public:
            static _Cp865CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp866CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp866CharacterSet(const _Cp866CharacterSet &);
            void                        operator = (const _Cp866CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp866CharacterSet();
            virtual ~_Cp866CharacterSet();
        public:
            static _Cp866CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp869CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp869CharacterSet(const _Cp869CharacterSet &);
            void                        operator = (const _Cp869CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp869CharacterSet();
            virtual ~_Cp869CharacterSet();
        public:
            static _Cp869CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  EBCDIC character sets
        class CARL_PUBLIC _EbcdicCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _EbcdicCharacterSetFamily(const _EbcdicCharacterSetFamily &);
            void                        operator = (const _EbcdicCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _EbcdicCharacterSetFamily();
            virtual ~_EbcdicCharacterSetFamily();
        public:
            static _EbcdicCharacterSetFamily *  getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Cp037CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp037CharacterSet(const _Cp037CharacterSet &);
            void                        operator = (const _Cp037CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp037CharacterSet();
            virtual ~_Cp037CharacterSet();
        public:
            static _Cp037CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp424CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp424CharacterSet(const _Cp424CharacterSet &);
            void                        operator = (const _Cp424CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp424CharacterSet();
            virtual ~_Cp424CharacterSet();
        public:
            static _Cp424CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp500CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp500CharacterSet(const _Cp500CharacterSet &);
            void                        operator = (const _Cp500CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp500CharacterSet();
            virtual ~_Cp500CharacterSet();
        public:
            static _Cp500CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp875CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp875CharacterSet(const _Cp875CharacterSet &);
            void                        operator = (const _Cp875CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp875CharacterSet();
            virtual ~_Cp875CharacterSet();
        public:
            static _Cp875CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Cp1026CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Cp1026CharacterSet(const _Cp1026CharacterSet &);
            void                        operator = (const _Cp1026CharacterSet &);

            //////////
            //  Singleton
        private:
            _Cp1026CharacterSet();
            virtual ~_Cp1026CharacterSet();
        public:
            static _Cp1026CharacterSet *    getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  Adobe character sets
        class CARL_PUBLIC _AdobeCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AdobeCharacterSetFamily(const _AdobeCharacterSetFamily &);
            void                        operator = (const _AdobeCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _AdobeCharacterSetFamily();
            virtual ~_AdobeCharacterSetFamily();
        public:
            static _AdobeCharacterSetFamily *   getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _AdobeStandardCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AdobeStandardCharacterSet(const _AdobeStandardCharacterSet &);
            void                        operator = (const _AdobeStandardCharacterSet &);

            //////////
            //  Singleton
        private:
            _AdobeStandardCharacterSet();
            virtual ~_AdobeStandardCharacterSet();
        public:
            static _AdobeStandardCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _AdobeSymbolCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AdobeSymbolCharacterSet(const _AdobeSymbolCharacterSet &);
            void                        operator = (const _AdobeSymbolCharacterSet &);

            //////////
            //  Singleton
        private:
            _AdobeSymbolCharacterSet();
            virtual ~_AdobeSymbolCharacterSet();
        public:
            static _AdobeSymbolCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _AdobeZapfDingbatsCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AdobeZapfDingbatsCharacterSet(const _AdobeZapfDingbatsCharacterSet &);
            void                        operator = (const _AdobeZapfDingbatsCharacterSet &);

            //////////
            //  Singleton
        private:
            _AdobeZapfDingbatsCharacterSet();
            virtual ~_AdobeZapfDingbatsCharacterSet();
        public:
            static _AdobeZapfDingbatsCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  KOI character sets
        class CARL_PUBLIC _KoiCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _KoiCharacterSetFamily(const _KoiCharacterSetFamily &);
            void                        operator = (const _KoiCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _KoiCharacterSetFamily();
            virtual ~_KoiCharacterSetFamily();
        public:
            static _KoiCharacterSetFamily * getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _Gost1976882CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Gost1976882CharacterSet(const _Gost1976882CharacterSet &);
            void                        operator = (const _Gost1976882CharacterSet &);

            //////////
            //  Singleton
        private:
            _Gost1976882CharacterSet();
            virtual ~_Gost1976882CharacterSet();
        public:
            static _Gost1976882CharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi7CharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi7CharacterSet(const _Koi7CharacterSet &);
            void                        operator = (const _Koi7CharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi7CharacterSet();
            virtual ~_Koi7CharacterSet();
        public:
            static _Koi7CharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8aCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8aCharacterSet(const _Koi8aCharacterSet &);
            void                        operator = (const _Koi8aCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8aCharacterSet();
            virtual ~_Koi8aCharacterSet();
        public:
            static _Koi8aCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8bCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8bCharacterSet(const _Koi8bCharacterSet &);
            void                        operator = (const _Koi8bCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8bCharacterSet();
            virtual ~_Koi8bCharacterSet();
        public:
            static _Koi8bCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8eCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8eCharacterSet(const _Koi8eCharacterSet &);
            void                        operator = (const _Koi8eCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8eCharacterSet();
            virtual ~_Koi8eCharacterSet();
        public:
            static _Koi8eCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8fCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8fCharacterSet(const _Koi8fCharacterSet &);
            void                        operator = (const _Koi8fCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8fCharacterSet();
            virtual ~_Koi8fCharacterSet();
        public:
            static _Koi8fCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8rCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8rCharacterSet(const _Koi8rCharacterSet &);
            void                        operator = (const _Koi8rCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8rCharacterSet();
            virtual ~_Koi8rCharacterSet();
        public:
            static _Koi8rCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8ruCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8ruCharacterSet(const _Koi8ruCharacterSet &);
            void                        operator = (const _Koi8ruCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8ruCharacterSet();
            virtual ~_Koi8ruCharacterSet();
        public:
            static _Koi8ruCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _Koi8uCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Koi8uCharacterSet(const _Koi8uCharacterSet &);
            void                        operator = (const _Koi8uCharacterSet &);

            //////////
            //  Singleton
        private:
            _Koi8uCharacterSet();
            virtual ~_Koi8uCharacterSet();
        public:
            static _Koi8uCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  MAC
        class CARL_PUBLIC _MacCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacCharacterSetFamily(const _MacCharacterSetFamily &);
            void                        operator = (const _MacCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _MacCharacterSetFamily();
            virtual ~_MacCharacterSetFamily();
        public:
            static _MacCharacterSetFamily * getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _MacArabicCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacArabicCharacterSet(const _MacArabicCharacterSet &);
            void                        operator = (const _MacArabicCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacArabicCharacterSet();
            virtual ~_MacArabicCharacterSet();
        public:
            static _MacArabicCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacCelticCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacCelticCharacterSet(const _MacCelticCharacterSet &);
            void                        operator = (const _MacCelticCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacCelticCharacterSet();
            virtual ~_MacCelticCharacterSet();
        public:
            static _MacCelticCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacCentralEuropeanCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacCentralEuropeanCharacterSet(const _MacCentralEuropeanCharacterSet &);
            void                        operator = (const _MacCentralEuropeanCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacCentralEuropeanCharacterSet();
            virtual ~_MacCentralEuropeanCharacterSet();
        public:
            static _MacCentralEuropeanCharacterSet *getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacCroatianCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacCroatianCharacterSet(const _MacCroatianCharacterSet &);
            void                        operator = (const _MacCroatianCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacCroatianCharacterSet();
            virtual ~_MacCroatianCharacterSet();
        public:
            static _MacCroatianCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacCyrillicCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacCyrillicCharacterSet(const _MacCyrillicCharacterSet &);
            void                        operator = (const _MacCyrillicCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacCyrillicCharacterSet();
            virtual ~_MacCyrillicCharacterSet();
        public:
            static _MacCyrillicCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacDevanagariCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacDevanagariCharacterSet(const _MacDevanagariCharacterSet &);
            void                        operator = (const _MacDevanagariCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacDevanagariCharacterSet();
            virtual ~_MacDevanagariCharacterSet();
        public:
            static _MacDevanagariCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacDingbatsCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacDingbatsCharacterSet(const _MacDingbatsCharacterSet &);
            void                        operator = (const _MacDingbatsCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacDingbatsCharacterSet();
            virtual ~_MacDingbatsCharacterSet();
        public:
            static _MacDingbatsCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacFarsiCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacFarsiCharacterSet(const _MacFarsiCharacterSet &);
            void                        operator = (const _MacFarsiCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacFarsiCharacterSet();
            virtual ~_MacFarsiCharacterSet();
        public:
            static _MacFarsiCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacGaelicCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacGaelicCharacterSet(const _MacGaelicCharacterSet &);
            void                        operator = (const _MacGaelicCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacGaelicCharacterSet();
            virtual ~_MacGaelicCharacterSet();
        public:
            static _MacGaelicCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacGreekCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacGreekCharacterSet(const _MacGreekCharacterSet &);
            void                        operator = (const _MacGreekCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacGreekCharacterSet();
            virtual ~_MacGreekCharacterSet();
        public:
            static _MacGreekCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacGujaratiCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacGujaratiCharacterSet(const _MacGujaratiCharacterSet &);
            void                        operator = (const _MacGujaratiCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacGujaratiCharacterSet();
            virtual ~_MacGujaratiCharacterSet();
        public:
            static _MacGujaratiCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacGurmukhiCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacGurmukhiCharacterSet(const _MacGurmukhiCharacterSet &);
            void                        operator = (const _MacGurmukhiCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacGurmukhiCharacterSet();
            virtual ~_MacGurmukhiCharacterSet();
        public:
            static _MacGurmukhiCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacHebrewCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacHebrewCharacterSet(const _MacHebrewCharacterSet &);
            void                        operator = (const _MacHebrewCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacHebrewCharacterSet();
            virtual ~_MacHebrewCharacterSet();
        public:
            static _MacHebrewCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacIcelandicCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacIcelandicCharacterSet(const _MacIcelandicCharacterSet &);
            void                        operator = (const _MacIcelandicCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacIcelandicCharacterSet();
            virtual ~_MacIcelandicCharacterSet();
        public:
            static _MacIcelandicCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacInuitCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacInuitCharacterSet(const _MacInuitCharacterSet &);
            void                        operator = (const _MacInuitCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacInuitCharacterSet();
            virtual ~_MacInuitCharacterSet();
        public:
            static _MacInuitCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacRomanCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacRomanCharacterSet(const _MacRomanCharacterSet &);
            void                        operator = (const _MacRomanCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacRomanCharacterSet();
            virtual ~_MacRomanCharacterSet();
        public:
            static _MacRomanCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacRomanianCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacRomanianCharacterSet(const _MacRomanianCharacterSet &);
            void                        operator = (const _MacRomanianCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacRomanianCharacterSet();
            virtual ~_MacRomanianCharacterSet();
        public:
            static _MacRomanianCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacSymbolCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacSymbolCharacterSet(const _MacSymbolCharacterSet &);
            void                        operator = (const _MacSymbolCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacSymbolCharacterSet();
            virtual ~_MacSymbolCharacterSet();
        public:
            static _MacSymbolCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacThaiCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacThaiCharacterSet(const _MacThaiCharacterSet &);
            void                        operator = (const _MacThaiCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacThaiCharacterSet();
            virtual ~_MacThaiCharacterSet();
        public:
            static _MacThaiCharacterSet *   getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacTurkishCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacTurkishCharacterSet(const _MacTurkishCharacterSet &);
            void                        operator = (const _MacTurkishCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacTurkishCharacterSet();
            virtual ~_MacTurkishCharacterSet();
        public:
            static _MacTurkishCharacterSet *    getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacUkrainianCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacUkrainianCharacterSet(const _MacUkrainianCharacterSet &);
            void                        operator = (const _MacUkrainianCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacUkrainianCharacterSet();
            virtual ~_MacUkrainianCharacterSet();
        public:
            static _MacUkrainianCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _MacChineseSimplifiedCharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacChineseSimplifiedCharacterSet(const _MacChineseSimplifiedCharacterSet &);
            void                        operator = (const _MacChineseSimplifiedCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacChineseSimplifiedCharacterSet();
            virtual ~_MacChineseSimplifiedCharacterSet();
        public:
            static _MacChineseSimplifiedCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _MacChineseTraditionalCharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacChineseTraditionalCharacterSet(const _MacChineseTraditionalCharacterSet &);
            void                        operator = (const _MacChineseTraditionalCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacChineseTraditionalCharacterSet();
            virtual ~_MacChineseTraditionalCharacterSet();
        public:
            static _MacChineseTraditionalCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _MacJapaneseCharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacJapaneseCharacterSet(const _MacJapaneseCharacterSet &);
            void                        operator = (const _MacJapaneseCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacJapaneseCharacterSet();
            virtual ~_MacJapaneseCharacterSet();
        public:
            static _MacJapaneseCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _MacKoreanCharacterSet : public _MultiByte2CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MacKoreanCharacterSet(const _MacKoreanCharacterSet &);
            void                        operator = (const _MacKoreanCharacterSet &);

            //////////
            //  Singleton
        private:
            _MacKoreanCharacterSet();
            virtual ~_MacKoreanCharacterSet();
        public:
            static _MacKoreanCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        //////////
        //  Miscellaneous character sets
        class CARL_PUBLIC _MiscellaneousCharacterSetFamily : public CharacterSetFamily
        {
            //////////
            //  Can't assign or copy construct
        private:
            _MiscellaneousCharacterSetFamily(const _MiscellaneousCharacterSetFamily &);
            void                        operator = (const _MiscellaneousCharacterSetFamily &);

            //////////
            //  Singleton
        private:
            _MiscellaneousCharacterSetFamily();
            virtual ~_MiscellaneousCharacterSetFamily();
        public:
            static _MiscellaneousCharacterSetFamily *   getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _AsciiCharacterSet : public CharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AsciiCharacterSet(const _AsciiCharacterSet &);
            void                        operator = (const _AsciiCharacterSet &);

            //////////
            //  Singleton
        private:
            _AsciiCharacterSet();
            virtual ~_AsciiCharacterSet();
        public:
            static _AsciiCharacterSet * getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual int                 getMaximumEncodingLength() const;
            virtual int                 getEncodingLengthForCharacter(Char c) const;
            virtual int                 encodeCharacter(Char c, void * buffer) const;
            virtual int                 getEncodingLengthForLeadByte(UInt8 leadByte) const;
            virtual int                 decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
        };

        class CARL_PUBLIC _NextStepCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NextStepCharacterSet(const _NextStepCharacterSet &);
            void                        operator = (const _NextStepCharacterSet &);

            //////////
            //  Singleton
        private:
            _NextStepCharacterSet();
            virtual ~_NextStepCharacterSet();
        public:
            static _NextStepCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        class CARL_PUBLIC _AtariStCharacterSet : public _SingleByteCharacterSet
        {
            //////////
            //  Can't assign or copy construct
        private:
            _AtariStCharacterSet(const _AtariStCharacterSet &);
            void                        operator = (const _AtariStCharacterSet &);

            //////////
            //  Singleton
        private:
            _AtariStCharacterSet();
            virtual ~_AtariStCharacterSet();
        public:
            static _AtariStCharacterSet *  getInstance();

            //////////
            //  Operations
        public:
            virtual CharacterSetFamily *    getFamily() const;
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual Char                getDefaultCharacter() const;
            virtual Array<UInt8>        getSignature() const;

            //////////
            //  Implementation
        private:
            Array<String>               _names;
            static Char                 _mappingTable[256];
        };

        //////////
        //  A platform-specific implementation of a character set manager
        class CARL_PUBLIC _CharsetManagerImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _CharsetManagerImpl(const _CharsetManagerImpl &);
            void                        operator = (const _CharsetManagerImpl &);

            //////////
            //  Singleton
        private:
            _CharsetManagerImpl();
            ~_CharsetManagerImpl();
        public:
            static _CharsetManagerImpl *    getInstance();

            //////////
            //  Operations
        public:
            MapResourceFactory *        getResourceFactory() const;
            Array<CharacterSet*>        getAllCharacterSets();
            CharacterSet *              findCharacterSet(const String & name);
            Array<CharacterSetFamily*>  getAllCharacterSetFamilies();
            CharacterSetFamily *        findCharacterSetFamily(const String & name);
            CharacterSet *              getSystem() const;
            CharacterSet *              getDefault() const;
            void                        setDefault(CharacterSet * pCharacterSet);
            void                        registerCharacterSet(CharacterSet * pCharacterSet) throw (InconsistentCharacterSetException, DuplicateCharacterSetException);
            void                        registerCharacterSetFamily(CharacterSetFamily * pCharacterSetFamily) throw (InconsistentCharacterSetFamilyException, DuplicateCharacterSetFamilyException);

            //////////
            //  Implementation
        private:
            mutable FrameworkResourceFactory    _resourceFactory;
            Array<CharacterSet*>        _characterSets;
            Array<CharacterSetFamily*>  _characterSetFamilies;
            HashMap<String, CharacterSet*>  _mapNamesToCharacterSets;   //  Names are UPPERCASED
            HashMap<String, CharacterSetFamily*>    _mapNamesToCharacterSetFamilies;    //  Names are UPPERCASED
            CharacterSet *              _pSystemCharacterSet;
            CharacterSet *              _pDefaultCharacterSet;
            mutable CriticalSection     _guard;
            //
            CharacterSet *              _determineSystemCharacterSet();
        };
    }
}

#endif  //  __CharsetImpl_hpp_included__
//  End of CharsetImpl.hpp
