//
//  CharsetManagerImpl.cpp - Implementation of the carl::_impl::_CharsetManagerImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/07 20:31:53 $
//  $Revision: 1.25 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"
#include <stdlib.h>
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <windows.h>
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)
    #include <localcharset.h>
    #include <string.h>
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC)
    #include <langinfo.h>
#else
    #error Unsupported platform
#endif

//////////
//  Singleton
_impl::_CharsetManagerImpl::_CharsetManagerImpl()
    :   _resourceFactory(new _CharsetResourceMap_en(), carl::String("CARL_CHARSET")),
        _mapNamesToCharacterSets(StandardHashFunctions::hashString),
        _mapNamesToCharacterSetFamilies(StandardHashFunctions::hashString)
{
    //  Register ISO-8859 character sets
    registerCharacterSetFamily(_Iso8859CharacterSetFamily::getInstance());
    registerCharacterSet(_Iso88591CharacterSet::getInstance());
    registerCharacterSet(_Iso88592CharacterSet::getInstance());
    registerCharacterSet(_Iso88593CharacterSet::getInstance());
    registerCharacterSet(_Iso88594CharacterSet::getInstance());
    registerCharacterSet(_Iso88595CharacterSet::getInstance());
    registerCharacterSet(_Iso88596CharacterSet::getInstance());
    registerCharacterSet(_Iso88597CharacterSet::getInstance());
    registerCharacterSet(_Iso88598CharacterSet::getInstance());
    registerCharacterSet(_Iso88599CharacterSet::getInstance());
    registerCharacterSet(_Iso885910CharacterSet::getInstance());
    registerCharacterSet(_Iso885911CharacterSet::getInstance());
    registerCharacterSet(_Iso885913CharacterSet::getInstance());
    registerCharacterSet(_Iso885914CharacterSet::getInstance());
    registerCharacterSet(_Iso885915CharacterSet::getInstance());
    //  Register ISO-10646 character sets
    registerCharacterSetFamily(_Iso10646CharacterSetFamily::getInstance());
    registerCharacterSet(_Utf32beCharacterSet::getInstance());
    registerCharacterSet(_Utf32leCharacterSet::getInstance());
    registerCharacterSet(_Utf16beCharacterSet::getInstance());
    registerCharacterSet(_Utf16leCharacterSet::getInstance());
    registerCharacterSet(_Utf8CharacterSet::getInstance());
    //  Register Windows character sets
    registerCharacterSetFamily(_WindowsCharacterSetFamily::getInstance());
    registerCharacterSet(_Windows1250CharacterSet::getInstance());
    registerCharacterSet(_Windows1251CharacterSet::getInstance());
    registerCharacterSet(_Windows1252CharacterSet::getInstance());
    registerCharacterSet(_Windows1253CharacterSet::getInstance());
    registerCharacterSet(_Windows1254CharacterSet::getInstance());
    registerCharacterSet(_Windows1255CharacterSet::getInstance());
    registerCharacterSet(_Windows1256CharacterSet::getInstance());
    registerCharacterSet(_Windows1257CharacterSet::getInstance());
    registerCharacterSet(_Windows1258CharacterSet::getInstance());
    registerCharacterSet(_Cp874CharacterSet::getInstance());
    registerCharacterSet(_Cp932CharacterSet::getInstance());
    registerCharacterSet(_Cp936CharacterSet::getInstance());
    registerCharacterSet(_Cp949CharacterSet::getInstance());
    registerCharacterSet(_Cp950CharacterSet::getInstance());
    //  Register PC character sets
    registerCharacterSetFamily(_PcCharacterSetFamily::getInstance());
    registerCharacterSet(_Cp437CharacterSet::getInstance());
    registerCharacterSet(_Cp737CharacterSet::getInstance());
    registerCharacterSet(_Cp775CharacterSet::getInstance());
    registerCharacterSet(_Cp850CharacterSet::getInstance());
    registerCharacterSet(_Cp852CharacterSet::getInstance());
    registerCharacterSet(_Cp855CharacterSet::getInstance());
    registerCharacterSet(_Cp856CharacterSet::getInstance());
    registerCharacterSet(_Cp857CharacterSet::getInstance());
    registerCharacterSet(_Cp860CharacterSet::getInstance());
    registerCharacterSet(_Cp861CharacterSet::getInstance());
    registerCharacterSet(_Cp862CharacterSet::getInstance());
    registerCharacterSet(_Cp863CharacterSet::getInstance());
    registerCharacterSet(_Cp864CharacterSet::getInstance());
    registerCharacterSet(_Cp865CharacterSet::getInstance());
    registerCharacterSet(_Cp866CharacterSet::getInstance());
    registerCharacterSet(_Cp869CharacterSet::getInstance());
    //  Register EBCDIC character sets
    registerCharacterSetFamily(_EbcdicCharacterSetFamily::getInstance());
    registerCharacterSet(_Cp037CharacterSet::getInstance());
    registerCharacterSet(_Cp424CharacterSet::getInstance());
    registerCharacterSet(_Cp500CharacterSet::getInstance());
    registerCharacterSet(_Cp875CharacterSet::getInstance());
    registerCharacterSet(_Cp1026CharacterSet::getInstance());
    //  Register Adobe character sets
    registerCharacterSetFamily(_AdobeCharacterSetFamily::getInstance());
    registerCharacterSet(_AdobeStandardCharacterSet::getInstance());
    registerCharacterSet(_AdobeSymbolCharacterSet::getInstance());
    registerCharacterSet(_AdobeZapfDingbatsCharacterSet::getInstance());
    //  Register KOI character sets
    registerCharacterSetFamily(_KoiCharacterSetFamily::getInstance());
    registerCharacterSet(_Gost1976882CharacterSet::getInstance());
    registerCharacterSet(_Koi7CharacterSet::getInstance());
    registerCharacterSet(_Koi8aCharacterSet::getInstance());
    registerCharacterSet(_Koi8bCharacterSet::getInstance());
    registerCharacterSet(_Koi8eCharacterSet::getInstance());
    registerCharacterSet(_Koi8fCharacterSet::getInstance());
    registerCharacterSet(_Koi8rCharacterSet::getInstance());
    registerCharacterSet(_Koi8ruCharacterSet::getInstance());
    registerCharacterSet(_Koi8uCharacterSet::getInstance());
    //  Register MAC character sets
    registerCharacterSetFamily(_MacCharacterSetFamily::getInstance());
    registerCharacterSet(_MacArabicCharacterSet::getInstance());
    registerCharacterSet(_MacCelticCharacterSet::getInstance());
    registerCharacterSet(_MacCentralEuropeanCharacterSet::getInstance());
    registerCharacterSet(_MacCroatianCharacterSet::getInstance());
    registerCharacterSet(_MacCyrillicCharacterSet::getInstance());
    registerCharacterSet(_MacDevanagariCharacterSet::getInstance());
    registerCharacterSet(_MacDingbatsCharacterSet::getInstance());
    registerCharacterSet(_MacFarsiCharacterSet::getInstance());
    registerCharacterSet(_MacGaelicCharacterSet::getInstance());
    registerCharacterSet(_MacGreekCharacterSet::getInstance());
    registerCharacterSet(_MacGujaratiCharacterSet::getInstance());
    registerCharacterSet(_MacGurmukhiCharacterSet::getInstance());
    registerCharacterSet(_MacHebrewCharacterSet::getInstance());
    registerCharacterSet(_MacIcelandicCharacterSet::getInstance());
    registerCharacterSet(_MacInuitCharacterSet::getInstance());
    registerCharacterSet(_MacRomanCharacterSet::getInstance());
    registerCharacterSet(_MacRomanianCharacterSet::getInstance());
    registerCharacterSet(_MacSymbolCharacterSet::getInstance());
    registerCharacterSet(_MacThaiCharacterSet::getInstance());
    registerCharacterSet(_MacTurkishCharacterSet::getInstance());
    registerCharacterSet(_MacUkrainianCharacterSet::getInstance());
    registerCharacterSet(_MacChineseSimplifiedCharacterSet::getInstance());
    registerCharacterSet(_MacChineseTraditionalCharacterSet::getInstance());
    registerCharacterSet(_MacKoreanCharacterSet::getInstance());
    registerCharacterSet(_MacJapaneseCharacterSet::getInstance());
    //  Register miscellaneous character sets
    registerCharacterSetFamily(_MiscellaneousCharacterSetFamily::getInstance());
    registerCharacterSet(_AsciiCharacterSet::getInstance());
    registerCharacterSet(_NextStepCharacterSet::getInstance());
    registerCharacterSet(_AtariStCharacterSet::getInstance());
    //  Prepare system character set
    _pSystemCharacterSet = _determineSystemCharacterSet();
    //  Initially, system character set is the default
    _pDefaultCharacterSet = _pSystemCharacterSet;
}

_impl::_CharsetManagerImpl::~_CharsetManagerImpl()
{
}

_impl::_CharsetManagerImpl * _impl::_CharsetManagerImpl::getInstance()
{
    static _CharsetManagerImpl instance;
    return &instance;
}

//////////
//  Operations
MapResourceFactory * _impl::_CharsetManagerImpl::getResourceFactory() const
{
    return &_resourceFactory;
}

Array<CharacterSet*> _impl::_CharsetManagerImpl::getAllCharacterSets()
{
    CriticalSectionLock lock(_guard);
    return _characterSets;
}

CharacterSet * _impl::_CharsetManagerImpl::findCharacterSet(const String & name)
{
    CriticalSectionLock lock(_guard);
    String nameUppercased = name.upperCased();
    if (_mapNamesToCharacterSets.containsKey(nameUppercased))
    {
        return _mapNamesToCharacterSets.get(nameUppercased);
    }
    return NULL;
}

Array<CharacterSetFamily*> _impl::_CharsetManagerImpl::getAllCharacterSetFamilies()
{
    CriticalSectionLock lock(_guard);
    return _characterSetFamilies;
}

CharacterSetFamily * _impl::_CharsetManagerImpl::findCharacterSetFamily(const String & name)
{
    CriticalSectionLock lock(_guard);
    String nameUppercased = name.upperCased();
    if (_mapNamesToCharacterSetFamilies.containsKey(nameUppercased))
    {
        return _mapNamesToCharacterSetFamilies.get(nameUppercased);
    }
    return NULL;
}

void _impl::_CharsetManagerImpl::registerCharacterSet(CharacterSet * pCharacterSet) throw (InconsistentCharacterSetException, DuplicateCharacterSetException)
{
    require(pCharacterSet != NULL);

    CriticalSectionLock lock(_guard);
    Array<String> names = pCharacterSet->getAllNames();
    for (UIntSize i = 0; i < names.size(); i++)
    {
        names[i].upperCase();
        if (_mapNamesToCharacterSets.containsKey(names[i]) ||
            _mapNamesToCharacterSetFamilies.containsKey(names[i]))
        {
            throw DuplicateCharacterSetException();
        }
    }
    if (!names.contains(pCharacterSet->getCanonicalName().upperCased()) ||
        !names.contains(pCharacterSet->getPreferredName().upperCased()) ||
        !_characterSetFamilies.contains(pCharacterSet->getFamily()))
    {
        throw InconsistentCharacterSetException();
    }
    if (pCharacterSet->getEncodingLengthForCharacter(pCharacterSet->getDefaultCharacter()) == -1)
    {
        throw InconsistentCharacterSetException();
    }
    //
    _characterSets.append(pCharacterSet);
    for (UIntSize i = 0; i < names.size(); i++)
    {
        _mapNamesToCharacterSets.insert(names[i], pCharacterSet);
    }
}

void _impl::_CharsetManagerImpl::registerCharacterSetFamily(CharacterSetFamily * pCharacterSetFamily) throw (InconsistentCharacterSetFamilyException, DuplicateCharacterSetFamilyException)
{
    require(pCharacterSetFamily != NULL);

    CriticalSectionLock lock(_guard);
    Array<String> names = pCharacterSetFamily->getAllNames();
    for (UIntSize i = 0; i < names.size(); i++)
    {
        names[i].upperCase();
        if (_mapNamesToCharacterSets.containsKey(names[i]) ||
            _mapNamesToCharacterSetFamilies.containsKey(names[i]))
        {
            throw DuplicateCharacterSetFamilyException();
        }
    }
    if (!names.contains(pCharacterSetFamily->getCanonicalName().upperCased()) ||
        !names.contains(pCharacterSetFamily->getPreferredName().upperCased()))
    {
        throw InconsistentCharacterSetFamilyException();
    }
    //
    _characterSetFamilies.append(pCharacterSetFamily);
    for (UIntSize i = 0; i < names.size(); i++)
    {
        _mapNamesToCharacterSetFamilies.insert(names[i], pCharacterSetFamily);
    }
}

CharacterSet * _impl::_CharsetManagerImpl::getSystem() const
{
    CriticalSectionLock lock(_guard);
    return _pSystemCharacterSet;
}

CharacterSet * _impl::_CharsetManagerImpl::getDefault() const
{
    CriticalSectionLock lock(_guard);
    return _pDefaultCharacterSet;
}

void _impl::_CharsetManagerImpl::setDefault(CharacterSet * pCharacterSet)
{
    require(pCharacterSet != NULL);

    CriticalSectionLock lock(_guard);
    _pDefaultCharacterSet = pCharacterSet;
}

//////////
//  Implementation helpers
CharacterSet * _impl::_CharsetManagerImpl::_determineSystemCharacterSet()
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    UINT acp = GetACP();
    switch (acp)
    {
        case 437:
            return _Cp437CharacterSet::getInstance();
        case 737:
            return _Cp737CharacterSet::getInstance();
        case 775:
            return _Cp775CharacterSet::getInstance();
        case 850:
            return _Cp850CharacterSet::getInstance();
        case 852:
            return _Cp852CharacterSet::getInstance();
        case 855:
            return _Cp855CharacterSet::getInstance();
        case 856:
            return _Cp856CharacterSet::getInstance();
        case 857:
            return _Cp857CharacterSet::getInstance();
        case 860:
            return _Cp860CharacterSet::getInstance();
        case 861:
            return _Cp861CharacterSet::getInstance();
        case 862:
            return _Cp862CharacterSet::getInstance();
        case 863:
            return _Cp863CharacterSet::getInstance();
        case 864:
            return _Cp864CharacterSet::getInstance();
        case 865:
            return _Cp865CharacterSet::getInstance();
        case 866:
            return _Cp866CharacterSet::getInstance();
        case 869:
            return _Cp869CharacterSet::getInstance();
        case 874:
            return _Cp874CharacterSet::getInstance();
        case 932:
            return _Cp932CharacterSet::getInstance();
        case 936:
            return _Cp936CharacterSet::getInstance();
        case 949:
            return _Cp949CharacterSet::getInstance();
        case 950:
            return _Cp950CharacterSet::getInstance();
        case 1250:
            return _Windows1250CharacterSet::getInstance();
        case 1251:
            return _Windows1251CharacterSet::getInstance();
        case 1252:
            return _Windows1252CharacterSet::getInstance();
        case 1253:
            return _Windows1253CharacterSet::getInstance();
        case 1254:
            return _Windows1254CharacterSet::getInstance();
        case 1255:
            return _Windows1255CharacterSet::getInstance();
        case 1256:
            return _Windows1256CharacterSet::getInstance();
        case 1257:
            return _Windows1257CharacterSet::getInstance();
        case 1258:
            return _Windows1258CharacterSet::getInstance();
        case 20127:
            return _AsciiCharacterSet::getInstance();
        case 20866:
            return _Koi8rCharacterSet::getInstance();
        case 21866:
            return _Koi8uCharacterSet::getInstance();
        case 28591:
            return _Iso88591CharacterSet::getInstance();
        case 28592:
            return _Iso88592CharacterSet::getInstance();
        case 28593:
            return _Iso88593CharacterSet::getInstance();
        case 28594:
            return _Iso88594CharacterSet::getInstance();
        case 28595:
            return _Iso88595CharacterSet::getInstance();
        case 28596:
            return _Iso88596CharacterSet::getInstance();
        case 28597:
            return _Iso88597CharacterSet::getInstance();
        case 28598:
            return _Iso88598CharacterSet::getInstance();
        case 28599:
            return _Iso88599CharacterSet::getInstance();
        case 28600:
            return _Iso885910CharacterSet::getInstance();
        case 28601:
            return _Iso885911CharacterSet::getInstance();
        case 28603:
            return _Iso885913CharacterSet::getInstance();
        case 28604:
            return _Iso885914CharacterSet::getInstance();
        case 28605:
            return _Iso885915CharacterSet::getInstance();
        case 65001:
            return _Utf8CharacterSet::getInstance();
        default:
            return _Iso88591CharacterSet::getInstance();
    }
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)
    const char * pCharsetName = locale_charset();
    if (strcmp(pCharsetName, "ANSI_X3.4-1968") == 0 ||
        strcmp(pCharsetName, "ANSI_X3.4-1986") == 0 ||
        strcmp(pCharsetName, "ASCII") == 0 ||
        strcmp(pCharsetName, "CP367") == 0 ||
        strcmp(pCharsetName, "IBM367") == 0 ||
        strcmp(pCharsetName, "ISO-IR-6") == 0 ||
        strcmp(pCharsetName, "ISO646-US") == 0 ||
        strcmp(pCharsetName, "ISO_646.IRV:1991") == 0 ||
        strcmp(pCharsetName, "US") == 0 ||
        strcmp(pCharsetName, "US-ASCII") == 0 ||
        strcmp(pCharsetName, "CSASCII") == 0)
    {
        return _AsciiCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UTF-8") == 0)
    {
        return _Utf8CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UTF-16") == 0)
    {
        if (System::HostByteOrder == ByteOrder_BigEndian)
        {
            return _Utf16beCharacterSet::getInstance();
        }
        else
        {
            return _Utf16leCharacterSet::getInstance();
        }
    }
    else if (strcmp(pCharsetName, "UTF-16BE") == 0)
    {
        return _Utf16beCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UTF-16LE") == 0)
    {
        return _Utf16leCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UTF-32") == 0)
    {
        if (System::HostByteOrder == ByteOrder_BigEndian)
        {
            return _Utf32beCharacterSet::getInstance();
        }
        else
        {
            return _Utf32leCharacterSet::getInstance();
        }
    }
    else if (strcmp(pCharsetName, "UTF-32BE") == 0)
    {
        return _Utf32beCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UTF-32LE") == 0)
    {
        return _Utf32leCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP819") == 0 ||
             strcmp(pCharsetName, "IBM819") == 0 ||
             strcmp(pCharsetName, "ISO-8859-1") == 0 ||
             strcmp(pCharsetName, "ISO-IR-100") == 0 ||
             strcmp(pCharsetName, "ISO8859-1") == 0 ||
             strcmp(pCharsetName, "ISO_8859-1") == 0 ||
             strcmp(pCharsetName, "ISO_8859-1:1987") == 0 ||
             strcmp(pCharsetName, "L1") == 0 ||
             strcmp(pCharsetName, "LATIN1") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN1") == 0)
    {
        return _Iso88591CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-2") == 0 ||
             strcmp(pCharsetName, "ISO-IR-101") == 0 ||
             strcmp(pCharsetName, "ISO8859-2") == 0 ||
             strcmp(pCharsetName, "ISO_8859-2") == 0 ||
             strcmp(pCharsetName, "ISO_8859-2:1987") == 0 ||
             strcmp(pCharsetName, "L2") == 0 ||
             strcmp(pCharsetName, "LATIN2") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN2") == 0)
    {
        return _Iso88592CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-3") == 0 ||
             strcmp(pCharsetName, "ISO-IR-109") == 0 ||
             strcmp(pCharsetName, "ISO8859-3") == 0 ||
             strcmp(pCharsetName, "ISO_8859-3") == 0 ||
             strcmp(pCharsetName, "ISO_8859-3:1988") == 0 ||
             strcmp(pCharsetName, "L3") == 0 ||
             strcmp(pCharsetName, "LATIN3") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN3") == 0)
    {
        return _Iso88593CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-4") == 0 ||
             strcmp(pCharsetName, "ISO-IR-110") == 0 ||
             strcmp(pCharsetName, "ISO8859-4") == 0 ||
             strcmp(pCharsetName, "ISO_8859-4") == 0 ||
             strcmp(pCharsetName, "ISO_8859-4:1988") == 0 ||
             strcmp(pCharsetName, "L4") == 0 ||
             strcmp(pCharsetName, "LATIN4") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN4") == 0)
    {
        return _Iso88594CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CYRILLIC") == 0 ||
             strcmp(pCharsetName, "ISO-8859-5") == 0 ||
             strcmp(pCharsetName, "ISO-IR-144") == 0 ||
             strcmp(pCharsetName, "ISO8859-5") == 0 ||
             strcmp(pCharsetName, "ISO_8859-5") == 0 ||
             strcmp(pCharsetName, "ISO_8859-5:1988") == 0 ||
             strcmp(pCharsetName, "CSISOLATINCYRILLIC") == 0)
    {
        return _Iso88595CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ARABIC") == 0 ||
             strcmp(pCharsetName, "ASMO-708") == 0 ||
             strcmp(pCharsetName, "ECMA-114") == 0 ||
             strcmp(pCharsetName, "ISO-8859-6") == 0 ||
             strcmp(pCharsetName, "ISO-IR-127") == 0 ||
             strcmp(pCharsetName, "ISO8859-6") == 0 ||
             strcmp(pCharsetName, "ISO_8859-6") == 0 ||
             strcmp(pCharsetName, "ISO_8859-6:1987") == 0 ||
             strcmp(pCharsetName, "CSISOLATINARABIC") == 0)
    {
        return _Iso88596CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ECMA-118") == 0 ||
             strcmp(pCharsetName, "ELOT_928") == 0 ||
             strcmp(pCharsetName, "GREEK GREEK8") == 0 ||
             strcmp(pCharsetName, "ISO-8859-7") == 0 ||
             strcmp(pCharsetName, "ISO-IR-126") == 0 ||
             strcmp(pCharsetName, "ISO8859-7") == 0 ||
             strcmp(pCharsetName, "ISO_8859-7") == 0 ||
             strcmp(pCharsetName, "ISO_8859-7:1987") == 0 ||
             strcmp(pCharsetName, "CSISOLATINGREEK") == 0)
    {
        return _Iso88597CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "HEBREW") == 0 ||
             strcmp(pCharsetName, "ISO-8859-8") == 0 ||
             strcmp(pCharsetName, "ISO-IR-138") == 0 ||
             strcmp(pCharsetName, "ISO8859-8") == 0 ||
             strcmp(pCharsetName, "ISO_8859-8") == 0 ||
             strcmp(pCharsetName, "ISO_8859-8:1988") == 0 ||
             strcmp(pCharsetName, "CSISOLATINHEBREW") == 0)
    {
        return _Iso88598CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-9") == 0 ||
             strcmp(pCharsetName, "ISO-IR-148") == 0 ||
             strcmp(pCharsetName, "ISO8859-9") == 0 ||
             strcmp(pCharsetName, "ISO_8859-9") == 0 ||
             strcmp(pCharsetName, "ISO_8859-9:1989") == 0 ||
             strcmp(pCharsetName, "L5") == 0 ||
             strcmp(pCharsetName, "LATIN5") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN5") == 0)
    {
        return _Iso88599CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-10") == 0 ||
             strcmp(pCharsetName, "ISO-IR-157") == 0 ||
             strcmp(pCharsetName, "ISO8859-10") == 0 ||
             strcmp(pCharsetName, "ISO_8859-10") == 0 ||
             strcmp(pCharsetName, "ISO_8859-10:1992") == 0 ||
             strcmp(pCharsetName, "L6") == 0 ||
             strcmp(pCharsetName, "LATIN6") == 0 ||
             strcmp(pCharsetName, "CSISOLATIN6") == 0)
    {
        return _Iso885910CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-13") == 0 ||
             strcmp(pCharsetName, "ISO-IR-179") == 0 ||
             strcmp(pCharsetName, "ISO8859-13") == 0 ||
             strcmp(pCharsetName, "ISO_8859-13") == 0 ||
             strcmp(pCharsetName, "L7") == 0 ||
             strcmp(pCharsetName, "LATIN7") == 0)
    {
        return _Iso885913CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-14") == 0 ||
             strcmp(pCharsetName, "ISO-CELTIC") == 0 ||
             strcmp(pCharsetName, "ISO-IR-199") == 0 ||
             strcmp(pCharsetName, "ISO8859-14") == 0 ||
             strcmp(pCharsetName, "ISO_8859-14") == 0 ||
             strcmp(pCharsetName, "ISO_8859-14:1998") == 0 ||
             strcmp(pCharsetName, "L8") == 0 ||
             strcmp(pCharsetName, "LATIN8") == 0)
    {
        return _Iso885914CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "ISO-8859-15") == 0 ||
             strcmp(pCharsetName, "ISO-IR-203") == 0 ||
             strcmp(pCharsetName, "ISO8859-15") == 0 ||
             strcmp(pCharsetName, "ISO_8859-15") == 0 ||
             strcmp(pCharsetName, "ISO_8859-15:1998") == 0 ||
             strcmp(pCharsetName, "LATIN-9") == 0)
    {
        return _Iso885915CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1250") == 0 ||
             strcmp(pCharsetName, "MS-EE") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1250") == 0)
    {
        return _Windows1250CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1251") == 0 ||
             strcmp(pCharsetName, "MS-CYRL") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1251") == 0)
    {
        return _Windows1251CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1252") == 0 ||
             strcmp(pCharsetName, "MS-ANSI") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1252") == 0)
    {
        return _Windows1252CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1253") == 0 ||
             strcmp(pCharsetName, "MS-GREEK") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1253") == 0)
    {
        return _Windows1253CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1254") == 0 ||
             strcmp(pCharsetName, "MS-TURK") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1254") == 0)
    {
        return _Windows1254CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1255") == 0 ||
             strcmp(pCharsetName, "MS-HEBR") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1255") == 0)
    {
        return _Windows1255CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1256") == 0 ||
             strcmp(pCharsetName, "MS-ARAB") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1256") == 0)
    {
        return _Windows1256CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1257") == 0 ||
             strcmp(pCharsetName, "WINBALTRIM") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1257") == 0)
    {
        return _Windows1257CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP1258") == 0 ||
             strcmp(pCharsetName, "WINDOWS-1258") == 0)
    {
        return _Windows1258CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "850") == 0 ||
             strcmp(pCharsetName, "CP850") == 0 ||
             strcmp(pCharsetName, "IBM850") == 0 ||
             strcmp(pCharsetName, "CSPC850MULTILINGUAL") == 0)
    {
        return _Cp850CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "862") == 0 ||
             strcmp(pCharsetName, "CP862") == 0 ||
             strcmp(pCharsetName, "IBM862") == 0 ||
             strcmp(pCharsetName, "CSPC862LATINHEBREW") == 0)
    {
        return _Cp862CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "866") == 0 ||
             strcmp(pCharsetName, "CP866") == 0 ||
             strcmp(pCharsetName, "IBM866") == 0 ||
             strcmp(pCharsetName, "CSIBM866") == 0)
    {
        return _Cp866CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP874") == 0 ||
             strcmp(pCharsetName, "WINDOWS-874") == 0)
    {
        return _Cp874CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "437") == 0 ||
             strcmp(pCharsetName, "CP437") == 0 ||
             strcmp(pCharsetName, "IBM437") == 0 ||
             strcmp(pCharsetName, "CSPC8CODEPAGE437") == 0)
    {
        return _Cp437CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP737") == 0)
    {
        return _Cp737CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP775") == 0 ||
             strcmp(pCharsetName, "IBM775") == 0 ||
             strcmp(pCharsetName, "CSPC775BALTIC") == 0)
    {
        return _Cp775CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "852") == 0 ||
             strcmp(pCharsetName, "CP852") == 0 ||
             strcmp(pCharsetName, "IBM852") == 0 ||
             strcmp(pCharsetName, "CSPCP852") == 0)
    {
        return _Cp852CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "855") == 0 ||
             strcmp(pCharsetName, "CP855") == 0 ||
             strcmp(pCharsetName, "IBM855") == 0 ||
             strcmp(pCharsetName, "CSPCP855") == 0)
    {
        return _Cp855CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "856") == 0 ||
             strcmp(pCharsetName, "CP856") == 0 ||
             strcmp(pCharsetName, "IBM856") == 0 ||
             strcmp(pCharsetName, "CSPCP856") == 0)
    {
        return _Cp856CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "857") == 0 ||
             strcmp(pCharsetName, "CP857") == 0 ||
             strcmp(pCharsetName, "IBM857") == 0 ||
             strcmp(pCharsetName, "CSPCP857") == 0)
    {
        return _Cp857CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP932") == 0)
    {
        return _Cp932CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP936") == 0 ||
             strcmp(pCharsetName, "GBK") == 0 ||
             strcmp(pCharsetName, "MS936") == 0 ||
             strcmp(pCharsetName, "WINDOWS-936") == 0)
    {
        return _Cp936CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP949") == 0 ||
             strcmp(pCharsetName, "UHC") == 0)
    {
        return _Cp949CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP950") == 0)
    {
        return _Cp950CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "860") == 0 ||
             strcmp(pCharsetName, "CP860") == 0 ||
             strcmp(pCharsetName, "IBM860") == 0 ||
             strcmp(pCharsetName, "CSIBM860") == 0)
    {
        return _Cp860CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "861") == 0 ||
             strcmp(pCharsetName, "CP-IS") == 0 ||
             strcmp(pCharsetName, "CP861") == 0 ||
             strcmp(pCharsetName, "IBM861") == 0 ||
             strcmp(pCharsetName, "CSIBM861") == 0)
    {
        return _Cp861CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "863") == 0 ||
             strcmp(pCharsetName, "CP863") == 0 ||
             strcmp(pCharsetName, "IBM863") == 0 ||
             strcmp(pCharsetName, "CSIBM863") == 0)
    {
        return _Cp863CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "865") == 0 ||
             strcmp(pCharsetName, "CP865") == 0 ||
             strcmp(pCharsetName, "IBM865") == 0 ||
             strcmp(pCharsetName, "CSIBM865") == 0)
    {
        return _Cp865CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "869") == 0 ||
             strcmp(pCharsetName, "CP-GR") == 0 ||
             strcmp(pCharsetName, "CP869") == 0 ||
             strcmp(pCharsetName, "IBM869") == 0 ||
             strcmp(pCharsetName, "CSIBM869") == 0)
    {
        return _Cp869CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "CP864") == 0 ||
             strcmp(pCharsetName, "IBM864") == 0 ||
             strcmp(pCharsetName, "CSIBM864") == 0)
    {
        return _Cp864CharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UCS-4BE") == 0)
    {
        return _Utf32beCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "UCS-4LE") == 0)
    {
        return _Utf32leCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "KOI8-R") == 0 || strcmp(pCharsetName, "CSKOI8R") == 0)
    {
        return _Koi8rCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "KOI8-U") == 0)
    {
        return _Koi8uCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "KOI8-RU") == 0)
    {
        return _Koi8ruCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "NEXTSTEP") == 0)
    {
        return _NextStepCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MAC") == 0 ||
             strcmp(pCharsetName, "MACINTOSH") == 0 ||
             strcmp(pCharsetName, "MACROMAN") == 0 ||
             strcmp(pCharsetName, "CSMACINTOSH") == 0)
    {
        return _MacRomanCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACCENTRALEUROPE") == 0)
    {
        return _MacCentralEuropeanCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACICELAND") == 0)
    {
        return _MacIcelandicCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACCROATIAN") == 0)
    {
        return _MacCroatianCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACROMANIA") == 0)
    {
        return _MacRomanianCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACCYRILLIC") == 0)
    {
        return _MacCyrillicCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACUKRAINE") == 0)
    {
        return _MacUkrainianCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACGREEK") == 0)
    {
        return _MacGreekCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACTURKISH") == 0)
    {
        return _MacTurkishCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACHEBREW") == 0)
    {
        return _MacHebrewCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACARABIC") == 0)
    {
        return _MacArabicCharacterSet::getInstance();
    }
    else if (strcmp(pCharsetName, "MACTHAI") == 0)
    {
        return _MacThaiCharacterSet::getInstance();
    }


    //  TODO Process the rest of the charset names below
/*
    else if (strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0 ||
             strcmp(pCharsetName, "") == 0)
    {
        return _Iso88591CharacterSet::getInstance();
    }
       
ISO-10646-UCS-2 UCS-2 CSUNICODE
UCS-2BE UNICODE-1-1 UNICODEBIG CSUNICODE11
UCS-2LE UNICODELITTLE
ISO-10646-UCS-4 UCS-4 CSUCS4
UNICODE-1-1-UTF-7 UTF-7 CSUNICODE11UTF7
UCS-2-INTERNAL
UCS-2-SWAPPED
UCS-4-INTERNAL
UCS-4-SWAPPED
C99
JAVA
HP-ROMAN8 R8 ROMAN8 CSHPROMAN8
ARMSCII-8
GEORGIAN-ACADEMY
GEORGIAN-PS
KOI8-T
MULELAO-1
CP1133 IBM-CP1133
ISO-IR-166 TIS-620 TIS620 TIS620-0 TIS620.2529-1 TIS620.2533-0 TIS620.2533-1
VISCII VISCII1.1-1 CSVISCII
TCVN TCVN-5712 TCVN5712-1 TCVN5712-1:1993
ISO-IR-14 ISO646-JP JIS_C6220-1969-RO JP CSISO14JISC6220RO
JISX0201-1976 JIS_X0201 X0201 CSHALFWIDTHKATAKANA
ISO-IR-87 JIS0208 JIS_C6226-1983 JIS_X0208 JIS_X0208-1983 JIS_X0208-1990 X0208 CSISO87JISX0208
ISO-IR-159 JIS_X0212 JIS_X0212-1990 JIS_X0212.1990-0 X0212 CSISO159JISX02121990
CN GB_1988-80 ISO-IR-57 ISO646-CN CSISO57GB1988
CHINESE GB_2312-80 ISO-IR-58 CSISO58GB231280
CN-GB-ISOIR165 ISO-IR-165
ISO-IR-149 KOREAN KSC_5601 KS_C_5601-1987 KS_C_5601-1989 CSKSC56011987
EUC-JP EUCJP EXTENDED_UNIX_CODE_PACKED_FORMAT_FOR_JAPANESE CSEUCPKDFMTJAPANESE
MS_KANJI SHIFT-JIS SHIFT_JIS SJIS CSSHIFTJIS
ISO-2022-JP CSISO2022JP
ISO-2022-JP-1
ISO-2022-JP-2 CSISO2022JP2
CN-GB EUC-CN EUCCN GB2312 CSGB2312
GB18030
ISO-2022-CN CSISO2022CN
ISO-2022-CN-EXT
HZ HZ-GB-2312
EUC-TW EUCTW CSEUCTW
BIG-5 BIG-FIVE BIG5 BIGFIVE CN-BIG5 CSBIG5
BIG5-HKSCS BIG5HKSCS
EUC-KR EUCKR CSEUCKR
CP1361 JOHAB
ISO-2022-KR CSISO2022KR
CP922
CP943
CP1046
CP1124
CP1129
CP1161 IBM-1161 IBM1161 CSIBM1161
CP1162 IBM-1162 IBM1162 CSIBM1162
CP1163 IBM-1163 IBM1163 CSIBM1163
DEC-KANJI
DEC-HANYU
CP853
CP858
CP1125
EUC-JISX0213
SHIFT_JISX0213
ISO-2022-JP-3
ISO-IR-230 TDS565
RISCOS-LATIN1
*/
    else
    {
        return _Iso88591CharacterSet::getInstance();
    }
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC)
    CharacterSet * pSystemCharacterSet = findCharacterSet(String(nl_langinfo(CODESET)));
    if (pSystemCharacterSet == NULL)
    {
        pSystemCharacterSet = _Iso88591CharacterSet::getInstance();
    }
    return pSystemCharacterSet;
#else
    #error Unsupported platform
#endif
}

//  End of CharsetManagerImpl.cpp
