//
//  CharsetResourceMap_en.cpp - Implementation of the carl::_CharsetResourceMap_en class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/07 19:25:06 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Construction/destruction
_impl::_CharsetResourceMap_en::_CharsetResourceMap_en()
    :   _mapKeysToValues(StandardHashFunctions::hashString)
{
    //  ISO-8859 character sets
    _mapKeysToValues.insert(String("CSF:ISO_8859"), String("ISO 8859"));
    _mapKeysToValues.insert(String("CS:ISO_8859-1:1987"), String("ISO Latin 1"));
    _mapKeysToValues.insert(String("CS:ISO_8859-2:1987"), String("ISO Latin 2"));
    _mapKeysToValues.insert(String("CS:ISO_8859-3:1988"), String("ISO Latin 3"));
    _mapKeysToValues.insert(String("CS:ISO_8859-4:1988"), String("ISO Latin 4"));
    _mapKeysToValues.insert(String("CS:ISO_8859-5:1988"), String("ISO Latin Cyrillic"));
    _mapKeysToValues.insert(String("CS:ISO_8859-6:1987"), String("ISO Latin Arabic"));
    _mapKeysToValues.insert(String("CS:ISO_8859-7:1987"), String("ISO Latin Greek"));
    _mapKeysToValues.insert(String("CS:ISO_8859-8:1988"), String("ISO Latin Hebrew"));
    _mapKeysToValues.insert(String("CS:ISO_8859-9:1989"), String("ISO Latin 5"));
    _mapKeysToValues.insert(String("CS:ISO_8859-10:1992"), String("ISO Latin 6"));
    _mapKeysToValues.insert(String("CS:ISO_8859-11:1999"), String("ISO Thai"));
    _mapKeysToValues.insert(String("CS:ISO_8859-13:1998"), String("ISO Latin 7"));
    _mapKeysToValues.insert(String("CS:ISO_8859-14:1998"), String("ISO Latin 8"));
    _mapKeysToValues.insert(String("CS:ISO_8859-15"), String("ISO Latin 9"));
    //  ISO 10646 character sets
    _mapKeysToValues.insert(String("CSF:ISO_10646"), String("ISO 10646"));
    _mapKeysToValues.insert(String("CS:UTF-32BE"), String("UTF-32 Big-Endian"));
    _mapKeysToValues.insert(String("CS:UTF-32LE"), String("UTF-32 Little-Endian"));
    _mapKeysToValues.insert(String("CS:UTF-16BE"), String("UTF-16 Big-Endian"));
    _mapKeysToValues.insert(String("CS:UTF-16LE"), String("UTF-16 Little-Endian"));
    _mapKeysToValues.insert(String("CS:UTF-8"), String("UTF-8"));
    //  Windows character sets
    _mapKeysToValues.insert(String("CSF:Windows"), String("Windows"));
    _mapKeysToValues.insert(String("CS:Windows-1250"), String("Windows-1250 (Central Europe)"));
    _mapKeysToValues.insert(String("CS:Windows-1251"), String("Windows-1251 (Cyrillic)"));
    _mapKeysToValues.insert(String("CS:Windows-1252"), String("Windows-1252 (Latin1)"));
    _mapKeysToValues.insert(String("CS:Windows-1253"), String("Windows-1253 (Greek)"));
    _mapKeysToValues.insert(String("CS:Windows-1254"), String("Windows-1254 (Turkish)"));
    _mapKeysToValues.insert(String("CS:Windows-1255"), String("Windows-1255 (Hebrew)"));
    _mapKeysToValues.insert(String("CS:Windows-1256"), String("Windows-1256 (Arabic)"));
    _mapKeysToValues.insert(String("CS:Windows-1257"), String("Windows-1257 (Baltic)"));
    _mapKeysToValues.insert(String("CS:Windows-1258"), String("Windows-1258 (Viet Nam)"));
    _mapKeysToValues.insert(String("CS:cp874"), String("Cp874 (Thai)"));
    _mapKeysToValues.insert(String("CS:CP932"), String("Cp932 (Japanese Shift-JIS)"));
    _mapKeysToValues.insert(String("CS:CP936"), String("Cp936 (Simplified Chinese GBK)"));
    _mapKeysToValues.insert(String("CS:CP949"), String("Cp949 (Korean)"));
    _mapKeysToValues.insert(String("CS:CP950"), String("Cp950 (Traditional Chinese Big5)"));
    //  PC character sets
    _mapKeysToValues.insert(String("CSF:PC"), String("IBM PC"));
    _mapKeysToValues.insert(String("CS:IBM437"), String("Cp437 (United States)"));
    _mapKeysToValues.insert(String("CS:IBM737"), String("Cp737 (Greek)"));
    _mapKeysToValues.insert(String("CS:IBM775"), String("Cp775 (Baltic)"));
    _mapKeysToValues.insert(String("CS:IBM850"), String("Cp850 (Multilingual Latin I)"));
    _mapKeysToValues.insert(String("CS:IBM852"), String("Cp852 (Latin II)"));
    _mapKeysToValues.insert(String("CS:IBM855"), String("Cp855 (Cyrillic)"));
    _mapKeysToValues.insert(String("CS:IBM856"), String("Cp856 (Hebrew)"));
    _mapKeysToValues.insert(String("CS:IBM857"), String("Cp857 (Turkish)"));
    _mapKeysToValues.insert(String("CS:IBM860"), String("Cp860 (Portuguese)"));
    _mapKeysToValues.insert(String("CS:IBM861"), String("Cp861 (Icelandic)"));
    _mapKeysToValues.insert(String("CS:IBM862"), String("Cp862 (Hebrew)"));
    _mapKeysToValues.insert(String("CS:IBM863"), String("Cp863 (Canadian French)"));
    _mapKeysToValues.insert(String("CS:IBM864"), String("Cp864 (Arabic)"));
    _mapKeysToValues.insert(String("CS:IBM865"), String("Cp865 (Nordic)"));
    _mapKeysToValues.insert(String("CS:IBM866"), String("Cp866 (Russian)"));
    _mapKeysToValues.insert(String("CS:IBM869"), String("Cp869 (Modern Greek)"));
    //  EBCDIC character sets
    _mapKeysToValues.insert(String("CSF:EBCDIC"), String("EBCDIC"));
    _mapKeysToValues.insert(String("CS:IBM037"), String("Cp037 (IBM US & Canada)"));
    _mapKeysToValues.insert(String("CS:IBM424"), String("Cp500 (IBM Hebrew)"));
    _mapKeysToValues.insert(String("CS:IBM500"), String("Cp500 (IBM International)"));
    _mapKeysToValues.insert(String("CS:IBM875"), String("Cp875 (IBM Greek)"));
    _mapKeysToValues.insert(String("CS:IBM1026"), String("Cp1026 (IBM Turkish)"));
    //  Adobe character sets
    _mapKeysToValues.insert(String("CSF:ADOBE"), String("Adobe"));
    _mapKeysToValues.insert(String("CS:ADOBESTANDARD"), String("Adobe Standard"));
    _mapKeysToValues.insert(String("CS:ADOBESYMBOL"), String("Adobe Symbol"));
    _mapKeysToValues.insert(String("CS:ADOBEZAPFDINGBATS"), String("Adobe Zapf Dingbats"));
    //  KOI character sets
    _mapKeysToValues.insert(String("CSF:KOI"), String("KOI"));
    _mapKeysToValues.insert(String("CS:GOST-19768-87"), String("GOST-19768-87"));
    _mapKeysToValues.insert(String("CS:KOI7"), String("KOI7"));
    _mapKeysToValues.insert(String("CS:KOI8-A"), String("KOI8-A"));
    _mapKeysToValues.insert(String("CS:KOI8-B"), String("KOI8-B"));
    _mapKeysToValues.insert(String("CS:KOI8-E"), String("KOI8-E"));
    _mapKeysToValues.insert(String("CS:KOI8-F"), String("KOI8-F"));
    _mapKeysToValues.insert(String("CS:KOI8-R"), String("KOI8-R"));
    _mapKeysToValues.insert(String("CS:KOI8-RU"), String("KOI8-RU"));
    _mapKeysToValues.insert(String("CS:KOI8-U"), String("KOI8-U"));
    //  MAC character sets
    _mapKeysToValues.insert(String("CSF:MAC"), String("Mac OS"));
    _mapKeysToValues.insert(String("CS:MacArabic"), String("Mac OS Arabic"));
    _mapKeysToValues.insert(String("CS:MacCeltic"), String("Mac OS Celtic"));
    _mapKeysToValues.insert(String("CS:MacCentEuro"), String("Mac OS Central European"));
    _mapKeysToValues.insert(String("CS:MacCroatian"), String("Mac OS Croatian"));
    _mapKeysToValues.insert(String("CS:MacCyrillic"), String("Mac OS Cyrillic"));
    _mapKeysToValues.insert(String("CS:MacDevanagari"), String("Mac OS Devanagari"));
    _mapKeysToValues.insert(String("CS:MacDingbats"), String("Mac OS Dingbats"));
    _mapKeysToValues.insert(String("CS:MacFarsi"), String("Mac OS Farsi"));
    _mapKeysToValues.insert(String("CS:MacGaelic"), String("Mac OS Gaelic"));
    _mapKeysToValues.insert(String("CS:MacGreek"), String("Mac OS Greek"));
    _mapKeysToValues.insert(String("CS:MacGujarati"), String("Mac OS Gujarati"));
    _mapKeysToValues.insert(String("CS:MacGurmukhi"), String("Mac OS Gurmukhi"));
    _mapKeysToValues.insert(String("CS:MacHebrew"), String("Mac OS Hebrew"));
    _mapKeysToValues.insert(String("CS:MacIcelandic"), String("Mac OS Icelandic"));
    _mapKeysToValues.insert(String("CS:MacInuit"), String("Mac OS Inuit"));
    _mapKeysToValues.insert(String("CS:MacRoman"), String("Mac OS Roman"));
    _mapKeysToValues.insert(String("CS:MacRomanian"), String("Mac OS Romanian"));
    _mapKeysToValues.insert(String("CS:MacSymbol"), String("Mac OS Symbol"));
    _mapKeysToValues.insert(String("CS:MacThai"), String("Mac OS Thai"));
    _mapKeysToValues.insert(String("CS:MacTurkish"), String("Mac OS Turkish"));
    _mapKeysToValues.insert(String("CS:MacUkrainian"), String("Mac OS Ukrainian"));
    _mapKeysToValues.insert(String("CS:MacChineseSimplified"), String("Mac OS Chinese Simplified"));
    _mapKeysToValues.insert(String("CS:MacChineseTraditional"), String("Mac OS Chinese Traditional"));
    _mapKeysToValues.insert(String("CS:MacKorean"), String("Mac OS Korean"));
    _mapKeysToValues.insert(String("CS:MacJapanese"), String("Mac OS Japanese"));
    //  Miscellaneous character sets
    _mapKeysToValues.insert(String("CSF:MISCELLANEOUS"), String("Miscellaneous"));
    _mapKeysToValues.insert(String("CS:ANSI_X3.4-1968"), String("US ASCII"));
    _mapKeysToValues.insert(String("CS:NEXTSTEP"), String("NextStep"));
    _mapKeysToValues.insert(String("CS:ATARIST"), String("Atari ST"));
}

_impl::_CharsetResourceMap_en::~_CharsetResourceMap_en()
{
}

//////////
//  Operations
String _impl::_CharsetResourceMap_en::getString(const String & key)
{
    if (_mapKeysToValues.containsKey(key))
    {
        return _mapKeysToValues.get(key);
    }
    throw MissingResourceError(key);
}

//  End of CharsetResourceMap_en.cpp
