//
//  Cipher.cpp - Implementation of the carl::Cipher class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Operations
void Cipher::registerInstance(Cipher * pCipher) throw (InconsistentCipherException, DuplicateCipherException)
{
    require(pCipher != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_EncryptionManagerImpl::getInstance()->registerCipher(pCipher);
}

Array<Cipher*> Cipher::getAllInstances()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_EncryptionManagerImpl::getInstance()->getAllCiphers();
}

Cipher * Cipher::getInstance(const char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

Cipher * Cipher::getInstance(const Char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

Cipher * Cipher::getInstance(const String & name)
{
    CriticalSectionLock lock(*_getStaticGuard());
    Cipher * pCipher = _impl::_EncryptionManagerImpl::getInstance()->findCipher(name);
    if (pCipher != NULL)
    {
        return pCipher;
    }
    throw NoSuchElementError();
}

//////////
//  Implementation
CriticalSection * Cipher::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of Cipher.cpp
