//
//  Collator.cpp - Implementation of the carl::Collator class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CollatorImpl.hpp"
#include <stdlib.h>

//////////
//  Operations
void Collator::registerInstance(const Locale & forLocale, Collator * pCollator)
{
    require(pCollator != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CollatorManager::getInstance()->registerCollator(forLocale, pCollator);
}

Collator * Collator::getInstance()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(Locale::getDefault());
}

Collator * Collator::getInstance(const Locale & forLocale)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_CollatorManager::getInstance()->getCollator(forLocale);
}

String Collator::performDecomposition(const String & s, Decomposition decomposition)
{
    switch (decomposition)
    {
        case Decomposition_None:
            return s;
        case Decomposition_Canonical:
            return Iso10646::performCanonicalDecomposition(s);
        case Decomposition_Full:
            return Iso10646::performCompatibilityDecomposition(s);
        default:
            require(false);
            return s;
    }
}

//////////
//  Implementation
CriticalSection * Collator::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of Collator.cpp
