//
//  CollatorBasedStringComparator.cpp - Implementation of the carl::CollatorBasedStringComparator class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/26 11:08:27 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
CollatorBasedStringComparator::CollatorBasedStringComparator(Collator * pCollator, Collator::CollationStrength collationStrength, Collator::Decomposition decomposition)
    :   _pCollator(pCollator),
        _collationStrength(collationStrength),
        _decomposition(decomposition)
{
    require(pCollator != NULL);
}

CollatorBasedStringComparator::CollatorBasedStringComparator(const CollatorBasedStringComparator & src)
    :   _pCollator(src._pCollator),
        _collationStrength(src._collationStrength),
        _decomposition(src._decomposition)
{
}

CollatorBasedStringComparator::~CollatorBasedStringComparator()
{
}

CollatorBasedStringComparator & CollatorBasedStringComparator::operator = (const CollatorBasedStringComparator & src)
{
    if (this != (&src))
    {
        _pCollator = src._pCollator;
        _collationStrength = src._collationStrength;
        _decomposition = src._decomposition;
    }
    return *this;
}

//////////
//  Operations
bool CollatorBasedStringComparator::equal(const String & value1, const String & value2) const
{
    return _pCollator->collate(value1, value2, _collationStrength, _decomposition) == 0;
}

bool CollatorBasedStringComparator::less(const String & value1, const String & value2) const
{
    return _pCollator->collate(value1, value2, _collationStrength, _decomposition) < 0;
}

bool CollatorBasedStringComparator::greater(const String & value1, const String & value2) const
{
    return _pCollator->collate(value1, value2, _collationStrength, _decomposition) > 0;
}

//  End of CollatorBasedStringComparator.cpp
