//
//  CollatorImpl.hpp - carl::_impl::_CollatorImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/26 11:08:27 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __CollatorImpl_hpp_included__
#define __CollatorImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Collator manager
        class CARL_PUBLIC _CollatorManager
        {
            //////////
            //  Can't assign or copy construct
        private:
            _CollatorManager(const _CollatorManager &);
            void                            operator = (const _CollatorManager &);

            //////////
            //  Singleton
        private:
            _CollatorManager();
            ~_CollatorManager() {}
        public:
            static _CollatorManager *       getInstance();

            //////////
            //  Operations
        public:
            void                            registerCollator(const Locale & forLocale, Collator * pCollator);
            Collator *                      getCollator(const Locale & forLocale);

            //////////
            //  Implementation
        private:
            HashMap<Locale, Collator*>      _mapLocalesToCollators;
            Collator *                      _pDefaultCollator;
            CriticalSection                 _guard;
        };

        //////////
        //  The default collator (uses ISO-10646 character codes for collation)
        class CARL_PUBLIC _DefaultCollator : public Collator
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultCollator(const _DefaultCollator &);
            void                            operator = (const _DefaultCollator &);

            //////////
            //  Singleton
        private:
            _DefaultCollator();
            virtual ~_DefaultCollator();
        public:
            static _DefaultCollator *       getInstance();

            //////////
            //  Operations
        public:
            virtual int                     collate(const String & s1, const String & s2, CollationStrength collationStrength = CollationStrength_Primary, Decomposition decomposition = Decomposition_Full);

            //////////
            //  Implementation
        private:
            static String                   _performDecomposition(const String & s, Decomposition decomposition);
            static int                      _calculatePrimaryDifference(Char c1, Char c2);
            static int                      _calculateSecondaryDifference(Char c1, Char c2);
            static int                      _calculateTertiaryDifference(Char c1, Char c2);
            static Char                     _getBaseCharacter(Char c);
        };
    }
}

#endif  //  __CollatorImpl_hpp_included__
//  End of CollatorImpl.hpp
