//
//  CompressionManagerImpl.cpp - Implementation of the carl::_impl::_CompressionManagerImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:49 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_CompressionManagerImpl::_CompressionManagerImpl()
    :   _resourceFactory(new _CompressionResourceMap_en(), carl::String("CARL_COMPRESSION")),
        _mapNamesToCompressionAlgorithms(StandardHashFunctions::hashString)
{
    registerCompressionAlgorithm(_NullCompressionAlgorithm::getInstance());
    registerCompressionAlgorithm(_RunLengthCompressionAlgorithm::getInstance());
    registerCompressionAlgorithm(_LzwCompressionAlgorithm::getInstance());
}

_impl::_CompressionManagerImpl * _impl::_CompressionManagerImpl::getInstance()
{
    static _CompressionManagerImpl instance;
    return &instance;
}

//////////
//  Operations
MapResourceFactory * _impl::_CompressionManagerImpl::getResourceFactory() const
{
    return &_resourceFactory;
}

Array<CompressionAlgorithm*> _impl::_CompressionManagerImpl::getAllCompressionAlgorithms()
{
    CriticalSectionLock lock(_guard);
    return _compressionAlgorithms;
}

CompressionAlgorithm * _impl::_CompressionManagerImpl::findCompressionAlgorithm(const String & name)
{
    CriticalSectionLock lock(_guard);
    String nameUppercased = name.upperCased();
    if (_mapNamesToCompressionAlgorithms.containsKey(nameUppercased))
    {
        return _mapNamesToCompressionAlgorithms.get(nameUppercased);
    }
    return NULL;
}

void _impl::_CompressionManagerImpl::registerCompressionAlgorithm(CompressionAlgorithm * pCompressionAlgorithm) throw (InconsistentCompressionAlgorithmException, DuplicateCompressionAlgorithmException)
{
    require(pCompressionAlgorithm != NULL);

    CriticalSectionLock lock(_guard);
    Array<String> names = pCompressionAlgorithm->getAllNames();
    for (UIntSize i = 0; i < names.size(); i++)
    {
        names[i].upperCase();
        if (_mapNamesToCompressionAlgorithms.containsKey(names[i]))
        {
            throw DuplicateCompressionAlgorithmException();
        }
    }
    if (!names.contains(pCompressionAlgorithm->getCanonicalName().upperCased()) ||
        !names.contains(pCompressionAlgorithm->getPreferredName().upperCased()))
    {
        throw InconsistentCompressionAlgorithmException();
    }
    //
    _compressionAlgorithms.append(pCompressionAlgorithm);
    for (UIntSize i = 0; i < names.size(); i++)
    {
        _mapNamesToCompressionAlgorithms.insert(names[i], pCompressionAlgorithm);
    }
}

//  End of CompressionManagerImpl.cpp
