//
//  CriticalSectionImpl.cpp - Implementation of the carl::_impl::_CriticalSectionImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CriticalSectionImpl.hpp"

//////////
//  Operations
void _impl::_CriticalSectionImpl::initialize()
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    InitializeCriticalSection(&_osCriticalSection);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    pthread_mutexattr_t mutexAttr;
    switch (pthread_mutexattr_init(&mutexAttr))
    {
        case 0:
            break;
        case ENOMEM:
            throw OutOfMemoryError(0);
        default:
            throw SystemError();
    }
    if (pthread_mutexattr_settype(&mutexAttr, PTHREAD_MUTEX_RECURSIVE) != 0)
    {
        throw SystemError();
    }
    switch (pthread_mutex_init(&_mutexData, &mutexAttr))
    {
        case 0:
            break;
        case ENOMEM:
            throw OutOfMemoryError(0);
        default:
            throw SystemError();
    }
#else
    #error Unsupported platform
#endif
}

void _impl::_CriticalSectionImpl::deinitialize()
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    DeleteCriticalSection(&_osCriticalSection);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    switch (pthread_mutex_destroy(&_mutexData))
    {
        case 0:
            break;
        case EBUSY:
            throw ProgramError();
        default:
            throw SystemError();
    }
#else
    #error Unsupported platform
#endif
}

void _impl::_CriticalSectionImpl::enter()
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    EnterCriticalSection(&_osCriticalSection);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    switch (pthread_mutex_lock(&_mutexData))
    {
        case 0:
            return;
        case EINVAL:
        case EAGAIN:
        case EDEADLK:
            throw ProgramError();
        default:
            throw SystemError();
    }
#else
    #error Unsupported platform
#endif
}

void _impl::_CriticalSectionImpl::leave()
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    LeaveCriticalSection(&_osCriticalSection);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    switch (pthread_mutex_unlock(&_mutexData))
    {
        case 0:
            return;
        case EPERM:
            throw ProgramError();
        default:
            throw SystemError();
    }
#else
    #error Unsupported platform
#endif
}

//  End of CriticalSectionImpl.cpp
