//
//  CriticalSectionImpl.hpp - carl::_impl::_CriticalSectionImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __CriticalSectionImpl_hpp_included__
#define __CriticalSectionImpl_hpp_included__

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <windows.h>
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #include <stdlib.h>
    #include <pthread.h>
    #include <errno.h>
#else
    #error Unsupported platform
#endif

namespace carl
{
    namespace _impl
    {
        //////////
        //  A platform-specific implementation of a critical section
        class CARL_PUBLIC _CriticalSectionImpl
        {
            //////////
            //  Data
            #if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
                CRITICAL_SECTION    _osCriticalSection;
            #elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
                  (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
                pthread_mutex_t     _mutexData;
            #else
                #error Unsupported platform
            #endif

            //////////
            //  Operations
        public:
            void                    initialize();
            void                    deinitialize();
            void                    enter();
            void                    leave();
        };
    }
}

#endif  //  __CriticalSectionImpl_hpp_included__
//  End of CriticalSectionImpl.hpp
