//
//  CryptoKey.cpp - Implementation of the carl::CryptoKey class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
CryptoKey::CryptoKey(const void *pBits, UIntSize keyLengthBits)
{
    require(pBits != NULL);

    _pBits = new UInt8[keyLengthBits / System::BitsPerByte + 1];
    Memory::copy(_pBits, pBits, keyLengthBits / System::BitsPerByte + 1);
    _keyLengthBits = keyLengthBits;
}

CryptoKey::CryptoKey(const CryptoKey & src)
{
    _pBits = new UInt8[src._keyLengthBits / System::BitsPerByte + 1];
    Memory::copy(_pBits, src._pBits, src._keyLengthBits / System::BitsPerByte + 1);
    _keyLengthBits = src._keyLengthBits;
}

CryptoKey & CryptoKey::operator = (const CryptoKey & src)
{
    if (this != (&src))
    {
        if (_keyLengthBits != src._keyLengthBits)
        {
            delete [] _pBits;
            _pBits = new UInt8[src._keyLengthBits / System::BitsPerByte + 1];
        }
        Memory::copy(_pBits, src._pBits, src._keyLengthBits / System::BitsPerByte + 1);
        _keyLengthBits = src._keyLengthBits;
    }
    return *this;
}

//  End of CryptoKey.cpp
