//
//  DataInputStream.cpp - Implementation of the carl::DataInputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdlib.h>

//////////
//  Construction/destruction
DataInputStream::DataInputStream(IByteInput * pSource)
{
    require(pSource != NULL);

    _pSource = pSource;
}

//////////
//  IDataInput
bool DataInputStream::readBool() throw(IOException)
{
    require(isOpen());

    UInt8 b;
    if (_pSource->read(&b, 1) != 1)
    {
        throw EOFException();
    }
    switch (b)
    {
        case 0:
            return false;
        case 1:
            return true;
        default:
            throw InvalidDataException();
    }
}

UInt8 DataInputStream::readUInt8() throw(IOException)
{
    require(isOpen());

    UInt8 b;
    if (_pSource->read(&b, 1) != 1)
    {
        throw EOFException();
    }
    return b;
}

UInt16 DataInputStream::readUInt16() throw(IOException)
{
    require(isOpen());

    UInt8 b[2];
    if (read(&b, 2) != 2)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedUInt16(b, ByteOrder_BigEndian);
}

UInt32 DataInputStream::readUInt32() throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    if (_pSource->read(&b, 4) != 4)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedUInt32(b, ByteOrder_BigEndian);
}

UInt64 DataInputStream::readUInt64() throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    if (_pSource->read(&b, 8) != 8)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedUInt64(b, ByteOrder_BigEndian);
}

Int8 DataInputStream::readInt8() throw(IOException)
{
    require(isOpen());

    Int8 b;
    if (_pSource->read(&b, 1) != 1)
    {
        throw EOFException();
    }
    return b;
}

Int16 DataInputStream::readInt16() throw(IOException)
{
    require(isOpen());

    UInt8 b[2];
    if (_pSource->read(&b, 2) != 2)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedInt16(b, ByteOrder_BigEndian);
}

Int32 DataInputStream::readInt32() throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    if (_pSource->read(&b, 4) != 4)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedInt32(b, ByteOrder_BigEndian);
}

Int64 DataInputStream::readInt64() throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    if (_pSource->read(&b, 8) != 8)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedInt64(b, ByteOrder_BigEndian);
}

Real32 DataInputStream::readReal32() throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    if (_pSource->read(&b, 4) != 4)
    {
        throw EOFException();
    }
    UInt32 valueBits = Memory::loadUnalignedUInt32(b, ByteOrder_BigEndian);
    return *(reinterpret_cast<Real32*>(&valueBits));
}

Real64 DataInputStream::readReal64() throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    if (_pSource->read(&b, 8) != 8)
    {
        throw EOFException();
    }
    UInt64 valueBits = Memory::loadUnalignedUInt64(b, ByteOrder_BigEndian);
    return *(reinterpret_cast<Real64*>(&valueBits));
}

Char DataInputStream::readChar() throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    if (_pSource->read(&b, 4) != 4)
    {
        throw EOFException();
    }
    return Memory::loadUnalignedUInt32(b, ByteOrder_BigEndian);
}

//  End of DataInputStream.cpp
