//
//  DataOutputStream.cpp - Implementation of the carl::DataOutputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdlib.h>

//////////
//  Construction/destruction
DataOutputStream::DataOutputStream(IByteOutput * pTarget)
{
    require(pTarget != NULL);
    _pTarget = pTarget;
}

//////////
//  IDataOutput
void DataOutputStream::writeBool(bool value) throw(IOException)
{
    require(isOpen());

    UInt8 b = (value) ? (1) : (0);
    _pTarget->write(&b, 1);
}

void DataOutputStream::writeUInt8(UInt8 value) throw(IOException)
{
    require(isOpen());

    _pTarget->write(&value, 1);
}

void DataOutputStream::writeUInt16(UInt16 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[2];
    Memory::storeUnalignedUInt16(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 2);
}

void DataOutputStream::writeUInt32(UInt32 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    Memory::storeUnalignedUInt32(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 4);
}

void DataOutputStream::writeUInt64(UInt64 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    Memory::storeUnalignedUInt64(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 8);
}

void DataOutputStream::writeInt8(Int8 value) throw(IOException)
{
    require(isOpen());

    _pTarget->write(&value, 1);
}

void DataOutputStream::writeInt16(Int16 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[2];
    Memory::storeUnalignedInt16(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 2);
}

void DataOutputStream::writeInt32(Int32 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    Memory::storeUnalignedInt32(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 4);
}

void DataOutputStream::writeInt64(Int64 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    Memory::storeUnalignedInt64(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 8);
}

void DataOutputStream::writeReal32(Real32 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    Memory::storeUnalignedUInt32(*(reinterpret_cast<UInt32*>(&value)), &b, ByteOrder_BigEndian);
    _pTarget->write(b, 4);
}

void DataOutputStream::writeReal64(Real64 value) throw(IOException)
{
    require(isOpen());

    UInt8 b[8];
    Memory::storeUnalignedUInt64(*(reinterpret_cast<UInt64*>(&value)), &b, ByteOrder_BigEndian);
    _pTarget->write(b, 8);
}

void DataOutputStream::writeChar(Char value) throw(IOException)
{
    require(isOpen());

    UInt8 b[4];
    Memory::storeUnalignedUInt32(value, &b, ByteOrder_BigEndian);
    _pTarget->write(b, 4);
}

//  End of DataOutputStream.cpp
