//
//  Date.cpp - Implementation of the carl::Date class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
Date::Date()
{
    _year = 0;
    _month = 0;
    _day = 0;
}

Date::Date(int year, int month, int day)
{
    require(year > 0);
    require(month >= 1 && month <= 12);
    require(day >= 1 && day <= Calendar::daysInMonth(year, month));

    _year = static_cast<UInt16>(year);
    _month = static_cast<UInt8>(month);
    _day = static_cast<UInt8>(day);
}

Date::Date(const Date & src)
{
    _year = src._year;
    _month = src._month;
    _day = src._day;
}

Date & Date::operator = (const Date & src)
{
    if (this != (&src))
    {
        _year = src._year;
        _month = src._month;
        _day = src._day;
    }
    return *this;
}

//////////
//  Operations
DayOfWeek Date::getDayOfWeek() const
{
    require(isValid());

    return Calendar::getDayOfWeek(_year, _month, _day);
}

int Date::getDayOfYear() const
{
    require(isValid());

    return Calendar::getDayOfYear(_year, _month, _day);
}

int Date::compare(const Date & date) const
{
    if (isValid())
    {
        if (date.isValid())
        {   //  <valid date> <compare to> <valid date>
            if (_year > date._year)
            {
                return 1;
            }
            if (_year < date._year)
            {
                return -1;
            }
            if (_month > date._month)
            {
                return 1;
            }
            if (_month < date._month)
            {
                return -1;
            }
            if (_day > date._day)
            {
                return 1;
            }
            if (_day < date._day)
            {
                return -1;
            }
            return 0;
        }
        else
        {   //  <valid date> <compare to> <invalid date>
            return 1;
        }
    }
    else
    {
        if (date.isValid())
        {   //  <invalid date> <compare to> <valid date>
            return -1;
        }
        else
        {   //  <invalid date> <compare to> <invalid date>
            return 0;
        }
    }
}

//////////
//  Hashing
UIntSize CARL_CALLBACK StandardHashFunctions::hashDate(const Date & key)
{
    return key.getDay() + 31 * (key.getMonth() + 12 * key.getYear());
}

//  End of Date.cpp
