//
//  DateParse.cpp - Implementation of the carl::DateParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:51 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Operations
void DateParse::registerInstance(const Locale & forLocale, DateParse * pDateParse)
{
    require(pDateParse != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_DateTimeFormatManager::getInstance()->registerDateParse(forLocale, pDateParse);
}

DateParse * DateParse::getInstance()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(Locale::getDefault());
}

DateParse * DateParse::getInstance(const Locale & forLocale)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeFormatManager::getInstance()->getDateParse(forLocale);
}

//  Parsing of full strings
Date DateParse::parse(const char * pDateString) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const char * pDateString, Style style) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan, style);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const char * pDateString, const String & format) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan, format);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const Char * pDateString) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const Char * pDateString, Style style) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan, style);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const Char * pDateString, const String & format) const throw(ParseException)
{
    require(pDateString != NULL);

    UIntSize scan = 0;
    Date result = parse(pDateString, scan, format);
    if (pDateString[scan] != 0)
    { 
        throw ParseException(String(pDateString), scan);
    }
    return result;
}

Date DateParse::parse(const String & dateString) const throw(ParseException)
{
    UIntSize scan = 0;
    Date result = parse(dateString, scan);
    if (scan < dateString.length())
    { 
        throw ParseException(dateString, scan);
    }
    return result;
}

Date DateParse::parse(const String & dateString, Style style) const throw(ParseException)
{
    UIntSize scan = 0;
    Date result = parse(dateString, scan, style);
    if (scan < dateString.length())
    { 
        throw ParseException(dateString, scan);
    }
    return result;
}

Date DateParse::parse(const String & dateString, const String & format) const throw(ParseException)
{
    UIntSize scan = 0;
    Date result = parse(dateString, scan, format);
    if (scan < dateString.length())
    { 
        throw ParseException(dateString, scan);
    }
    return result;
}

//////////
//  Implementation
CriticalSection * DateParse::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of DateTimeFormat.cpp
