//
//  DateTimeFormatImpl.hpp - carl::DateTimeFormat implementation helpers
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.11 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __DateTimeFormatImpl_hpp_included__
#define __DateTimeFormatImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Date/time format resources
        class CARL_PUBLIC _DateTimeFormatResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DateTimeFormatResourceMap_en(const _DateTimeFormatResourceMap_en &);
            void                        operator = (const _DateTimeFormatResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _DateTimeFormatResourceMap_en();
            virtual ~_DateTimeFormatResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String              getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>     _mapKeysToValues;
        };

        //////////
        //  Date+Time format manager
        class CARL_PUBLIC _DateTimeFormatManager
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DateTimeFormatManager(const _DateTimeFormatManager &);
            void                                operator = (const _DateTimeFormatManager &);

            //////////
            //  Singleton
        private:
            _DateTimeFormatManager();
            ~_DateTimeFormatManager();
        public:
            static _DateTimeFormatManager *     getInstance();

            //////////
            //  Operations
        public:
            MapResourceFactory *                getResourceFactory() const;
            DateTimeFormat *                    getDateTimeFormat(const Locale & forLocale);
            DateFormat *                        getDateFormat(const Locale & forLocale);
            TimeFormat *                        getTimeFormat(const Locale & forLocale);
            DateTimeParse *                     getDateTimeParse(const Locale & forLocale);
            DateParse *                         getDateParse(const Locale & forLocale);
            TimeParse *                         getTimeParse(const Locale & forLocale);
            void                                registerDateTimeFormat(const Locale & forLocale, DateTimeFormat * pDateTimeFormat);
            void                                registerDateFormat(const Locale & forLocale, DateFormat * pDateFormat);
            void                                registerTimeFormat(const Locale & forLocale, TimeFormat * pTimeFormat);
            void                                registerDateTimeParse(const Locale & forLocale, DateTimeParse * pDateTimeParse);
            void                                registerDateParse(const Locale & forLocale, DateParse * pDateParse);
            void                                registerTimeParse(const Locale & forLocale, TimeParse * pTimeParse);

            //////////
            //  Resource keys
        public:
            static const char *                 ShortDateTimeFormatStringMinutes;
            static const char *                 MediumDateTimeFormatStringMinutes;
            static const char *                 LongDateTimeFormatStringMinutes;
            static const char *                 FullDateTimeFormatStringMinutes;

            static const char *                 ShortDateTimeFormatStringSeconds;
            static const char *                 MediumDateTimeFormatStringSeconds;
            static const char *                 LongDateTimeFormatStringSeconds;
            static const char *                 FullDateTimeFormatStringSeconds;

            static const char *                 ShortDateTimeFormatStringMilliseconds;
            static const char *                 MediumDateTimeFormatStringMilliseconds;
            static const char *                 LongDateTimeFormatStringMilliseconds;
            static const char *                 FullDateTimeFormatStringMilliseconds;

            static const char *                 ShortDateTimeFormatStringMicroseconds;
            static const char *                 MediumDateTimeFormatStringMicroseconds;
            static const char *                 LongDateTimeFormatStringMicroseconds;
            static const char *                 FullDateTimeFormatStringMicroseconds;

            static const char *                 ShortDateTimeFormatStringNanoseconds;
            static const char *                 MediumDateTimeFormatStringNanoseconds;
            static const char *                 LongDateTimeFormatStringNanoseconds;
            static const char *                 FullDateTimeFormatStringNanoseconds;

            static const char *                 ShortDateFormatString;
            static const char *                 MediumDateFormatString;
            static const char *                 LongDateFormatString;
            static const char *                 FullDateFormatString;

            static const char *                 TimeFormatStringMinutes;
            static const char *                 TimeFormatStringSeconds;
            static const char *                 TimeFormatStringMilliseconds;
            static const char *                 TimeFormatStringMicroseconds;
            static const char *                 TimeFormatStringNanoseconds;

            static const char *                 ShortJanuaryMonthName;
            static const char *                 ShortFebruaryMonthName;
            static const char *                 ShortMarchMonthName;
            static const char *                 ShortAprilMonthName;
            static const char *                 ShortMayMonthName;
            static const char *                 ShortJuneMonthName;
            static const char *                 ShortJulyMonthName;
            static const char *                 ShortAugustMonthName;
            static const char *                 ShortSeptemberMonthName;
            static const char *                 ShortOctoberMonthName;
            static const char *                 ShortNovemberMonthName;
            static const char *                 ShortDecemberMonthName;

            static const char *                 LongJanuaryMonthName;
            static const char *                 LongFebruaryMonthName;
            static const char *                 LongMarchMonthName;
            static const char *                 LongAprilMonthName;
            static const char *                 LongMayMonthName;
            static const char *                 LongJuneMonthName;
            static const char *                 LongJulyMonthName;
            static const char *                 LongAugustMonthName;
            static const char *                 LongSeptemberMonthName;
            static const char *                 LongOctoberMonthName;
            static const char *                 LongNovemberMonthName;
            static const char *                 LongDecemberMonthName;

            static const char *                 ShortSundayWeekdayName;
            static const char *                 ShortMondayWeekdayName;
            static const char *                 ShortTuesdayWeekdayName;
            static const char *                 ShortWednesdayWeekdayName;
            static const char *                 ShortThursdayWeekdayName;
            static const char *                 ShortFridayWeekdayName;
            static const char *                 ShortSaturdayWeekdayName;

            static const char *                 LongSundayWeekdayName;
            static const char *                 LongMondayWeekdayName;
            static const char *                 LongTuesdayWeekdayName;
            static const char *                 LongWednesdayWeekdayName;
            static const char *                 LongThursdayWeekdayName;
            static const char *                 LongFridayWeekdayName;
            static const char *                 LongSaturdayWeekdayName;

            static const char *                 AmString;
            static const char *                 PmString;

            //////////
            //  Implementation
        private:
            mutable FrameworkResourceFactory    _resourceFactory;
            HashMap<Locale, DateTimeFormat*>    _mapLocalesToDateTimeFormats;
            HashMap<Locale, DateFormat*>        _mapLocalesToDateFormats;
            HashMap<Locale, TimeFormat*>        _mapLocalesToTimeFormats;
            HashMap<Locale, DateTimeParse*>     _mapLocalesToDateTimeParses;
            HashMap<Locale, DateParse*>         _mapLocalesToDateParses;
            HashMap<Locale, TimeParse*>         _mapLocalesToTimeParses;
            Array<DateTimeFormat*>              _autoDeletedDateTimeFormats;
            Array<DateFormat*>                  _autoDeletedDateFormats;
            Array<TimeFormat*>                  _autoDeletedTimeFormats;
            Array<DateTimeParse*>               _autoDeletedDateTimeParses;
            Array<DateParse*>                   _autoDeletedDateParses;
            Array<TimeParse*>                   _autoDeletedTimeParses;
            CriticalSection                     _guard;
        };

        //////////
        //  A default date/time format uses resources provided by the date/time
        //  format manager to represent dates/times
        class CARL_PUBLIC _DefaultDateTimeFormat;
        class CARL_PUBLIC _DefaultDateFormat;
        class CARL_PUBLIC _DefaultTimeFormat;
        class CARL_PUBLIC _DefaultDateTimeFormatImpl
        {
            friend class _DefaultDateTimeFormat;
            friend class _DefaultDateFormat;
            friend class _DefaultTimeFormat;
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateTimeFormatImpl(const _DefaultDateTimeFormatImpl &);
            void                                operator = (const _DefaultDateTimeFormatImpl &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateTimeFormatImpl(const Locale & forLocale);
            virtual ~_DefaultDateTimeFormatImpl();

            //////////
            //  Implementation
        private:
            Locale                              _forLocale;
            NumberFormat *                      _pNumberFormat;

            //  Helpers
            bool                                _formatDateTime(String & result, const String & format,
                                                                bool yearKnown, int year,
                                                                bool monthKnown, int month,
                                                                bool dayKnown, int day,
                                                                bool dayOfWeekKnown, DayOfWeek dayOfWeek,
                                                                bool hourKnown, int hour,
                                                                bool minuteKnown, int minute,
                                                                bool secondKnown, int second,
                                                                bool nanosecondKnown, UInt32 nanosecond) const;
        };

        class CARL_PUBLIC _DefaultDateTimeFormat : public DateTimeFormat,
                                                   private _DefaultDateTimeFormatImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateTimeFormat(const _DefaultDateTimeFormat &);
            void                                operator = (const _DefaultDateTimeFormat &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateTimeFormat(const Locale & forLocale);
            virtual ~_DefaultDateTimeFormat();

            //////////
            //  Operations
        public:
            virtual String                      format(const DateTime & dateTime, DateTimeFormat::Style style, DateTimeFormat::Precision precision) const;
            virtual String                      format(const DateTime & dateTime, const String & format) const;
        };

        class CARL_PUBLIC _DefaultDateFormat : public DateFormat,
                                               private _DefaultDateTimeFormatImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateFormat(const _DefaultDateFormat &);
            void                                operator = (const _DefaultDateFormat &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateFormat(const Locale & forLocale);
            virtual ~_DefaultDateFormat();

            //////////
            //  Operations
        public:
            virtual String                      format(const Date & date, DateFormat::Style style) const;
            virtual String                      format(const Date & date, const String & format) const;
        };

        class CARL_PUBLIC _DefaultTimeFormat : public TimeFormat,
                                               private _DefaultDateTimeFormatImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultTimeFormat(const _DefaultTimeFormat &);
            void                                operator = (const _DefaultTimeFormat &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultTimeFormat(const Locale & forLocale);
            virtual ~_DefaultTimeFormat();

            //////////
            //  Operations
        public:
            virtual String                      format(const Time & time, TimeFormat::Precision precision) const;
            virtual String                      format(const Time & time, const String & format) const;
        };

        //////////
        //  A default date/time parse uses resources provided by the date/time
        //  format manager to represent dates/times
        class CARL_PUBLIC _DefaultDateTimeParse;
        class CARL_PUBLIC _DefaultDateParse;
        class CARL_PUBLIC _DefaultTimeParse;
        class CARL_PUBLIC _DefaultDateTimeParseImpl
        {
            friend class _DefaultDateTimeParse;
            friend class _DefaultDateParse;
            friend class _DefaultTimeParse;
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateTimeParseImpl(const _DefaultDateTimeParseImpl &);
            void                                operator = (const _DefaultDateTimeParseImpl &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateTimeParseImpl(const Locale & forLocale);
            virtual ~_DefaultDateTimeParseImpl();

            //////////
            //  Implementation
        private:
            Locale                              _forLocale;
            NumberParse *                       _pNumberParse;

            //  Helpers
            bool                                _parseDateTime(const String & dateTimeString, UIntSize & position, 
                                                               const String & format,
                                                               bool & yearKnown, int & year,
                                                               bool & monthKnown, int & month,
                                                               bool & dayKnown, int & day,
                                                               bool & dayOfWeekKnown, DayOfWeek & dayOfWeek,
                                                               bool & hourKnown, int & hour,
                                                               bool & minuteKnown, int & minute,
                                                               bool & secondKnown, int & second,
                                                               bool & nanosecondKnown, UInt32 & nanosecond) const;
            bool                                _matchFormatLiteralCharacter(const String & dateTimeString, UIntSize & position,
                                                                             const String & format, UIntSize & formatScan) const;
        };

        class CARL_PUBLIC _DefaultDateTimeParse : private _DefaultDateTimeParseImpl,
                                                  public DateTimeParse
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateTimeParse(const _DefaultDateTimeParse &);
            void                                operator = (const _DefaultDateTimeParse &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateTimeParse(const Locale & forLocale);
            virtual ~_DefaultDateTimeParse();

            //////////
            //  Operations
        public:
            virtual DateTime                    parse(const char * pDateTimeString, UIntSize & position) const throw(ParseException);
            virtual DateTime                    parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException);
            virtual DateTime                    parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const char * pDateTimeString, UIntSize & position, const String & format) const throw(ParseException);
            virtual DateTime                    parse(const Char * pDateTimeString, UIntSize & position) const throw(ParseException);
            virtual DateTime                    parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException);
            virtual DateTime                    parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const Char * pDateTimeString, UIntSize & position, const String & format) const throw(ParseException);
            virtual DateTime                    parse(const String & dateTimeString, UIntSize & position) const throw(ParseException);
            virtual DateTime                    parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException);
            virtual DateTime                    parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException);
            virtual DateTime                    parse(const String & dateTimeString, UIntSize & position, const String & format) const throw(ParseException);
        };

        class CARL_PUBLIC _DefaultDateParse : private _DefaultDateTimeParseImpl,
                                              public DateParse
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultDateParse(const _DefaultDateParse &);
            void                                operator = (const _DefaultDateParse &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultDateParse(const Locale & forLocale);
            virtual ~_DefaultDateParse();

            //////////
            //  Operations
        public:
            virtual Date                        parse(const char * pDateString, UIntSize & position) const throw(ParseException);
            virtual Date                        parse(const char * pDateString, UIntSize & position, DateParse::Style style) const throw(ParseException);
            virtual Date                        parse(const char * pDateString, UIntSize & position, const String & format) const throw(ParseException);
            virtual Date                        parse(const Char * pDateString, UIntSize & position) const throw(ParseException);
            virtual Date                        parse(const Char * pDateString, UIntSize & position, DateParse::Style style) const throw(ParseException);
            virtual Date                        parse(const Char * pDateString, UIntSize & position, const String & format) const throw(ParseException);
            virtual Date                        parse(const String & dateString, UIntSize & position) const throw(ParseException);
            virtual Date                        parse(const String & dateString, UIntSize & position, DateParse::Style style) const throw(ParseException);
            virtual Date                        parse(const String & dateString, UIntSize & position, const String & format) const throw(ParseException);
        };

        class CARL_PUBLIC _DefaultTimeParse : private _DefaultDateTimeParseImpl,
                                              public TimeParse
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultTimeParse(const _DefaultTimeParse &);
            void                                operator = (const _DefaultTimeParse &);

            //////////
            //  Construction/destruction
        public:
            explicit _DefaultTimeParse(const Locale & forLocale);
            virtual ~_DefaultTimeParse();

            //////////
            //  Operations
        public:
            virtual Time                        parse(const char * pTimeString, UIntSize & position) const throw(ParseException);
            virtual Time                        parse(const char * pTimeString, UIntSize & position, Precision precision) const throw(ParseException);
            virtual Time                        parse(const char * pTimeString, UIntSize & position, const String & format) const throw(ParseException);
            virtual Time                        parse(const Char * pTimeString, UIntSize & position) const throw(ParseException);
            virtual Time                        parse(const Char * pTimeString, UIntSize & position, Precision precision) const throw(ParseException);
            virtual Time                        parse(const Char * pTimeString, UIntSize & position, const String & format) const throw(ParseException);
            virtual Time                        parse(const String & timeString, UIntSize & position) const throw(ParseException);
            virtual Time                        parse(const String & timeString, UIntSize & position, Precision precision) const throw(ParseException);
            virtual Time                        parse(const String & timeString, UIntSize & position, const String & format) const throw(ParseException);
        };
    }
}

#endif  //  __NumberFormatImpl_hpp_included__
//  End of NumberFormatImpl.hpp
