//
//  DateTimeImpl.hpp - carl::_impl::_DateTimeImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.18 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __DateTimeImpl_hpp_included__
#define __DateTimeImpl_hpp_included__
#include <time.h>
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <windows.h>
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #include <sys/time.h>
#else
    #error Unsupported platform
#endif  //  Host selection

namespace carl
{
    namespace _impl
    {
        //////////
        //  Date/time resources
        class CARL_PUBLIC _DateTimeResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DateTimeResourceMap_en(const _DateTimeResourceMap_en &);
            void                        operator = (const _DateTimeResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _DateTimeResourceMap_en();
            virtual ~_DateTimeResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String              getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>     _mapKeysToValues;
        };

        //////////
        //  A parameterized time zone with an optional DST
        enum _DaylightSwitchpointType
        {
            _DaylightSwitchpointType_Absolute   = 1,    //  switch occurs on a specific day of month
            _DaylightSwitchpointType_DayOfWeek  = 2     //  switch occurs on a specific day of week in a specific month
        };

        struct _AbsoluteDaylightSwitchpoint
        {
            _AbsoluteDaylightSwitchpoint()
            {
                this->month = 0;
                this->day = 0;
                this->hour = 0;
                this->minute = 0;
                this->second = 0;
                this->nanosecond = 0;
            }

            _AbsoluteDaylightSwitchpoint(UInt8 month,
                                         UInt8 day,
                                         UInt8 hour,
                                         UInt8 minute,
                                         UInt8 second,
                                         UInt32 nanosecond)
            {
                this->month = month;
                this->day = day;
                this->hour = hour;
                this->minute = minute;
                this->second = second;
                this->nanosecond = nanosecond;
            }
            UInt8                       month;      //  1..12
            UInt8                       day;        //  1..31
            UInt8                       hour;       //  0..23
            UInt8                       minute;     //  0..59
            UInt8                       second;     //  0..59
            UInt32                      nanosecond; //  0..999999999
        };

        struct _DayOfWeekDaylightSwitchpoint
        {
            _DayOfWeekDaylightSwitchpoint()
            {
                this->month = 0;
                this->dayOfWeek = 0;
                this->occurence = 0;
                this->hour = 0;
                this->minute = 0;
                this->second = 0;
                this->nanosecond = 0;
            }

            _DayOfWeekDaylightSwitchpoint(UInt8 month,
                                          UInt8 dayOfWeek,
                                          UInt8 occurence,
                                          UInt8 hour,
                                          UInt8 minute,
                                          UInt8 second,
                                          UInt32 nanosecond)
            {
                this->month = month;
                this->dayOfWeek = dayOfWeek;
                this->occurence = occurence;
                this->hour = hour;
                this->minute = minute;
                this->second = second;
                this->nanosecond = nanosecond;
            }

            UInt8                       month;  //  1..12
            UInt8                       dayOfWeek;  //  0..6 (Monday..Saturday)
            UInt8                       occurence;  //  1..4 (1st..4th) or 5 (last)
            UInt8                       hour;       //  0..23
            UInt8                       minute;     //  0..59
            UInt8                       second;     //  0..59
            UInt32                      nanosecond; //  0..999999999
        };

        struct _DaylightSwitchpoint
        {
            _DaylightSwitchpoint()
            {
                type = _DaylightSwitchpointType_Absolute;
            }

            _DaylightSwitchpoint(const _AbsoluteDaylightSwitchpoint & absoluteDaylightSwitchpoint)
            {
                type = _DaylightSwitchpointType_Absolute;
                this->absoluteDaylightSwitchpoint = absoluteDaylightSwitchpoint;
            }

            _DaylightSwitchpoint(const _DayOfWeekDaylightSwitchpoint & dayOfWeekDaylightSwitchpoint)
            {
                type = _DaylightSwitchpointType_DayOfWeek;
                this->dayOfWeekDaylightSwitchpoint = dayOfWeekDaylightSwitchpoint;
            }

            _DaylightSwitchpointType        type;
            _AbsoluteDaylightSwitchpoint    absoluteDaylightSwitchpoint;
            _DayOfWeekDaylightSwitchpoint   dayOfWeekDaylightSwitchpoint;
        };

        class CARL_PUBLIC _ParameterizedDstAwareTimeZone : public TimeZone
        {
            //////////
            //  Can't assign or copy construct
        private:
            _ParameterizedDstAwareTimeZone(const _ParameterizedDstAwareTimeZone &);
            void                        operator = (const _ParameterizedDstAwareTimeZone &);

            //////////
            //  Construction/destruction - from derived classes only
        protected:
            explicit _ParameterizedDstAwareTimeZone(Int64 bias);
            _ParameterizedDstAwareTimeZone(Int64 bias, 
                                           Int64 standardBias, const _DaylightSwitchpoint & standardSwitchpoint, 
                                           Int64 daylightBias, const _DaylightSwitchpoint & daylightSwitchpoint);
            virtual ~_ParameterizedDstAwareTimeZone();

            //////////
            //  Operations
        public:
            virtual DateTime            timeToLocal(const TimeStamp & timeStamp) const;
            virtual TimeStamp           timeFromLocal(const DateTime & dateTime) const;
            Int64                       getBias() const;
            bool                        usesDst() const;
            Int64                       getStandardBias() const;
            _DaylightSwitchpoint        getStandardSwitchpoint() const;
            Int64                       getDaylightBias() const;
            _DaylightSwitchpoint        getDaylightSwitchpoint() const;

            //////////
            //  Implementation
        private:
            Int64                       _bias;  //  nanoseconds, UTC = local time + bias
            bool                        _usesDst;       //  if false, the following attributes are unused
            Int64                       _standardBias;  //  nanoseconds, extra bias during standard time
            _DaylightSwitchpoint        _standardSwitchpoint;   //  when to switch to standard time
            Int64                       _daylightBias;  //  nanoseconds, extra bias during daylight time
            _DaylightSwitchpoint        _daylightSwitchpoint;   //  when to switch to daylight time
            //  Helpers
            static const UInt64         _NanosecondsInSecond = 1000000000;
            static const UInt64         _NanosecondsInMinute = _NanosecondsInSecond * 60;
            static const UInt64         _NanosecondsInHour = _NanosecondsInMinute * 60;
            static const UInt64         _NanosecondsInDay = _NanosecondsInHour * 24;
            static DateTime             _convertTimeStampToDateTime(const TimeStamp & timeStamp);
            static TimeStamp            _convertDateTimeToTimeStamp(const DateTime & dateTime);
            static TimeStamp            _applyBias(const TimeStamp & timeStamp, Int64 bias);
            static _AbsoluteDaylightSwitchpoint _calculateAbsoluteDaylightSwitchpoint(const _DaylightSwitchpoint & daylightSwitchpoint, int year);
            static int                  _compare(const _AbsoluteDaylightSwitchpoint & switchpoint1, const _AbsoluteDaylightSwitchpoint & switchpoint2);
            static int                  _compare(const _AbsoluteDaylightSwitchpoint & switchpoint, const DateTime & dateTime);
        protected:
            bool                        _isDst(const DateTime & dateTime) const;
        };

        class CARL_PUBLIC _SystemTimeZone : public _ParameterizedDstAwareTimeZone
        {
            //////////
            //  Can't assign or copy construct
        private:
            _SystemTimeZone(const _SystemTimeZone &);
            void                        operator = (const _SystemTimeZone &);

            //////////
            //  Construction/destruction
        public:
            _SystemTimeZone(Int64 bias,
                            const String & name);
            _SystemTimeZone(Int64 bias, 
                            Int64 standardBias, const _DaylightSwitchpoint & standardSwitchpoint, 
                            Int64 daylightBias, const _DaylightSwitchpoint & daylightSwitchpoint,
                            const String & name);
            virtual ~_SystemTimeZone();

            //////////
            //  Operations
        public:
            virtual Array<String>       getAllNames() const;
            virtual String              getCanonicalName() const;
            virtual String              getPreferredName() const;
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual String              getDisplayName(const DateTime & dateTime) const;
            virtual String              getDisplayName(const DateTime & dateTime, const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            String                      _name;
        };

        class CARL_PUBLIC _NonstandardSystemTimeZone : public _SystemTimeZone
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NonstandardSystemTimeZone(const _NonstandardSystemTimeZone &);
            void                        operator = (const _NonstandardSystemTimeZone &);

            //////////
            //  Construction/destruction
        public:
            _NonstandardSystemTimeZone(Int64 bias,
                            const String & name,
                            const String & displayName);
            _NonstandardSystemTimeZone(Int64 bias, 
                            Int64 standardBias, const _DaylightSwitchpoint & standardSwitchpoint, 
                            Int64 daylightBias, const _DaylightSwitchpoint & daylightSwitchpoint,
                            const String & name,
                            const String & displayName,
                            const String & standardDisplayName,
                            const String & daylightDisplayName);
            virtual ~_NonstandardSystemTimeZone();

            //////////
            //  Operations
        public:
            virtual String              getDisplayName() const;
            virtual String              getDisplayName(const Locale & forLocale) const;
            virtual String              getDisplayName(const DateTime & dateTime) const;
            virtual String              getDisplayName(const DateTime & dateTime, const Locale & forLocale) const;

            //////////
            //  Implementation
        private:
            String                      _displayName;
            String                      _standardDisplayName;
            String                      _daylightDisplayName;
        };

        //////////
        //  A platform-specific implementation of a date/time manager
        class CARL_PUBLIC _DateTimeManagerImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DateTimeManagerImpl(const _DateTimeManagerImpl &);
            void                        operator = (const _DateTimeManagerImpl &);

            //////////
            //  Singleton
        private:
            _DateTimeManagerImpl();
            ~_DateTimeManagerImpl();
        public:
            static _DateTimeManagerImpl * getInstance();

            //////////
            //  Operations
        public:
            MapResourceFactory *        getResourceFactory() const;
            void                        registerTimeZone(TimeZone * pTimeZone) throw (InconsistentTimeZoneException, DuplicateTimeZoneException);
            Array<TimeZone*>            getAllTimeZones();
            TimeZone *                  findTimeZone(const String & name);
            TimeZone *                  getSystem();
            TimeZone *                  getDefault();
            void                        setDefault(TimeZone * pTimeZone);
            static TimeStamp            convertTimeStampFromCrtFormat(time_t crtTime);
            static time_t               convertTimeStampToCrtFormat(const TimeStamp & t);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
            static TimeStamp            convertTimeStampFromWin32Format(const FILETIME & t);
            static FILETIME             convertTimeStampToWin32Format(const TimeStamp & t);
#endif

            //////////
            //  Resource keys
        public:
            static const char *         SundayShortDisplayName;
            static const char *         MondayShortDisplayName;
            static const char *         TuesdayShortDisplayName;
            static const char *         WednesdayShortDisplayName;
            static const char *         ThursdayShortDisplayName;
            static const char *         FridayShortDisplayName;
            static const char *         SaturdayShortDisplayName;

            static const char *         SundayLongDisplayName;
            static const char *         MondayLongDisplayName;
            static const char *         TuesdayLongDisplayName;
            static const char *         WednesdayLongDisplayName;
            static const char *         ThursdayLongDisplayName;
            static const char *         FridayLongDisplayName;
            static const char *         SaturdayLongDisplayName;

            static const char *         JanuaryShortDisplayName;
            static const char *         FebruaryShortDisplayName;
            static const char *         MarchShortDisplayName;
            static const char *         AprilShortDisplayName;
            static const char *         MayShortDisplayName;
            static const char *         JuneShortDisplayName;
            static const char *         JulyShortDisplayName;
            static const char *         AugustShortDisplayName;
            static const char *         SeptemberShortDisplayName;
            static const char *         OctoberShortDisplayName;
            static const char *         NovemberShortDisplayName;
            static const char *         DecemberShortDisplayName;

            static const char *         JanuaryLongDisplayName;
            static const char *         FebruaryLongDisplayName;
            static const char *         MarchLongDisplayName;
            static const char *         AprilLongDisplayName;
            static const char *         MayLongDisplayName;
            static const char *         JuneLongDisplayName;
            static const char *         JulyLongDisplayName;
            static const char *         AugustLongDisplayName;
            static const char *         SeptemberLongDisplayName;
            static const char *         OctoberLongDisplayName;
            static const char *         NovemberLongDisplayName;
            static const char *         DecemberLongDisplayName;

            //////////
            //  Implementation
        private:
            mutable FrameworkResourceFactory    _resourceFactory;
            Array<TimeZone*>            _allTimeZones;
            TimeZone *                  _pSystemTimeZone;
            TimeZone *                  _pDefaultTimeZone;
            mutable CriticalSection     _guard;
            HashMap<String, TimeZone *> _mapNamesToTimeZones;
        };
    }
}

#endif  //  __DateTimeImpl_hpp_included__
//  End of DateTimeImpl.hpp
