//
//  DateTimeManagerImpl.cpp - Implementation of the carl::_impl::_DateTimeManagerImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.22 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeImpl.hpp"

namespace
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    _impl::_DaylightSwitchpoint _translateDstSwitchpoint(const SYSTEMTIME & st)
    {
        _impl::_DaylightSwitchpoint result;

        if (st.wYear != 0)
        {
            result.type = _impl::_DaylightSwitchpointType_Absolute;
            result.absoluteDaylightSwitchpoint.month = static_cast<UInt8>(st.wMonth);
            result.absoluteDaylightSwitchpoint.day = static_cast<UInt8>(st.wDay);
            result.absoluteDaylightSwitchpoint.hour = static_cast<UInt8>(st.wHour);
            result.absoluteDaylightSwitchpoint.minute = static_cast<UInt8>(st.wMinute);
            result.absoluteDaylightSwitchpoint.second = static_cast<UInt8>(st.wSecond);
            result.absoluteDaylightSwitchpoint.nanosecond = st.wMilliseconds * static_cast<UInt32>(1000000);
        }
        else
        {
            result.type = _impl::_DaylightSwitchpointType_DayOfWeek;
            result.dayOfWeekDaylightSwitchpoint.month = static_cast<UInt8>(st.wMonth);
            result.dayOfWeekDaylightSwitchpoint.dayOfWeek = static_cast<UInt8>(st.wDayOfWeek);
            result.dayOfWeekDaylightSwitchpoint.occurence = static_cast<UInt8>(st.wDay);
            result.dayOfWeekDaylightSwitchpoint.hour = static_cast<UInt8>(st.wHour);
            result.dayOfWeekDaylightSwitchpoint.minute = static_cast<UInt8>(st.wMinute);
            result.dayOfWeekDaylightSwitchpoint.second = static_cast<UInt8>(st.wSecond);
            result.dayOfWeekDaylightSwitchpoint.nanosecond = st.wMilliseconds * static_cast<UInt32>(1000000);
        }
        return result;
    }

    bool operator == (const SYSTEMTIME & switchpoint1, const _impl::_DaylightSwitchpoint switchpoint2)
    {
        if (switchpoint1.wYear != 0)
        {
            return switchpoint2.type == _impl::_DaylightSwitchpointType_Absolute &&
                   switchpoint2.absoluteDaylightSwitchpoint.month == static_cast<UInt8>(switchpoint1.wMonth) &&
                   switchpoint2.absoluteDaylightSwitchpoint.day == static_cast<UInt8>(switchpoint1.wDay) &&
                   switchpoint2.absoluteDaylightSwitchpoint.hour == static_cast<UInt8>(switchpoint1.wHour) &&
                   switchpoint2.absoluteDaylightSwitchpoint.minute == static_cast<UInt8>(switchpoint1.wMinute) &&
                   switchpoint2.absoluteDaylightSwitchpoint.second == static_cast<UInt8>(switchpoint1.wSecond) &&
                   switchpoint2.absoluteDaylightSwitchpoint.nanosecond == switchpoint1.wMilliseconds * static_cast<UInt32>(1000000);
        }
        else
        {
            return switchpoint2.type == _impl::_DaylightSwitchpointType_DayOfWeek &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.month == static_cast<UInt8>(switchpoint1.wMonth) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.dayOfWeek == static_cast<UInt8>(switchpoint1.wDayOfWeek) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.occurence == static_cast<UInt8>(switchpoint1.wDay) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.hour == static_cast<UInt8>(switchpoint1.wHour) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.minute == static_cast<UInt8>(switchpoint1.wMinute) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.second == static_cast<UInt8>(switchpoint1.wSecond) &&
                   switchpoint2.dayOfWeekDaylightSwitchpoint.nanosecond == switchpoint1.wMilliseconds * static_cast<UInt32>(1000000);
        }
    }
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
#else
    #error Unsupported platform
#endif

    Array<TimeZone*> _createStandardTimeZones()
    {
        Array<TimeZone*> result;

        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-16200000000000), String("Afghanistan Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(32400000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Alaskan Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-10800000000000), String("Arab Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-14400000000000), String("Arabian Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-10800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 1, 4, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 3, 0, 0, 0)), String("Arabic Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(14400000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Atlantic Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-34200000000000), String("AUS Central Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-36000000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), String("AUS Eastern Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(3600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Azores Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(21600000000000), String("Canada Central Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(3600000000000), String("Cape Verde Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-14400000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Caucasus Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-34200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), String("Cen. Australia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(21600000000000), String("Central America Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-21600000000000), String("Central Asia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-3600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Central Europe Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-3600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Central European Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-39600000000000), String("Central Pacific Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-21600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Central Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), String("China Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(43200000000000), String("Dateline Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-10800000000000), String("E. Africa Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-36000000000000), String("E. Australia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 1, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 0, 0, 0, 0)), String("E. Europe Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(10800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(2, 0, 2, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 3, 2, 0, 0, 0)), String("E. South America Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(18000000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Eastern Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(9, 3, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(5, 5, 1, 2, 0, 0, 0)), String("Egypt Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-18000000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Ekaterinburg Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-43200000000000), String("Fiji Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 4, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 3, 0, 0, 0)), String("FLE Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(0), 0, _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 1, 0, 0, 0)), String("GMT Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(10800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Greenland Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(0), String("Greenwich Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("GTB Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(36000000000000), String("Hawaiian Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-19800000000000), String("India Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-13600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(9, 2, 4, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 1, 2, 0, 0, 0)), String("Iran Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), String("Israel Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-32400000000000), String("Korea Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(21600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Mexico Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(25200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Mexico Standard Time 2")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(7200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(9, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Mid-Atlantic Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(25200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Mountain Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-23400000000000), String("Myanmar Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-21600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("N. Central Asia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-20700000000000), String("Nepal Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-43200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 3, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 1, 2, 0, 0, 0)), String("New Zealand Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-12600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Newfoundland Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("North Asia East Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-25200000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("North Asia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(14400000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 6, 2, 0, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 6, 2, 0, 0, 0, 0)), String("Pacific SA Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(28800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 2, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(4, 0, 1, 2, 0, 0, 0)), String("Pacific Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-3600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Romance Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-10800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Russian Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(10800000000000), String("SA Eastern Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(18000000000000), String("SA Pacific Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(14400000000000), String("SA Western Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(39600000000000), String("Samoa Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-25200000000000), String("SE Asia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), String("Singapore Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-7200000000000), String("South Africa Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-21600000000000), String("Sri Lanka Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), String("Taipei Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 1, 2, 0, 0, 0)), String("Tasmania Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-32400000000000), String("Tokyo Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-46800000000000), String("Tonga Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(18000000000000), String("US Eastern Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(25200000000000), String("US Mountain Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-36000000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Vladivostok Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-28800000000000), String("W. Australia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-3600000000000), String("W. Central Africa Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-3600000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("W. Europe Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-18000000000000), String("West Asia Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-36000000000000), String("West Pacific Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(-32400000000000), CARL_INT64_CONSTANT(0), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(10, 0, 5, 3, 0, 0, 0)), CARL_INT64_CONSTANT(-3600000000000), _impl::_DaylightSwitchpoint(_impl::_DayOfWeekDaylightSwitchpoint(3, 0, 5, 2, 0, 0, 0)), String("Yakutsk Standard Time")));
        result.append(new _impl::_SystemTimeZone(CARL_INT64_CONSTANT(0), String("UTC")));

        //  Done
        return result;
    }

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    TimeZone * _determineSystemTimeZone(Array<TimeZone*> & timeZones)
    {
        const Int64 NanosecondsPerMinute = static_cast<Int64>(60) * static_cast<Int64>(1000000000);

        //  Obtain the information about current system time zone
        TIME_ZONE_INFORMATION tzi;
        String name, displayName, stdName, dltName;
        if (GetTimeZoneInformation(&tzi) == TIME_ZONE_ID_INVALID)
        {   //  Could not obtain the system time zone information - assume UTC
            name = _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString("UTC");
            displayName = _impl::_DateTimeManagerImpl::getInstance()->getResourceFactory()->getString("UTC");
            //
            tzi.Bias = 0;
            tzi.DaylightBias = 0;
            tzi.DaylightDate.wMonth = 0;
            tzi.StandardBias = 0;
            tzi.StandardDate.wMonth = 0;
        }
        else
        {
            for (int i = 0; i < sizeof(tzi.StandardName) / sizeof(tzi.StandardName[0]); i++)
            {
                if (tzi.StandardName[i] == 0)
                {
                    break;
                }
                name += static_cast<Char>(tzi.StandardName[i]);
            }
            stdName = displayName = name;
            for (int i = 0; i < sizeof(tzi.DaylightName) / sizeof(tzi.DaylightName[0]); i++)
            {
                if (tzi.DaylightName[i] == 0)
                {
                    break;
                }
                dltName += static_cast<Char>(tzi.DaylightName[i]);
            }
        }

        //  Does the time zone with the specified properties exist ?
        for (UIntSize i = 0; i < timeZones.size(); i++)
        {
            TimeZone * pTZ = timeZones[i];
            _impl::_ParameterizedDstAwareTimeZone * pPDATZ = dynamic_cast<_impl::_ParameterizedDstAwareTimeZone*>(pTZ);
            if (pPDATZ != NULL)
            {   //  This time zone is a parameterized DST-aware time zone
                if (pPDATZ->usesDst())
                {   //  This time zone uses DST
                    if (pPDATZ->getBias() == tzi.Bias * NanosecondsPerMinute &&
                        pPDATZ->getStandardBias() == tzi.StandardBias * NanosecondsPerMinute &&
                        pPDATZ->getDaylightBias() == tzi.DaylightBias * NanosecondsPerMinute &&
                        tzi.StandardDate == pPDATZ->getStandardSwitchpoint() &&
                        tzi.DaylightDate == pPDATZ->getDaylightSwitchpoint())
                    {   //  Found it!
                        return pPDATZ;
                    }
                }
                else
                {   //  This time zone does not use DST
                    if (tzi.DaylightDate.wMonth == 0 && tzi.StandardDate.wMonth == 0 &&
                        pPDATZ->getBias() == tzi.Bias * NanosecondsPerMinute)
                    {   //  Found it!
                        return pPDATZ;
                    }
                }
            }
        }

        //  The list of installed time zones does not include the current system time zone
        _impl::_SystemTimeZone * pSystemTimeZone;
        if (tzi.StandardDate.wMonth != 0 && tzi.DaylightDate.wMonth != 0)
        {   //  Time zone uses DST
            _impl::_DaylightSwitchpoint stdSwitchpoint = _translateDstSwitchpoint(tzi.StandardDate);
            _impl::_DaylightSwitchpoint dltSwitchpoint = _translateDstSwitchpoint(tzi.DaylightDate);
            pSystemTimeZone = new _impl::_NonstandardSystemTimeZone(tzi.Bias * NanosecondsPerMinute, 
                                                                    tzi.StandardBias * NanosecondsPerMinute,
                                                                    stdSwitchpoint,
                                                                    tzi.DaylightBias * NanosecondsPerMinute,
                                                                    dltSwitchpoint,
                                                                    name,
                                                                    displayName,
                                                                    stdName,
                                                                    dltName);
        }
        else
        {   //  Time zone does not use DST
            pSystemTimeZone = new _impl::_NonstandardSystemTimeZone(tzi.Bias * NanosecondsPerMinute, 
                                                                    name,
                                                                    displayName);
        }
        timeZones.append(pSystemTimeZone);

        //  Done
        return pSystemTimeZone;
    }
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)
    TimeZone * _determineSystemTimeZone(Array<TimeZone*> & timeZones)
    {
        tzset();
        Int64 standardBias = static_cast<Int64>(_timezone) * 1000000000;
        for (UIntSize i = 0; i < timeZones.size(); i++)
        {
            TimeZone * pTZ = timeZones[i];
            _impl::_ParameterizedDstAwareTimeZone * pPDATZ = dynamic_cast<_impl::_ParameterizedDstAwareTimeZone*>(pTZ);
            if (pPDATZ != NULL)
            {
                if (pPDATZ->usesDst() == (_daylight != 0))
                {   //  DST usage matched
                    if (pPDATZ->getBias() == standardBias)
                    {   //  Found it!
                        return pPDATZ;
                    }
                }
            }
        }
        //  Not found - create a new one
        const char * pTimeZoneName = (_tzname[0] == NULL) ? ("") : (_tzname[0]);
        _impl::_SystemTimeZone * pSystemTimeZone =
            new _impl::_NonstandardSystemTimeZone(standardBias, String(pTimeZoneName), String(pTimeZoneName));
        timeZones.append(pSystemTimeZone);
        return pSystemTimeZone;
    }
#elif defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC)
    TimeZone * _determineSystemTimeZone(Array<TimeZone*> & timeZones)
    {
        tzset();
        Int64 standardBias = static_cast<Int64>(timezone) * 1000000000;
        for (UIntSize i = 0; i < timeZones.size(); i++)
        {
            TimeZone * pTZ = timeZones[i];
            _impl::_ParameterizedDstAwareTimeZone * pPDATZ = dynamic_cast<_impl::_ParameterizedDstAwareTimeZone*>(pTZ);
            if (pPDATZ != NULL)
            {
                if (pPDATZ->usesDst() == (daylight != 0))
                {   //  DST usage matched
                    if (pPDATZ->getBias() == standardBias)
                    {   //  Found it!
                        return pPDATZ;
                    }
                }
            }
        }
        //  Not found - create a new one
        const char * pTimeZoneName = (tzname[0] == NULL) ? ("") : (tzname[0]);
        _impl::_SystemTimeZone * pSystemTimeZone =
            new _impl::_NonstandardSystemTimeZone(standardBias, String(pTimeZoneName), String(pTimeZoneName));
        timeZones.append(pSystemTimeZone);
        return pSystemTimeZone;
    }
#else
    #error Unsupported platform
#endif
}

//////////
//  Singleton
_impl::_DateTimeManagerImpl::_DateTimeManagerImpl()
    :   _resourceFactory(new _DateTimeResourceMap_en(), carl::String("CARL_DATETIME")),
        _mapNamesToTimeZones(StandardHashFunctions::hashString)
{
    //  Build a list of all known time zones
    Array<TimeZone*> allTimeZones = _createStandardTimeZones();
    for (UIntSize i = 0; i < allTimeZones.size(); i++)
    {
        registerTimeZone(allTimeZones[i]);
    }
    //  Set up system time zone
    _pSystemTimeZone = _determineSystemTimeZone(_allTimeZones);
    //  Initiallay system time zone is the default
    _pDefaultTimeZone = _pSystemTimeZone;
}

_impl::_DateTimeManagerImpl::~_DateTimeManagerImpl()
{
}

_impl::_DateTimeManagerImpl * _impl::_DateTimeManagerImpl::getInstance()
{
    static _DateTimeManagerImpl instance;
    return &instance;
}

//////////
//  Operations
MapResourceFactory * _impl::_DateTimeManagerImpl::getResourceFactory() const
{
    return &_resourceFactory;
}

void _impl::_DateTimeManagerImpl::registerTimeZone(TimeZone * pTimeZone) throw (InconsistentTimeZoneException, DuplicateTimeZoneException)
{
    require(pTimeZone != NULL);

    CriticalSectionLock lock(_guard);
    Array<String> names = pTimeZone->getAllNames();
    for (UIntSize i = 0; i < names.size(); i++)
    {
        names[i].upperCase();
        if (_mapNamesToTimeZones.containsKey(names[i]))
        {
            throw DuplicateTimeZoneException();
        }
    }
    if (!names.contains(pTimeZone->getCanonicalName().upperCased()) ||
        !names.contains(pTimeZone->getPreferredName().upperCased()))
    {
        throw InconsistentTimeZoneException();
    }
    //
    _allTimeZones.append(pTimeZone);
    for (UIntSize i = 0; i < names.size(); i++)
    {
        _mapNamesToTimeZones.insert(names[i], pTimeZone);
    }
}

Array<TimeZone*> _impl::_DateTimeManagerImpl::getAllTimeZones()
{
    CriticalSectionLock lock(_guard);
    return _allTimeZones;
}

TimeZone * _impl::_DateTimeManagerImpl::findTimeZone(const String & name)
{
    CriticalSectionLock lock(_guard);
    String nameUppercased = name.upperCased();
    if (_mapNamesToTimeZones.containsKey(nameUppercased))
    {
        return _mapNamesToTimeZones.get(nameUppercased);
    }
    return NULL;
}

TimeZone * _impl::_DateTimeManagerImpl::getSystem()
{
    CriticalSectionLock lock(_guard);
    return _pSystemTimeZone;
}

TimeZone * _impl::_DateTimeManagerImpl::getDefault()
{
    CriticalSectionLock lock(_guard);
    return _pDefaultTimeZone;
}

void _impl::_DateTimeManagerImpl::setDefault(TimeZone * pTimeZone)
{
    require(pTimeZone != NULL);

    CriticalSectionLock lock(_guard);
    _pDefaultTimeZone = pTimeZone;
}

TimeStamp _impl::_DateTimeManagerImpl::convertTimeStampFromCrtFormat(time_t crtTime)
{
    if (crtTime < 0)
    {
        return TimeStamp();
    }
    return TimeStamp(static_cast<UInt64>(crtTime) * 1000000000);
}

time_t _impl::_DateTimeManagerImpl::convertTimeStampToCrtFormat(const TimeStamp & t)
{
    if (t.isValid())
    {
        long secondsSinseEpoch = static_cast<long>(t.getNanosecondsSinceEpoch() / 1000000000);
        if (secondsSinseEpoch >= 0)
        {
            return static_cast<time_t>(secondsSinseEpoch);
        }
    }
    return static_cast<time_t>(-1);
}

#if defined(CARL_OS_WIN32)
TimeStamp _impl::_DateTimeManagerImpl::convertTimeStampFromWin32Format(const FILETIME & t)
{
    static bool referencePointCalculated = false;
    static ULARGE_INTEGER referencePoint;

    //  Unknown time requires special treatment
    if (t.dwHighDateTime == 0 && t.dwLowDateTime == 0)
    {
        return TimeStamp();
    }

    //  Calculate reference point if necessary
    if (!referencePointCalculated)
    {
        SYSTEMTIME st;
        st.wYear = 1970;
        st.wMonth = 1;
        st.wDay = 1;
        st.wHour = 0;
        st.wMinute = 0;
        st.wSecond = 0;
        st.wMilliseconds = 0;
        FILETIME ft;
        if (!SystemTimeToFileTime(&st,&ft))
        {   //  Couldn't obtain a reference timestamp
            return TimeStamp();
        }
        referencePoint.HighPart = ft.dwHighDateTime;
        referencePoint.LowPart = ft.dwLowDateTime;
    }

    //  Translate the time
    ULARGE_INTEGER tConverted;
    tConverted.HighPart = t.dwHighDateTime;
    tConverted.LowPart = t.dwLowDateTime;
    if (tConverted.QuadPart < referencePoint.QuadPart)
    {   //  Time stamp is before reference point
        return TimeStamp();
    }
    return TimeStamp((tConverted.QuadPart - referencePoint.QuadPart) * 100);
}

FILETIME _impl::_DateTimeManagerImpl::convertTimeStampToWin32Format(const TimeStamp & t)
{
    static bool referencePointCalculated = false;
    static ULARGE_INTEGER referencePoint;
    FILETIME result;

    //  Unknown time requires special treatment
    if (!t.isValid())
    {
        result.dwHighDateTime = result.dwLowDateTime = 0;
        return result;
    }

    //  Calculate reference point if necessary
    if (!referencePointCalculated)
    {
        SYSTEMTIME st;
        st.wYear = 1970;
        st.wMonth = 1;
        st.wDay = 1;
        st.wHour = 0;
        st.wMinute = 0;
        st.wSecond = 0;
        st.wMilliseconds = 0;
        FILETIME ft;
        if (!SystemTimeToFileTime(&st,&ft))
        {   //  Couldn't obtain a reference timestamp
            result.dwHighDateTime = result.dwLowDateTime = 0;
            return result;
        }
        referencePoint.HighPart = ft.dwHighDateTime;
        referencePoint.LowPart = ft.dwLowDateTime;
    }

    //  Translate the time
    ULARGE_INTEGER tConverted;
    tConverted.QuadPart = t.getNanosecondsSinceEpoch() / 100 + referencePoint.QuadPart;
    result.dwHighDateTime = tConverted.HighPart;
    result.dwLowDateTime = tConverted.LowPart;
    return result;
}
#endif  //  defined(CARL_OS_WIN32)

//////////
//  Resource keys
const char * _impl::_DateTimeManagerImpl::SundayShortDisplayName = "SundayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::MondayShortDisplayName = "MondayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::TuesdayShortDisplayName = "TuesdayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::WednesdayShortDisplayName = "WednesdayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::ThursdayShortDisplayName = "ThursdayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::FridayShortDisplayName = "FridayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::SaturdayShortDisplayName = "SaturdayShortDisplayName";

const char * _impl::_DateTimeManagerImpl::SundayLongDisplayName = "SundayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::MondayLongDisplayName = "MondayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::TuesdayLongDisplayName = "TuesdayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::WednesdayLongDisplayName = "WednesdayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::ThursdayLongDisplayName = "ThursdayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::FridayLongDisplayName = "FridayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::SaturdayLongDisplayName = "SaturdayLongDisplayName";

const char * _impl::_DateTimeManagerImpl::JanuaryShortDisplayName = "JanuaryShortDisplayName";
const char * _impl::_DateTimeManagerImpl::FebruaryShortDisplayName = "FebruaryShortDisplayName";
const char * _impl::_DateTimeManagerImpl::MarchShortDisplayName = "MarchShortDisplayName";
const char * _impl::_DateTimeManagerImpl::AprilShortDisplayName = "AprilShortDisplayName";
const char * _impl::_DateTimeManagerImpl::MayShortDisplayName = "MayShortDisplayName";
const char * _impl::_DateTimeManagerImpl::JuneShortDisplayName = "JuneShortDisplayName";
const char * _impl::_DateTimeManagerImpl::JulyShortDisplayName = "JulyShortDisplayName";
const char * _impl::_DateTimeManagerImpl::AugustShortDisplayName = "AugustShortDisplayName";
const char * _impl::_DateTimeManagerImpl::SeptemberShortDisplayName = "SeptemberShortDisplayName";
const char * _impl::_DateTimeManagerImpl::OctoberShortDisplayName = "OctoberShortDisplayName";
const char * _impl::_DateTimeManagerImpl::NovemberShortDisplayName = "NovemberShortDisplayName";
const char * _impl::_DateTimeManagerImpl::DecemberShortDisplayName = "DecemberShortDisplayName";

const char * _impl::_DateTimeManagerImpl::JanuaryLongDisplayName = "JanuaryLongDisplayName";
const char * _impl::_DateTimeManagerImpl::FebruaryLongDisplayName = "FebruaryLongDisplayName";
const char * _impl::_DateTimeManagerImpl::MarchLongDisplayName = "MarchLongDisplayName";
const char * _impl::_DateTimeManagerImpl::AprilLongDisplayName = "AprilLongDisplayName";
const char * _impl::_DateTimeManagerImpl::MayLongDisplayName = "MayLongDisplayName";
const char * _impl::_DateTimeManagerImpl::JuneLongDisplayName = "JuneLongDisplayName";
const char * _impl::_DateTimeManagerImpl::JulyLongDisplayName = "JulyLongDisplayName";
const char * _impl::_DateTimeManagerImpl::AugustLongDisplayName = "AugustLongDisplayName";
const char * _impl::_DateTimeManagerImpl::SeptemberLongDisplayName = "SeptemberLongDisplayName";
const char * _impl::_DateTimeManagerImpl::OctoberLongDisplayName = "OctoberLongDisplayName";
const char * _impl::_DateTimeManagerImpl::NovemberLongDisplayName = "NovemberLongDisplayName";
const char * _impl::_DateTimeManagerImpl::DecemberLongDisplayName = "DecemberLongDisplayName";

//  End of DateTimeManagerImpl.cpp
