//
//  DateTimeParse.cpp - Implementation of the carl::DateTimeParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Operations
void DateTimeParse::registerInstance(const Locale & forLocale, DateTimeParse * pDateTimeParse)
{
    require(pDateTimeParse != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_DateTimeFormatManager::getInstance()->registerDateTimeParse(forLocale, pDateTimeParse);
}

DateTimeParse * DateTimeParse::getInstance()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(Locale::getDefault());
}

DateTimeParse * DateTimeParse::getInstance(const Locale & forLocale)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeFormatManager::getInstance()->getDateTimeParse(forLocale);
}

//  Parsing of full strings
DateTime DateTimeParse::parse(const char * pDateTimeString) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const char * pDateTimeString, Style style) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, style);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const char * pDateTimeString, Precision precision) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, precision);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const char * pDateTimeString, Style style, Precision precision) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, style, precision);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const char * pDateTimeString, const String & format) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, format);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const Char * pDateTimeString) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const Char * pDateTimeString, Style style) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, style);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const Char * pDateTimeString, Precision precision) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, precision);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const Char * pDateTimeString, Style style, Precision precision) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, style, precision);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const Char * pDateTimeString, const String & format) const throw(ParseException)
{
    require(pDateTimeString != NULL);

    UIntSize scan = 0;
    DateTime result = parse(pDateTimeString, scan, format);
    if (pDateTimeString[scan] != 0)
    { 
        throw ParseException(String(pDateTimeString), scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const String & dateTimeString) const throw(ParseException)
{
    UIntSize scan = 0;
    DateTime result = parse(dateTimeString, scan);
    if (scan < dateTimeString.length())
    { 
        throw ParseException(dateTimeString, scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const String & dateTimeString, Style style) const throw(ParseException)
{
    UIntSize scan = 0;
    DateTime result = parse(dateTimeString, scan, style);
    if (scan < dateTimeString.length())
    { 
        throw ParseException(dateTimeString, scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const String & dateTimeString, Precision precision) const throw(ParseException)
{
    UIntSize scan = 0;
    DateTime result = parse(dateTimeString, scan, precision);
    if (scan < dateTimeString.length())
    { 
        throw ParseException(dateTimeString, scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const String & dateTimeString, Style style, Precision precision) const throw(ParseException)
{
    UIntSize scan = 0;
    DateTime result = parse(dateTimeString, scan, style, precision);
    if (scan < dateTimeString.length())
    { 
        throw ParseException(dateTimeString, scan);
    }
    return result;
}

DateTime DateTimeParse::parse(const String & dateTimeString, const String & format) const throw(ParseException)
{
    UIntSize scan = 0;
    DateTime result = parse(dateTimeString, scan, format);
    if (scan < dateTimeString.length())
    { 
        throw ParseException(dateTimeString, scan);
    }
    return result;
}

//////////
//  Implementation
CriticalSection * DateTimeParse::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of DateTimeFormat.cpp
