//
//  DefaultDateFormat.cpp - Implementation of the carl::_impl::DefaultDateFormat class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateFormat::_DefaultDateFormat(const Locale & forLocale)
    :   _DefaultDateTimeFormatImpl(forLocale)
{
    _pNumberFormat = NumberFormat::getInstance(forLocale);
}

_impl::_DefaultDateFormat::~_DefaultDateFormat()
{
}

//////////
//  Operations
String _impl::_DefaultDateFormat::format(const Date & date, DateFormat::Style style) const
{
    //  Obtain the format to use
    String format;
    switch (style)
    {
        case DateFormat::Style_Short:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateFormatString, _forLocale);
            break;
        case DateFormat::Style_Medium:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateFormatString, _forLocale);
            break;
        case DateFormat::Style_Long:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateFormatString, _forLocale);
            break;
        case DateFormat::Style_Full:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateFormatString, _forLocale);
            break;
        default:
            require(false);
    }
    //  Perform the formatting
    return _DefaultDateFormat::format(date, format);
}

String _impl::_DefaultDateFormat::format(const Date & date, const String & format) const
{
    //  Perform the formatting
    String result;
    if (!_formatDateTime(result, format,
                         true, date.getYear(),
                         true, date.getMonth(),
                         true, date.getDay(),
                         true, date.getDayOfWeek(),
                         false, 0,
                         false, 0,
                         false, 0,
                         false, 0))
    {   //  Failure to format means invalid format string
        require(false);
    }
    return result;
}

//  End of DefaultDateTimeFormat.cpp
