//
//  DefaultDateParse.cpp - Implementation of the carl::_impl::_DefaultDateParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/09 13:18:17 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateParse::_DefaultDateParse(const Locale & forLocale)
    :   _DefaultDateTimeParseImpl(forLocale)
{
    _pNumberParse = NumberParse::getInstance(forLocale);
}

_impl::_DefaultDateParse::~_DefaultDateParse()
{
}

//////////
//  Operations
Date _impl::_DefaultDateParse::parse(const char * pDateString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pDateString), position);
}

Date _impl::_DefaultDateParse::parse(const char * pDateString, UIntSize & position, DateParse::Style style) const throw(ParseException)
{
    return parse(String(pDateString), position, style);
}

Date _impl::_DefaultDateParse::parse(const char * pDateString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pDateString), position, format);
}

Date _impl::_DefaultDateParse::parse(const Char * pDateString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pDateString), position);
}

Date _impl::_DefaultDateParse::parse(const Char * pDateString, UIntSize & position, DateParse::Style style) const throw(ParseException)
{
    return parse(String(pDateString), position, style);
}

Date _impl::_DefaultDateParse::parse(const Char * pDateString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pDateString), position, format);
}

Date _impl::_DefaultDateParse::parse(const String & dateString, UIntSize & position) const throw(ParseException)
{
    UIntSize savedDateStringScan = position;

    //  Try all styles
    for (int i = DateParse::Style_Full; i >= DateParse::Style_Short; i--)
    {
        try
        {
            return parse(dateString, position, (DateParse::Style)i);
        }
        catch (ParseException)
        {
            position = savedDateStringScan;
        }
    }
    throw ParseException(dateString, position);
}

Date _impl::_DefaultDateParse::parse(const String & dateString, UIntSize & position, DateParse::Style style) const throw(ParseException)
{
    //  Obtain the format to use
    String format;
    switch (style)
    {
        case DateParse::Style_Short:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateFormatString, _forLocale);
            break;
        case DateParse::Style_Medium:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateFormatString, _forLocale);
            break;
        case DateParse::Style_Long:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateFormatString, _forLocale);
            break;
        case DateParse::Style_Full:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateFormatString, _forLocale);
            break;
        default:
            require(false);
    }
    //  Perform the formatting
    return parse(dateString, position, format);
}

Date _impl::_DefaultDateParse::parse(const String & dateString, UIntSize & position, const String & format) const throw(ParseException)
{
    bool yearKnown = false;
    int year = 0;
    bool monthKnown = false; 
    int month = 0;
    bool dayKnown = false; 
    int day = 0;
    bool dayOfWeekKnown = false; 
    DayOfWeek dayOfWeek = DayOfWeek_Monday;
    bool hourKnown = false; 
    int hour = 0;
    bool minuteKnown = false; 
    int minute = 0;
    bool secondKnown = false; 
    int second = 0;
    bool nanosecondKnown = false; 
    UInt32 nanosecond = 0;

    if (!_parseDateTime(dateString, position, 
                        format,
                        yearKnown, year,
                        monthKnown, month,
                        dayKnown, day,
                        dayOfWeekKnown, dayOfWeek,
                        hourKnown, hour,
                        minuteKnown, minute,
                        secondKnown, second,
                        nanosecondKnown, nanosecond))
    {
        throw ParseException(dateString, position);
    }
    if (!yearKnown || !monthKnown || !dayKnown || hourKnown || minuteKnown || secondKnown || nanosecondKnown)
    {
        throw ParseException(dateString, position);
    }
    if (day > Calendar::daysInMonth(year, month))
    {
        throw ParseException(dateString, position);
    }
    Date result(year, month, day);
    if (dayOfWeekKnown && result.getDayOfWeek() != dayOfWeek)
    {
        throw ParseException(dateString, position);
    }
    return result;
}

//  End of DefaultDateParse.cpp
