//
//  DefaultDateTimeFormat.cpp - Implementation of the carl::_impl::DefaultDateTimeFormat class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateTimeFormat::_DefaultDateTimeFormat(const Locale & forLocale)
    :   _DefaultDateTimeFormatImpl(forLocale)
{
    _pNumberFormat = NumberFormat::getInstance(forLocale);
}

_impl::_DefaultDateTimeFormat::~_DefaultDateTimeFormat()
{
}

//////////
//  Operations
String _impl::_DefaultDateTimeFormat::format(const DateTime & dateTime, DateTimeFormat::Style style, DateTimeFormat::Precision precision) const
{
    //  Obtain the format to use
    String format;
    switch (style)
    {
        case DateTimeFormat::Style_Short:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Medium:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Long:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Full:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        default:
            require(false);
    }
    //  Perform the formatting
    return _DefaultDateTimeFormat::format(dateTime, format);
}

String _impl::_DefaultDateTimeFormat::format(const DateTime & dateTime, const String & format) const
{
    //  Perform the formatting
    String result;
    if (!_formatDateTime(result, format,
                         true, dateTime.getYear(),
                         true, dateTime.getMonth(),
                         true, dateTime.getDay(),
                         true, dateTime.getDayOfWeek(),
                         true, dateTime.getHour(),
                         true, dateTime.getMinute(),
                         true, dateTime.getSecond(),
                         true, dateTime.getNanosecond()))
    {   //  Failure to format means invalid format string
        require(false);
    }
    return result;
}

//  End of DefaultDateTimeFormat.cpp
