//
//  DefaultDateTimeFormatImpl.cpp - Implementation of the carl::_impl::DefaultDateTimeFormatImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateTimeFormatImpl::_DefaultDateTimeFormatImpl(const Locale & forLocale)
    :   _forLocale(forLocale)
{
    _pNumberFormat = NumberFormat::getInstance(forLocale);
}

_impl::_DefaultDateTimeFormatImpl::~_DefaultDateTimeFormatImpl()
{
}

//////////
//  Helpers
bool _impl::_DefaultDateTimeFormatImpl::_formatDateTime(String & result, const String & format,
                                                        bool yearKnown, int year,
                                                        bool monthKnown, int month,
                                                        bool dayKnown, int day,
                                                        bool dayOfWeekKnown, DayOfWeek dayOfWeek,
                                                        bool hourKnown, int hour,
                                                        bool minuteKnown, int minute,
                                                        bool secondKnown, int second,
                                                        bool nanosecondKnown, UInt32 nanosecond) const
{
    result.setLength(0);

    for (UIntSize scan = 0; scan < format.length(); )
    {
        if (scan + 1 < format.length() && format[scan] == '\\')
        {   //  \X -> X
            scan++;
            result += format[scan++];
        }
        else if (format.mid(scan).startsWith("{year:full}"))
        {   //  Full year "YYYY"
            scan += 11;
            if (!yearKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(4).setMaximumDigits(4);
            result += _pNumberFormat->format(year, ifp);
        }
        else if (format.mid(scan).startsWith("{year:short}"))
        {   //  Short year "YY"
            scan += 12;
            if (!yearKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(year % 100, ifp);
        }
        else if (format.mid(scan).startsWith("{month:full}"))
        {   //  Full month "MM"
            scan += 12;
            if (!monthKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(month, ifp);
        }
        else if (format.mid(scan).startsWith("{month:short}"))
        {   //  Short month "[M]M"
            scan += 13;
            if (!monthKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            result += _pNumberFormat->format(month, ifp);
        }
        else if (format.mid(scan).startsWith("{month_name:full}"))
        {   //  Full month "name"
            scan += 17;
            if (!monthKnown)
            {
                return false;
            }
            result += Calendar::getMonthFullDisplayName(month);
        }
        else if (format.mid(scan).startsWith("{month_name:short}"))
        {   //  Short month "name"
            scan += 18;
            if (!monthKnown)
            {
                return false;
            }
            result += Calendar::getMonthShortDisplayName(month);
        }
        else if (format.mid(scan).startsWith("{day:full}"))
        {   //  Full day "DD"
            scan += 10;
            if (!dayKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(day, ifp);
        }
        else if (format.mid(scan).startsWith("{day:short}"))
        {   //  Short day "[D]D"
            scan += 11;
            if (!dayKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            result += _pNumberFormat->format(day, ifp);
        }
        else if (format.mid(scan).startsWith("{day_of_week_name:full}"))
        {   //  Full day of week "name"
            scan += 23;
            if (!dayOfWeekKnown)
            {
                return false;
            }
            result += Calendar::getDayOfWeekFullDisplayName(dayOfWeek);
        }
        else if (format.mid(scan).startsWith("{day_of_week_name:short}"))
        {   //  Short day of week "name"
            scan += 24;
            if (!dayOfWeekKnown)
            {
                return false;
            }
            result += Calendar::getDayOfWeekShortDisplayName(dayOfWeek);
        }
        else if (format.mid(scan).startsWith("{hour12:full}"))
        {   //  Full 12-hour "HH"
            scan += 13;
            if (!hourKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            int hour12 = hour % 12;
            if (hour12 == 0)
            {
                hour12 = 12;
            }
            result += _pNumberFormat->format(hour12, ifp);
        }
        else if (format.mid(scan).startsWith("{hour12:short}"))
        {   //  Short 12-hour "[H]H"
            scan += 14;
            if (!hourKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            int hour12 = hour % 12;
            if (hour12 == 0)
            {
                hour12 = 12;
            }
            result += _pNumberFormat->format(hour12, ifp);
        }
        else if (format.mid(scan).startsWith("{hour24:full}"))
        {   //  Full 24-hour "HH"
            scan += 13;
            if (!hourKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(hour, ifp);
        }
        else if (format.mid(scan).startsWith("{hour24:short}"))
        {   //  Short 24-hour "[H]H"
            scan += 14;
            if (!hourKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            result += _pNumberFormat->format(hour, ifp);
        }
        else if (format.mid(scan).startsWith("{minute:full}"))
        {   //  Full minute "MM"
            scan += 13;
            if (!minuteKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(minute, ifp);
        }
        else if (format.mid(scan).startsWith("{minute:short}"))
        {   //  Short minute "[M]M"
            scan += 14;
            if (!minuteKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            result += _pNumberFormat->format(minute, ifp);
        }
        else if (format.mid(scan).startsWith("{second:full}"))
        {   //  Full second "SS"
            scan += 13;
            if (!secondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(2).setMaximumDigits(2);
            result += _pNumberFormat->format(second, ifp);
        }
        else if (format.mid(scan).startsWith("{second:short}"))
        {   //  Short second "[S]S"
            scan += 14;
            if (!secondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(2);
            result += _pNumberFormat->format(second, ifp);
        }
        else if (format.mid(scan).startsWith("{millisecond:full}"))
        {   //  Full millisecond "MMM"
            scan += 18;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(3).setMaximumDigits(3);
            result += _pNumberFormat->format(nanosecond / 1000000, ifp);
        }
        else if (format.mid(scan).startsWith("{millisecond:short}"))
        {   //  Short millisecond "[[M]M]M"
            scan += 19;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(3);
            result += _pNumberFormat->format(nanosecond / 1000000, ifp);
        }
        else if (format.mid(scan).startsWith("{microsecond:full}"))
        {   //  Full microsecond "MMMMMM"
            scan += 18;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(6).setMaximumDigits(6);
            result += _pNumberFormat->format(nanosecond / 1000, ifp);
        }
        else if (format.mid(scan).startsWith("{microsecond:short}"))
        {   //  Short microsecond "[[[[[M]M]M]M]M]M"
            scan += 19;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(6);
            result += _pNumberFormat->format(nanosecond / 1000, ifp);
        }
        else if (format.mid(scan).startsWith("{nanosecond:full}"))
        {   //  Full nanosecond "NNNNNNNNN"
            scan += 17;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(9).setMaximumDigits(9);
            result += _pNumberFormat->format(nanosecond, ifp);
        }
        else if (format.mid(scan).startsWith("{nanosecond:short}"))
        {   //  Short nanosecond "[[[[[[[[N]N]N]N]N]N]N]N]N"
            scan += 18;
            if (!nanosecondKnown)
            {
                return false;
            }
            IntegerFormatProperties ifp;
            ifp.setMinimumDigits(1).setMaximumDigits(9);
            result += _pNumberFormat->format(nanosecond, ifp);
        }
        else if (format.mid(scan).startsWith("{ampm}"))
        {   //  AM/PM indicator
            scan += 6;
            if (!hourKnown)
            {
                return false;
            }
            if (hour < 12)
            {
                result += _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::AmString, _forLocale);
            }
            else
            {
                result += _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::PmString, _forLocale);
            }
        }
        else
        {   //  Copy 1 character "as is"
            result += format[scan++];
        }
    }
    return true;
}

//  End of DefaultDateTimeFormatImpl.cpp
