//
//  DefaultDateTimeParse.cpp - Implementation of the carl::_impl::_DefaultDateTimeParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/09 20:13:45 $
//  $Revision: 1.13 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateTimeParse::_DefaultDateTimeParse(const Locale & forLocale)
    :   _DefaultDateTimeParseImpl(forLocale)
{
    _pNumberParse = NumberParse::getInstance(forLocale);
}

_impl::_DefaultDateTimeParse::~_DefaultDateTimeParse()
{
}

//////////
//  Operations
DateTime _impl::_DefaultDateTimeParse::parse(const char * pDateTimeString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pDateTimeString), position);
}

DateTime _impl::_DefaultDateTimeParse::parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, style);
}

DateTime _impl::_DefaultDateTimeParse::parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, precision);
}

DateTime _impl::_DefaultDateTimeParse::parse(const char * pDateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, style, precision);
}

DateTime _impl::_DefaultDateTimeParse::parse(const char * pDateTimeString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, format);
}

DateTime _impl::_DefaultDateTimeParse::parse(const Char * pDateTimeString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pDateTimeString), position);
}

DateTime _impl::_DefaultDateTimeParse::parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, style);
}

DateTime _impl::_DefaultDateTimeParse::parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, precision);
}

DateTime _impl::_DefaultDateTimeParse::parse(const Char * pDateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, style, precision);
}

DateTime _impl::_DefaultDateTimeParse::parse(const Char * pDateTimeString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pDateTimeString), position, format);
}

DateTime _impl::_DefaultDateTimeParse::parse(const String & dateTimeString, UIntSize & position) const throw(ParseException)
{
    UIntSize savedPosition = position;

    //  Try all styles & precisions
    for (int i = DateTimeParse::Style_Full; i >= DateTimeParse::Style_Short; i--)
    {
        for (int j = DateTimeParse::Precision_Nanosecond; j >= DateTimeParse::Precision_Minute; j--)
        {
            try
            {
                return parse(dateTimeString, position, (DateTimeParse::Style)i, (DateTimeParse::Precision)j);
            }
            catch (ParseException)
            {
                position = savedPosition;
            }
        }
    }
    throw ParseException(dateTimeString, position);
}

DateTime _impl::_DefaultDateTimeParse::parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Style style) const throw(ParseException)
{
    UIntSize savedPosition = position;

    //  Try all precisions for the specified style
    for (int i = DateTimeParse::Precision_Nanosecond; i >= DateTimeParse::Precision_Minute; i--)
    {
        try
        {
            return parse(dateTimeString, position, style, (DateTimeParse::Precision)i);
        }
        catch (ParseException)
        {
            position = savedPosition;
        }
    }
    throw ParseException(dateTimeString, position);
}

DateTime _impl::_DefaultDateTimeParse::parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Precision precision) const throw(ParseException)
{
    UIntSize savedPosition = position;

    //  Try all styles for the specified precision
    for (int i = DateTimeParse::Style_Full; i >= DateTimeParse::Style_Short; i--)
    {
        try
        {
            return parse(dateTimeString, position, (DateTimeParse::Style)i, precision);
        }
        catch (ParseException)
        {
            position = savedPosition;
        }
    }
    throw ParseException(dateTimeString, position);
}

DateTime _impl::_DefaultDateTimeParse::parse(const String & dateTimeString, UIntSize & position, DateTimeParse::Style style, DateTimeParse::Precision precision) const throw(ParseException)
{
    //  Obtain the format to use
    String format;
    switch (style)
    {
        case DateTimeFormat::Style_Short:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::ShortDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Medium:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::MediumDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Long:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::LongDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        case DateTimeFormat::Style_Full:
            switch (precision)
            {
                case DateTimeFormat::Precision_Minute:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMinutes, _forLocale);
                    break;
                case DateTimeFormat::Precision_Second:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringSeconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Millisecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMilliseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Microsecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringMicroseconds, _forLocale);
                    break;
                case DateTimeFormat::Precision_Nanosecond:
                    format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::FullDateTimeFormatStringNanoseconds, _forLocale);
                    break;
                default:
                    require(false);
            }
            break;
        default:
            require(false);
    }
    //  Perform the parsing
    return parse(dateTimeString, position, format);
}

DateTime _impl::_DefaultDateTimeParse::parse(const String & dateTimeString, UIntSize & position, const String & format) const throw(ParseException)
{
    bool yearKnown = false;
    int year = 0;
    bool monthKnown = false; 
    int month = 0;
    bool dayKnown = false; 
    int day = 0;
    bool dayOfWeekKnown = false; 
    DayOfWeek dayOfWeek = DayOfWeek_Monday;
    bool hourKnown = false; 
    int hour = 0;
    bool minuteKnown = false; 
    int minute = 0;
    bool secondKnown = false; 
    int second = 0;
    bool nanosecondKnown = false; 
    UInt32 nanosecond = 0;

    if (!_parseDateTime(dateTimeString, position, 
                        format,
                        yearKnown, year,
                        monthKnown, month,
                        dayKnown, day,
                        dayOfWeekKnown, dayOfWeek,
                        hourKnown, hour,
                        minuteKnown, minute,
                        secondKnown, second,
                        nanosecondKnown, nanosecond))
    {
        throw ParseException(dateTimeString, position);
    }
    if (!yearKnown || !monthKnown || !dayKnown || !hourKnown || !minuteKnown)
    {
        throw ParseException(dateTimeString, position);
    }
    if (day > Calendar::daysInMonth(year, month))
    {
        throw ParseException(dateTimeString, position);
    }
    DateTime result(year, month, day, hour, minute, second, nanosecond);
    if (dayOfWeekKnown && result.getDayOfWeek() != dayOfWeek)
    {
        throw ParseException(dateTimeString, position);
    }
    return result;
}

//  End of DefaultDateTimeParse.cpp
