//
//  DefaultDateTimeParseImpl.cpp - Implementation of the carl::_impl::_DefaultDateTimeParseImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/08/09 14:13:19 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultDateTimeParseImpl::_DefaultDateTimeParseImpl(const Locale & forLocale)
    :   _forLocale(forLocale)
{
    _pNumberParse = NumberParse::getInstance(forLocale);
}

_impl::_DefaultDateTimeParseImpl::~_DefaultDateTimeParseImpl()
{
}

//////////
//  Operations
bool _impl::_DefaultDateTimeParseImpl::_parseDateTime(const String & dateTimeString, UIntSize & position, 
                                                      const String & format,
                                                      bool & yearKnown, int & year,
                                                      bool & monthKnown, int & month,
                                                      bool & dayKnown, int & day,
                                                      bool & dayOfWeekKnown, DayOfWeek & dayOfWeek,
                                                      bool & hourKnown, int & hour,
                                                      bool & minuteKnown, int & minute,
                                                      bool & secondKnown, int & second,
                                                      bool & nanosecondKnown, UInt32 & nanosecond) const
{
    require(position <= dateTimeString.length());

    yearKnown = monthKnown = dayKnown = dayOfWeekKnown = hourKnown = minuteKnown = secondKnown = nanosecondKnown = false;
    bool hour12Known = false;
    int hour12 = 0;
    bool amPmKnown = false;
    bool pm = false;
    for (UIntSize formatScan = 0; formatScan < format.length(); )
    {
        if (formatScan + 1 < format.length() && format[formatScan] == '\\')
        {   //  \X -> X
            formatScan++;
            if (!_matchFormatLiteralCharacter(dateTimeString, position, format, formatScan))
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{year:full}"))
        {   //  Full year "YYYY"
            formatScan += 11;
            if (yearKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(4).setMaximumWidth(4);
                bool overflow;
                UInt32 parsedYear = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedYear == 0 || parsedYear > 9999 || overflow)
                {
                    return false;
                }
                year = (int)parsedYear;
                yearKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{year:short}"))
        {   //  Short year "YY"
            formatScan += 12;
            if (yearKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(2).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedYear = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedYear > 99 || overflow)
                {
                    return false;
                }
                year = (int)((parsedYear >= 80) ? (parsedYear + 1900) : (parsedYear + 2000));
                yearKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{month:full}"))
        {   //  Full month "MM"
            formatScan += 12;
            if (monthKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedMonth = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMonth < 1 || parsedMonth > 12 || overflow)
                {
                    return false;
                }
                month = (int)parsedMonth;
                monthKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{month:short}"))
        {   //  Short month "[M]M"
            formatScan += 13;
            if (monthKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedMonth = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMonth < 1 || parsedMonth > 12 || overflow)
                {
                    return false;
                }
                month = (int)parsedMonth;
                monthKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{month_name:full}"))
        {   //  Full month "name"
            formatScan += 17;
            if (monthKnown)
            {
                return false;
            }
            for (int i = Month_January; i <= Month_December; i++)
            {
                String monthName = Calendar::getMonthFullDisplayName((Month)i, _forLocale);
                if (dateTimeString.mid(position).startsWithIgnoreCase(monthName))
                {
                    position += monthName.length();
                    month = i;
                    monthKnown = true;
                    break;
                }
            }
            if (!monthKnown)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{month_name:short}"))
        {   //  Short month "name"
            formatScan += 18;
            if (monthKnown)
            {
                return false;
            }
            for (int i = Month_January; i <= Month_December; i++)
            {
                String monthName = Calendar::getMonthShortDisplayName((Month)i, _forLocale);
                if (dateTimeString.mid(position).startsWithIgnoreCase(monthName))
                {
                    position += monthName.length();
                    month = i;
                    monthKnown = true;
                    break;
                }
            }
            if (!monthKnown)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{day:full}"))
        {   //  Full day "DD"
            formatScan += 10;
            if (dayKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedDay = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedDay < 1 || parsedDay > 31 || overflow)
                {
                    return false;
                }
                day = (int)parsedDay;
                dayKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{day:short}"))
        {   //  Short day "[D]D"
            formatScan += 11;
            if (dayKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedDay = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedDay < 1 || parsedDay > 31 || overflow)
                {
                    return false;
                }
                day = (int)parsedDay;
                dayKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{day_of_week_name:full}"))
        {   //  Full day of week "name"
            formatScan += 23;
            if (dayOfWeekKnown)
            {
                return false;
            }
            for (int i = DayOfWeek_Monday; i <= DayOfWeek_Sunday; i++)
            {
                String dayOfWeekName = Calendar::getDayOfWeekFullDisplayName(static_cast<DayOfWeek>(i), _forLocale);
                if (dateTimeString.mid(position).startsWithIgnoreCase(dayOfWeekName))
                {
                    position += dayOfWeekName.length();
                    dayOfWeek = static_cast<DayOfWeek>(i);
                    dayOfWeekKnown = true;
                    break;
                }
            }
            if (!dayOfWeekKnown)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{day_of_week_name:short}"))
        {   //  Short day of week "name"
            formatScan += 24;
            if (dayOfWeekKnown)
            {
                return false;
            }
            for (int i = DayOfWeek_Monday; i <= DayOfWeek_Sunday; i++)
            {
                String dayOfWeekName = Calendar::getDayOfWeekShortDisplayName(static_cast<DayOfWeek>(i), _forLocale);
                if (dateTimeString.mid(position).startsWithIgnoreCase(dayOfWeekName))
                {
                    position += dayOfWeekName.length();
                    dayOfWeek = static_cast<DayOfWeek>(i);
                    dayOfWeekKnown = true;
                    break;
                }
            }
            if (!dayOfWeekKnown)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{hour12:full}"))
        {   //  Full 12-hour "HH"
            formatScan += 13;
            if (hour12Known || hourKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedHour12 = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedHour12 == 0 || parsedHour12 > 12 || overflow)
                {
                    return false;
                }
                hour12 = (int)parsedHour12;
                hour12Known = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{hour12:short}"))
        {   //  Short 12-hour "[H]H"
            formatScan += 14;
            if (hour12Known || hourKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedHour12 = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedHour12 == 0 || parsedHour12 > 12 || overflow)
                {
                    return false;
                }
                hour12 = (int)parsedHour12;
                hour12Known = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{hour24:full}"))
        {   //  Full 12-hour "HH"
            formatScan += 13;
            if (hourKnown || hour12Known)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedHour = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedHour > 23 || overflow)
                {
                    return false;
                }
                hour = (int)parsedHour;
                hourKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{hour24:short}"))
        {   //  Short 12-hour "[H]H"
            formatScan += 14;
            if (hourKnown || hour12Known)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedHour = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedHour > 23 || overflow)
                {
                    return false;
                }
                hour = (int)parsedHour;
                hourKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{minute:full}"))
        {   //  Full minute "MM"
            formatScan += 13;
            if (minuteKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedMinute = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMinute > 59 || overflow)
                {
                    return false;
                }
                minute = (int)parsedMinute;
                minuteKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{minute:short}"))
        {   //  Short minute "[M]M"
            formatScan += 14;
            if (minuteKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedMinute = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMinute > 59 || overflow)
                {
                    return false;
                }
                minute = (int)parsedMinute;
                minuteKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{second:full}"))
        {   //  Full second "SS"
            formatScan += 13;
            if (secondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedSecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedSecond > 59 || overflow)
                {
                    return false;
                }
                second = (int)parsedSecond;
                secondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{second:short}"))
        {   //  Short second "[S]S"
            formatScan += 14;
            if (secondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(2);
                bool overflow;
                UInt32 parsedSecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedSecond > 59 || overflow)
                {
                    return false;
                }
                second = (int)parsedSecond;
                secondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{millisecond:full}"))
        {   //  Full millisecond "MMM"
            formatScan += 18;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(3).setMaximumWidth(3);
                bool overflow;
                UInt32 parsedMillisecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMillisecond > 999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedMillisecond * 1000000;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{millisecond:short}"))
        {   //  Short millisecond "[[M]M]M"
            formatScan += 19;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(3);
                bool overflow;
                UInt32 parsedMillisecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMillisecond > 999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedMillisecond * 1000000;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{microsecond:full}"))
        {   //  Full microsecond "MMMMMM"
            formatScan += 18;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(6).setMaximumWidth(6);
                bool overflow;
                UInt32 parsedMicrosecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMicrosecond > 999999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedMicrosecond * 1000;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{microsecond:short}"))
        {   //  Short microsecond "[[[[[M]M]M]M]M]M"
            formatScan += 19;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(6);
                bool overflow;
                UInt32 parsedMicrosecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedMicrosecond > 999999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedMicrosecond * 1000;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{nanosecond:full}"))
        {   //  Full nanosecond "NNNNNNNNN"
            formatScan += 17;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(9).setMaximumWidth(9);
                bool overflow;
                UInt32 parsedNanosecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedNanosecond > 999999999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedNanosecond;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{nanosecond:short}"))
        {   //  Short nanosecond "[[[[[[[[N]N]N]N]N]N]N]N]N"
            formatScan += 18;
            if (nanosecondKnown)
            {
                return false;
            }
            try
            {
                IntegerParseProperties parseProperties;
                parseProperties.setMinimumWidth(1).setMaximumWidth(9);
                bool overflow;
                UInt32 parsedNanosecond = _pNumberParse->parseUInt32(dateTimeString, position, parseProperties, &overflow);
                if (parsedNanosecond > 999999999 || overflow)
                {
                    return false;
                }
                nanosecond = parsedNanosecond;
                nanosecondKnown = true;
            }
            catch (ParseException)
            {
                return false;
            }
        }
        else if (format.mid(formatScan).startsWith("{ampm}"))
        {   //  AM/PM indicator
            formatScan += 6;
            if (hourKnown || amPmKnown)
            {
                return false;
            }
            String amString = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::AmString, _forLocale);
            String pmString = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::PmString, _forLocale);
            if (dateTimeString.mid(position).startsWithIgnoreCase(amString))
            {
                pm = false;
                position += amString.length();
            }
            else if (dateTimeString.mid(position).startsWithIgnoreCase(pmString))
            {
                pm = true;
                position += pmString.length();
            }
            else
            {
                return false;
            }
            amPmKnown = true;
        }
        else
        {   //  Match character literally
            if (!_matchFormatLiteralCharacter(dateTimeString, position, format, formatScan))
            {
                return false;
            }
        }
    }
    //  Adjust 12-hour if necessary
    if (!hourKnown && hour12Known && amPmKnown)
    {
        hourKnown = true;
        if (pm)
        {
            hour = hour12 % 12 + 12;
        }
        else
        {
            hour = hour12 % 12;
        }
    }
    //  Done
    return true;
}

bool _impl::_DefaultDateTimeParseImpl::_matchFormatLiteralCharacter(const String & dateTimeString, UIntSize & position,
                                                                    const String & format, UIntSize & formatScan) const
{
    require(position <= dateTimeString.length());
    require(formatScan < format.length());

    if (Iso10646::isSpace(format[formatScan]))
    {   //  Space matches any number of spaces, including none
        while (position < dateTimeString.length() &&
               Iso10646::isSpace(dateTimeString[position]))
        {
            position++;
        }
        formatScan++;
        return true;
    }
    else
    {   //  Any other character matches itself case-insensitively
        if (position >= dateTimeString.length())
        {
            return false;
        }
        else if (Iso10646::toUpper(format[formatScan]) == Iso10646::toUpper(dateTimeString[position]))
        {
            formatScan++;
            position++;
            return true;
        }
        else
        {
            return false;
        }
    }
}

//  End of DefaultDateTimeParseImpl.cpp
