//
//  DefaultNumberParse.cpp - Implementation of the carl::_impl::_DefaultNumberParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/09/12 11:36:17 $
//  $Revision: 1.16 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "NumberParseImpl.hpp"
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <float.h>
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    #include <math.h>
#else
    #error Unsupported platform
#endif

//////////
//  Construction/destruction
_impl::_DefaultNumberParse::_DefaultNumberParse(Char decimalPointChar, Char separatorChar)
{
    _decimalPointChar = decimalPointChar;
    _separatorChar = separatorChar;
}

_impl::_DefaultNumberParse::~_DefaultNumberParse()
{
}

//////////
//  Operations
Int8 _impl::_DefaultNumberParse::parseInt8(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int8 result = static_cast<Int8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int16 _impl::_DefaultNumberParse::parseInt16(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int16 result = static_cast<Int16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int32 _impl::_DefaultNumberParse::parseInt32(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int32 result = static_cast<Int32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int64 _impl::_DefaultNumberParse::parseInt64(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseInt64(parsedString, position, parseProperties, pOverflow);
}

UInt8 _impl::_DefaultNumberParse::parseUInt8(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt8 result = static_cast<UInt8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt16 _impl::_DefaultNumberParse::parseUInt16(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt16 result = static_cast<UInt16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt32 _impl::_DefaultNumberParse::parseUInt32(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt32 result = static_cast<UInt32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt64 _impl::_DefaultNumberParse::parseUInt64(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseUInt64(parsedString, position, parseProperties, pOverflow);
}

BinaryInteger _impl::_DefaultNumberParse::parseBinaryInteger(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseBinaryInteger(parsedString, position, parseProperties);
}

DecimalInteger _impl::_DefaultNumberParse::parseDecimalInteger(const char * pString, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseDecimalInteger(parsedString, position, parseProperties);
}

Real32 _impl::_DefaultNumberParse::parseReal32(const char * pString, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return static_cast<Real32>(_parseReal64(parsedString, position, parseProperties, pOverflow));
}

Real64 _impl::_DefaultNumberParse::parseReal64(const char * pString, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseReal64(parsedString, position, parseProperties, pOverflow);
}

BinaryFixed _impl::_DefaultNumberParse::parseBinaryFixed(const char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseBinaryFixed(parsedString, position, parseProperties);
}

BinaryFloat _impl::_DefaultNumberParse::parseBinaryFloat(const char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseBinaryFloat(parsedString, position, parseProperties);
}

DecimalFixed _impl::_DefaultNumberParse::parseDecimalFixed(const char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseDecimalFixed(parsedString, position, parseProperties);
}

DecimalFloat _impl::_DefaultNumberParse::parseDecimalFloat(const char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedAsciiString parsedString(pString);
    return _parseDecimalFloat(parsedString, position, parseProperties);
}

Int8 _impl::_DefaultNumberParse::parseInt8(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int8 result = static_cast<Int8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int16 _impl::_DefaultNumberParse::parseInt16(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int16 result = static_cast<Int16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int32 _impl::_DefaultNumberParse::parseInt32(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int32 result = static_cast<Int32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int64 _impl::_DefaultNumberParse::parseInt64(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseInt64(parsedString, position, parseProperties, pOverflow);
}

UInt8 _impl::_DefaultNumberParse::parseUInt8(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt8 result = static_cast<UInt8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt16 _impl::_DefaultNumberParse::parseUInt16(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt16 result = static_cast<UInt16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt32 _impl::_DefaultNumberParse::parseUInt32(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt32 result = static_cast<UInt32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt64 _impl::_DefaultNumberParse::parseUInt64(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseUInt64(parsedString, position, parseProperties, pOverflow);
}

BinaryInteger _impl::_DefaultNumberParse::parseBinaryInteger(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseBinaryInteger(parsedString, position, parseProperties);
}

DecimalInteger _impl::_DefaultNumberParse::parseDecimalInteger(const Char * pString, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseDecimalInteger(parsedString, position, parseProperties);
}

Real32 _impl::_DefaultNumberParse::parseReal32(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return static_cast<Real32>(_parseReal64(parsedString, position, parseProperties, pOverflow));
}

Real64 _impl::_DefaultNumberParse::parseReal64(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseReal64(parsedString, position, parseProperties, pOverflow);
}

BinaryFixed _impl::_DefaultNumberParse::parseBinaryFixed(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseBinaryFixed(parsedString, position, parseProperties);
}

BinaryFloat _impl::_DefaultNumberParse::parseBinaryFloat(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseBinaryFloat(parsedString, position, parseProperties);
}

DecimalFixed _impl::_DefaultNumberParse::parseDecimalFixed(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseDecimalFixed(parsedString, position, parseProperties);
}

DecimalFloat _impl::_DefaultNumberParse::parseDecimalFloat(const Char * pString, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedIso10646String parsedString(pString);
    return _parseDecimalFloat(parsedString, position, parseProperties);
}

Int8 _impl::_DefaultNumberParse::parseInt8(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int8 result = static_cast<Int8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int16 _impl::_DefaultNumberParse::parseInt16(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int16 result = static_cast<Int16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int32 _impl::_DefaultNumberParse::parseInt32(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    Int64 value = _parseInt64(parsedString, position, parseProperties, pOverflow);
    Int32 result = static_cast<Int32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

Int64 _impl::_DefaultNumberParse::parseInt64(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseInt64(parsedString, position, parseProperties, pOverflow);
}

UInt8 _impl::_DefaultNumberParse::parseUInt8(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt8 result = static_cast<UInt8>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt16 _impl::_DefaultNumberParse::parseUInt16(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt16 result = static_cast<UInt16>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt32 _impl::_DefaultNumberParse::parseUInt32(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    UInt64 value = _parseUInt64(parsedString, position, parseProperties, pOverflow);
    UInt32 result = static_cast<UInt32>(value);
    if (result != value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return result;
}

UInt64 _impl::_DefaultNumberParse::parseUInt64(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseUInt64(parsedString, position, parseProperties, pOverflow);
}

BinaryInteger _impl::_DefaultNumberParse::parseBinaryInteger(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseBinaryInteger(parsedString, position, parseProperties);
}

DecimalInteger _impl::_DefaultNumberParse::parseDecimalInteger(const String & string, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseDecimalInteger(parsedString, position, parseProperties);
}

Real32 _impl::_DefaultNumberParse::parseReal32(const String & string, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return static_cast<Real32>(_parseReal64(parsedString, position, parseProperties, pOverflow));
}

Real64 _impl::_DefaultNumberParse::parseReal64(const String & string, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseReal64(parsedString, position, parseProperties, pOverflow);
}

BinaryFixed _impl::_DefaultNumberParse::parseBinaryFixed(const String & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseBinaryFixed(parsedString, position, parseProperties);
}

BinaryFloat _impl::_DefaultNumberParse::parseBinaryFloat(const String & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseBinaryFloat(parsedString, position, parseProperties);
}

DecimalFixed _impl::_DefaultNumberParse::parseDecimalFixed(const String & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseDecimalFixed(parsedString, position, parseProperties);
}

DecimalFloat _impl::_DefaultNumberParse::parseDecimalFloat(const String & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    _ParsedCARLString parsedString(string);
    return _parseDecimalFloat(parsedString, position, parseProperties);
}

//////////
//  Helpers
Int64 _impl::_DefaultNumberParse::_parseInt64(const _IParsedString & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow) const throw(ParseException)
{
    if (pOverflow != NULL)
    {
        (*pOverflow) = false;
    }
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process sign if one is specified
    Int32 sign = 1;
    if (!string.atEnd(position) && string.charAt(position) == '+')
    {
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    else if (!string.atEnd(position) && string.charAt(position) == '-')
    {
        sign = -1;
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Skip prefix
    if (parseProperties.getRadix() == 16 && parseProperties.getAllowPrefix() &&
        !string.atEnd(position) && !string.atEnd(position + 1) &&
        string.charAt(position) == '0' && Iso10646::toUpper(string.charAt(position + 1)) == 'X')
    {   //  The "0x" prefix is found
        position += 2;
        charactersProcessed += 2;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process digits
    UIntSize numDigits = 0;
    UInt64 value = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            break;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            break;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {   //  Not a digit
                break;
            }
            //
            UInt64 newValue = value * parseProperties.getRadix() + digit;
            if (pOverflow != NULL && newValue / parseProperties.getRadix() != value)
            {
                (*pOverflow) = true;
            }
            value = newValue;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    Int64 valueSigned = static_cast<Int64>(value);
    if ((pOverflow != NULL) && (valueSigned < 0) && (-valueSigned != valueSigned))
    {
        (*pOverflow) = true;
    }
    return valueSigned * sign;
}

UInt64 _impl::_DefaultNumberParse::_parseUInt64(const _IParsedString & string, UIntSize & position, const IntegerParseProperties & parseProperties, bool * pOverflow) const throw(ParseException)
{
    if (pOverflow != NULL)
    {
        (*pOverflow) = false;
    }
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Skip prefix
    if (parseProperties.getRadix() == 16 && parseProperties.getAllowPrefix() &&
        !string.atEnd(position) && !string.atEnd(position + 1) &&
        string.charAt(position) == '0' && Iso10646::toUpper(string.charAt(position + 1)) == 'X')
    {   //  The "0x" prefix is found
        position += 2;
        charactersProcessed += 2;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process digits
    UIntSize numDigits = 0;
    UInt64 value = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            break;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            break;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {   //  Not a digit
                break;
            }
            //
            UInt64 newValue = value * parseProperties.getRadix() + digit;
            if (pOverflow != NULL && newValue / parseProperties.getRadix() != value)
            {
                (*pOverflow) = true;
            }
            value = newValue;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return value;
}

BinaryInteger _impl::_DefaultNumberParse::_parseBinaryInteger(const _IParsedString & string, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process sign if one is specified
    bool isNegative = false;
    if (!string.atEnd(position) && string.charAt(position) == '+')
    {
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    else if (!string.atEnd(position) && string.charAt(position) == '-')
    {
        isNegative = true;
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Skip prefix
    if (parseProperties.getRadix() == 16 && parseProperties.getAllowPrefix() &&
        !string.atEnd(position) && !string.atEnd(position + 1) &&
        string.charAt(position) == '0' && Iso10646::toUpper(string.charAt(position + 1)) == 'X')
    {   //  The "0x" prefix is found
        position += 2;
        charactersProcessed += 2;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process digits
    UIntSize numDigits = 0;
    BinaryInteger value = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            break;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            break;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {   //  Not a digit
                break;
            }
            //
            value = value * parseProperties.getRadix() + digit;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return isNegative ? (-value) : value;
}

DecimalInteger _impl::_DefaultNumberParse::_parseDecimalInteger(const _IParsedString & string, UIntSize & position, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process sign if one is specified
    bool isNegative = false;
    if (!string.atEnd(position) && string.charAt(position) == '+')
    {
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    else if (!string.atEnd(position) && string.charAt(position) == '-')
    {
        isNegative = true;
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Skip prefix
    if (parseProperties.getRadix() == 16 && parseProperties.getAllowPrefix() &&
        !string.atEnd(position) && !string.atEnd(position + 1) &&
        string.charAt(position) == '0' && Iso10646::toUpper(string.charAt(position + 1)) == 'X')
    {   //  The "0x" prefix is found
        position += 2;
        charactersProcessed += 2;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process digits
    UIntSize numDigits = 0;
    DecimalInteger value = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            break;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            break;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {   //  Not a digit
                break;
            }
            //
            value = value * parseProperties.getRadix() + digit;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return isNegative ? (-value) : value;
}

Real64 _impl::_DefaultNumberParse::_parseReal64(const _IParsedString & string, UIntSize & position, const RealParseProperties & parseProperties, bool * pOverflow) const throw(ParseException)
{
    if (pOverflow != NULL)
    {
        (*pOverflow) = false;
    }
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Parse the value
    Real64 value = 0;
    //  Process special cases
    if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
        !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'S' &&
        !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
        !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
        !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #SNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        *(UInt64*)(&value) = CARL_UINT64_CONSTANT(0x7FF7FFFFFFFFFFFF);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        require(_fpclass(value) == _FPCLASS_SNAN || _fpclass(value) == _FPCLASS_QNAN);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
        require(isnan(value));
#else
    #error Unsupported platform
#endif
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'Q' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #QNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        *(UInt64*)(&value) = CARL_UINT64_CONSTANT(0x7FFFFFFFFFFFFFFF);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        require(_fpclass(value) == _FPCLASS_QNAN);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
        require(isnan(value));
#else
    #error Unsupported platform
#endif
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '+' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #+INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        *(UInt64*)(&value) = CARL_UINT64_CONSTANT(0x7FF0000000000000);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        require(_fpclass(value) == _FPCLASS_PINF);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
        require(isinf(value) == 1);
#else
    #error Unsupported platform
#endif
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '-' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #-INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        *(UInt64*)(&value) = CARL_UINT64_CONSTANT(0xFFF0000000000000);
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
        require(_fpclass(value) == _FPCLASS_NINF);
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
        require(isinf(value) == -11);
#else
    #error Unsupported platform
#endif
    }
    else
    {   //  A "normal" real number
        //  Process sign if one is specified
        bool negative = false;
        if (!string.atEnd(position) && string.charAt(position) == '+')
        {
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        else if (!string.atEnd(position) && string.charAt(position) == '-')
        {
            negative = true;
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        //  Process integer digits
        UIntSize numDigits = 0;
        UIntSize numFractionalDigirs = 0;
        bool exponentNegative = false;
        UIntSize exponent = 0;
        for (; ; )
        {
            if (string.atEnd(position))
            {   //  End of parsed string
                goto EndOfNumber;
            }
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters
                goto EndOfNumber;
            }
            if (!string.atEnd(position + 1) &&
                (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
            {   //  Exponent starts here
                break;
            }
            //
            Char c = string.charAt(position);
            if (parseProperties.getAllowSeparator() && c == _separatorChar)
            {
                position++;
                charactersProcessed++;
            }
            else
            {
                UInt32 digit;
                if (c >= '0' && c <= '9')
                {
                    digit = c - '0';
                }
                else if (c >= 'a' && c <= 'z')
                {
                    digit = c - 'a' + 10;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    digit = c - 'A' + 10;
                }
                else
                {   //  Not a digit
                    break;
                }
                if (digit >= parseProperties.getRadix())
                {
                    break;
                }
                value = value * parseProperties.getRadix() + digit;
                numDigits++;
                position++;
                charactersProcessed++;
            }
        }
        //  Process fractional digits if there are any
        if (!string.atEnd(position) && string.charAt(position) == '.')
        {
            position++;
            charactersProcessed++;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfNumber;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfNumber;
                }
                if (!string.atEnd(position + 1) &&
                    (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                    (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
                {   //  Exponent starts here
                    break;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    value = value * parseProperties.getRadix() + digit;
                    numDigits++;
                    numFractionalDigirs++;
                    position++;
                    charactersProcessed++;
                }
            }
        }
        //  Process exponent if one is present
        if (!string.atEnd(position) && (string.charAt(position) == 'e' || string.charAt(position) == 'E'))
        {
            position++;
            charactersProcessed++;
            //  Process exponent sign
            if (!string.atEnd(position) && string.charAt(position) == '+')
            {
                position++;
                charactersProcessed++;
            }
            else if (!string.atEnd(position) && string.charAt(position) == '-')
            {
                exponentNegative = true;
                position++;
                charactersProcessed++;
            }
            //  Process exponent digits
            UIntSize numExponentDigits = 0;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfExponent;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfExponent;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    exponent = exponent * parseProperties.getRadix() + digit;
                    numExponentDigits++;
                    position++;
                    charactersProcessed++;
                }
            }
            //
EndOfExponent:
            if (numExponentDigits == 0)
            {
                throw ParseException(string.toString(), position);
            }
        }
EndOfNumber:
        if (numDigits == 0)
        {
            throw ParseException(string.toString(), position);
        }
        for (; numFractionalDigirs != 0; numFractionalDigirs--)
        {
            value /= parseProperties.getRadix();
        }
        if (exponentNegative)
        {
            for (; exponent != 0; exponent--)
            {
                value /= parseProperties.getRadix();
            }
        }
        else
        {
            for (; exponent != 0; exponent--)
            {
                value *= parseProperties.getRadix();
            }
        }
        if (negative)
        {
            value = -value;
        }
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    if (value != 0 && value / parseProperties.getRadix() == value && pOverflow != NULL)
    {
        (*pOverflow) = true;
    }
    return value;
}

BinaryFixed _impl::_DefaultNumberParse::_parseBinaryFixed(const _IParsedString & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Parse the value
    BinaryFixed value = 0;
    //  Process sign if one is specified
    bool negative = false;
    if (!string.atEnd(position) && string.charAt(position) == '+')
    {
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    else if (!string.atEnd(position) && string.charAt(position) == '-')
    {
        negative = true;
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process integer digits
    UIntSize numDigits = 0;
    UIntSize numFractionalDigirs = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            goto EndOfNumber;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            goto EndOfNumber;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {
                break;
            }
            value = value * parseProperties.getRadix() + digit;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    //  Process fractional digits if there are any
    if (!string.atEnd(position) && string.charAt(position) == '.')
    {
        position++;
        charactersProcessed++;
        for (; ; )
        {
            if (string.atEnd(position))
            {   //  End of parsed string
                goto EndOfNumber;
            }
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters
                goto EndOfNumber;
            }
            //
            Char c = string.charAt(position);
            if (parseProperties.getAllowSeparator() && c == _separatorChar)
            {
                position++;
                charactersProcessed++;
            }
            else
            {
                UInt32 digit;
                if (c >= '0' && c <= '9')
                {
                    digit = c - '0';
                }
                else if (c >= 'a' && c <= 'z')
                {
                    digit = c - 'a' + 10;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    digit = c - 'A' + 10;
                }
                else
                {   //  Not a digit
                    break;
                }
                if (digit >= parseProperties.getRadix())
                {
                    break;
                }
                value = value * parseProperties.getRadix() + digit;
                numDigits++;
                numFractionalDigirs++;
                position++;
                charactersProcessed++;
            }
        }
    }
EndOfNumber:
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    BinaryFixed divisor = 1;
    for (; numFractionalDigirs != 0; numFractionalDigirs--)
    {
        divisor *= parseProperties.getRadix();
    }
    value /= divisor;
    if (negative)
    {
        value = -value;
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return value;
}

BinaryFloat _impl::_DefaultNumberParse::_parseBinaryFloat(const _IParsedString & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Parse the value
    BinaryFloat value = 0;
    //  Process special cases
    if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
        !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'S' &&
        !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
        !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
        !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #SNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = BinaryFloat(BinaryFloat::ValueCategory_SignalingNan);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'Q' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #QNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = BinaryFloat(BinaryFloat::ValueCategory_QuietNan);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '+' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #+INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = BinaryFloat(BinaryFloat::ValueCategory_PositiveInfinity);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '-' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #-INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = BinaryFloat(BinaryFloat::ValueCategory_NegativeInfinity);
    }
    else
    {   //  A "normal" real number
        //  Process sign if one is specified
        bool negative = false;
        if (!string.atEnd(position) && string.charAt(position) == '+')
        {
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        else if (!string.atEnd(position) && string.charAt(position) == '-')
        {
            negative = true;
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        //  Process integer digits
        UIntSize numDigits = 0;
        UIntSize numFractionalDigirs = 0;
        bool exponentNegative = false;
        BinaryInteger exponent = 0;
        for (; ; )
        {
            if (string.atEnd(position))
            {   //  End of parsed string
                goto EndOfNumber;
            }
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters
                goto EndOfNumber;
            }
            if (!string.atEnd(position + 1) &&
                (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
            {   //  Exponent starts here
                break;
            }
            //
            Char c = string.charAt(position);
            if (parseProperties.getAllowSeparator() && c == _separatorChar)
            {
                position++;
                charactersProcessed++;
            }
            else
            {
                UInt32 digit;
                if (c >= '0' && c <= '9')
                {
                    digit = c - '0';
                }
                else if (c >= 'a' && c <= 'z')
                {
                    digit = c - 'a' + 10;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    digit = c - 'A' + 10;
                }
                else
                {   //  Not a digit
                    break;
                }
                if (digit >= parseProperties.getRadix())
                {
                    break;
                }
                value = value * parseProperties.getRadix() + digit;
                numDigits++;
                position++;
                charactersProcessed++;
            }
        }
        //  Process fractional digits if there are any
        if (!string.atEnd(position) && string.charAt(position) == '.')
        {
            position++;
            charactersProcessed++;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfNumber;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfNumber;
                }
                if (!string.atEnd(position + 1) &&
                    (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                    (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
                {   //  Exponent starts here
                    break;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    value = value * parseProperties.getRadix() + digit;
                    numDigits++;
                    numFractionalDigirs++;
                    position++;
                    charactersProcessed++;
                }
            }
        }
        //  Process exponent if one is present
        if (!string.atEnd(position) && (string.charAt(position) == 'e' || string.charAt(position) == 'E'))
        {
            position++;
            charactersProcessed++;
            //  Process exponent sign
            if (!string.atEnd(position) && string.charAt(position) == '+')
            {
                position++;
                charactersProcessed++;
            }
            else if (!string.atEnd(position) && string.charAt(position) == '-')
            {
                exponentNegative = true;
                position++;
                charactersProcessed++;
            }
            //  Process exponent digits
            UIntSize numExponentDigits = 0;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfExponent;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfExponent;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    exponent = exponent * parseProperties.getRadix() + digit;
                    numExponentDigits++;
                    position++;
                    charactersProcessed++;
                }
            }
            //
EndOfExponent:
            if (numExponentDigits == 0)
            {
                throw ParseException(string.toString(), position);
            }
        }
EndOfNumber:
        if (numDigits == 0)
        {
            throw ParseException(string.toString(), position);
        }
        BinaryFloat divisor = 1;
        for (; numFractionalDigirs != 0; numFractionalDigirs--)
        {
            divisor *= parseProperties.getRadix();
        }
        value /= divisor;
        if (exponentNegative)
        {
            BinaryFloat scale = 1;
            for (; exponent != 0; exponent--)
            {
                scale *= parseProperties.getRadix();
            }
            value /= scale;
        }
        else
        {
            BinaryFloat scale = 1;
            for (; exponent != 0; exponent--)
            {
                scale *= parseProperties.getRadix();
            }
            value *= scale;
        }
        if (negative)
        {
            value = -value;
        }
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return value;
}

DecimalFixed _impl::_DefaultNumberParse::_parseDecimalFixed(const _IParsedString & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Parse the value
    DecimalFixed value = 0;
    //  Process sign if one is specified
    bool negative = false;
    if (!string.atEnd(position) && string.charAt(position) == '+')
    {
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    else if (!string.atEnd(position) && string.charAt(position) == '-')
    {
        negative = true;
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Process integer digits
    UIntSize numDigits = 0;
    UIntSize numFractionalDigirs = 0;
    for (; ; )
    {
        if (string.atEnd(position))
        {   //  End of parsed string
            goto EndOfNumber;
        }
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters
            goto EndOfNumber;
        }
        //
        Char c = string.charAt(position);
        if (parseProperties.getAllowSeparator() && c == _separatorChar)
        {
            position++;
            charactersProcessed++;
        }
        else
        {
            UInt32 digit;
            if (c >= '0' && c <= '9')
            {
                digit = c - '0';
            }
            else if (c >= 'a' && c <= 'z')
            {
                digit = c - 'a' + 10;
            }
            else if (c >= 'A' && c <= 'Z')
            {
                digit = c - 'A' + 10;
            }
            else
            {   //  Not a digit
                break;
            }
            if (digit >= parseProperties.getRadix())
            {
                break;
            }
            value = value * parseProperties.getRadix() + digit;
            numDigits++;
            position++;
            charactersProcessed++;
        }
    }
    //  Process fractional digits if there are any
    if (!string.atEnd(position) && string.charAt(position) == '.')
    {
        position++;
        charactersProcessed++;
        for (; ; )
        {
            if (string.atEnd(position))
            {   //  End of parsed string
                goto EndOfNumber;
            }
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters
                goto EndOfNumber;
            }
            //
            Char c = string.charAt(position);
            if (parseProperties.getAllowSeparator() && c == _separatorChar)
            {
                position++;
                charactersProcessed++;
            }
            else
            {
                UInt32 digit;
                if (c >= '0' && c <= '9')
                {
                    digit = c - '0';
                }
                else if (c >= 'a' && c <= 'z')
                {
                    digit = c - 'a' + 10;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    digit = c - 'A' + 10;
                }
                else
                {   //  Not a digit
                    break;
                }
                if (digit >= parseProperties.getRadix())
                {
                    break;
                }
                value = value * parseProperties.getRadix() + digit;
                numDigits++;
                numFractionalDigirs++;
                position++;
                charactersProcessed++;
            }
        }
    }
EndOfNumber:
    if (numDigits == 0)
    {
        throw ParseException(string.toString(), position);
    }
    DecimalFixed divisor = 1;
    for (; numFractionalDigirs != 0; numFractionalDigirs--)
    {
        divisor *= parseProperties.getRadix();
    }
    value /= divisor;
    if (negative)
    {
        value = -value;
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return value;
}

DecimalFloat _impl::_DefaultNumberParse::_parseDecimalFloat(const _IParsedString & string, UIntSize & position, const RealParseProperties & parseProperties) const throw(ParseException)
{
    //  Prepare to parse
    UIntSize charactersProcessed = 0;
    UIntSize minimumCharactersToProcess = parseProperties.getMinimumWidth();
    UIntSize maximumCharactersToProcess = parseProperties.getMaximumWidth();
    //  Skip initial spaces
    while (!string.atEnd(position) && Iso10646::isSpace(string.charAt(position)))
    {   //  The current character is a space
        position++;
        charactersProcessed++;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
    }
    //  Parse the value
    DecimalFloat value = 0;
    //  Process special cases
    if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
        !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'S' &&
        !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
        !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
        !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #SNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = DecimalFloat(DecimalFloat::ValueCategory_SignalingNan);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == 'Q' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'N' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'A' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'N')
    {   //  #QNAN
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = DecimalFloat(DecimalFloat::ValueCategory_QuietNan);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '+' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #+INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = DecimalFloat(DecimalFloat::ValueCategory_PositiveInfinity);
    }
    else if (!string.atEnd(position) && Iso10646::toUpper(string.charAt(position)) == '#' &&
             !string.atEnd(position + 1) && Iso10646::toUpper(string.charAt(position + 1)) == '-' &&
             !string.atEnd(position + 2) && Iso10646::toUpper(string.charAt(position + 2)) == 'I' &&
             !string.atEnd(position + 3) && Iso10646::toUpper(string.charAt(position + 3)) == 'N' &&
             !string.atEnd(position + 4) && Iso10646::toUpper(string.charAt(position + 4)) == 'F')
    {   //  #-INF
        position += 5;
        charactersProcessed += 5;
        if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
        {   //  Processed the maximum allowed number of characters and still no number found
            throw ParseException(string.toString(), position);
        }
        value = DecimalFloat(DecimalFloat::ValueCategory_NegativeInfinity);
    }
    else
    {   //  A "normal" real number
        //  Process sign if one is specified
        bool negative = false;
        if (!string.atEnd(position) && string.charAt(position) == '+')
        {
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        else if (!string.atEnd(position) && string.charAt(position) == '-')
        {
            negative = true;
            position++;
            charactersProcessed++;
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters and still no number found
                throw ParseException(string.toString(), position);
            }
        }
        //  Process integer digits
        UIntSize numDigits = 0;
        UIntSize numFractionalDigirs = 0;
        bool exponentNegative = false;
        DecimalInteger exponent = 0;
        for (; ; )
        {
            if (string.atEnd(position))
            {   //  End of parsed string
                goto EndOfNumber;
            }
            if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
            {   //  Processed the maximum allowed number of characters
                goto EndOfNumber;
            }
            if (!string.atEnd(position + 1) &&
                (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
            {   //  Exponent starts here
                break;
            }
            //
            Char c = string.charAt(position);
            if (parseProperties.getAllowSeparator() && c == _separatorChar)
            {
                position++;
                charactersProcessed++;
            }
            else
            {
                UInt32 digit;
                if (c >= '0' && c <= '9')
                {
                    digit = c - '0';
                }
                else if (c >= 'a' && c <= 'z')
                {
                    digit = c - 'a' + 10;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    digit = c - 'A' + 10;
                }
                else
                {   //  Not a digit
                    break;
                }
                if (digit >= parseProperties.getRadix())
                {
                    break;
                }
                value = value * parseProperties.getRadix() + digit;
                numDigits++;
                position++;
                charactersProcessed++;
            }
        }
        //  Process fractional digits if there are any
        if (!string.atEnd(position) && string.charAt(position) == '.')
        {
            position++;
            charactersProcessed++;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfNumber;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfNumber;
                }
                if (!string.atEnd(position + 1) &&
                    (string.charAt(position) == 'e' || string.charAt(position) == 'E') &&
                    (string.charAt(position + 1) == '+' || string.charAt(position + 1) == '-'))
                {   //  Exponent starts here
                    break;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    value = value * parseProperties.getRadix() + digit;
                    numDigits++;
                    numFractionalDigirs++;
                    position++;
                    charactersProcessed++;
                }
            }
        }
        //  Process exponent if one is present
        if (!string.atEnd(position) && (string.charAt(position) == 'e' || string.charAt(position) == 'E'))
        {
            position++;
            charactersProcessed++;
            //  Process exponent sign
            if (!string.atEnd(position) && string.charAt(position) == '+')
            {
                position++;
                charactersProcessed++;
            }
            else if (!string.atEnd(position) && string.charAt(position) == '-')
            {
                exponentNegative = true;
                position++;
                charactersProcessed++;
            }
            //  Process exponent digits
            UIntSize numExponentDigits = 0;
            for (; ; )
            {
                if (string.atEnd(position))
                {   //  End of parsed string
                    goto EndOfExponent;
                }
                if (maximumCharactersToProcess != 0 && charactersProcessed >= maximumCharactersToProcess)
                {   //  Processed the maximum allowed number of characters
                    goto EndOfExponent;
                }
                //
                Char c = string.charAt(position);
                if (parseProperties.getAllowSeparator() && c == _separatorChar)
                {
                    position++;
                    charactersProcessed++;
                }
                else
                {
                    UInt32 digit;
                    if (c >= '0' && c <= '9')
                    {
                        digit = c - '0';
                    }
                    else if (c >= 'a' && c <= 'z')
                    {
                        digit = c - 'a' + 10;
                    }
                    else if (c >= 'A' && c <= 'Z')
                    {
                        digit = c - 'A' + 10;
                    }
                    else
                    {   //  Not a digit
                        break;
                    }
                    if (digit >= parseProperties.getRadix())
                    {
                        break;
                    }
                    exponent = exponent * parseProperties.getRadix() + digit;
                    numExponentDigits++;
                    position++;
                    charactersProcessed++;
                }
            }
            //
EndOfExponent:
            if (numExponentDigits == 0)
            {
                throw ParseException(string.toString(), position);
            }
        }
EndOfNumber:
        if (numDigits == 0)
        {
            throw ParseException(string.toString(), position);
        }
        DecimalFloat divisor = 1;
        for (; numFractionalDigirs != 0; numFractionalDigirs--)
        {
            divisor *= parseProperties.getRadix();
        }
        value /= divisor;
        if (exponentNegative)
        {
            DecimalFloat scale = 1;
            for (; exponent != 0; exponent--)
            {
                scale *= parseProperties.getRadix();
            }
            value /= scale;
        }
        else
        {
            DecimalFloat scale = 1;
            for (; exponent != 0; exponent--)
            {
                scale *= parseProperties.getRadix();
            }
            value *= scale;
        }
        if (negative)
        {
            value = -value;
        }
    }
    //  If the minimum required number of characters has not yet been processed, the
    //  rest of them must all be spaces
    if (minimumCharactersToProcess != 0 && charactersProcessed < minimumCharactersToProcess)
    {
        while (charactersProcessed < minimumCharactersToProcess)
        {
            if (string.atEnd(position))
            {
                throw ParseException(string.toString(), position);
            }
            if (!Iso10646::isSpace(string.charAt(position)))
            {
                throw ParseException(string.toString(), position);
            }
            position++;
            charactersProcessed++;
        }
    }
    //  Done
    return value;
}

//  End of DefaultNumberParse.cpp
