//
//  DefaultTimeFormat.cpp - Implementation of the carl::_impl::DefaultTimeFormat class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultTimeFormat::_DefaultTimeFormat(const Locale & forLocale)
    :   _DefaultDateTimeFormatImpl(forLocale)
{
    _pNumberFormat = NumberFormat::getInstance(forLocale);
}

_impl::_DefaultTimeFormat::~_DefaultTimeFormat()
{
}

//////////
//  Operations
String _impl::_DefaultTimeFormat::format(const Time & time, TimeFormat::Precision precision) const
{
    String format;
    switch (precision)
    {
        case TimeFormat::Precision_Minute:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMinutes, _forLocale);
            break;
        case TimeFormat::Precision_Second:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringSeconds, _forLocale);
            break;
        case TimeFormat::Precision_Millisecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMilliseconds, _forLocale);
            break;
        case TimeFormat::Precision_Microsecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMicroseconds, _forLocale);
            break;
        case TimeFormat::Precision_Nanosecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringNanoseconds, _forLocale);
            break;
        default:
            require(false);
    }
    //  Perform the formatting
    return _DefaultTimeFormat::format(time, format);
}

String _impl::_DefaultTimeFormat::format(const Time & time, const String & format) const
{
    //  Perform the formatting
    String result;
    if (!_formatDateTime(result, format,
                         false, 0,
                         false, 0,
                         false, 0,
                         false, DayOfWeek_Monday,
                         true, time.getHour(),
                         true, time.getMinute(),
                         true, time.getSecond(),
                         true, time.getNanosecond()))
    {   //  Failure to format means invalid format string
        require(false);
    }
    return result;
}

//  End of DefaultTimeFormat.cpp
