//
//  DefaultTimeParse.cpp - Implementation of the carl::_impl::_DefaultTimeParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Construction/destruction
_impl::_DefaultTimeParse::_DefaultTimeParse(const Locale & forLocale)
    :   _DefaultDateTimeParseImpl(forLocale)
{
    _pNumberParse = NumberParse::getInstance(forLocale);
}

_impl::_DefaultTimeParse::~_DefaultTimeParse()
{
}

//////////
//  Operations
Time _impl::_DefaultTimeParse::parse(const char * pTimeString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pTimeString), position);
}

Time _impl::_DefaultTimeParse::parse(const char * pTimeString, UIntSize & position, Precision precision) const throw(ParseException)
{
    return parse(String(pTimeString), position, precision);
}

Time _impl::_DefaultTimeParse::parse(const char * pTimeString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pTimeString), position, format);
}

Time _impl::_DefaultTimeParse::parse(const Char * pTimeString, UIntSize & position) const throw(ParseException)
{
    return parse(String(pTimeString), position);
}

Time _impl::_DefaultTimeParse::parse(const Char * pTimeString, UIntSize & position, Precision precision) const throw(ParseException)
{
    return parse(String(pTimeString), position, precision);
}

Time _impl::_DefaultTimeParse::parse(const Char * pTimeString, UIntSize & position, const String & format) const throw(ParseException)
{
    return parse(String(pTimeString), position, format);
}

Time _impl::_DefaultTimeParse::parse(const String & timeString, UIntSize & position) const throw(ParseException)
{
    UIntSize savedTimeStringScan = position;

    //  Try all precisions
    for (int i = TimeParse::Precision_Nanosecond; i >= TimeParse::Precision_Minute; i--)
    {
        try
        {
            return parse(timeString, position, (TimeParse::Precision)i);
        }
        catch (ParseException)
        {
            position = savedTimeStringScan;
        }
    }
    throw ParseException(timeString, position);
}

Time _impl::_DefaultTimeParse::parse(const String & timeString, UIntSize & position, Precision precision) const throw(ParseException)
{
    String format;
    switch (precision)
    {
        case TimeParse::Precision_Minute:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMinutes, _forLocale);
            break;
        case TimeParse::Precision_Second:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringSeconds, _forLocale);
            break;
        case TimeParse::Precision_Millisecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMilliseconds, _forLocale);
            break;
        case TimeParse::Precision_Microsecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringMicroseconds, _forLocale);
            break;
        case TimeParse::Precision_Nanosecond:
            format = _DateTimeFormatManager::getInstance()->getResourceFactory()->getString(_DateTimeFormatManager::TimeFormatStringNanoseconds, _forLocale);
            break;
        default:
            require(false);
    }
    return parse(timeString, position, format);
}

Time _impl::_DefaultTimeParse::parse(const String & timeString, UIntSize & position, const String & format) const throw(ParseException)
{
    bool yearKnown = false;
    int year = 0;
    bool monthKnown = false; 
    int month = 0;
    bool dayKnown = false; 
    int day = 0;
    bool dayOfWeekKnown = false; 
    DayOfWeek dayOfWeek = DayOfWeek_Monday;
    bool hourKnown = false; 
    int hour = 0;
    bool minuteKnown = false; 
    int minute = 0;
    bool secondKnown = false; 
    int second = 0;
    bool nanosecondKnown = false; 
    UInt32 nanosecond = 0;

    if (!_parseDateTime(timeString, position, 
                        format,
                        yearKnown, year,
                        monthKnown, month,
                        dayKnown, day,
                        dayOfWeekKnown, dayOfWeek,
                        hourKnown, hour,
                        minuteKnown, minute,
                        secondKnown, second,
                        nanosecondKnown, nanosecond))
    {
        throw ParseException(timeString, position);
    }
    if (yearKnown || monthKnown || dayKnown || dayOfWeekKnown || !hourKnown || !minuteKnown)
    {
        throw ParseException(timeString, position);
    }
    Time result(hour, minute, second, nanosecond);
    return result;
}

//  End of DefaultTimeParse.cpp
