//
//  EmulatedSettingsNode.cpp - Implementation of the carl::_impl::_EmulatedSettingsNode class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EmulatedSettingsImpl.hpp"

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
_impl::_EmulatedSettingsNode::_EmulatedSettingsNode(_EmulatedSettingsNode * pParent, const String & name)
    :   _pParent(pParent),
        _name(name)
{
}

_impl::_EmulatedSettingsNode::~_EmulatedSettingsNode()
{
    for (UIntSize i = 0; i < _childNodes.size(); i++)
    {
        delete _childNodes[i];
    }
    for (UIntSize i = 0; i < _values.size(); i++)
    {
        delete _values[i];
    }
}

//////////
//  Operations
_impl::_EmulatedSettingsNode * _impl::_EmulatedSettingsNode::getParent() const
{
    return _pParent;
}

String _impl::_EmulatedSettingsNode::getName() const
{
    return _name;
}

String _impl::_EmulatedSettingsNode::getPath() const
{
    if (_pParent == NULL)
    {
        return String('/');
    }
    else
    {
        return _pParent->getPath() + '/' + _name;
    }
}

UIntSize _impl::_EmulatedSettingsNode::getChildNodeCount() const
{
    return _childNodes.size();
}

_impl::_EmulatedSettingsNode * _impl::_EmulatedSettingsNode::getChildNodeAt(UIntSize index) const
{
    return _childNodes[index];
}

_impl::_EmulatedSettingsNode * _impl::_EmulatedSettingsNode::findChildNode(const String & name) const
{
    for (UIntSize i = 0; i < _childNodes.size(); i++)
    {
        if (_childNodes[i]->getName() == name)
        {
            return _childNodes[i];
        }
    }
    return NULL;
}

_impl::_EmulatedSettingsNode * _impl::_EmulatedSettingsNode::createChildNode(const String & name)
{
    require (findChildNode(name) == NULL);

    _EmulatedSettingsNode * pNode = new _EmulatedSettingsNode(this, name);
    _childNodes.append(pNode);
    return pNode;
}

void _impl::_EmulatedSettingsNode::destroyChildNode(const String & name)
{
    _EmulatedSettingsNode * pNode = findChildNode(name);
    require(pNode != NULL);

    _childNodes.removeFirst(pNode);
}

UIntSize _impl::_EmulatedSettingsNode::getValueCount() const
{
    return _values.size();
}

_impl::_EmulatedSettingsValue * _impl::_EmulatedSettingsNode::getValueAt(UIntSize index) const
{
    return _values[index];
}

_impl::_EmulatedSettingsValue * _impl::_EmulatedSettingsNode::findValue(const String & name) const
{
    for (UIntSize i = 0; i < _values.size(); i++)
    {
        if (_values[i]->getName() == name)
        {
            return _values[i];
        }
    }
    return NULL;
}

_impl::_EmulatedSettingsValue * _impl::_EmulatedSettingsNode::createValue(const String & name, const String & value)
{
    require (findValue(name) == NULL);

    _EmulatedSettingsValue * pValue = new _EmulatedSettingsValue(name, value);
    _values.append(pValue);
    return pValue;
}

void _impl::_EmulatedSettingsNode::destroyValue(const String & name)
{
    _EmulatedSettingsValue * pValue = findValue(name);
    require(pValue != NULL);

    _values.removeFirst(pValue);
}

void _impl::_EmulatedSettingsNode::write(IDataOutput & dos) const throw(IOException)
{
    dos << _name;
    ULeb128::writeUIntSize(dos, _childNodes.size());
    for (UIntSize i = 0; i < _childNodes.size(); i++)
    {
        _childNodes[i]->write(dos);
    }
    ULeb128::writeUIntSize(dos, _values.size());
    for (UIntSize i = 0; i < _values.size(); i++)
    {
        _values[i]->write(dos);
    }
}

void _impl::_EmulatedSettingsNode::read(IDataInput & dis) throw(IOException)
{
    dis >> _name;
    UIntSize childNodeCount = ULeb128::readUIntSize(dis);
    while (_childNodes.size() < childNodeCount)
    {
        _childNodes.append(new _EmulatedSettingsNode(this, String()));
    }
    while (_childNodes.size() > childNodeCount)
    {
        delete _childNodes[_childNodes.size() - 1];
        _childNodes.setSize(_childNodes.size() - 1);
    }
    for (UIntSize i = 0; i < _childNodes.size(); i++)
    {
        _childNodes[i]->read(dis);
    }
    UIntSize valueCount = ULeb128::readUIntSize(dis);
    while (_values.size() < valueCount)
    {
        _values.append(new _EmulatedSettingsValue(String(), String()));
    }
    while (_values.size() > valueCount)
    {
        delete _values[_values.size() - 1];
        _values.setSize(_values.size() - 1);
    }
    for (UIntSize i = 0; i < _values.size(); i++)
    {
        _values[i]->read(dis);
    }
}

#else
    #error Unsupported platform
#endif

//  End of EmulatedSettingsNode.cpp
