//
//  EncryptionImpl.hpp - carl::_impl::_EncryptionImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/28 10:40:28 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __EncryptionImpl_hpp_included__
#define __EncryptionImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Encryption resources
        class CARL_PUBLIC _EncryptionResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _EncryptionResourceMap_en(const _EncryptionResourceMap_en &);
            void                            operator = (const _EncryptionResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _EncryptionResourceMap_en();
            virtual ~_EncryptionResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String                  getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>         _mapKeysToValues;
        };

        //////////
        //  A platform-specific implementation of an encryption manager
        class CARL_PUBLIC _EncryptionManagerImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _EncryptionManagerImpl(const _EncryptionManagerImpl &);
            void                            operator = (const _EncryptionManagerImpl &);

            //////////
            //  Singleton
        private:
            _EncryptionManagerImpl();
            ~_EncryptionManagerImpl();
        public:
            static _EncryptionManagerImpl *    getInstance();

            //////////
            //  Operations
        public:
            MapResourceFactory *            getResourceFactory() const;
            Array<Cipher*>                  getAllCiphers();
            Cipher *                        findCipher(const String & name);
            Array<StreamCipher*>            getAllStreamCiphers();
            StreamCipher *                  findStreamCipher(const String & name);
            Array<BlockCipher*>             getAllBlockCiphers();
            BlockCipher *                   findBlockCipher(const String & name);
            Array<SymmetricCipher*>         getAllSymmetricCiphers();
            SymmetricCipher *               findSymmetricCipher(const String & name);
            Array<AsymmetricCipher*>        getAllAsymmetricCiphers();
            AsymmetricCipher *              findAsymmetricCipher(const String & name);
            Array<PaddingAlgorithm*>        getAllPaddingAlgorithms();
            PaddingAlgorithm *              findPaddingAlgorithm(const String & name);
            void                            registerCipher(Cipher * pCipher) throw (InconsistentCipherException, DuplicateCipherException);
            void                            registerPaddingAlgorithm(PaddingAlgorithm * pPaddingAlgorithm) throw (InconsistentPaddingAlgorithmException, DuplicatePaddingAlgorithmException);

            //////////
            //  Implementation
        private:
            mutable FrameworkResourceFactory    _resourceFactory;
            Array<Cipher*>                  _ciphers;
            HashMap<String, Cipher*>        _mapNamesToCiphers;   //  Names are UPPERCASED
            Array<PaddingAlgorithm*>        _paddingAlgorithms;
            HashMap<String, PaddingAlgorithm*>  _mapNamesToPaddingAlgorithms;   //  Names are UPPERCASED
            CriticalSection                 _guard;
        };

        //////////
        //  The PKCS5 padding algorithm
        class CARL_PUBLIC _Pkcs5PaddingAlgorithm : public PaddingAlgorithm
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Pkcs5PaddingAlgorithm(const _Pkcs5PaddingAlgorithm & src);
            _Pkcs5PaddingAlgorithm &        operator = (const _Pkcs5PaddingAlgorithm & src);

            //////////
            //  Singleton
        private:
            _Pkcs5PaddingAlgorithm();
            virtual ~_Pkcs5PaddingAlgorithm();
        public:
            static _Pkcs5PaddingAlgorithm * getInstance();

            //////////
            //  Operations
        public:
            virtual Array<String>           getAllNames() const;
            virtual String                  getCanonicalName() const;
            virtual String                  getPreferredName() const;
            virtual String                  getDisplayName() const;
            virtual String                  getDisplayName(const Locale & forLocale) const;
            virtual UIntSize                getPaddingBitsRequiredForData(const void * pDataBits, UIntSize dataBitsCount, UIntSize blockSizeBits) const;
            virtual UIntSize                createPaddingBitsForData(const void * pDataBits, UIntSize dataBitsCount, UIntSize blockSizeBits, void * pPaddingBits) const;
            virtual UIntSize                getPaddingBitsStoredInData(const void * pDataBits, UIntSize dataBitsCount, UIntSize blockSizeBits) const throw(PaddedDataCorruptException);
        };

        //////////
        //  The null cipher
        class CARL_PUBLIC _NullEncryptor : public StreamEncryptor
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NullEncryptor(const _NullEncryptor & src);
            _NullEncryptor &                operator = (const _NullEncryptor & src);

            //////////
            //  Construction/destruction
        public:
            _NullEncryptor();
            virtual ~_NullEncryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual int                     encryptBit(int cleartextBit);
        };

        class CARL_PUBLIC _NullDecryptor : public StreamDecryptor
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NullDecryptor(const _NullDecryptor & src);
            _NullDecryptor &                operator = (const _NullDecryptor & src);

            //////////
            //  Construction/destruction
        public:
            _NullDecryptor();
            virtual ~_NullDecryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual int                     decryptBit(int ciphertextBit) throw(EncryptedDataCorruptException);
        };

        class CARL_PUBLIC _NullCipher : public virtual StreamCipher, public virtual SymmetricCipher
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NullCipher(const _NullCipher & src);
            _NullCipher &                   operator = (const _NullCipher & src);

            //////////
            //  Singleton
        private:
            _NullCipher();
            virtual ~_NullCipher();
        public:
            static _NullCipher *            getInstance();

            //////////
            //  SymmetricCipher
        public:
            virtual Array<UIntSize>         getSupportedKeyLenghts() const;
            virtual SecretKey               generateKey(UInt64 seed, UIntSize keyLengthBits) const;
            virtual SecretKey               generateRandomKey(UIntSize keyLengthBits) const;

            //////////
            //  Cipher
        public:
            virtual Array<String>           getAllNames() const;
            virtual String                  getCanonicalName() const;
            virtual String                  getPreferredName() const;
            virtual String                  getDisplayName() const;
            virtual String                  getDisplayName(const Locale & forLocale) const;
            virtual Encryptor *             createEncryptor(const CryptoKey & key) const throw(KeyCorruptException);
            virtual Decryptor *             createDecryptor(const CryptoKey & key) const throw(KeyCorruptException);
        };

        //////////
        //  The blowfish cipher
        class CARL_PUBLIC _BlowfishTransformer
        {
            //////////
            //  Can't assign or copy construct
        private:
            _BlowfishTransformer(const _BlowfishTransformer & src);
            _BlowfishTransformer &          operator = (const _BlowfishTransformer & src);

            //////////
            //  Construction/destruction
        protected:
            explicit _BlowfishTransformer(const CryptoKey & key);
            virtual ~_BlowfishTransformer();

            //////////
            //  Implementation
        protected:
            static const int                _N = 16;

            UInt32                          _p[_N + 2];
            UInt32                          _s[4][256];

            void                            _encrypt(UInt32 & xl, UInt32 & xr);
            void                            _decrypt(UInt32 & xl, UInt32 & xr);
            UInt32                          _F(UInt32 x);
        };

        class CARL_PUBLIC _BlowfishEncryptor : public BlockEncryptor, private _BlowfishTransformer
        {
            //////////
            //  Can't assign or copy construct
        private:
            _BlowfishEncryptor(const _BlowfishEncryptor & src);
            _BlowfishEncryptor &            operator = (const _BlowfishEncryptor & src);

            //////////
            //  Construction/destruction
        public:
            explicit _BlowfishEncryptor(const CryptoKey & key);
            virtual ~_BlowfishEncryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual UIntSize                getCiphertextSizeForCleartext(const void * pCleartextBits, UIntSize cleartextBitsCount);
            virtual UIntSize                encryptBlock(const void * pCleartextBits, UIntSize cleartextBitsCount, void * pCiphertextBits);
        };

        class CARL_PUBLIC _BlowfishDecryptor : public BlockDecryptor, private _BlowfishTransformer
        {
            //////////
            //  Can't assign or copy construct
        private:
            _BlowfishDecryptor(const _BlowfishDecryptor & src);
            _BlowfishDecryptor &            operator = (const _BlowfishDecryptor & src);

            //////////
            //  Construction/destruction
        public:
            explicit _BlowfishDecryptor(const CryptoKey & key);
            virtual ~_BlowfishDecryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual UIntSize                getCleartextSizeForCiphertext(const void * pCiphertextBits, UIntSize ciphertextBitsCount);
            virtual UIntSize                decryptBlock(const void * pCiphertextBits, UIntSize ciphertextBitsCount, void * pCleartextBits) throw(EncryptedDataCorruptException);
        };

        class CARL_PUBLIC _BlowfishCipher : public virtual BlockCipher, public virtual SymmetricCipher
        {
            //////////
            //  Can't assign or copy construct
        private:
            _BlowfishCipher(const _BlowfishCipher & src);
            _BlowfishCipher &                   operator = (const _BlowfishCipher & src);

            //////////
            //  Singleton
        private:
            _BlowfishCipher();
            virtual ~_BlowfishCipher();
        public:
            static _BlowfishCipher *        getInstance();

            //////////
            //  BlockCipher
        public:
            virtual Array<UIntSize>         getSupportedBlockLenghts() const;

            //////////
            //  SymmetricCipher
        public:
            virtual Array<UIntSize>         getSupportedKeyLenghts() const;
            virtual SecretKey               generateKey(UInt64 seed, UIntSize keyLengthBits) const;
            virtual SecretKey               generateRandomKey(UIntSize keyLengthBits) const;

            //////////
            //  Cipher
        public:
            virtual Array<String>           getAllNames() const;
            virtual String                  getCanonicalName() const;
            virtual String                  getPreferredName() const;
            virtual String                  getDisplayName() const;
            virtual String                  getDisplayName(const Locale & forLocale) const;
            virtual Encryptor *             createEncryptor(const CryptoKey & key) const throw(KeyCorruptException);
            virtual Decryptor *             createDecryptor(const CryptoKey & key) const throw(KeyCorruptException);
        };

        //////////
        //  The RSA cipher
        class CARL_PUBLIC _RsaEncryptor : public BlockEncryptor
        {
            //////////
            //  Can't assign or copy construct
        private:
            _RsaEncryptor(const _RsaEncryptor & src);
            _RsaEncryptor &            operator = (const _RsaEncryptor & src);

            //////////
            //  Construction/destruction
        public:
            _RsaEncryptor(const BinaryInteger & modulus, const BinaryInteger & exponent, UIntSize cleartextChunkSize);
            virtual ~_RsaEncryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual UIntSize                getCiphertextSizeForCleartext(const void * pCleartextBits, UIntSize cleartextBitsCount);
            virtual UIntSize                encryptBlock(const void * pCleartextBits, UIntSize cleartextBitsCount, void * pCiphertextBits);

            //////////
            //  Implementation
        private:
            BinaryInteger                   _modulus;
            BinaryInteger                   _exponent;
            UIntSize                        _cleartextChunkSize;    //  in bytes; a single chunk is encrypted as a single number

            //  Helpers
            Array<UInt8>                    _encrypt(const UInt8 * pCleartextBytes, UIntSize cleartextBytesCount);
        };

        class CARL_PUBLIC _RsaDecryptor : public BlockDecryptor
        {
            //////////
            //  Can't assign or copy construct
        private:
            _RsaDecryptor(const _RsaDecryptor & src);
            _RsaDecryptor &            operator = (const _RsaDecryptor & src);

            //////////
            //  Construction/destruction
        public:
            _RsaDecryptor(const BinaryInteger & modulus, const BinaryInteger & exponent, UIntSize cleartextChunkSize);
            virtual ~_RsaDecryptor();

            //////////
            //  Operations
        public:
            virtual Cipher *                getCipher() const;
            virtual UIntSize                getCleartextSizeForCiphertext(const void * pCiphertextBits, UIntSize ciphertextBitsCount);
            virtual UIntSize                decryptBlock(const void * pCiphertextBits, UIntSize ciphertextBitsCount, void * pCleartextBits) throw(EncryptedDataCorruptException);

            //////////
            //  Implementation
        private:
            BinaryInteger                   _modulus;
            BinaryInteger                   _exponent;
            UIntSize                        _cleartextChunkSize;    //  in bytes; a single chunk is decrypted as a single number

            //  Helpers
            Array<UInt8>                    _decrypt(const void * pCiphertextBytes, UIntSize ciphertextBytesCount)  throw(EncryptedDataCorruptException);
        };

        class CARL_PUBLIC _RsaCipher : public virtual BlockCipher, public virtual AsymmetricCipher
        {
            //////////
            //  Can't assign or copy construct
        private:
            _RsaCipher(const _RsaCipher & src);
            _RsaCipher &                   operator = (const _RsaCipher & src);

            //////////
            //  Singleton
        private:
            _RsaCipher();
            virtual ~_RsaCipher();
        public:
            static _RsaCipher *        getInstance();

            //////////
            //  BlockCipher
        public:
            virtual Array<UIntSize>         getSupportedBlockLenghts() const;

            //////////
            //  AsymmetricCipher
        public:
            virtual Array<Pair<UIntSize,UIntSize> > getSupportedKeyPairLenghts() const;
            virtual KeyPair                 generateKeyPair(UInt64 seed, UIntSize publicKeyLengthBits, UIntSize privateKeyLengthBits) const;
            virtual KeyPair                 generateRandomKeyPair(UIntSize publicKeyLengthBits, UIntSize privateKeyLengthBits) const;

            //////////
            //  Cipher
        public:
            virtual Array<String>           getAllNames() const;
            virtual String                  getCanonicalName() const;
            virtual String                  getPreferredName() const;
            virtual String                  getDisplayName() const;
            virtual String                  getDisplayName(const Locale & forLocale) const;
            virtual Encryptor *             createEncryptor(const CryptoKey & key) const throw(KeyCorruptException);
            virtual Decryptor *             createDecryptor(const CryptoKey & key) const throw(KeyCorruptException);

            //////////
            //  Helpers
        private:
            static Array<UInt8>             _assembleKeyBits(const BinaryInteger & modulus, const BinaryInteger & exponent, UIntSize keyLength);
            static bool                     _disassembleKeyBits(const UInt8 * pKeyBytes, UIntSize keyBytesCount, BinaryInteger & modulus, BinaryInteger & exponent, UIntSize & keyLength);
        };
    }
}

#endif  //  __EncryptionImpl_hpp_included__
//  End of EncryptionImpl.hpp
