//
//  Environment.cpp - Implementation of the carl::Environment class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Operations
Array<String> Environment::getAllVariableNames() const
{
    Array<String> result;

    for (Iterator<String> it = _mapNamesToValues.enumerateKeys(); !it.finished(); it.advance())
    {
        result.append(it.item());
    }
    return result;
}

bool Environment::hasVariable(const char * pName) const
{
    return hasVariable(String(pName));
}

bool Environment::hasVariable(const Char * pName) const
{
    return hasVariable(String(pName));
}

bool Environment::hasVariable(const String & name) const
{
    if (CaseSensitiveVariableNames)
    {
        return _mapNamesToValues.containsKey(name.upperCased());
    }
    else
    {
        return _mapNamesToValues.containsKey(name);
    }
}

String Environment::getVariableValue(const char * pName, const char * pDefaultValue) const
{
    return getVariableValue(String(pName), String(pDefaultValue));
}

String Environment::getVariableValue(const char * pName, const Char * pDefaultValue) const
{
    return getVariableValue(String(pName), String(pDefaultValue));
}

String Environment::getVariableValue(const char * pName, const String & defaultValue) const
{
    return getVariableValue(String(pName), defaultValue);
}

String Environment::getVariableValue(const Char * pName, const char * pDefaultValue) const
{
    return getVariableValue(String(pName), String(pDefaultValue));
}

String Environment::getVariableValue(const Char * pName, const Char * pDefaultValue) const
{
    return getVariableValue(String(pName), String(pDefaultValue));
}

String Environment::getVariableValue(const Char * pName, const String & defaultValue) const
{
    return getVariableValue(String(pName), defaultValue);
}

String Environment::getVariableValue(const String & name, const char * pDefaultValue) const
{
    return getVariableValue(name, String(pDefaultValue));
}

String Environment::getVariableValue(const String & name, const Char * pDefaultValue) const
{
    return getVariableValue(name, String(pDefaultValue));
}

String Environment::getVariableValue(const String & name, const String & defaultValue) const
{
    if (CaseSensitiveVariableNames)
    {
        if (_mapNamesToValues.containsKey(name.upperCased()))
        {
            return _mapNamesToValues.get(name.upperCased());
        }
        return defaultValue;
    }
    else
    {
        if (_mapNamesToValues.containsKey(name))
        {
            return _mapNamesToValues.get(name);
        }
        return defaultValue;
    }
}

void Environment::setVariableValue(const char * pName, const char * pValue)
{
    setVariableValue(String(pName), String(pValue));
}

void Environment::setVariableValue(const char * pName, const Char * pValue)
{
    setVariableValue(String(pName), String(pValue));
}

void Environment::setVariableValue(const char * pName, const String & value)
{
    setVariableValue(String(pName), value);
}

void Environment::setVariableValue(const Char * pName, const char * pValue)
{
    setVariableValue(String(pName), String(pValue));
}

void Environment::setVariableValue(const Char * pName, const Char * pValue)
{
    setVariableValue(String(pName), String(pValue));
}

void Environment::setVariableValue(const Char * pName, const String & value)
{
    setVariableValue(String(pName), value);
}

void Environment::setVariableValue(const String & name, const char * pValue)
{
    setVariableValue(name, String(pValue));
}

void Environment::setVariableValue(const String & name, const Char * pValue)
{
    setVariableValue(name, String(pValue));
}

void Environment::setVariableValue(const String & name, const String & value)
{
    if (CaseSensitiveVariableNames)
    {
        _mapNamesToValues.insert(name.upperCased(), value);
    }
    else
    {
        _mapNamesToValues.insert(name, value);
    }
}

void Environment::removeVariable(const char * pName)
{
    removeVariable(String(pName));
}

void Environment::removeVariable(const Char * pName)
{
    removeVariable(String(pName));
}

void Environment::removeVariable(const String & name)
{
    if (CaseSensitiveVariableNames)
    {
        _mapNamesToValues.removeKey(name.upperCased());
    }
    else
    {
        _mapNamesToValues.removeKey(name);
    }
}

//  End of Environment.cpp
