//
//  FrameworkResourceFactory.cpp - Implementation of the carl::FrameworkResourceFactory class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:52 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
FrameworkResourceFactory::FrameworkResourceFactory(ResourceMap * pDefaultResourceMap, const String & resourceFileNameBase)
    :   FileResourceFactory(pDefaultResourceMap, resourceFileNameBase, carl::String(".res"), _determineResourceSearchPaths())
{
}

//////////
//  Implementation
Array<String> FrameworkResourceFactory::_determineResourceSearchPaths()
{
    Array<String> result;

    //  Look in directories mentioned by CARL_1_0_RESOURCES environment variable
    Environment * pEnvironment = Process::getCurrentProcess()->getEnvironment();
    if (pEnvironment->hasVariable("CARL_1_0_RESOURCES"))
    {
        String searchPathList = pEnvironment->getVariableValue("CARL_1_0_RESOURCES", "");
        while (searchPathList.length() != 0)
        {
            UIntSize separatorPosition;
            String path;
            if (searchPathList.findFirst(FileSystem::PathListSeparator, separatorPosition))
            {
                path = searchPathList.left(separatorPosition);
                searchPathList = searchPathList.mid(separatorPosition + 1);
            }
            else
            {
                path = searchPathList;
                searchPathList.setLength(0);
            }
            path.trim();
            if (path.length() != 0)
            {
                result.append(path);
            }
        }
    }
    //  Also look in startup directory and under "/resources" there
    String startupDirectory = PathName(Process::getCurrentProcess()->getApplicationFileName()).getDirectory();
    result.append(startupDirectory);
    result.append(PathName(startupDirectory, "resources").toString());
    //  Done
    return result;
}

//  End of FrameworkResourceFactory.cpp
