//
//  IntegerParseProperties.cpp - Implementation of the carl::IntegerParseProperties class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
IntegerParseProperties::IntegerParseProperties(const IntegerParseProperties & src)
{
    _radix = src._radix;
    _minimumWidth = src._minimumWidth;
    _maximumWidth = src._maximumWidth;
    _allowPrefix = src._allowPrefix;
    _allowSeparator = src._allowSeparator;
}

IntegerParseProperties & IntegerParseProperties::operator = (const IntegerParseProperties & src)
{
    if (this != (&src))
    {
        _radix = src._radix;
        _minimumWidth = src._minimumWidth;
        _maximumWidth = src._maximumWidth;
        _allowPrefix = src._allowPrefix;
        _allowSeparator = src._allowSeparator;
    }
    return *this;
}

//////////
//  Operations
IntegerParseProperties & IntegerParseProperties::setRadix(UIntSize radix)
{
    require(radix >= 2 && radix <= 36);

    _radix = radix;
    return *this;
}

IntegerParseProperties & IntegerParseProperties::setMinimumWidth(UIntSize width)
{
    _minimumWidth = width;
    return *this;
}

IntegerParseProperties & IntegerParseProperties::setMaximumWidth(UIntSize width)
{
    _maximumWidth = width;
    return *this;
}

IntegerParseProperties & IntegerParseProperties::setAllowPrefix(bool allowPrefix)
{
    _allowPrefix = allowPrefix;
    return *this;
}

IntegerParseProperties & IntegerParseProperties::setAllowSeparator(bool allowSeparator)
{
    _allowSeparator = allowSeparator;
    return *this;
}

//////////
//  Implementation helpers
void IntegerParseProperties::_setStandardProperties()
{
    _radix = 10;
    _minimumWidth = 0;
    _maximumWidth = 0;
    _allowPrefix = false;
    _allowSeparator = false;
}

void IntegerParseProperties::_parseCrtFormatString(const String & crtFormatString)
{
    _setStandardProperties();
    UIntSize scan = 0;
    //  Skip optional leading '%'
    if (scan < crtFormatString.length() && crtFormatString[scan] == '%')
    {
        scan++;
    }
    //  Process width
    if (scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]))
    {
        for (; scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]); scan++)
        {
            _maximumWidth = 10 * _maximumWidth + (crtFormatString[scan] - '0');
        }
    }
    //  Process type
    require(scan < crtFormatString.length());
    switch (crtFormatString[scan])
    {
        case 'd':   case 'i':   case 'u':
            scan++;
            break;
        case 'o':
            scan++;
            _radix  = 8;
            break;
        case 'x':   case 'X':
            scan++;
            _radix  = 16;
            break;
        default:
            require(false);
    }
    if (scan != crtFormatString.length())
    {
        require(false);
    }
}

//  End of IntegerParseProperties.cpp
