//
//  Iso3166.cpp - Implementation of the carl::Iso3166 class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "Iso3166Impl.hpp"
#include <stdlib.h>
#include <ctype.h>

//////////
//  carl::Iso3166::Country
Iso3166::Country::Country(const char * pCode2, const char * pCode3, int codeN)
{
    require(pCode2 != NULL && pCode2[0] != 0 && pCode2[1] != 0 && pCode2[2] == 0);
    require(pCode3 != NULL && pCode3[0] != 0 && pCode3[1] != 0 && pCode3[2] != 0 && pCode3[3] == 0);
    require(codeN > 0 && codeN <= 999);

    _code2[0] = toupper(static_cast<unsigned char>(pCode2[0]));
    _code2[1] = toupper(static_cast<unsigned char>(pCode2[1]));
    _code3[0] = toupper(static_cast<unsigned char>(pCode3[0]));
    _code3[1] = toupper(static_cast<unsigned char>(pCode3[1]));
    _code3[2] = toupper(static_cast<unsigned char>(pCode3[2]));
    _codeN = codeN;
}

String Iso3166::Country::getDisplayName(const Locale & forLocale) const
{
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    try
    {
        return pIso3166Impl->getResourceFactory()->getString(String(_code2, 2), forLocale);
    }
    catch (MissingResourceError)
    {
        return String(_code2, 2);
    }
}

//////////
//  Operations
bool Iso3166::isValidCode2(const char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(pCode2[0], pCode2[1])) != NULL;
}

bool Iso3166::isValidCode2(const Char * pCode2)
{
    require(pCode2 != NULL);

    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(pCode2[0], pCode2[1])) != NULL;
}

bool Iso3166::isValidCode2(const String & code2)
{
    if (code2.length() != 2)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(code2[0], code2[1])) != NULL;
}

bool Iso3166::isValidCode3(const char * pCode3)
{
    require(pCode3 != NULL);

    if (pCode3[0] == 0 || pCode3[1] == 0 || pCode3[2] == 0 || pCode3[3] != 0)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(pCode3[0], pCode3[1], pCode3[2])) != NULL;
}

bool Iso3166::isValidCode3(const Char * pCode3)
{
    require(pCode3 != NULL);

    if (pCode3[0] == 0 || pCode3[1] == 0 || pCode3[2] == 0 || pCode3[3] != 0)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(pCode3[0], pCode3[1], pCode3[2])) != NULL;
}

bool Iso3166::isValidCode3(const String & code3)
{
    if (code3.length() != 3)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(code3[0], code3[1], code3[2])) != NULL;
}

bool Iso3166::isValidCodeN(int codeN)
{
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCodeN(codeN) != NULL;
}

const Iso3166::Country * Iso3166::findCountryByCode2(const char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return NULL;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(pCode2[0], pCode2[1]));
}

Array<const Iso3166::Country*> Iso3166::getAllCountries()
{
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->getAllCountries();
}

const Iso3166::Country * Iso3166::findCountryByCode2(const Char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return NULL;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(pCode2[0], pCode2[1]));
}

const Iso3166::Country * Iso3166::findCountryByCode2(const String & code2)
{
    if (code2.length() != 2)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode2(_impl::_Iso3166Code2(code2[0], code2[1]));
}

const Iso3166::Country * Iso3166::findCountryByCode3(const char * pCode3)
{
    require(pCode3 != NULL);

    if (pCode3[0] == 0 || pCode3[1] == 0 || pCode3[2] == 0 || pCode3[3] != 0)
    {
        return NULL;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(pCode3[0], pCode3[1], pCode3[2]));
}

const Iso3166::Country * Iso3166::findCountryByCode3(const Char * pCode3)
{
    require(pCode3 != NULL);

    if (pCode3[0] == 0 || pCode3[1] == 0 || pCode3[2] == 0 || pCode3[3] != 0)
    {
        return NULL;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(pCode3[0], pCode3[1], pCode3[2]));
}

const Iso3166::Country * Iso3166::findCountryByCode3(const String & code3)
{
    if (code3.length() != 3)
    {
        return false;
    }
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCode3(_impl::_Iso3166Code3(code3[0], code3[1], code3[2]));
}

const Iso3166::Country * Iso3166::findCountryByCodeN(int codeN)
{
    _impl::_Iso3166Impl * pIso3166Impl = _impl::_Iso3166Impl::getInstance();
    return pIso3166Impl->findCountryByCodeN(codeN);
}

//  End of Iso3166.cpp
