//
//  Iso3166Impl.cpp - Implementation of the carl::_impl::_Iso3166Impl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "Iso3166Impl.hpp"
#include <stdlib.h>

//////////
//  Singleton
_impl::_Iso3166Impl::_Iso3166Impl()
    :   _mapCode2ToCountry(_impl::_hashIso3166Code2),
        _mapCode3ToCountry(_impl::_hashIso3166Code3),
        _mapCodeNToCountry(StandardHashFunctions::hashSignedInt),
        _resourceFactory(new _Iso3166ResourceMap_en(), carl::String("CARL_ISO3166"))
{
    //  Set up the countries
    _registerCountry(new Iso3166::Country("AF", "AFG",   4));
    _registerCountry(new Iso3166::Country("AL", "ALB",   8));
    _registerCountry(new Iso3166::Country("DZ", "DZA",  12));
    _registerCountry(new Iso3166::Country("AS", "ASM",  16));
    _registerCountry(new Iso3166::Country("AD", "AND",  20));
    _registerCountry(new Iso3166::Country("AO", "AGO",  24));
    _registerCountry(new Iso3166::Country("AI", "AIA", 660));
    _registerCountry(new Iso3166::Country("AQ", "ATA",  10));
    _registerCountry(new Iso3166::Country("AG", "ATG",  28));
    _registerCountry(new Iso3166::Country("AR", "ARG",  32));
    _registerCountry(new Iso3166::Country("AM", "ARM",  51));
    _registerCountry(new Iso3166::Country("AW", "ABW", 533));
    _registerCountry(new Iso3166::Country("AU", "AUS",  36));
    _registerCountry(new Iso3166::Country("AT", "AUT",  40));
    _registerCountry(new Iso3166::Country("AZ", "AZE",  31));
    _registerCountry(new Iso3166::Country("BS", "BHS",  44));
    _registerCountry(new Iso3166::Country("BH", "BHR",  48));
    _registerCountry(new Iso3166::Country("BD", "BGD",  50));
    _registerCountry(new Iso3166::Country("BB", "BRB",  52));
    _registerCountry(new Iso3166::Country("BY", "BLR", 112));
    _registerCountry(new Iso3166::Country("BE", "BEL",  56));
    _registerCountry(new Iso3166::Country("BZ", "BLZ",  84));
    _registerCountry(new Iso3166::Country("BJ", "BEN", 204));
    _registerCountry(new Iso3166::Country("BM", "BMU",  60));
    _registerCountry(new Iso3166::Country("BT", "BTN",  64));
    _registerCountry(new Iso3166::Country("BO", "BOL",  68));
    _registerCountry(new Iso3166::Country("BA", "BIH",  70));
    _registerCountry(new Iso3166::Country("BW", "BWA",  72));
    _registerCountry(new Iso3166::Country("BV", "BVT",  74));
    _registerCountry(new Iso3166::Country("BR", "BRA",  76));
    _registerCountry(new Iso3166::Country("IO", "IOT",  86));
    _registerCountry(new Iso3166::Country("BN", "BRN",  96));
    _registerCountry(new Iso3166::Country("BG", "BGR", 100));
    _registerCountry(new Iso3166::Country("BF", "BFA", 854));
    _registerCountry(new Iso3166::Country("BI", "BDI", 108));
    _registerCountry(new Iso3166::Country("KH", "KHM", 116));
    _registerCountry(new Iso3166::Country("CM", "CMR", 120));
    _registerCountry(new Iso3166::Country("CA", "CAN", 124));
    _registerCountry(new Iso3166::Country("CV", "CPV", 132));
    _registerCountry(new Iso3166::Country("KY", "CYM", 136));
    _registerCountry(new Iso3166::Country("CF", "CAF", 140));
    _registerCountry(new Iso3166::Country("TD", "TCD", 148));
    _registerCountry(new Iso3166::Country("CL", "CHL", 152));
    _registerCountry(new Iso3166::Country("CN", "CHN", 156));
    _registerCountry(new Iso3166::Country("CX", "CXR", 162));
    _registerCountry(new Iso3166::Country("CC", "CCK", 166));
    _registerCountry(new Iso3166::Country("CO", "COL", 170));
    _registerCountry(new Iso3166::Country("KM", "COM", 174));
    _registerCountry(new Iso3166::Country("CG", "COG", 178));
    _registerCountry(new Iso3166::Country("CD", "COD", 180));
    _registerCountry(new Iso3166::Country("CK", "COK", 184));
    _registerCountry(new Iso3166::Country("CR", "CRI", 188));
    _registerCountry(new Iso3166::Country("CI", "CIV", 384));
    _registerCountry(new Iso3166::Country("HR", "HRV", 191));
    _registerCountry(new Iso3166::Country("CU", "CUB", 192));
    _registerCountry(new Iso3166::Country("CY", "CYP", 196));
    _registerCountry(new Iso3166::Country("CZ", "CZE", 203));
    _registerCountry(new Iso3166::Country("DK", "DNK", 208));
    _registerCountry(new Iso3166::Country("DJ", "DJI", 262));
    _registerCountry(new Iso3166::Country("DM", "DMA", 212));
    _registerCountry(new Iso3166::Country("DO", "DOM", 214));
    _registerCountry(new Iso3166::Country("TP", "TMP", 626));
    _registerCountry(new Iso3166::Country("EC", "ECU", 218));
    _registerCountry(new Iso3166::Country("EG", "EGY", 818));
    _registerCountry(new Iso3166::Country("SV", "SLV", 222));
    _registerCountry(new Iso3166::Country("GQ", "GNQ", 226));
    _registerCountry(new Iso3166::Country("ER", "ERI", 232));
    _registerCountry(new Iso3166::Country("EE", "EST", 233));
    _registerCountry(new Iso3166::Country("ET", "ETH", 231));
    _registerCountry(new Iso3166::Country("FK", "FLK", 238));
    _registerCountry(new Iso3166::Country("FO", "FRO", 234));
    _registerCountry(new Iso3166::Country("FJ", "FJI", 242));
    _registerCountry(new Iso3166::Country("FI", "FIN", 246));
    _registerCountry(new Iso3166::Country("FR", "FRA", 250));
    _registerCountry(new Iso3166::Country("FX", "FXX", 249));
    _registerCountry(new Iso3166::Country("GF", "GUF", 254));
    _registerCountry(new Iso3166::Country("PF", "PYF", 258));
    _registerCountry(new Iso3166::Country("TF", "ATF", 260));
    _registerCountry(new Iso3166::Country("GA", "GAB", 266));
    _registerCountry(new Iso3166::Country("GM", "GMB", 270));
    _registerCountry(new Iso3166::Country("GE", "GEO", 268));
    _registerCountry(new Iso3166::Country("DE", "DEU", 276));
    _registerCountry(new Iso3166::Country("GH", "GHA", 288));
    _registerCountry(new Iso3166::Country("GI", "GIB", 292));
    _registerCountry(new Iso3166::Country("GR", "GRC", 300));
    _registerCountry(new Iso3166::Country("GL", "GRL", 304));
    _registerCountry(new Iso3166::Country("GD", "GRD", 308));
    _registerCountry(new Iso3166::Country("GP", "GLP", 312));
    _registerCountry(new Iso3166::Country("GU", "GUM", 316));
    _registerCountry(new Iso3166::Country("GT", "GTM", 320));
    _registerCountry(new Iso3166::Country("GN", "GIN", 324));
    _registerCountry(new Iso3166::Country("GW", "GNB", 624));
    _registerCountry(new Iso3166::Country("GY", "GUY", 328));
    _registerCountry(new Iso3166::Country("HT", "HTI", 332));
    _registerCountry(new Iso3166::Country("HM", "HMD", 334));
    _registerCountry(new Iso3166::Country("VA", "VAT", 336));
    _registerCountry(new Iso3166::Country("HN", "HND", 340));
    _registerCountry(new Iso3166::Country("HK", "HKG", 344));
    _registerCountry(new Iso3166::Country("HU", "HUN", 348));
    _registerCountry(new Iso3166::Country("IS", "ISL", 352));
    _registerCountry(new Iso3166::Country("IN", "IND", 356));
    _registerCountry(new Iso3166::Country("ID", "IDN", 360));
    _registerCountry(new Iso3166::Country("IR", "IRN", 364));
    _registerCountry(new Iso3166::Country("IQ", "IRQ", 368));
    _registerCountry(new Iso3166::Country("IE", "IRL", 372));
    _registerCountry(new Iso3166::Country("IL", "ISR", 376));
    _registerCountry(new Iso3166::Country("IT", "ITA", 380));
    _registerCountry(new Iso3166::Country("JM", "JAM", 388));
    _registerCountry(new Iso3166::Country("JP", "JPN", 392));
    _registerCountry(new Iso3166::Country("JO", "JOR", 400));
    _registerCountry(new Iso3166::Country("KZ", "KAZ", 398));
    _registerCountry(new Iso3166::Country("KE", "KEN", 404));
    _registerCountry(new Iso3166::Country("KI", "KIR", 296));
    _registerCountry(new Iso3166::Country("KP", "PRK", 408));
    _registerCountry(new Iso3166::Country("KR", "KOR", 410));
    _registerCountry(new Iso3166::Country("KW", "KWT", 414));
    _registerCountry(new Iso3166::Country("KG", "KGZ", 417));
    _registerCountry(new Iso3166::Country("LA", "LAO", 418));
    _registerCountry(new Iso3166::Country("LV", "LVA", 428));
    _registerCountry(new Iso3166::Country("LB", "LBN", 422));
    _registerCountry(new Iso3166::Country("LS", "LSO", 426));
    _registerCountry(new Iso3166::Country("LR", "LBR", 430));
    _registerCountry(new Iso3166::Country("LY", "LBY", 434));
    _registerCountry(new Iso3166::Country("LI", "LIE", 438));
    _registerCountry(new Iso3166::Country("LT", "LTU", 440));
    _registerCountry(new Iso3166::Country("LU", "LUX", 442));
    _registerCountry(new Iso3166::Country("MO", "MAC", 446));
    _registerCountry(new Iso3166::Country("MK", "MKD", 807));
    _registerCountry(new Iso3166::Country("MG", "MDG", 450));
    _registerCountry(new Iso3166::Country("MW", "MWI", 454));
    _registerCountry(new Iso3166::Country("MY", "MYS", 458));
    _registerCountry(new Iso3166::Country("MV", "MDV", 462));
    _registerCountry(new Iso3166::Country("ML", "MLI", 466));
    _registerCountry(new Iso3166::Country("MT", "MLT", 470));
    _registerCountry(new Iso3166::Country("MH", "MHL", 584));
    _registerCountry(new Iso3166::Country("MQ", "MTQ", 474));
    _registerCountry(new Iso3166::Country("MR", "MRT", 478));
    _registerCountry(new Iso3166::Country("MU", "MUS", 480));
    _registerCountry(new Iso3166::Country("YT", "MYT", 175));
    _registerCountry(new Iso3166::Country("MX", "MEX", 484));
    _registerCountry(new Iso3166::Country("FM", "FSM", 583));
    _registerCountry(new Iso3166::Country("MD", "MDA", 498));
    _registerCountry(new Iso3166::Country("MC", "MCO", 492));
    _registerCountry(new Iso3166::Country("MN", "MNG", 496));
    _registerCountry(new Iso3166::Country("MS", "MSR", 500));
    _registerCountry(new Iso3166::Country("MA", "MAR", 504));
    _registerCountry(new Iso3166::Country("MZ", "MOZ", 508));
    _registerCountry(new Iso3166::Country("MM", "MMR", 104));
    _registerCountry(new Iso3166::Country("NA", "NAM", 516));
    _registerCountry(new Iso3166::Country("NR", "NRU", 520));
    _registerCountry(new Iso3166::Country("NP", "NPL", 524));
    _registerCountry(new Iso3166::Country("NL", "NLD", 528));
    _registerCountry(new Iso3166::Country("AN", "ANT", 530));
    _registerCountry(new Iso3166::Country("NC", "NCL", 540));
    _registerCountry(new Iso3166::Country("NZ", "NZL", 554));
    _registerCountry(new Iso3166::Country("NI", "NIC", 558));
    _registerCountry(new Iso3166::Country("NE", "NER", 562));
    _registerCountry(new Iso3166::Country("NG", "NGA", 566));
    _registerCountry(new Iso3166::Country("NU", "NIU", 570));
    _registerCountry(new Iso3166::Country("NF", "NFK", 574));
    _registerCountry(new Iso3166::Country("MP", "MNP", 580));
    _registerCountry(new Iso3166::Country("NO", "NOR", 578));
    _registerCountry(new Iso3166::Country("OM", "OMN", 512));
    _registerCountry(new Iso3166::Country("PK", "PAK", 586));
    _registerCountry(new Iso3166::Country("PW", "PLW", 585));
    _registerCountry(new Iso3166::Country("PA", "PAN", 591));
    _registerCountry(new Iso3166::Country("PG", "PNG", 598));
    _registerCountry(new Iso3166::Country("PY", "PRY", 600));
    _registerCountry(new Iso3166::Country("PE", "PER", 604));
    _registerCountry(new Iso3166::Country("PH", "PHL", 608));
    _registerCountry(new Iso3166::Country("PN", "PCN", 612));
    _registerCountry(new Iso3166::Country("PL", "POL", 616));
    _registerCountry(new Iso3166::Country("PT", "PRT", 620));
    _registerCountry(new Iso3166::Country("PR", "PRI", 630));
    _registerCountry(new Iso3166::Country("QA", "QAT", 634));
    _registerCountry(new Iso3166::Country("RE", "REU", 638));
    _registerCountry(new Iso3166::Country("RO", "ROM", 642));
    _registerCountry(new Iso3166::Country("RU", "RUS", 643));
    _registerCountry(new Iso3166::Country("RW", "RWA", 646));
    _registerCountry(new Iso3166::Country("KN", "KNA", 659));
    _registerCountry(new Iso3166::Country("LC", "LCA", 662));
    _registerCountry(new Iso3166::Country("VC", "VCT", 670));
    _registerCountry(new Iso3166::Country("WS", "WSM", 882));
    _registerCountry(new Iso3166::Country("SM", "SMR", 674));
    _registerCountry(new Iso3166::Country("ST", "STP", 678));
    _registerCountry(new Iso3166::Country("SA", "SAU", 682));
    _registerCountry(new Iso3166::Country("SN", "SEN", 686));
    _registerCountry(new Iso3166::Country("SC", "SYC", 690));
    _registerCountry(new Iso3166::Country("SL", "SLE", 694));
    _registerCountry(new Iso3166::Country("SG", "SGP", 702));
    _registerCountry(new Iso3166::Country("SK", "SVK", 703));
    _registerCountry(new Iso3166::Country("SI", "SVN", 705));
    _registerCountry(new Iso3166::Country("SB", "SLB",  90));
    _registerCountry(new Iso3166::Country("SO", "SOM", 706));
    _registerCountry(new Iso3166::Country("ZA", "ZAF", 710));
    _registerCountry(new Iso3166::Country("GS", "SGS", 239));
    _registerCountry(new Iso3166::Country("ES", "ESP", 724));
    _registerCountry(new Iso3166::Country("LK", "LKA", 144));
    _registerCountry(new Iso3166::Country("SH", "SHN", 654));
    _registerCountry(new Iso3166::Country("PM", "SPM", 666));
    _registerCountry(new Iso3166::Country("SD", "SDN", 736));
    _registerCountry(new Iso3166::Country("SR", "SUR", 740));
    _registerCountry(new Iso3166::Country("SJ", "SJM", 744));
    _registerCountry(new Iso3166::Country("SZ", "SWZ", 748));
    _registerCountry(new Iso3166::Country("SE", "SWE", 752));
    _registerCountry(new Iso3166::Country("CH", "CHE", 756));
    _registerCountry(new Iso3166::Country("SY", "SYR", 760));
    _registerCountry(new Iso3166::Country("TW", "TWN", 158));
    _registerCountry(new Iso3166::Country("TJ", "TJK", 762));
    _registerCountry(new Iso3166::Country("TZ", "TZA", 834));
    _registerCountry(new Iso3166::Country("TH", "THA", 764));
    _registerCountry(new Iso3166::Country("TG", "TGO", 768));
    _registerCountry(new Iso3166::Country("TK", "TKL", 772));
    _registerCountry(new Iso3166::Country("TO", "TON", 776));
    _registerCountry(new Iso3166::Country("TT", "TTO", 780));
    _registerCountry(new Iso3166::Country("TN", "TUN", 788));
    _registerCountry(new Iso3166::Country("TR", "TUR", 792));
    _registerCountry(new Iso3166::Country("TM", "TKM", 795));
    _registerCountry(new Iso3166::Country("TC", "TCA", 796));
    _registerCountry(new Iso3166::Country("TV", "TUV", 798));
    _registerCountry(new Iso3166::Country("UG", "UGA", 800));
    _registerCountry(new Iso3166::Country("UA", "UKR", 804));
    _registerCountry(new Iso3166::Country("AE", "ARE", 784));
    _registerCountry(new Iso3166::Country("GB", "GBR", 826));
    _registerCountry(new Iso3166::Country("US", "USA", 840));
    _registerCountry(new Iso3166::Country("UM", "UMI", 581));
    _registerCountry(new Iso3166::Country("UY", "URY", 858));
    _registerCountry(new Iso3166::Country("UZ", "UZB", 860));
    _registerCountry(new Iso3166::Country("VU", "VUT", 548));
    _registerCountry(new Iso3166::Country("VE", "VEN", 862));
    _registerCountry(new Iso3166::Country("VN", "VNM", 704));
    _registerCountry(new Iso3166::Country("VG", "VGB",  92));
    _registerCountry(new Iso3166::Country("VI", "VIR", 850));
    _registerCountry(new Iso3166::Country("WF", "WLF", 876));
    _registerCountry(new Iso3166::Country("EH", "ESH", 732));
    _registerCountry(new Iso3166::Country("YE", "YEM", 887));
    _registerCountry(new Iso3166::Country("YU", "YUG", 891));
    _registerCountry(new Iso3166::Country("ZM", "ZMB", 894));
    _registerCountry(new Iso3166::Country("ZW", "ZWE", 716));
}

_impl::_Iso3166Impl::~_Iso3166Impl()
{
    for (UIntSize i = 0; i < _countries.size(); i++)
    {
        delete _countries[i];
    }
}

_impl::_Iso3166Impl * _impl::_Iso3166Impl::getInstance()
{
    static _Iso3166Impl instance;
    return &instance;
}

//////////
//  Operations
Array<const Iso3166::Country*> _impl::_Iso3166Impl::getAllCountries() const
{
    return _countries;
}

const Iso3166::Country * _impl::_Iso3166Impl::findCountryByCode2(const _Iso3166Code2 & code2) const
{
    if (_mapCode2ToCountry.containsKey(code2))
    {
        return _mapCode2ToCountry.get(code2);
    }
    else
    {
        return NULL;
    }
}

const Iso3166::Country * _impl::_Iso3166Impl::findCountryByCode3(const _Iso3166Code3 & code3) const
{
    if (_mapCode3ToCountry.containsKey(code3))
    {
        return _mapCode3ToCountry.get(code3);
    }
    else
    {
        return NULL;
    }
}

const Iso3166::Country * _impl::_Iso3166Impl::findCountryByCodeN(int codeN) const
{
    if (_mapCodeNToCountry.containsKey(codeN))
    {
        return _mapCodeNToCountry.get(codeN);
    }
    else
    {
        return NULL;
    }
}

MapResourceFactory * _impl::_Iso3166Impl::getResourceFactory() const
{
    return &_resourceFactory;
}

//////////
//  Implementation helpers
void _impl::_Iso3166Impl::_registerCountry(const Iso3166::Country * pCountry)
{
    require(pCountry != NULL);

    String code2String = pCountry ->getCode2();
    String code3String = pCountry ->getCode3();
    _Iso3166Code2 code2(code2String[0], code2String[1]);
    _Iso3166Code3 code3(code3String[0], code3String[1], code3String[2]);
    require(!_mapCode2ToCountry.containsKey(code2));
    require(!_mapCode3ToCountry.containsKey(code3));
    require(!_mapCodeNToCountry.containsKey(pCountry ->getCodeN()));
    _countries.append(pCountry);
    _mapCode2ToCountry.insert(code2, pCountry);
    _mapCode3ToCountry.insert(code3, pCountry);
    _mapCodeNToCountry.insert(pCountry ->getCodeN(), pCountry);
}

//  End of Iso3166Impl.cpp
