//
//  Iso3166Impl.hpp - ISO 3166 implementation
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __Iso3166Impl_hpp_included__
#define __Iso3166Impl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Codes used internally by implementation
        class CARL_PUBLIC _Iso3166Code2
        {
            //////////
            //  Construction/destruction/assignment
        public:
            _Iso3166Code2();
            _Iso3166Code2(Char ch1, Char ch2);
            _Iso3166Code2(const _Iso3166Code2 & src);
            ~_Iso3166Code2();

            _Iso3166Code2 &                 operator = (const _Iso3166Code2 & src);

            //////////
            //  Operations
        public:
            bool                            operator == (const _Iso3166Code2 & code2) const;
            bool                            operator != (const _Iso3166Code2 & code2) const;
            UIntSize                        hashCode() const;

            //////////
            //  Implementation
        private:
            Char                            _code2[2];
        };

        class CARL_PUBLIC _Iso3166Code3
        {
            //////////
            //  Construction/destruction/assignment
        public:
            _Iso3166Code3();
            _Iso3166Code3(Char ch1, Char ch2, Char ch3);
            _Iso3166Code3(const _Iso3166Code3 & src);
            ~_Iso3166Code3();

            _Iso3166Code3 &                 operator = (const _Iso3166Code3 & src);

            //////////
            //  Operations
        public:
            bool                            operator == (const _Iso3166Code3 & code3) const;
            bool                            operator != (const _Iso3166Code3 & code3) const;
            UIntSize                        hashCode() const;

            //////////
            //  Implementation
        private:
            Char                            _code3[3];
        };

        UIntSize _hashIso3166Code2(const _Iso3166Code2 & key);
        UIntSize _hashIso3166Code3(const _Iso3166Code3 & key);

        //////////
        //  ISO 3166 resources
        class CARL_PUBLIC _Iso3166ResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso3166ResourceMap_en(const _Iso3166ResourceMap_en &);
            void                            operator = (const _Iso3166ResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _Iso3166ResourceMap_en();
            virtual ~_Iso3166ResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String                  getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>         _mapCodes2ToCountryDisplayNames;
        };

        //////////
        //  An encapsulated ISO 3166 implementation
        class CARL_PUBLIC _Iso3166Impl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso3166Impl(const _Iso3166Impl &);
            void                            operator = (const _Iso3166Impl &);

            //////////
            //  Singleton
        private:
            _Iso3166Impl();
            ~_Iso3166Impl();
        public:
            static _Iso3166Impl *           getInstance();

            //////////
            //  Operations
        public:
            Array<const Iso3166::Country*>      getAllCountries() const;
            const Iso3166::Country *            findCountryByCode2(const _Iso3166Code2 & code2) const;
            const Iso3166::Country *            findCountryByCode3(const _Iso3166Code3 & code3) const;
            const Iso3166::Country *            findCountryByCodeN(int codeN) const;
            MapResourceFactory *                getResourceFactory() const;

            //////////
            //  Implementation
        private:
            Array<const Iso3166::Country*>      _countries;
            HashMap<_Iso3166Code2, const Iso3166::Country*> _mapCode2ToCountry;
            HashMap<_Iso3166Code3, const Iso3166::Country*> _mapCode3ToCountry;
            HashMap<int, const Iso3166::Country*>   _mapCodeNToCountry;
            mutable FrameworkResourceFactory    _resourceFactory;

            void                                _registerCountry(const Iso3166::Country * pCountry);
        };
    }
}

#endif  //  __Iso3166Impl_hpp_included__
//  End of Iso3166Impl.hpp
