//
//  Iso639.cpp - Implementation of the carl::Iso639 class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "Iso639Impl.hpp"
#include <stdlib.h>
#include <ctype.h>

//////////
//  carl::Iso639::Language
Iso639::Language::Language(const char * pCode2)
{
    require(pCode2 != NULL && pCode2[0] != 0 && pCode2[1] != 0 && pCode2[2] == 0);

    _code2[0] = tolower(static_cast<unsigned char>(pCode2[0]));
    _code2[1] = tolower(static_cast<unsigned char>(pCode2[1]));
}

String Iso639::Language::getDisplayName(const Locale & forLocale) const
{
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    try
    {
        return pIso639Impl->getResourceFactory()->getString(String(_code2, 2), forLocale);
    }
    catch (MissingResourceError)
    {
        return String(_code2, 2);
    }
}

//////////
//  Operations
bool Iso639::isValidCode2(const char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return false;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(pCode2[0], pCode2[1])) != NULL;
}

bool Iso639::isValidCode2(const Char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return false;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(pCode2[0], pCode2[1])) != NULL;
}

bool Iso639::isValidCode2(const String & code2)
{
    if (code2.length() != 2)
    {
        return false;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(code2[0], code2[1])) != NULL;
}

const Iso639::Language * Iso639::findLanguageByCode2(const char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return NULL;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(pCode2[0], pCode2[1]));
}

Array<const Iso639::Language*> Iso639::getAllLanguages()
{
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->getAllLanguages();
}

const Iso639::Language * Iso639::findLanguageByCode2(const Char * pCode2)
{
    require(pCode2 != NULL);
    
    if (pCode2[0] == 0 || pCode2[1] == 0 || pCode2[2] != 0)
    {
        return NULL;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(pCode2[0], pCode2[1]));
}

const Iso639::Language * Iso639::findLanguageByCode2(const String & code2)
{
    if (code2.length() != 2)
    {
        return NULL;
    }
    _impl::_Iso639Impl * pIso639Impl = _impl::_Iso639Impl::getInstance();
    return pIso639Impl->findLanguageByCode2(_impl::_Iso639Code2(code2[0], code2[1]));
}

//  End of Iso639.cpp
