//
//  Iso639Impl.hpp - ISO 639 implementation
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __Iso639Impl_hpp_included__
#define __Iso639Impl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Codes used internally by implementation
        class CARL_PUBLIC _Iso639Code2
        {
            //////////
            //  Construction/destruction/assignment
        public:
            _Iso639Code2();
            _Iso639Code2(Char ch1, Char ch2);
            _Iso639Code2(const _Iso639Code2 & src);
            ~_Iso639Code2();

            _Iso639Code2 &                  operator = (const _Iso639Code2 & src);

            //////////
            //  Operations
        public:
            bool                            operator == (const _Iso639Code2 & code2) const;
            bool                            operator != (const _Iso639Code2 & code2) const;
            UIntSize                        hashCode() const;

            //////////
            //  Implementation
        private:
            Char                            _code2[2];
        };

        UIntSize _hashIso639Code2(const _Iso639Code2 & key);

        //////////
        //  ISO 639 resources
        class CARL_PUBLIC _Iso639ResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso639ResourceMap_en(const _Iso639ResourceMap_en &);
            void                            operator = (const _Iso639ResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _Iso639ResourceMap_en();
            virtual ~_Iso639ResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String                  getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>         _mapCodes2ToLanguageDisplayNames;
        };

        //////////
        //  An encapsulated ISO 639 implementation
        class CARL_PUBLIC _Iso639Impl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _Iso639Impl(const _Iso639Impl &);
            void                            operator = (const _Iso639Impl &);

            //////////
            //  Singleton
        private:
            _Iso639Impl();
            ~_Iso639Impl();
        public:
            static _Iso639Impl *            getInstance();

            //////////
            //  Operations
        public:
            Array<const Iso639::Language*>  getAllLanguages();
            const Iso639::Language *        findLanguageByCode2(const _Iso639Code2 & code2) const;
            MapResourceFactory *            getResourceFactory() const;

            //////////
            //  Implementation
        private:
            Array<const Iso639::Language*>  _languages;
            HashMap<_Iso639Code2, const Iso639::Language*>  _mapCode2ToLanguage;
            mutable FrameworkResourceFactory    _resourceFactory;

            void                            _registerLanguage(const Iso639::Language * pLanguage);
        };
    }
}

#endif  //  __Iso639Impl_hpp_included__
//  End of Iso639Impl.hpp
