//
//  Iso88591CharacterSet.cpp - Implementation of the carl::_impl::_Iso88591CharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Singleton
_impl::_Iso88591CharacterSet::_Iso88591CharacterSet()
{
    _names.append(String("ISO_8859-1:1987"));
    _names.append(String("iso-ir-100"));
    _names.append(String("ISO_8859-1"));
    _names.append(String("ISO-8859-1"));
    _names.append(String("latin1"));
    _names.append(String("l1"));
    _names.append(String("IBM819"));
    _names.append(String("CP819"));
    _names.append(String("csISOLatin1"));
}

_impl::_Iso88591CharacterSet::~_Iso88591CharacterSet()
{
}

_impl::_Iso88591CharacterSet * _impl::_Iso88591CharacterSet::getInstance()
{
    static _Iso88591CharacterSet instance;
    return &instance;
}

//////////
//  Operations
CharacterSetFamily * _impl::_Iso88591CharacterSet::getFamily() const
{
    return _Iso8859CharacterSetFamily::getInstance();
}

Array<String> _impl::_Iso88591CharacterSet::getAllNames() const
{
    return _names;
}

String _impl::_Iso88591CharacterSet::getCanonicalName() const
{
    return String("ISO_8859-1:1987");
}

String _impl::_Iso88591CharacterSet::getPreferredName() const
{
    return String("ISO-8859-1");
}

String _impl::_Iso88591CharacterSet::getDisplayName() const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), Locale::getDefault());
}

int _impl::_Iso88591CharacterSet::getMaximumEncodingLength() const
{
    return 1;
}

int _impl::_Iso88591CharacterSet::getEncodingLengthForCharacter(Char c) const
{
    if ((c & 0xFFFFFF00) == 0)
    {
        return 1;
    }
    return -1;
}

int _impl::_Iso88591CharacterSet::encodeCharacter(Char c, void * buffer) const
{
    if ((c & 0xFFFFFF00) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(c);
        return 1;
    }
    return -1;
}

int _impl::_Iso88591CharacterSet::getEncodingLengthForLeadByte(UInt8 /*leadByte*/) const
{
    return 1;
}

int _impl::_Iso88591CharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
    if (bytesAvailable >= 1)
    {
        c = pBytes[0];
        return 1;
    }
    return -1;
}

String _impl::_Iso88591CharacterSet::getDisplayName(const Locale & forLocale) const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), forLocale);
}

Char _impl::_Iso88591CharacterSet::getDefaultCharacter() const
{
    return '?';
}

Array<UInt8> _impl::_Iso88591CharacterSet::getSignature() const
{
    return Array<UInt8>();
}

//  End of Iso88591CharacterSet.cpp
