//
//  Locale.cpp - Implementation of the carl::Locale class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.12 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "LocaleImpl.hpp"
#include <stdlib.h>

//////////
//  Construction/destruction/assignment
Locale::Locale()
{
    _language[0] = _language[1] = 0;
    _country[0] = _country[1] = 0;
}

Locale::Locale(const char * pLanguage)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);

    _language[0] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[0]));
    _language[1] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[1]));
    _country[0] = _country[1] = 0;
}

Locale::Locale(const char * pLanguage, const char * pCountry)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);
    require(pCountry != NULL && pCountry[0] != 0 && pCountry[1] != 0 && pCountry[2] == 0);

    _language[0] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[0]));
    _language[1] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[1]));
    _country[0] = Iso10646::toUpper(static_cast<unsigned char>(pCountry[0]));
    _country[1] = Iso10646::toUpper(static_cast<unsigned char>(pCountry[1]));
}

Locale::Locale(const char * pLanguage, const char * pCountry, const char * pVariant)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);
    require(pCountry != NULL && pCountry[0] != 0 && pCountry[1] != 0 && pCountry[2] == 0);
    require(pVariant != NULL && pVariant[0] != 0);

    _language[0] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[0]));
    _language[1] = Iso10646::toLower(static_cast<unsigned char>(pLanguage[1]));
    _country[0] = Iso10646::toUpper(static_cast<unsigned char>(pCountry[0]));
    _country[1] = Iso10646::toUpper(static_cast<unsigned char>(pCountry[1]));
    _variant = String(pVariant);
}

Locale::Locale(const Char * pLanguage)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);

    _language[0] = Iso10646::toLower(pLanguage[0]);
    _language[1] = Iso10646::toLower(pLanguage[1]);
    _country[0] = _country[1] = 0;
}

Locale::Locale(const Char * pLanguage, const Char * pCountry)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);
    require(pCountry != NULL && pCountry[0] != 0 && pCountry[1] != 0 && pCountry[2] == 0);

    _language[0] = Iso10646::toLower(pLanguage[0]);
    _language[1] = Iso10646::toLower(pLanguage[1]);
    _country[0] = Iso10646::toUpper(pCountry[0]);
    _country[1] = Iso10646::toUpper(pCountry[1]);
}

Locale::Locale(const Char * pLanguage, const Char * pCountry, const Char * pVariant)
{
    require(pLanguage != NULL && pLanguage[0] != 0 && pLanguage[1] != 0 && pLanguage[2] == 0);
    require(pCountry != NULL && pCountry[0] != 0 && pCountry[1] != 0 && pCountry[2] == 0);
    require(pVariant != NULL && pVariant[0] != 0);

    _language[0] = Iso10646::toLower(pLanguage[0]);
    _language[1] = Iso10646::toLower(pLanguage[1]);
    _country[0] = Iso10646::toUpper(pCountry[0]);
    _country[1] = Iso10646::toUpper(pCountry[1]);
    _variant = String(pVariant);
}

Locale::Locale(const String & language)
{
    require(language.length() == 2);

    _language[0] = Iso10646::toLower(language[0]);
    _language[1] = Iso10646::toLower(language[1]);
    _country[0] = _country[1] = 0;
}

Locale::Locale(const String & language, const String & country)
{
    require(language.length() == 2);
    require(country.length() == 2);

    _language[0] = Iso10646::toLower(language[0]);
    _language[1] = Iso10646::toLower(language[1]);
    _country[0] = Iso10646::toUpper(country[0]);
    _country[1] = Iso10646::toUpper(country[1]);
}

Locale::Locale(const String & language, const String & country, const String & variant)
{
    require(language.length() == 2);
    require(country.length() == 2);
    require(variant.length() != 0);

    _language[0] = Iso10646::toLower(language[0]);
    _language[1] = Iso10646::toLower(language[1]);
    _country[0] = Iso10646::toUpper(country[0]);
    _country[1] = Iso10646::toUpper(country[1]);
    _variant = variant;
}

Locale::Locale(const Locale & src)
{
    _language[0] = src._language[0];
    _language[1] = src._language[1];
    _country[0] = src._country[0];
    _country[1] = src._country[1];
    _variant = src._variant;
}

Locale & Locale::operator = (const Locale & src)
{
    if (this != (&src))
    {
        _language[0] = src._language[0];
        _language[1] = src._language[1];
        _country[0] = src._country[0];
        _country[1] = src._country[1];
        _variant = src._variant;
    }
    return *this;
}

//////////
//  Operations
Locale Locale::getSystem()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_LocaleManagerImpl::getInstance()->getSystemLocale();
}

Locale Locale::getDefault()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_LocaleManagerImpl::getInstance()->getDefaultLocale();
}

void Locale::setDefault(const Locale & locale)
{
    _impl::_LocaleManagerImpl::getInstance()->setDefaultLocale(locale);
}

String Locale::getDisplayLanguage(const Locale & forLocale) const
{
    String language = getLanguage();
    const Iso639::Language * pLanguage = Iso639::findLanguageByCode2(language);
    if (pLanguage == NULL)
    {
        return language;
    }
    else
    {
        return pLanguage->getDisplayName(forLocale);
    }
}

String Locale::getDisplayCountry(const Locale & forLocale) const
{
    String country = getCountry();
    const Iso3166::Country * pCountry = Iso3166::findCountryByCode2(country);
    if (pCountry == NULL)
    {
        return country;
    }
    else
    {
        return pCountry->getDisplayName(forLocale);
    }
}

String Locale::getDisplayName(const Locale & forLocale) const
{
    try
    {   //  Try to lookup locale display name in resources
        return _impl::_LocaleManagerImpl::getInstance()->getResourceFactory()->getString(toString(), forLocale);
    }
    catch (MissingResourceError)
    {   //  Lookup failed - construct locale display name using standard rules
        String displayLanguage = getDisplayLanguage(forLocale);
        String displayCountry = getDisplayCountry(forLocale);
        String displayVariant = getDisplayVariant(forLocale);

        if (displayLanguage.length() != 0 && displayCountry.length() != 0 && displayVariant.length() != 0)
        {
            return displayLanguage + " (" + displayCountry + ") " + displayVariant;
        }
        else if (displayLanguage.length() != 0 && displayCountry.length() != 0 && displayVariant.length() == 0)
        {
            return displayLanguage + " (" + displayCountry + ")";
        }
        else if (displayLanguage.length() != 0 && displayCountry.length() == 0 && displayVariant.length() == 0)
        {
            return displayLanguage;
        }
        else
        {
            return _impl::_LocaleManagerImpl::getInstance()->getResourceFactory()->getString(_impl::_LocaleManagerImpl::InvariantLocaleDisplayName, forLocale);
        }
    }
}

Locale Locale::getParent() const
{
    if (_variant.length() != 0)
    {
        Char language[3] = { _language[0], _language[1], 0 };
        Char country[3] = { _country[0], _country[1], 0 };
        return Locale(language, country);
    }
    else if (_country[0] != 0)
    {
        Char language[3] = { _language[0], _language[1], 0 };
        return Locale(language);
    }
    else
    {
        return Locale();
    }
}

bool Locale::operator == (const Locale & locale) const
{
    return (_language[0] == locale._language[0]) &&
           (_language[1] == locale._language[1]) &&
           (_country[0] == locale._country[0]) &&
           (_country[1] == locale._country[1]) &&
           (_variant == locale._variant);
}

bool Locale::operator != (const Locale & locale) const
{
    return (_language[0] != locale._language[0]) ||
           (_language[1] != locale._language[1]) ||
           (_country[0] != locale._country[0]) ||
           (_country[1] != locale._country[1]) ||
           (_variant != locale._variant);
}

String Locale::toString() const
{
    if (isFull())
    {
        return String(_language, 2) + '_' + String(_country, 2) + '_' + _variant;
    }
    else if (isCulture())
    {
        return String(_language, 2) + '_' + String(_country, 2);
    }
    else if (isNeutral())
    {
        return String(_language, 2);
    }
    else
    {
        return String();
    }
}

Locale Locale::fromString(const char * pLocaleString)
{
    require(pLocaleString != NULL);

    return fromString(String(pLocaleString));
}

Locale Locale::fromString(const Char * pLocaleString)
{
    require(pLocaleString != NULL);

    return fromString(String(pLocaleString));
}

Locale Locale::fromString(const String & localeString)
{
    if (localeString.length() > 6 && 
        Iso10646::isAlpha(localeString[0]) &&
        Iso10646::isAlpha(localeString[1]) &&
        localeString[2] == '_' && 
        Iso10646::isAlpha(localeString[3]) &&
        Iso10646::isAlpha(localeString[4]) &&
        localeString[5] == '_')
    {
        return Locale(localeString.mid(0, 2), localeString.mid(3, 2), localeString.mid(6));
    }
    else if (localeString.length() == 5 && 
             Iso10646::isAlpha(localeString[0]) &&
             Iso10646::isAlpha(localeString[1]) &&
             localeString[2] == '_' &&
             Iso10646::isAlpha(localeString[3]) &&
             Iso10646::isAlpha(localeString[4]))
    {
        return Locale(localeString.mid(0, 2), localeString.mid(3, 2));
    }
    else if (localeString.length() == 2 &&
             Iso10646::isAlpha(localeString[0]) &&
             Iso10646::isAlpha(localeString[1]))
    {
        return Locale(localeString);
    }
    else if (localeString.length() == 0)
    {
        return Locale();
    }
    else
    {
        return getDefault();
    }
}

//////////
//  Hashing locales
UIntSize CARL_CALLBACK StandardHashFunctions::hashLocale(const Locale & key)
{
    return hashString(key.getLanguage()) * 31 + hashString(key.getCountry());
}

CriticalSection * Locale::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of Locale.cpp
