//
//  LocaleImpl.hpp - carl::_impl::_LocaleImpl interface
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __LocaleImpl_hpp_included__
#define __LocaleImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Locale resources
        class CARL_PUBLIC _LocaleResourceMap_en : public ResourceMap
        {
            //////////
            //  Can't assign or copy construct
        private:
            _LocaleResourceMap_en(const _LocaleResourceMap_en &);
            void                        operator = (const _LocaleResourceMap_en &);

            //////////
            //  Construction/destruction
        public:
            _LocaleResourceMap_en();
            virtual ~_LocaleResourceMap_en();

            //////////
            //  Operations
        public:
            virtual String              getString(const String & key);

            //////////
            //  Implementation
        private:
            HashMap<String, String>     _mapKeysToValues;
        };

        //////////
        //  A platform-specific implementation of a locale manager
        class CARL_PUBLIC _LocaleManagerImpl
        {
            //////////
            //  Can't assign or copy construct
        private:
            _LocaleManagerImpl(const _LocaleManagerImpl &);
            void                        operator = (const _LocaleManagerImpl &);

            //////////
            //  Singleton
        private:
            _LocaleManagerImpl();
            ~_LocaleManagerImpl();
        public:
            static _LocaleManagerImpl * getInstance();

            //////////
            //  Operations
        public:
            MapResourceFactory *        getResourceFactory() const;
            Locale                      getSystemLocale() const;
            Locale                      getDefaultLocale() const;
            void                        setDefaultLocale(const Locale & locale);

            //////////
            //  Resource keys
        public:
            static const char *         InvariantLocaleDisplayName;

            //////////
            //  Implementation
        private:
            mutable FrameworkResourceFactory    _resourceFactory;
            Locale                      _systemLocale;
            Locale                      _defaultLocale;
            mutable CriticalSection     _guard;
        };
    }
}

#endif  //  __LocaleImpl_hpp_included__
//  End of LocaleImpl.hpp
