//
//  LocaleManagerImpl.cpp - Implementation of the carl::_impl::_LocaleManagerImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.15 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "LocaleImpl.hpp"

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <windows.h>
#endif

//////////
//  Helpers
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
namespace
{
    Locale _translatePlatformLocale(LCID lcid)
    {
        switch (PRIMARYLANGID(LANGIDFROMLCID(lcid)))
        {
            case LANG_ARABIC:       //   Arabic 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_ARABIC_SAUDI_ARABIA:           //  Arabic (Saudi Arabia) 
                        return Locale("ar", "SA");
                    case SUBLANG_ARABIC_IRAQ:                   //  Arabic (Iraq) 
                        return Locale("ar", "IQ");
                    case SUBLANG_ARABIC_EGYPT:                  //  Arabic (Egypt) 
                        return Locale("ar", "EG");
                    case SUBLANG_ARABIC_LIBYA:                  //  Arabic (Libya) 
                        return Locale("ar", "LY");
                    case SUBLANG_ARABIC_ALGERIA:                //  Arabic (Algeria) 
                        return Locale("ar", "DZ");
                    case SUBLANG_ARABIC_MOROCCO:                //  Arabic (Morocco) 
                        return Locale("ar", "MA");
                    case SUBLANG_ARABIC_TUNISIA:                //  Arabic (Tunisia) 
                        return Locale("ar", "TN");
                    case SUBLANG_ARABIC_OMAN:                   //  Arabic (Oman) 
                        return Locale("ar", "OM");
                    case SUBLANG_ARABIC_YEMEN:                  //  Arabic (Yemen) 
                        return Locale("ar", "YE");
                    case SUBLANG_ARABIC_SYRIA:                  //  Arabic (Syria) 
                        return Locale("ar", "SY");
                    case SUBLANG_ARABIC_JORDAN:                 //  Arabic (Jordan) 
                        return Locale("ar", "JO");
                    case SUBLANG_ARABIC_LEBANON:                //  Arabic (Lebanon) 
                        return Locale("ar", "LB");
                    case SUBLANG_ARABIC_KUWAIT:                 //  Arabic (Kuwait) 
                        return Locale("ar", "KW");
                    case SUBLANG_ARABIC_UAE:                    //  Arabic (U.A.E.) 
                        return Locale("ar", "AE");
                    case SUBLANG_ARABIC_BAHRAIN:                //  Arabic (Bahrain) 
                        return Locale("ar", "BH");
                    case SUBLANG_ARABIC_QATAR:                  //  Arabic (Qatar) 
                        return Locale("ar", "QA");
                    default:
                        return Locale("ar");
                }
            case LANG_BULGARIAN:    //   Bulgarian 
                return Locale("bg", "BG");
            case LANG_CATALAN:      //   Catalan  
                return Locale("ca");
            case LANG_CHINESE:      //   Chinese 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_CHINESE_TRADITIONAL:           //  Chinese (Traditional) 
                        return Locale("zh", "CN", "Traditional");
                    case SUBLANG_CHINESE_SIMPLIFIED:            //  Chinese (Simplified) 
                        return Locale("zh", "CN", "Simplified");
                    case SUBLANG_CHINESE_HONGKONG:              //  Chinese (Hong Kong SAR, PRC) 
                        return Locale("zh", "HK");
                    case SUBLANG_CHINESE_SINGAPORE:             //  Chinese (Singapore) 
                        return Locale("zh", "SG");
                    case SUBLANG_CHINESE_MACAU:                 //  Chinese (Macao SAR) 
                        return Locale("zh", "MO");
                    default:
                        return Locale("zh", "CN");
                }
            case LANG_CZECH:        //   Czech 
                return Locale("cs", "CZ");
            case LANG_DANISH:       //   Danish 
                return Locale("da", "DK");
            case LANG_GERMAN:       //   German 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_GERMAN:                        //  German 
                        return Locale("de", "DE");
                    case SUBLANG_GERMAN_SWISS:                  //  German (Swiss) 
                        return Locale("de", "CH");
                    case SUBLANG_GERMAN_AUSTRIAN:               //  German (Austrian) 
                        return Locale("de", "AT");
                    case SUBLANG_GERMAN_LUXEMBOURG:             //  German (Luxembourg) 
                        return Locale("de", "LU");
                    case SUBLANG_GERMAN_LIECHTENSTEIN:          //  German (Liechtenstein) 
                        return Locale("de", "LI");
                    default:
                        return Locale("de", "DE");
                }
            case LANG_GREEK:        //   Greek 
                return Locale("el", "GR");
            case LANG_ENGLISH:      //   English  
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_ENGLISH_US:                    //  English (US) 
                        return Locale("en", "US");
                    case SUBLANG_ENGLISH_UK:                    //  English (UK) 
                        return Locale("en", "GB");
                    case SUBLANG_ENGLISH_AUS:                   //  English (Australian) 
                        return Locale("en", "AU");
                    case SUBLANG_ENGLISH_CAN:                   //  English (Canadian) 
                        return Locale("en", "CA");
                    case SUBLANG_ENGLISH_NZ:                    //  English (New Zealand) 
                        return Locale("en", "NZ");
                    case SUBLANG_ENGLISH_EIRE:                  //  English (Ireland) 
                        return Locale("en", "IE");
                    case SUBLANG_ENGLISH_SOUTH_AFRICA:          //  English (South Africa) 
                        return Locale("en", "ZA");
                    case SUBLANG_ENGLISH_JAMAICA:               //  English (Jamaica) 
                        return Locale("en", "JM");
                    case SUBLANG_ENGLISH_BELIZE:                //  English (Belize) 
                        return Locale("en", "BZ");
                    case SUBLANG_ENGLISH_TRINIDAD:              //  English (Trinidad) 
                        return Locale("en", "TT");
                    case SUBLANG_ENGLISH_ZIMBABWE:              //  English (Zimbabwe) 
                        return Locale("en", "ZW");
                    case SUBLANG_ENGLISH_PHILIPPINES:           //  English (Philippines) 
                        return Locale("en", "PH");
                    default:
                        return Locale("en", "GB");
                }
            case LANG_SPANISH:      //   Spanish  
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_SPANISH:                       //  Spanish (Castilian) 
                        return Locale("es", "ES");
                    case SUBLANG_SPANISH_MEXICAN:               //  Spanish (Mexican) 
                        return Locale("es", "MX");
                    case SUBLANG_SPANISH_MODERN:                //  Spanish (Spain) 
                        return Locale("es", "ES");
                    case SUBLANG_SPANISH_GUATEMALA:             //  Spanish (Guatemala) 
                        return Locale("es", "GT");
                    case SUBLANG_SPANISH_COSTA_RICA:            //  Spanish (Costa Rica) 
                        return Locale("es", "CR");
                    case SUBLANG_SPANISH_PANAMA:                //  Spanish (Panama) 
                        return Locale("es", "PA");
                    case SUBLANG_SPANISH_DOMINICAN_REPUBLIC:    //  Spanish (Dominican Republic) 
                        return Locale("es", "DO");
                    case SUBLANG_SPANISH_VENEZUELA:             //  Spanish (Venezuela) 
                        return Locale("es", "VE");
                    case SUBLANG_SPANISH_COLOMBIA:              //  Spanish (Colombia) 
                        return Locale("es", "CO");
                    case SUBLANG_SPANISH_PERU:                  //  Spanish (Peru) 
                        return Locale("es", "PE");
                    case SUBLANG_SPANISH_ARGENTINA:             //  Spanish (Argentina) 
                        return Locale("es", "AR");
                    case SUBLANG_SPANISH_ECUADOR:               //  Spanish (Ecuador) 
                        return Locale("es", "EC");
                    case SUBLANG_SPANISH_CHILE:                 //  Spanish (Chile) 
                        return Locale("es", "CL");
                    case SUBLANG_SPANISH_URUGUAY:               //  Spanish (Uruguay) 
                        return Locale("es", "UY");
                    case SUBLANG_SPANISH_PARAGUAY:              //  Spanish (Paraguay) 
                        return Locale("es", "PY");
                    case SUBLANG_SPANISH_BOLIVIA:               //  Spanish (Bolivia) 
                        return Locale("es", "BO");
                    case SUBLANG_SPANISH_EL_SALVADOR:           //  Spanish (El Salvador) 
                        return Locale("es", "SV");
                    case SUBLANG_SPANISH_HONDURAS:              //  Spanish (Honduras) 
                        return Locale("es", "HN");
                    case SUBLANG_SPANISH_NICARAGUA:             //  Spanish (Nicaragua) 
                        return Locale("es", "NI");
                    case SUBLANG_SPANISH_PUERTO_RICO:           //  Spanish (Puerto Rico) 
                        return Locale("es", "PR");
                    default:
                        return Locale("es", "ES");
                }
            case LANG_FINNISH:      //   Finnish 
                return Locale("fi", "FI");
            case LANG_FRENCH:       //   French 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_FRENCH:                        //  French 
                        return Locale("fr", "FR");
                    case SUBLANG_FRENCH_BELGIAN:                //  French (Belgian) 
                        return Locale("fr", "BE");
                    case SUBLANG_FRENCH_CANADIAN:               //  French (Canadian) 
                        return Locale("fr", "CA");
                    case SUBLANG_FRENCH_SWISS:                  //  French (Swiss) 
                        return Locale("fr", "CH");
                    case SUBLANG_FRENCH_LUXEMBOURG:             //  French (Luxembourg) 
                        return Locale("fr", "LU");
                    case SUBLANG_FRENCH_MONACO:                 //  French (Monaco) 
                        return Locale("fr", "MC");
                    default:
                        return Locale("fr", "FR");
                }
            case LANG_HEBREW:       //   Hebrew 
                return Locale("iw", "IL");
            case LANG_HUNGARIAN:    //   Hungarian 
                return Locale("hu", "HU");
            case LANG_ICELANDIC:    //   Icelandic 
                return Locale("is", "IS");
            case LANG_ITALIAN:      //   Italian  
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_ITALIAN:                       //  Italian 
                        return Locale("it", "IT");
                    case SUBLANG_ITALIAN_SWISS:                 //  Italian (Swiss) 
                        return Locale("it", "CH");
                    default:
                        return Locale("it", "IT");
                }
            case LANG_JAPANESE:     //   Japanese 
                return Locale("ja", "JP");
            case LANG_KOREAN:       //   Korean 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_KOREAN:                        //  Korean 
                        return Locale("ko", "KR");
                    default:
                        return Locale("ko");
                }
            case LANG_DUTCH:        //   Dutch 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_DUTCH:                         //  Dutch 
                        return Locale("nl", "NL");
                    case SUBLANG_DUTCH_BELGIAN:                 //  Dutch (Belgian)  
                        return Locale("nl", "BE");
                    default:
                        return Locale("nl", "NL");
                }
            case LANG_NORWEGIAN:    //   Norwegian 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_NORWEGIAN_BOKMAL:              //  Norwegian (Bokmal) 
                        return Locale("no", "NO", "Bokmal");
                    case SUBLANG_NORWEGIAN_NYNORSK:             //  Norwegian (Nynorsk) 
                        return Locale("no", "NO", "Nynorsk");
                    default:
                        return Locale("no", "NO");
                }
            case LANG_POLISH:       //   Polish 
                return Locale("pl", "PL");
            case LANG_PORTUGUESE:   //   Portuguese 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_PORTUGUESE_BRAZILIAN:          //  Portuguese (Brazil) 
                        return Locale("pt", "BR");
                    case SUBLANG_PORTUGUESE:                    //  Portuguese (Portugal) 
                        return Locale("pt", "PT");
                    default:
                        return Locale("pt", "PT");
                }
            case LANG_ROMANIAN:     //   Romanian 
                return Locale("ro", "RO");
            case LANG_RUSSIAN:      //   Russian 
                return Locale("ru", "RU");
            case LANG_SERBIAN:     //   Serbian, same value as LANG_CROATIAN (Croatian)
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_SERBIAN_LATIN:                 //  Serbian (Latin) 
                        return Locale("sr", "HR", "Latin");
                    case SUBLANG_SERBIAN_CYRILLIC:              //  Serbian (Cyrillic) 
                        return Locale("sr", "HR", "Cyrillic");
                    default:
                        return Locale("sr", "HR");
                }
            case LANG_SLOVAK:       //   Slovak 
                return Locale("sk", "SK");
            case LANG_ALBANIAN:     //   Albanian 
                return Locale("sq", "AL");
            case LANG_SWEDISH:      //   Swedish  
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_SWEDISH:                       //  Swedish 
                        return Locale("sv", "SE");
                    case SUBLANG_SWEDISH_FINLAND:               //  Swedish (Finland) 
                        return Locale("sv", "FI");
                    default:
                        return Locale("sv", "SE");
                }
            case LANG_THAI:         //   Thai 
                return Locale("th", "TH");
            case LANG_TURKISH:      //   Turkish  
                return Locale("tr", "TR");
            case LANG_URDU:         //   Urdu 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_URDU_PAKISTAN:                 //  Urdu (Pakistan) 
                        return Locale("ur", "PK");
                    case SUBLANG_URDU_INDIA:                    //  Urdu (India) 
                        return Locale("ur", "IN");
                    default:
                        return Locale("ur");
                }
            case LANG_INDONESIAN:   //   Indonesian 
                return Locale("in", "ID");
            case LANG_UKRAINIAN:    //   Ukrainian 
                return Locale("uk", "UA");
            case LANG_BELARUSIAN:   //   Belarusian 
                return Locale("be", "BY");
            case LANG_SLOVENIAN:    //   Slovenian 
                return Locale("sl", "SI");
            case LANG_ESTONIAN:     //   Estonian 
                return Locale("et", "EE");
            case LANG_LATVIAN:      //   Latvian 
                return Locale("lv", "LV");
            case LANG_LITHUANIAN:   //   Lithuanian 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_LITHUANIAN:                    //  Lithuanian 
                        return Locale("lt", "LT");
                    default:
                        return Locale("lt", "LT");
                }
            case LANG_FARSI:        //   Farsi 
                return Locale("fa", "IR");
            case LANG_VIETNAMESE:   //   Vietnamese 
                return Locale("vi", "VN");
            case LANG_ARMENIAN:     //   Armenian 
                return Locale("hy", "AM");
            case LANG_AZERI:        //   Azeri 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_AZERI_LATIN:                   //  Azeri (Latin) 
                        return Locale("az", "AZ", "Latin");
                    case SUBLANG_AZERI_CYRILLIC:                //  Azeri (Cyrillic) 
                        return Locale("az", "AZ", "Cyrillic");
                    default:
                        return Locale("az", "AZ");
                }
            case LANG_BASQUE:       //  Basque 
                return Locale("eu");
            case LANG_MACEDONIAN:   //  Macedonian 
                return Locale("mk", "MK");
            case LANG_AFRIKAANS:    //  Afrikaans 
                return Locale("af", "ZA");
            case LANG_GEORGIAN:     //   Georgian 
                return Locale("ka", "GE");
            case LANG_HINDI:        //   Hindi 
                return Locale("hi");
            case LANG_MALAY:        //   Malay 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_MALAY_MALAYSIA:                //  Malay (Malaysia) 
                        return Locale("ms", "MY");
                    case SUBLANG_MALAY_BRUNEI_DARUSSALAM:       //  Malay (Brunei Darassalam) 
                        return Locale("ms", "BN");
                    default:
                        return Locale("ms");
                }
            case LANG_KAZAK:        //   Kazak 
                return Locale("kk", "KZ");
            case LANG_KYRGYZ:       //   Kyrgyz 
                return Locale("ky", "KG");
            case LANG_SWAHILI:      //   Swahili 
                return Locale("sw");
            case LANG_UZBEK:        //   Uzbek 
                switch (SUBLANGID(LANGIDFROMLCID(lcid)))
                {
                    case SUBLANG_UZBEK_LATIN:                   //  Uzbek (Latin) 
                        return Locale("uz", "UZ", "Latin");
                    case SUBLANG_UZBEK_CYRILLIC:                //  Uzbek (Cyrillic) 
                        return Locale("uz", "UZ", "Cyrillic");
                    default:
                        return Locale("uz", "UZ");
                }
            case LANG_TATAR:        //   Tatar 
                return Locale("tt");
            case LANG_BENGALI:      //   Not supported. 
                return Locale("bn");
            case LANG_PUNJABI:      //   Punjabi 
                return Locale("pa");
            case LANG_GUJARATI:     //   Gujarati 
                return Locale("gu");
            case LANG_ORIYA:        //   Not supported. 
                return Locale("or");
            case LANG_TAMIL:        //   Tamil 
                return Locale("ta");
            case LANG_TELUGU:       //   Telugu 
                return Locale("te");
            case LANG_KANNADA:      //   Kannada 
                return Locale("kn");
            case LANG_MALAYALAM:    //   Not supported. 
                return Locale("ml");
            case LANG_ASSAMESE:     //   Not supported. 
                return Locale("as");
            case LANG_MARATHI:      //   Marathi 
                return Locale("mr");
            case LANG_SANSKRIT:     //   Sanskrit 
                return Locale("sa");
            case LANG_MONGOLIAN:    //   Mongolian 
                return Locale("mn", "MN");
            case LANG_GALICIAN:     //   Galician 
                return Locale("gl");
            case LANG_SINDHI:       //   Not supported. 
                return Locale("sd");
            case LANG_KASHMIRI:     //   Not supported. 
                return Locale("ks");
            case LANG_NEPALI:       //   Not supported. 
                return Locale("ne");
            default:    //  Default system locale is invariant
                return Locale();
        }
    }
}
#endif

//////////
//  Singleton
_impl::_LocaleManagerImpl::_LocaleManagerImpl()
    :   _resourceFactory(new _LocaleResourceMap_en(), carl::String("CARL_LOCALE"))
{
    //  Set up the system locale
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    _systemLocale = _translatePlatformLocale(GetSystemDefaultLCID());
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    _systemLocale = Locale("en");
#else
    #error Unsupported platform
#endif
    //  Initially, default locale os the same as system locale
    _defaultLocale = _systemLocale;
}

_impl::_LocaleManagerImpl::~_LocaleManagerImpl()
{
}

_impl::_LocaleManagerImpl * _impl::_LocaleManagerImpl::getInstance()
{
    static _LocaleManagerImpl instance;
    return &instance;
}

//////////
//  Operations
MapResourceFactory * _impl::_LocaleManagerImpl::getResourceFactory() const
{
    return &_resourceFactory;
}

Locale _impl::_LocaleManagerImpl::getSystemLocale() const
{
    CriticalSectionLock lock(_guard);
    return _systemLocale;
}

Locale _impl::_LocaleManagerImpl::getDefaultLocale() const
{
    CriticalSectionLock lock(_guard);
    return _defaultLocale;
}

void _impl::_LocaleManagerImpl::setDefaultLocale(const Locale & locale)
{
    CriticalSectionLock lock(_guard);
    _defaultLocale = locale;
}

//////////
//  Resource keys
const char * _impl::_LocaleManagerImpl::InvariantLocaleDisplayName = "InvariantLocaleDisplayName";

//  End of LocaleManagerImpl.cpp
