//
//  LzwCompressor.cpp - Implementation of the carl::_impl::_LzwCompressor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_LzwCompressor::_LzwCompressor()
{
}

_impl::_LzwCompressor::~_LzwCompressor()
{
}

_impl::_LzwCompressor * _impl::_LzwCompressor::getInstance()
{
    static _LzwCompressor instance;
    return &instance;
}

//////////
//  Operations
CompressionAlgorithm * _impl::_LzwCompressor::getCompressionAlgorithm()
{
    return _LzwCompressionAlgorithm::getInstance();
}

UIntSize _impl::_LzwCompressor::getRequiredOutputBufferSize(const void * pUncompressedData, UIntSize uncompressedDataSize)
{
    const carl::UInt8 * pUncompressedBytes = static_cast<const carl::UInt8 *>(pUncompressedData);
    carl::UIntSize offs1 = 0;
    carl::UIntSize len = uncompressedDataSize, outlen = 0, srclen, destlen;

    while (len > 0)
    {
        srclen = min(len, _LZ_BUFFER_SIZE);
        destlen = _CompressLine(pUncompressedBytes + offs1, (carl::Int16)srclen, _buf);
        if (destlen < srclen - 3)
        {   //  Save compressed
            outlen += 5;
            outlen += destlen;
        }
        else
        {   //  Save uncompressed
            outlen += 3;
            outlen += srclen;
        }
        offs1 += srclen;
        len -= srclen;
    }
    return outlen;
}

UIntSize _impl::_LzwCompressor::compress(const void * pUncompressedData, UIntSize uncompressedDataSize, void * pCompressedData)
{
    const carl::UInt8 * pUncompressedBytes = static_cast<const carl::UInt8 *>(pUncompressedData);
    carl::UInt8 * pCompressedBytes = static_cast<carl::UInt8 *>(pCompressedData);
    carl::UIntSize offs1 = 0;
    carl::UIntSize len = uncompressedDataSize, outlen = 0, srclen, destlen;

    while (len > 0)
    {
        srclen = min(len, _LZ_BUFFER_SIZE);
        destlen = _CompressLine(pUncompressedBytes + offs1, (carl::Int16)srclen, _buf);
        if (destlen < srclen - 3)
        {   //  Save compressed
            pCompressedBytes[outlen++] = 1;
            pCompressedBytes[outlen++] = (carl::UInt8)(destlen >> 8);
            pCompressedBytes[outlen++] = (carl::UInt8)(destlen);
            pCompressedBytes[outlen++] = (carl::UInt8)(srclen >> 8);
            pCompressedBytes[outlen++] = (carl::UInt8)(srclen);
            Memory::copy(pCompressedBytes + outlen, _buf, destlen);
            outlen += destlen;
        }
        else
        {   //  Save uncompressed
            pCompressedBytes[outlen++] = 2;
            pCompressedBytes[outlen++] = (carl::UInt8)(srclen >> 8);
            pCompressedBytes[outlen++] = (carl::UInt8)(srclen);
            Memory::copy(pCompressedBytes + outlen, pUncompressedBytes + offs1, srclen);
            outlen += srclen;
        }
        offs1 += srclen;
        len -= srclen;
    }
    return outlen;
}

//////////
//  Hash table support
void _impl::_LzwCompressor::_clearHashTable(UInt32 * Table)
{
    for (Int16 i = 0; static_cast<UIntSize>(i) < _HT_SIZE; i++)
    {
        Table[i] = 0xFFFFFFFFL;
    }
}

void _impl::_LzwCompressor::_initHashTable(void)
{
    _clearHashTable(_hashTable);
}

Int16 _impl::_LzwCompressor::_KeyItem(UInt32 item)
{
    return (Int16)(((item >> 12) ^ item) & _HT_KEY_MASK);
}

void _impl::_LzwCompressor::_insertHashTable(UInt32 * Table, UInt32 Key, Int16 code)
{
    Int16 HKey = _KeyItem(Key);
    UInt32 * HTable = Table;

    while (_HT_GET_KEY(HTable[HKey]) != 0xFFFFFl)
    {
        HKey = (HKey+1) & _HT_KEY_MASK;
    }
    HTable[HKey] = _HT_PUT_KEY(Key) | _HT_PUT_CODE(code);
}

Int16 _impl::_LzwCompressor::_existHashTable(UInt32 * table, UInt32 Key)
{
    Int16 HKey = _KeyItem(Key);
    UInt32 *HTable = table, HTKey;

    while ((HTKey = _HT_GET_KEY(HTable[HKey])) != 0xFFFFFL)
    {
        if (Key == HTKey)
        {
            return (Int16)(_HT_GET_CODE(HTable[HKey]));
        }
        HKey = (HKey + 1) & _HT_KEY_MASK;
    }
    return -1;
}

//////////
//  Compressor
Int16 _impl::_LzwCompressor::_CompressLine(const UInt8 * line, Int16 lineLen, UInt8 * dest)
{
    Int16 i = 0;

    _InitCompress();
    if (static_cast<UIntSize>(_curCode) == _FIRST_CODE)
    {
        _curCode = line[i++];
    }
    while (i < lineLen)
    {
        _pixel = line[i++];
        _newKey = (((UInt32) _curCode) << 8) + _pixel;
        if ((_newCode = _existHashTable(_hashTable,_newKey)) >= 0)
        {
            _curCode = _newCode;
        }
        else
        {
            _CompressCode(_curCode);
            _curCode = _pixel;
            if (static_cast<UIntSize>(_runCode) >= _LZ_MAX_CODE)
            {
                _CompressCode(_clearCode);
                _runCode = _EOFCode + 1;
                _runningBits = _HT_BITS_PER_PIXEL + 1;
                _maxCodeMult2 = 1 << _runningBits;
                _clearHashTable(_hashTable);
            }
            else
            {
                _insertHashTable(_hashTable, _newKey, _runCode++);
            }
        }
    }
    if (_pixelCount == 0)
    {
        _CompressCode(_curCode);
        _CompressCode(_EOFCode);
        _CompressCode(_FLUSH_OUT);
    }
    Memory::copy(dest, _buffer, _bufferLength);
    return _bufferLength;
}

Int16 _impl::_LzwCompressor::_OutByte(UInt8 * buffer, Int16 code)
{
    if (static_cast<UIntSize>(code) != _FLUSH_OUT)
    {
        buffer[_bufferLength++] = (UInt8)code;
    }
    return 1;
}

Int16 _impl::_LzwCompressor::_CompressCode(Int16 code)
{
    if (static_cast<UIntSize>(code) == _FLUSH_OUT)
    {
        while (_curBufferShift > 0)
        {
            _OutByte(_buffer, (Int16)(_codeBuffer & 0xFF));
            _codeBuffer >>= 8;
            _curBufferShift -= 8;
        }
        _curBufferShift = 0;
        _OutByte(_buffer, _FLUSH_OUT);
    }
    else
    {
        _codeBuffer |= ((Int32)code) << _curBufferShift;
        _curBufferShift = (Int16)(_curBufferShift + _runningBits);
        while (_curBufferShift >= 8)
        {
            _OutByte(_buffer, (Int16)(_codeBuffer & 0xFF));
            _codeBuffer >>= 8;
            _curBufferShift -= 8;
        }
    }
    if (_runCode >= _maxCodeMult2 && code <= 4095)
    {
        _maxCodeMult2 = 1 << (++_runningBits);
    }
    return 1;
}

Int16 _impl::_LzwCompressor::_InitCompress(void)
{
     _clearCode = 1 << _HT_BITS_PER_PIXEL;
     _EOFCode = _clearCode + 1;
     _runCode = _EOFCode + 1;
     _runningBits = _HT_BITS_PER_PIXEL + 1;
     _maxCodeMult2 = 1 << _runningBits;
     _curBufferShift = 0;
     _codeBuffer = 0;
     _initHashTable();
     _bufferLength = 0;
     _curCode = _FIRST_CODE;
     _CompressCode(_clearCode);
     return 1;
}

//  End of LzwCompressor.cpp
