//
//  LzwDecompressor.cpp - Implementation of the carl::_impl::_LzwDecompressor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_LzwDecompressor::_LzwDecompressor()
{
}

_impl::_LzwDecompressor::~_LzwDecompressor()
{
}

_impl::_LzwDecompressor * _impl::_LzwDecompressor::getInstance()
{
    static _LzwDecompressor instance;
    return &instance;
}

//////////
//  Operations
CompressionAlgorithm * _impl::_LzwDecompressor::getCompressionAlgorithm()
{
    return _LzwCompressionAlgorithm::getInstance();
}

UIntSize _impl::_LzwDecompressor::getRequiredOutputBufferSize(const void * pCompressedData, UIntSize compressedDataSize) throw(CompressedDataCorruptException)
{
    const UInt8 * pCompressedBytes = static_cast<const UInt8 *>(pCompressedData);
    UIntSize offs1 = 0, offs2 = 0;
    UInt8 srch, srcl, destl, desth;
    UIntSize srclen, destlen;

    while (offs1 != compressedDataSize)
    {
        if (offs1 > compressedDataSize)
        {
            throw CompressedDataCorruptException();
        }
        if (pCompressedBytes[offs1] == 2)
        {   //  Uncompressed
            if (offs1 + 3 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            offs1++;
            srch  = pCompressedBytes[offs1++];
            srcl = pCompressedBytes[offs1++];
            srclen = (srch << 8) | srcl;
            if (offs1 + srclen > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            offs1 += srclen;
            offs2 += srclen;
        }
        else if (pCompressedBytes[offs1] == 1)
        {   //  Compressed
            if (offs1 + 5 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            offs1++;
            srch = pCompressedBytes[offs1++];
            srcl = pCompressedBytes[offs1++];
            srclen = (srch << 8) | srcl;
            desth = pCompressedBytes[offs1++];
            destl = pCompressedBytes[offs1++];
            destlen = (desth << 8) | destl;
            if (offs1 + srclen > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            _DecompressLine(pCompressedBytes + offs1, (UInt16)srclen, 
                            _buf, (UInt16)destlen);
            offs1 += srclen;
            offs2 += destlen;
        }
        else
        {
            throw CompressedDataCorruptException();
        }
    }
    return offs2;
}

UIntSize _impl::_LzwDecompressor::decompress(const void * pCompressedData, UIntSize compressedDataSize, void * pUncompressedData) throw(CompressedDataCorruptException)
{
    const UInt8 * pCompressedBytes = static_cast<const UInt8 *>(pCompressedData);
    UInt8 * pUncompressedBytes = static_cast<UInt8 *>(pUncompressedData);
    UIntSize offs1 = 0, offs2 = 0;
    UInt8 srch, srcl, destl, desth;
    UIntSize srclen, destlen;

    while (offs1 != compressedDataSize)
    {
        if (offs1 > compressedDataSize)
        {
            throw CompressedDataCorruptException();
        }
        if (pCompressedBytes[offs1] == 2)
        {   //  Uncompressed
            if (offs1 + 3 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            offs1++;
            srch  = pCompressedBytes[offs1++];
            srcl = pCompressedBytes[offs1++];
            srclen = (srch << 8) | srcl;
            if (offs1 + srclen > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            Memory::copy(pUncompressedBytes + offs2, pCompressedBytes + offs1, srclen);
            offs1 += srclen;
            offs2 += srclen;
        }
        else if (pCompressedBytes[offs1] == 1)
        {   //  Compressed
            if (offs1 + 5 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            offs1++;
            srch = pCompressedBytes[offs1++];
            srcl = pCompressedBytes[offs1++];
            srclen = (srch << 8) | srcl;
            desth = pCompressedBytes[offs1++];
            destl = pCompressedBytes[offs1++];
            destlen = (desth << 8) | destl;
            if (offs1 + srclen > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            _DecompressLine(pCompressedBytes + offs1, (UInt16)srclen, 
                            pUncompressedBytes + offs2, (carl::UInt16)destlen);
            offs1 += srclen;
            offs2 += destlen;
        }
        else
        {
            throw CompressedDataCorruptException();
        }
    }
    return offs2;
}

//////////
//  Hash table support
void _impl::_LzwDecompressor::_clearHashTable(UInt32 * Table)
{
    for (Int16 i = 0; static_cast<UIntSize>(i) < _HT_SIZE; i++)
    {
        Table[i] = 0xFFFFFFFFL;
    }
}

void _impl::_LzwDecompressor::_initHashTable(void)
{
    _clearHashTable(_hashTable);
}

Int16 _impl::_LzwDecompressor::_KeyItem(UInt32 item)
{
    return (Int16)(((item >> 12) ^ item) & _HT_KEY_MASK);
}

void _impl::_LzwDecompressor::_insertHashTable(UInt32 * Table, UInt32 Key, Int16 code)
{
    Int16 HKey = _KeyItem(Key);
    UInt32 * HTable = Table;

    while (_HT_GET_KEY(HTable[HKey]) != 0xFFFFFl)
    {
        HKey = (HKey+1) & _HT_KEY_MASK;
    }
    HTable[HKey] = _HT_PUT_KEY(Key) | _HT_PUT_CODE(code);
}

Int16 _impl::_LzwDecompressor::_existHashTable(UInt32 * table, UInt32 Key)
{
    Int16 HKey = _KeyItem(Key);
    UInt32 *HTable = table, HTKey;

    while ((HTKey = _HT_GET_KEY(HTable[HKey])) != 0xFFFFFL)
    {
        if (Key == HTKey)
        {
            return (Int16)(_HT_GET_CODE(HTable[HKey]));
        }
        HKey = (HKey + 1) & _HT_KEY_MASK;
    }
    return -1;
}

//////////
//  Decompressor
Int16 _impl::_LzwDecompressor::_DecompressLine(const UInt8 * from, Int16 fromLen,UInt8 * line, Int16 lineLen)
{
    Int16 i = 0, j = 0, code;

    Memory::copy(_buffer, from, fromLen);
    _InitDecompress();
    if (_stackCount != 0)
    {
        while (_stackCount != 0 && i < lineLen)
        {
            line[i++] = (UInt8)_stack()[--_stackCount];
        }
    }
    while (i < lineLen)
    {
        _DecompressCode(&code);
        if (code == _EOFCode)
        {
            i++;
        }
        else if (code == _clearCode)
        {
            for (j = 0; static_cast<UIntSize>(j) < _LZ_MAX_CODE; j++) 
            {
                _prefix()[j] = _NOT_CODE;
            }
            _RunCode = _EOFCode + 1;
            _bitsPerCode = _HT_BITS_PER_PIXEL + 1;
            _maxCodeMult2 = 1 << _bitsPerCode;
            _oldCode = _NOT_CODE;
        }
        else
        {
            if (code < _clearCode)
            {
                line[i++] = (UInt8)code;
            }
            else
            {
                if (static_cast<UIntSize>(_prefix()[code]) == _NOT_CODE)
                {
                    _curPrefix = _oldCode;
                    _sufix()[_RunCode-2] = _getPrefixChar((UInt16*)_prefix(), _oldCode, _clearCode);
                    _stack()[_stackCount++] = _sufix()[_RunCode - 2];
                }
                else
                {
                    _curPrefix = code;
                }
                while (_curPrefix > _clearCode && static_cast<UIntSize>(_curPrefix) <= _LZ_MAX_CODE)
                {
                    _stack()[_stackCount++] = _sufix()[_curPrefix];
                    _curPrefix = _prefix()[_curPrefix];
                }
                _stack()[_stackCount++] = _curPrefix;
                while (_stackCount != 0 && i < lineLen)
                {
                    line[i++] = (UInt8)_stack()[--_stackCount];
                }
            }
            if (static_cast<UIntSize>(_oldCode) != _NOT_CODE)
            {
                _prefix()[_RunCode - 2] = _oldCode;
                if ((UInt32)code == _RunCode - 2)
                {
                    _sufix()[_RunCode - 2] = _getPrefixChar((UInt16*)_prefix(), _oldCode, _clearCode);
                }
                else
                {
                    _sufix()[_RunCode - 2] = _getPrefixChar((UInt16*)_prefix(), code, _clearCode);
                }
            }
            _oldCode = code;
        }
    }
    return i;
}

void _impl::_LzwDecompressor::_getByte(UInt8 * buffer,UInt8 * byte)
{
    *byte = buffer[_nextByte++];
}

void _impl::_LzwDecompressor::_DecompressCode(Int16 * code)
{
    UInt8 byte;
    UInt32 longByte;

    while (_curBufferShift < _bitsPerCode)
    {
        _getByte(_buffer, &byte);
        longByte = (UInt32)byte;
        longByte <<= _curBufferShift;
        _codeBuffer |= longByte;
        _curBufferShift += 8;
    }
    *code = (Int16)(_codeBuffer & (0xFFFFFFFFl >> (Int32)(32 - _bitsPerCode)));
    _codeBuffer >>= _bitsPerCode;
    _curBufferShift = (Int16)(_curBufferShift - _bitsPerCode);
    if (++_RunCode > (UInt32)_maxCodeMult2 && static_cast<UIntSize>(_bitsPerCode) < _MAX_BIT_PER_CODE)
    {
        _maxCodeMult2 <<= 1;
        _bitsPerCode++;
    }
}

Int16 _impl::_LzwDecompressor::_getPrefixChar(UInt16  * prefix, Int16 code, Int16 ClearCode)
{
    Int16 i = 0;

    while(code > ClearCode && static_cast<UIntSize>(i++) < _LZ_MAX_CODE)
    {
        code = prefix[code];
    }
    return code;
}

Int16 _impl::_LzwDecompressor::_InitDecompress(void)
{
    Int16 k;

    _nextByte = 0;
    _clearCode = 1 << _HT_BITS_PER_PIXEL;
    _EOFCode = _clearCode + 1;
    _RunCode = _EOFCode + 1;
    _bitsPerCode = _HT_BITS_PER_PIXEL + 1;
    _maxCodeMult2 = 1 << _bitsPerCode;
    _stackCount = 0;
    _oldCode = _NOT_CODE;
    _curBufferShift = 0;
    _codeBuffer = 0;
    for(k = 0; static_cast<UIntSize>(k) < _LZ_MAX_CODE; k++)
    {
        _prefix()[k] = _NOT_CODE;
    }
    return 0;
}

//  End of LzwDecompressor.cpp
