//
//  Math.cpp - Implementation of the carl::Math class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.14 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <math.h>
#include <errno.h>
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    #include <float.h>
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
#else
    #error Unsupported platform
#endif

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
int _matherr(struct _exception * /*except*/)
{
    errno = EDOM;
    return 1;
}
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
#else
    #error Unsupported platform
#endif

//////////
//  Operations
Real32 Math::acos(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::acosf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::acos(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::acos(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::acos(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::acos(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::asin(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::asinf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::asin(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::asin(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::asin(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::asin(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::atan(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::atanf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::atan(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::atan(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::atan(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::atan(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::atan2(Real32 y, Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::atan2f(y, x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::atan2(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::atan2(Real64 y, Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::atan2(y, x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::atan2(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::ceiling(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::ceilf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::ceiling(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::ceiling(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::ceil(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::ceiling(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::cos(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::cosf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::cos(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::cos(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::cos(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::cos(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::cosh(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::coshf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::cosh(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::cosh(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::cosh(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::cosh(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::exp(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::expf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::exp(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::exp(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::exp(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::exp(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::floor(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::floorf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::floor(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::floor(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::floor(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::floor(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::log(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::logf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::log(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::log(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::log(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::log(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::log10(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::log10f(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::log10(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::log10(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::log10(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::log10(): valid argument", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::pow(Real32 x, Real32 y)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::powf(x, y);
    if (errno == EDOM)
    {
        throw AssertionError("Math::pow(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::pow(Real64 x, Real64 y)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::pow(x, y);
    if (errno == EDOM)
    {
        throw AssertionError("Math::pow(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::sin(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::sinf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sin(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::sin(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::sin(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sin(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::sinh(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::sinhf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sinh(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::sinh(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::sinh(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sinh(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::sqrt(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::sqrtf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sqrt(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::sqrt(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::sqrt(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::sqrt(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::tan(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::tanf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::tan(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::tan(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::tan(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::tan(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real32 Math::tanh(Real32 x)
{
    int oldErrno = errno;
    errno = 0;
    Real32 result = ::tanhf(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::tanh(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

Real64 Math::tanh(Real64 x)
{
    int oldErrno = errno;
    errno = 0;
    Real64 result = ::tanh(x);
    if (errno == EDOM)
    {
        throw AssertionError("Math::tanh(): valid argument(s)", __FILE__, __LINE__);
    }
    errno = oldErrno;
    return result;
}

bool Math::isNan(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    switch (_fpclass(x))
    {
        case _FPCLASS_SNAN:
        case _FPCLASS_QNAN:
            return true;
        default:
            return false;
    }
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isnan(x);
#else
    #error Unsupported platform
#endif
}

bool Math::isNan(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    switch (_fpclass(x))
    {
        case _FPCLASS_SNAN:
        case _FPCLASS_QNAN:
            return true;
        default:
            return false;
    }
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isnan(x);
#else
    #error Unsupported platform
#endif
}

bool Math::isNan(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_SignalingNan ||
           x.getCategory() == BinaryFloat::ValueCategory_QuietNan;
}

bool Math::isNan(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_SignalingNan ||
           x.getCategory() == DecimalFloat::ValueCategory_QuietNan;
}

bool Math::isSignalingNan(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_SNAN;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return false;
#else
    #error Unsupported platform
#endif
}

bool Math::isSignalingNan(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_SNAN;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return false;
#else
    #error Unsupported platform
#endif
}

bool Math::isSignalingNan(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_SignalingNan;
}

bool Math::isSignalingNan(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_SignalingNan;
}

bool Math::isQuietNan(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_QNAN;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isnan(x);
#else
    #error Unsupported platform
#endif
}

bool Math::isQuietNan(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_QNAN;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isnan(x);
#else
    #error Unsupported platform
#endif
}

bool Math::isQuietNan(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_QuietNan;
}

bool Math::isQuietNan(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_QuietNan;
}

bool Math::isInfinite(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    switch (_fpclass(x))
    {
        case _FPCLASS_NINF:
        case _FPCLASS_PINF:
            return true;
        default:
            return false;
    }
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    int isInf = isinf(x);
    return isInf == -1 || isInf == 1;
#else
    #error Unsupported platform
#endif
}

bool Math::isInfinite(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    switch (_fpclass(x))
    {
        case _FPCLASS_NINF:
        case _FPCLASS_PINF:
            return true;
        default:
            return false;
    }
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    int isInf = isinf(x);
    return isInf == -1 || isInf == 1;
#else
    #error Unsupported platform
#endif
}

bool Math::isInfinite(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_NegativeInfinity ||
           x.getCategory() == BinaryFloat::ValueCategory_PositiveInfinity;
}

bool Math::isInfinite(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_NegativeInfinity ||
           x.getCategory() == DecimalFloat::ValueCategory_PositiveInfinity;
}

bool Math::isPositiveInfinite(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_PINF;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isinf(x) == 1;
#else
    #error Unsupported platform
#endif
}

bool Math::isPositiveInfinite(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_PINF;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isinf(x) == 1;
#else
    #error Unsupported platform
#endif
}

bool Math::isPositiveInfinite(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_PositiveInfinity;
}

bool Math::isPositiveInfinite(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_PositiveInfinity;
}

bool Math::isNegativeInfinite(Real32 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_NINF;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isinf(x) == -1;
#else
    #error Unsupported platform
#endif
}

bool Math::isNegativeInfinite(Real64 x)
{
#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
    return _fpclass(x) == _FPCLASS_NINF;
#elif (defined(CARL_CPU_IA32) && defined(CARL_OS_CYGWIN32) && defined(CARL_TOOLCHAIN_GCC)) || \
      (defined(CARL_CPU_IA32) && defined(CARL_OS_LINUX32) && defined(CARL_TOOLCHAIN_GCC))
    return isinf(x) == -1;
#else
    #error Unsupported platform
#endif
}

bool Math::isNegativeInfinite(const BinaryFloat & x)
{
    return x.getCategory() == BinaryFloat::ValueCategory_NegativeInfinity;
}

bool Math::isNegativeInfinite(const DecimalFloat & x)
{
    return x.getCategory() == DecimalFloat::ValueCategory_NegativeInfinity;
}

//  End of Math.cpp
