//
//  Memory.cpp - Implementation of the carl::Memory class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <string.h>

//////////
//  Byte order services - unaligned access
UInt16 Memory::loadUnalignedUInt16(const void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    const unsigned char * bytes = static_cast<const unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        return (static_cast<UInt16>(bytes[0]) << 8) |
                static_cast<UInt16>(bytes[1]);
    }
    else
    {
        return  static_cast<UInt16>(bytes[0]) |
               (static_cast<UInt16>(bytes[1]) << 8);
    }
}

UInt32 Memory::loadUnalignedUInt32(const void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    const unsigned char * bytes = static_cast<const unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        return (static_cast<UInt32>(bytes[0]) << 24) |
               (static_cast<UInt32>(bytes[1]) << 16) |
               (static_cast<UInt32>(bytes[2]) <<  8) |
                static_cast<UInt32>(bytes[3]);
    }
    else
    {
        return  static_cast<UInt32>(bytes[0]) |
               (static_cast<UInt32>(bytes[1]) <<  8) |
               (static_cast<UInt32>(bytes[2]) << 16) |
               (static_cast<UInt32>(bytes[3]) << 24);
    }
}

UInt64 Memory::loadUnalignedUInt64(const void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    const unsigned char * bytes = static_cast<const unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        return (static_cast<UInt64>(bytes[0]) << 56) |
               (static_cast<UInt64>(bytes[1]) << 48) |
               (static_cast<UInt64>(bytes[2]) << 40) |
               (static_cast<UInt64>(bytes[3]) << 32) |
               (static_cast<UInt64>(bytes[4]) << 24) |
               (static_cast<UInt64>(bytes[5]) << 16) |
               (static_cast<UInt64>(bytes[6]) <<  8) |
                static_cast<UInt64>(bytes[7]);
    }
    else
    {
        return  static_cast<UInt64>(bytes[0]) |
               (static_cast<UInt64>(bytes[1]) <<  8) |
               (static_cast<UInt64>(bytes[2]) << 16) |
               (static_cast<UInt64>(bytes[3]) << 24) |
               (static_cast<UInt64>(bytes[4]) << 32) |
               (static_cast<UInt64>(bytes[5]) << 40) |
               (static_cast<UInt64>(bytes[6]) << 48) |
               (static_cast<UInt64>(bytes[7]) << 56);
    }
}

Int16 Memory::loadUnalignedInt16(const void * pRawData, ByteOrder rawDataByteOrder)
{
    return static_cast<Int16>(loadUnalignedUInt16(pRawData, rawDataByteOrder));
}

Int32 Memory::loadUnalignedInt32(const void * pRawData, ByteOrder rawDataByteOrder)
{
    return static_cast<Int32>(loadUnalignedUInt32(pRawData, rawDataByteOrder));
}

Int64 Memory::loadUnalignedInt64(const void * pRawData, ByteOrder rawDataByteOrder)
{
    return static_cast<Int64>(loadUnalignedUInt64(pRawData, rawDataByteOrder));
}

Real32 Memory::loadUnalignedReal32(const void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    carl::UInt32 bits = loadUnalignedUInt32(pRawData, rawDataByteOrder);
    return *(carl::Real32*)(&bits);
}

Real64 Memory::loadUnalignedReal64(const void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    carl::UInt64 bits = loadUnalignedUInt64(pRawData, rawDataByteOrder);
    return *(carl::Real64*)(&bits);
}

void Memory::storeUnalignedUInt16(UInt16 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    unsigned char * bytes = static_cast<unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        bytes[0] = static_cast<UInt8>(value >> 8);
        bytes[1] = static_cast<UInt8>(value);
    }
    else
    {
        bytes[0] = static_cast<UInt8>(value);
        bytes[1] = static_cast<UInt8>(value >> 8);
    }
}

void Memory::storeUnalignedUInt32(UInt32 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    unsigned char * bytes = static_cast<unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        bytes[0] = static_cast<UInt8>(value >> 24);
        bytes[1] = static_cast<UInt8>(value >> 16);
        bytes[2] = static_cast<UInt8>(value >> 8);
        bytes[3] = static_cast<UInt8>(value);
    }
    else
    {
        bytes[0] = static_cast<UInt8>(value);
        bytes[1] = static_cast<UInt8>(value >> 8);
        bytes[2] = static_cast<UInt8>(value >> 16);
        bytes[3] = static_cast<UInt8>(value >> 24);
    }
}

void Memory::storeUnalignedUInt64(UInt64 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    unsigned char * bytes = static_cast<unsigned char *>(pRawData);
    if (rawDataByteOrder == ByteOrder_BigEndian)
    {
        bytes[0] = static_cast<UInt8>(value >> 56);
        bytes[1] = static_cast<UInt8>(value >> 48);
        bytes[2] = static_cast<UInt8>(value >> 40);
        bytes[3] = static_cast<UInt8>(value >> 32);
        bytes[4] = static_cast<UInt8>(value >> 24);
        bytes[5] = static_cast<UInt8>(value >> 16);
        bytes[6] = static_cast<UInt8>(value >> 8);
        bytes[7] = static_cast<UInt8>(value);
    }
    else
    {
        bytes[0] = static_cast<UInt8>(value);
        bytes[1] = static_cast<UInt8>(value >> 8);
        bytes[2] = static_cast<UInt8>(value >> 16);
        bytes[3] = static_cast<UInt8>(value >> 24);
        bytes[4] = static_cast<UInt8>(value >> 32);
        bytes[5] = static_cast<UInt8>(value >> 40);
        bytes[6] = static_cast<UInt8>(value >> 48);
        bytes[7] = static_cast<UInt8>(value >> 56);
    }
}

void Memory::storeUnalignedInt16(Int16 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    storeUnalignedUInt16(static_cast<UInt16>(value), pRawData, rawDataByteOrder);
}

void Memory::storeUnalignedInt32(Int32 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    storeUnalignedUInt32(static_cast<UInt32>(value), pRawData, rawDataByteOrder);
}

void Memory::storeUnalignedInt64(Int64 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    storeUnalignedUInt64(static_cast<UInt64>(value), pRawData, rawDataByteOrder);
}

void Memory::storeUnalignedReal32(Real32 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    carl::UInt32 bits;
    *(carl::Real32*)(&bits) = value;
    storeUnalignedUInt32(bits, pRawData, rawDataByteOrder);
}

void Memory::storeUnalignedReal64(Real64 value, void * pRawData, ByteOrder rawDataByteOrder)
{
    require(pRawData != NULL);

    carl::UInt64 bits;
    *(carl::Real64*)(&bits) = value;
    storeUnalignedUInt64(bits, pRawData, rawDataByteOrder);
}

//////////
//  pointer <-> integer translations
UIntPtr Memory::addressOf(const void * pointer)
{
    return static_cast<UIntPtr>(reinterpret_cast<size_t>(pointer));
}

void * Memory::pointerTo(UIntPtr address)
{
    return reinterpret_cast<void*>(static_cast<size_t>(address));
}

//////////
//  Operations on raw memory blocks
void Memory::copy(void * pDestination, const void * pSource, UIntSize byteCount)
{
    memmove(pDestination, pSource, byteCount);
}

void Memory::set(void * pDestination, UInt8 byte, UIntSize byteCount)
{
    memset(pDestination, byte, byteCount);
}

int Memory::compare(const void * pBlock1, const void * pBlock2, UIntSize byteCount)
{
    return memcmp(pBlock1, pBlock2, byteCount);
}

//  End of Memory.cpp
