//
//  MemoryInputStream.cpp - Implementation of the carl::MemoryInputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
MemoryInputStream::MemoryInputStream(const void * pBuffer, UIntSize bufferSize, bool makePrivateCopy)
{
    require(pBuffer != NULL || bufferSize == 0);

    _isOpen = true;
    if (makePrivateCopy)
    {
        if (bufferSize == 0)
        {
            _pData = 0;
        }
        else
        {
            _pData = new UInt8[bufferSize];
            Memory::copy(const_cast<UInt8*>(_pData), pBuffer, bufferSize);
        }
        _dataSize = bufferSize;
    }
    else
    {
        _pData = static_cast<const UInt8*>(pBuffer);
        _dataSize = bufferSize;
    }
    _currentPosition = 0;
    _isPrivateData = makePrivateCopy;
}

//////////
//  Operations
void MemoryInputStream::close()
{
    if (_isOpen)
    {
        if (_isPrivateData)
        {
            delete [] _pData;
        }
        _isOpen = false;
    }
}

//////////
//  IByteInput
UIntSize MemoryInputStream::available() throw(IOException)
{
    require(_isOpen);

    return _dataSize - _currentPosition;
}

int MemoryInputStream::read() throw(IOException)
{
    require(_isOpen);

    if (_currentPosition < _dataSize)
    {
        return _pData[_currentPosition++];
    }
    else
    {
        return -1;
    }
}

UIntSize MemoryInputStream::read(void * pBuffer, UIntSize bytesToRead) throw(IOException)
{
    require(pBuffer != NULL);
    require(_isOpen);

    if (bytesToRead >= available())
    {
        UIntSize bytesRead = available();
        Memory::copy(pBuffer, _pData + _currentPosition, bytesRead);
        _currentPosition = _dataSize;
        return bytesRead;
    }
    else
    {
        Memory::copy(pBuffer, _pData + _currentPosition, bytesToRead);
        _currentPosition += bytesToRead;
        return bytesToRead;
    }
}

UIntSize MemoryInputStream::skip(UIntSize bytesToSkip) throw(IOException)
{
    require(_isOpen);

    if (bytesToSkip >= available())
    {
        UIntSize bytesSkipped = available();
        _currentPosition = _dataSize;
        return bytesSkipped;
    }
    else
    {
        _currentPosition += bytesToSkip;
        return bytesToSkip;
    }
}

//  End of MemoryInputStream.cpp
