//
//  NoSuchHeapBlockError.cpp - Implementation of the carl::NoSuchHeapBlockError class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "ExceptionImpl.hpp"

//////////
//  Construction/destruction/assignment
NoSuchHeapBlockError::NoSuchHeapBlockError(const void * pBlock, const char * pSourceFileName, UIntSize sourceLineNumber)
    :   _pBlock(pBlock),
        _pSourceFileName(pSourceFileName),
        _sourceLineNumber(sourceLineNumber)
{
}

NoSuchHeapBlockError::NoSuchHeapBlockError(const NoSuchHeapBlockError & src)
    :   _pBlock(src._pBlock),
        _pSourceFileName(src._pSourceFileName),
        _sourceLineNumber(src._sourceLineNumber)
{
}

NoSuchHeapBlockError::~NoSuchHeapBlockError()
{
}

NoSuchHeapBlockError & NoSuchHeapBlockError::operator = (const NoSuchHeapBlockError & src)
{
    if (this != (&src))
    {
        _pBlock = src._pBlock;
        _pSourceFileName = src._pSourceFileName;
        _sourceLineNumber = src._sourceLineNumber;
    }
    return *this;
}

//////////
//  Operations
const void * NoSuchHeapBlockError::getBlock() const
{
    return _pBlock;
}

const char * NoSuchHeapBlockError::getSourceFileName() const
{
    return _pSourceFileName;
}

UIntSize NoSuchHeapBlockError::getSourceLineNumber() const
{
    return _sourceLineNumber;
}

String NoSuchHeapBlockError::toString() const
{
    String blockAddress = NumberFormat::getInstance()->format(Memory::addressOf(_pBlock), IntegerFormatProperties("%8.8X"));
    String fileName = (_pSourceFileName == NULL) ? String("?") : String(_pSourceFileName);
    String lineNumber = (_sourceLineNumber == 0) ? String("?") : NumberFormat::getInstance()->format(_sourceLineNumber);
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("NoSuchHeapBlockError", blockAddress, fileName, lineNumber);
}

String NoSuchHeapBlockError::toString(const Locale & forLocale) const
{
    String blockAddress = NumberFormat::getInstance()->format(Memory::addressOf(_pBlock), IntegerFormatProperties("%8.8X"));
    String fileName = (_pSourceFileName == NULL) ? String("?") : String(_pSourceFileName);
    String lineNumber = (_sourceLineNumber == 0) ? String("?") : NumberFormat::getInstance()->format(_sourceLineNumber);
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("NoSuchHeapBlockError", blockAddress, fileName, lineNumber, forLocale);
}

//  End of NoSuchHeapBlockError.cpp
