//
//  NonstandardSystemTimeZone.cpp - Implementation of the carl::_impl::_NonstandardSystemTimeZone class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.4 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeImpl.hpp"

//////////
//  Construction/destruction
_impl::_NonstandardSystemTimeZone::_NonstandardSystemTimeZone(Int64 bias,
                const String & name,
                const String & displayName)
    :   _SystemTimeZone(bias, name)
{
    _displayName = _standardDisplayName = _daylightDisplayName = displayName;
}

_impl::_NonstandardSystemTimeZone::_NonstandardSystemTimeZone(Int64 bias, 
                Int64 standardBias, const _DaylightSwitchpoint & standardSwitchpoint, 
                Int64 daylightBias, const _DaylightSwitchpoint & daylightSwitchpoint,
                const String & name,
                const String & displayName,
                const String & standardDisplayName,
                const String & daylightDisplayName)
    :   _SystemTimeZone(bias, standardBias, standardSwitchpoint, daylightBias, daylightSwitchpoint, name)
{
    _displayName = displayName;
    _standardDisplayName = standardDisplayName;
    _daylightDisplayName = daylightDisplayName;
}

_impl::_NonstandardSystemTimeZone::~_NonstandardSystemTimeZone()
{
}

//////////
//  Operations
String _impl::_NonstandardSystemTimeZone::getDisplayName() const
{
    return _displayName;
}

String _impl::_NonstandardSystemTimeZone::getDisplayName(const Locale & /*forLocale*/) const
{
    return _displayName;
}

String _impl::_NonstandardSystemTimeZone::getDisplayName(const DateTime & dateTime) const
{
    if (_isDst(dateTime))
    {
        return _daylightDisplayName;
    }
    else
    {
        return _standardDisplayName;
    }
}

String _impl::_NonstandardSystemTimeZone::getDisplayName(const DateTime & dateTime, const Locale & /*forLocale*/) const
{
    if (_isDst(dateTime))
    {
        return _daylightDisplayName;
    }
    else
    {
        return _standardDisplayName;
    }
}

//  End of NonstandardSystemTimeZone.cpp
