//
//  NullCipher.cpp - Implementation of the carl::_impl::_NullCipher class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Singleton
_impl::_NullCipher::_NullCipher()
{
}

_impl::_NullCipher::~_NullCipher()
{
}

_impl::_NullCipher * _impl::_NullCipher::getInstance()
{
    static _NullCipher instance;
    return &instance;
}

//////////
//  SymmetricCipher
Array<UIntSize> _impl::_NullCipher::getSupportedKeyLenghts() const
{
    Array<UIntSize> result;
    result.append(0);
    return result;
}

SecretKey _impl::_NullCipher::generateKey(UInt64 /*seed*/, UIntSize keyLengthBits) const
{
    if (keyLengthBits != 0)
    {
        throw InvalidKeyLengthError();
    }
    UInt8 b = 0;
    return SecretKey(&b, 0);
}

SecretKey _impl::_NullCipher::generateRandomKey(UIntSize keyLengthBits) const
{
    if (keyLengthBits != 0)
    {
        throw InvalidKeyLengthError();
    }
    UInt8 b = 0;
    return SecretKey(&b, 0);
}

//////////
//  Cipher
Array<String> _impl::_NullCipher::getAllNames() const
{
    Array<String> result;
    result.append(getCanonicalName());
    return result;
}

String _impl::_NullCipher::getCanonicalName() const
{
    return String("NULL");
}

String _impl::_NullCipher::getPreferredName() const
{
    return getCanonicalName();
}

String _impl::_NullCipher::getDisplayName() const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName());
}

String _impl::_NullCipher::getDisplayName(const Locale & forLocale) const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName(), forLocale);
}

Encryptor * _impl::_NullCipher::createEncryptor(const CryptoKey & key) const throw(KeyCorruptException)
{
    if (key.getKeyLengthBits() != 0)
    {
        throw InvalidKeyLengthError();
    }
    return new _NullEncryptor();
}

Decryptor * _impl::_NullCipher::createDecryptor(const CryptoKey & key) const throw(KeyCorruptException)
{
    if (key.getKeyLengthBits() != 0)
    {
        throw InvalidKeyLengthError();
    }
    return new _NullDecryptor();
}

//  End of NullCipher.cpp
