//
//  NullCompressionAlgorithm.cpp - Implementation of the carl::_impl::_NullCompressionAlgorithm class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_NullCompressionAlgorithm::_NullCompressionAlgorithm()
{
}

_impl::_NullCompressionAlgorithm::~_NullCompressionAlgorithm()
{
}

_impl::_NullCompressionAlgorithm * _impl::_NullCompressionAlgorithm::getInstance()
{
    static _NullCompressionAlgorithm instance;
    return &instance;
}

//////////
//  Operations
Array<String> _impl::_NullCompressionAlgorithm::getAllNames() const
{
    Array<String> result;
    result.append(getCanonicalName());
    return result;
}

String _impl::_NullCompressionAlgorithm::getCanonicalName() const
{
    return String("NULL");
}

String _impl::_NullCompressionAlgorithm::getPreferredName() const
{
    return getCanonicalName();
}

String _impl::_NullCompressionAlgorithm::getDisplayName() const
{
    return _CompressionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName());
}

String _impl::_NullCompressionAlgorithm::getDisplayName(const Locale & forLocale) const
{
    return _CompressionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName(), forLocale);
}

UIntSize _impl::_NullCompressionAlgorithm::getMinimumBlockSize() const
{
    return 0;
}

UIntSize _impl::_NullCompressionAlgorithm::getPreferredBlockSize() const
{
    return static_cast<UIntSize>(~static_cast<UIntSize>(0));
}

UIntSize _impl::_NullCompressionAlgorithm::getMaximumBlockSize() const
{
    return static_cast<UIntSize>(~static_cast<UIntSize>(0));
}

Compressor * _impl::_NullCompressionAlgorithm::getCompressor() const
{
    return _NullCompressor::getInstance();
}

Decompressor * _impl::_NullCompressionAlgorithm::getDecompressor() const
{
    return _NullDecompressor::getInstance();
}

//  End of NullCompressionAlgorithm.cpp
