//
//  NumberFormatImpl.hpp - carl::NumberFormat implementation helpers
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.12 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#ifndef __NumberFormatImpl_hpp_included__
#define __NumberFormatImpl_hpp_included__

namespace carl
{
    namespace _impl
    {
        //////////
        //  Number format manager
        class CARL_PUBLIC _NumberFormatManager
        {
            //////////
            //  Can't assign or copy construct
        private:
            _NumberFormatManager(const _NumberFormatManager &);
            void                            operator = (const _NumberFormatManager &);

            //////////
            //  Singleton
        private:
            _NumberFormatManager();
            ~_NumberFormatManager();
        public:
            static _NumberFormatManager *   getInstance();

            //////////
            //  Operations
        public:
            void                            registerNumberFormat(const Locale & forLocale, NumberFormat * pNumberFormat);
            NumberFormat *                  getNumberFormat(const Locale & forLocale);

            //////////
            //  Implementation
        private:
            HashMap<Locale, NumberFormat*>  _mapLocalesToNumberFormats;
            NumberFormat *                  _pDefaultNumberFormat;
            CriticalSection                 _guard;
        };

        //////////
        //  Default (invariant) number format
        class CARL_PUBLIC _DefaultNumberFormat : public NumberFormat
        {
            //////////
            //  Can't assign or copy construct
        private:
            _DefaultNumberFormat(const _DefaultNumberFormat &);
            void                    operator = (const _DefaultNumberFormat &);

            //////////
            //  Construction/destruction
        public:
            _DefaultNumberFormat(Char decimalPointChar, Char separatorChar);
            virtual ~_DefaultNumberFormat();

            //////////
            //  Operations
        public:
            //  Custom formatting
            virtual String          format(Int8 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(Int16 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(Int32 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(Int64 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(UInt8 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(UInt16 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(UInt32 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(UInt64 value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(const BinaryInteger & value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(const DecimalInteger & value, const IntegerFormatProperties & formatProperties) const;
            virtual String          format(Real32 value, const RealFormatProperties & formatProperties) const;
            virtual String          format(Real64 value, const RealFormatProperties & formatProperties) const;
            virtual String          format(const BinaryFixed & value, const RealFormatProperties & formatProperties) const;
            virtual String          format(const BinaryFloat & value, const RealFormatProperties & formatProperties) const;
            virtual String          format(const DecimalFixed & value, const RealFormatProperties & formatProperties) const;
            virtual String          format(const DecimalFloat & value, const RealFormatProperties & formatProperties) const;

            //////////
            //  Implementation
        private:
            Char                    _decimalPointChar;
            Char                    _separatorChar;
            //  Helpers
            String                  _postFormatIntegerValue(String valueString, bool valueIsNegative, const IntegerFormatProperties & formatProperties) const;
            static String           _postFormatRealValue(String valueString, const RealFormatProperties & formatProperties);
        };
    }
}

#endif  //  __NumberFormatImpl_hpp_included__
//  End of NumberFormatImpl.hpp
