//
//  NumberParse.cpp - Implementation of the carl::NumberParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.12 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "NumberParseImpl.hpp"
#include <ctype.h>

//////////
//  Operations
void NumberParse::registerInstance(const Locale & forLocale, NumberParse * pNumberParse)
{
    require(pNumberParse != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_NumberParseManager::getInstance()->registerNumberParse(forLocale, pNumberParse);
}

NumberParse * NumberParse::getInstance()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(Locale::getDefault());
}

NumberParse * NumberParse::getInstance(const Locale & forLocale)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_NumberParseManager::getInstance()->getNumberParse(forLocale);
}

Int8 NumberParse::parseInt8(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int16 NumberParse::parseInt16(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int32 NumberParse::parseInt32(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int64 NumberParse::parseInt64(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(pString, position);
    _ensureEos(pString, position);
    return value;
}

Real32 NumberParse::parseReal32(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Real64 NumberParse::parseReal64(const char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(pString, position);
    _ensureEos(pString, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(pString, position);
    _ensureEos(pString, position);
    return value;
}

Int8 NumberParse::parseInt8(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int16 NumberParse::parseInt16(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int32 NumberParse::parseInt32(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int64 NumberParse::parseInt64(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(pString, position);
    _ensureEos(pString, position);
    return value;
}

Real32 NumberParse::parseReal32(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Real64 NumberParse::parseReal64(const Char * pString, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(pString, position, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(pString, position);
    _ensureEos(pString, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(pString, position);
    _ensureEos(pString, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const Char * pString) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(pString, position);
    _ensureEos(pString, position);
    return value;
}

Int8 NumberParse::parseInt8(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int16 NumberParse::parseInt16(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int32 NumberParse::parseInt32(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int64 NumberParse::parseInt64(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(string, position);
    _ensureEos(string, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(string, position);
    _ensureEos(string, position);
    return value;
}

Real32 NumberParse::parseReal32(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

Real64 NumberParse::parseReal64(const String & string, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(string, position, pOverflow);
    _ensureEos(string, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(string, position);
    _ensureEos(string, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(string, position);
    _ensureEos(string, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(string, position);
    _ensureEos(string, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const String & string) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(string, position);
    _ensureEos(string, position);
    return value;
}

Int8 NumberParse::parseInt8(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int16 NumberParse::parseInt16(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int32 NumberParse::parseInt32(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int64 NumberParse::parseInt64(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const char * pString, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const char * pString, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

Real32 NumberParse::parseReal32(const char * pString, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Real64 NumberParse::parseReal64(const char * pString, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

Int8 NumberParse::parseInt8(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int16 NumberParse::parseInt16(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int32 NumberParse::parseInt32(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Int64 NumberParse::parseInt64(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const Char * pString, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const Char * pString, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const Char * pString, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

Real32 NumberParse::parseReal32(const Char * pString, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

Real64 NumberParse::parseReal64(const Char * pString, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(pString, position, parseProperties, pOverflow);
    _ensureEos(pString, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const Char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const Char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const Char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const Char * pString, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(pString, position, parseProperties);
    _ensureEos(pString, position);
    return value;
}

Int8 NumberParse::parseInt8(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int8 value = parseInt8(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int16 NumberParse::parseInt16(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int16 value = parseInt16(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int32 NumberParse::parseInt32(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int32 value = parseInt32(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

Int64 NumberParse::parseInt64(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Int64 value = parseInt64(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt8 NumberParse::parseUInt8(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt8 value = parseUInt8(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt16 NumberParse::parseUInt16(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt16 value = parseUInt16(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt32 NumberParse::parseUInt32(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt32 value = parseUInt32(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

UInt64 NumberParse::parseUInt64(const String & string, const IntegerParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    UInt64 value = parseUInt64(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

BinaryInteger NumberParse::parseBinaryInteger(const String & string, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryInteger value = parseBinaryInteger(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

DecimalInteger NumberParse::parseDecimalInteger(const String & string, const IntegerParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalInteger value = parseDecimalInteger(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

Real32 NumberParse::parseReal32(const String & string, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real32 value = parseReal32(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

Real64 NumberParse::parseReal64(const String & string, const RealParseProperties & parseProperties, bool * pOverflow /*= NULL*/) const throw(ParseException)
{
    UIntSize position = 0;
    Real64 value = parseReal64(string, position, parseProperties, pOverflow);
    _ensureEos(string, position);
    return value;
}

BinaryFixed NumberParse::parseBinaryFixed(const String & string, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFixed value = parseBinaryFixed(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

BinaryFloat NumberParse::parseBinaryFloat(const String & string, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    BinaryFloat value = parseBinaryFloat(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

DecimalFixed NumberParse::parseDecimalFixed(const String & string, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFixed value = parseDecimalFixed(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

DecimalFloat NumberParse::parseDecimalFloat(const String & string, const RealParseProperties & parseProperties) const throw(ParseException)
{
    UIntSize position = 0;
    DecimalFloat value = parseDecimalFloat(string, position, parseProperties);
    _ensureEos(string, position);
    return value;
}

//////////
//  Implementationh elpers
void NumberParse::_skipSpaces(const char * pString, UIntSize & position)
{
    while (pString[position] != 0 &&
           isspace(static_cast<unsigned char>(pString[position])))
    {
        position++;
    }
}

void NumberParse::_skipSpaces(const Char * pString, UIntSize & position)
{
    while (pString[position] != 0 && Iso10646::isSpace(pString[position]))
    {
        position++;
    }
}

void NumberParse::_skipSpaces(const String & string, UIntSize & position)
{
    while (position < string.length() && Iso10646::isSpace(string[position]))
    {
        position++;
    }
}

void NumberParse::_ensureEos(const char * pString, UIntSize position)
{
    _skipSpaces(pString, position);
    if (pString[position] != 0)
    {
        throw ParseException(String(pString), position);
    }
}

void NumberParse::_ensureEos(const Char * pString, UIntSize position)
{
    _skipSpaces(pString, position);
    if (pString[position] != 0)
    {
        throw ParseException(String(pString), position);
    }
}

void NumberParse::_ensureEos(const String & string, UIntSize position)
{
    _skipSpaces(string, position);
    if (position < string.length())
    {
        throw ParseException(string, position);
    }
}

//////////
//  Implementation
CriticalSection * NumberParse::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of NumberParse.cpp
