//
//  NumberParseManager.cpp - Implementation of the carl::_impl::_NumberParseManager class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "NumberParseImpl.hpp"

//////////
//  Singleton
_impl::_NumberParseManager::_NumberParseManager()
    :   _mapLocalesToNumberParses(StandardHashFunctions::hashLocale)
{
    //  Set up the default parse
    _pDefaultNumberParse = new _DefaultNumberParse('.', ',');
}

_impl::_NumberParseManager::~_NumberParseManager()
{
    delete _pDefaultNumberParse;
}

_impl::_NumberParseManager * _impl::_NumberParseManager::getInstance()
{
    static _NumberParseManager instance;
    return &instance;
}

//////////
//  Operations
void _impl::_NumberParseManager::registerNumberParse(const Locale & forLocale, NumberParse * pNumberParse)
{
    require(pNumberParse != NULL);

    CriticalSectionLock lock(_guard);
    _mapLocalesToNumberParses.insert(forLocale, pNumberParse);
}

NumberParse * _impl::_NumberParseManager::getNumberParse(const Locale & forLocale)
{
    CriticalSectionLock lock(_guard);
    for (Locale locale = forLocale; ; locale = locale.getParent())
    {
        if (_mapLocalesToNumberParses.containsKey(locale))
        {
            return _mapLocalesToNumberParses.get(locale);
        }
        if (locale.isInvariant())
        {
            break;
        }
    }
    return _pDefaultNumberParse;
}

//  End of NumberParseManager.cpp
