//
//  PaddingAlgorithm.cpp - Implementation of the carl::PaddingAlgorithm class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Operations
void PaddingAlgorithm::registerInstance(PaddingAlgorithm * pPaddingAlgorithm) throw (InconsistentPaddingAlgorithmException, DuplicatePaddingAlgorithmException)
{
    require(pPaddingAlgorithm != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_EncryptionManagerImpl::getInstance()->registerPaddingAlgorithm(pPaddingAlgorithm);
}

Array<PaddingAlgorithm*> PaddingAlgorithm::getAllInstances()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_EncryptionManagerImpl::getInstance()->getAllPaddingAlgorithms();
}

PaddingAlgorithm * PaddingAlgorithm::getInstance(const char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

PaddingAlgorithm * PaddingAlgorithm::getInstance(const Char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

PaddingAlgorithm * PaddingAlgorithm::getInstance(const String & name)
{
    CriticalSectionLock lock(*_getStaticGuard());
    PaddingAlgorithm * pPaddingAlgorithm = _impl::_EncryptionManagerImpl::getInstance()->findPaddingAlgorithm(name);
    if (pPaddingAlgorithm != NULL)
    {
        return pPaddingAlgorithm;
    }
    throw NoSuchElementError();
}

//////////
//  Implementation
CriticalSection * PaddingAlgorithm::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of PaddingAlgorithm.cpp
