//
//  ParseException.cpp - Implementation of the carl::ParseException class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "ExceptionImpl.hpp"

//////////
//  Construction/destruction/assignment
ParseException::ParseException(const String & parsedString, UIntSize parsePosition)
    :   _parsedString(parsedString),
        _parsePosition(parsePosition)
{
}

ParseException::ParseException(const ParseException & src)
    :   _parsedString(src._parsedString),
        _parsePosition(src._parsePosition)
{
}

ParseException::~ParseException()
{
}

ParseException & ParseException::operator = (const ParseException & src)
{
    if (this != (&src))
    {
        _parsedString = src._parsedString;
        _parsePosition = src._parsePosition;
    }
    return *this;
}

//////////
//  Operations
String ParseException::getParsedString() const
{
    return _parsedString;
}

UIntSize ParseException::getParsePosition() const
{
    return _parsePosition;
}

String ParseException::toString() const
{
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("ParseException", _parsedString, NumberFormat::getInstance()->format(_parsePosition));
}

String ParseException::toString(const Locale & forLocale) const
{
    return _impl::_ExceptionManagerImpl::getInstance()->getResourceFactory()->getParameterizedString("ParseException", _parsedString, NumberFormat::getInstance()->format(_parsePosition), forLocale);
}

//  End of ParseException.cpp
