//
//  Pkcs5PaddingAlgorithm.cpp - Implementation of the carl::_impl::_Pkcs5PaddingAlgorithm class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Singleton
_impl::_Pkcs5PaddingAlgorithm::_Pkcs5PaddingAlgorithm()
{
}

_impl::_Pkcs5PaddingAlgorithm::~_Pkcs5PaddingAlgorithm()
{
}

_impl::_Pkcs5PaddingAlgorithm * _impl::_Pkcs5PaddingAlgorithm::getInstance()
{
    static _Pkcs5PaddingAlgorithm instance;
    return &instance;
}

//////////
//  Operations
Array<String> _impl::_Pkcs5PaddingAlgorithm::getAllNames() const
{
    Array<String> result;
    result.append(getCanonicalName());
    return result;
}

String _impl::_Pkcs5PaddingAlgorithm::getCanonicalName() const
{
    return String("PKCS5");
}

String _impl::_Pkcs5PaddingAlgorithm::getPreferredName() const
{
    return getCanonicalName();
}

String _impl::_Pkcs5PaddingAlgorithm::getDisplayName() const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName());
}

String _impl::_Pkcs5PaddingAlgorithm::getDisplayName(const Locale & forLocale) const
{
    return _EncryptionManagerImpl::getInstance()->getResourceFactory()->getString(getCanonicalName(), forLocale);
}

UIntSize _impl::_Pkcs5PaddingAlgorithm::getPaddingBitsRequiredForData(const void * /*pDataBits*/, UIntSize dataBitsCount, UIntSize blockSizeBits) const
{
    require(System::BitsPerByte == 8);
    require(blockSizeBits % 8 == 0);
    require(blockSizeBits / 8 <= 255);
    require(dataBitsCount % 8 == 0);
    require(dataBitsCount <= blockSizeBits);

    UIntSize result = blockSizeBits - dataBitsCount;
    if (result == 0)
    {
        result = blockSizeBits;
    }
    return result;

}

UIntSize _impl::_Pkcs5PaddingAlgorithm::createPaddingBitsForData(const void * /*pDataBits*/, UIntSize dataBitsCount, UIntSize blockSizeBits, void * pPaddingBits) const
{
    require(System::BitsPerByte == 8);
    require(blockSizeBits % 8 == 0);
    require(blockSizeBits / 8 <= 255);
    require(dataBitsCount % 8 == 0);
    require(dataBitsCount <= blockSizeBits);

    UIntSize paddingBitsNeeded = blockSizeBits - dataBitsCount;
    if (paddingBitsNeeded == 0)
    {
        paddingBitsNeeded = blockSizeBits;
    }
    UIntSize paddingBytetsNeeded = paddingBitsNeeded / 8;
    UInt8 * pPaddingBytes = static_cast<UInt8*>(pPaddingBits);
    for (UIntSize i = 0; i < paddingBytetsNeeded; i++)
    {
        pPaddingBytes[i] = static_cast<UInt8>(paddingBytetsNeeded);
    }
    return paddingBitsNeeded;
}

UIntSize _impl::_Pkcs5PaddingAlgorithm::getPaddingBitsStoredInData(const void * pDataBits, UIntSize dataBitsCount, UIntSize blockSizeBits) const throw(PaddedDataCorruptException)
{
    require(System::BitsPerByte == 8);
    require(blockSizeBits % 8 == 0);
    require(blockSizeBits / 8 <= 255);
    require(dataBitsCount % 8 == 0);
    require(dataBitsCount == blockSizeBits);
    require(dataBitsCount != 0);

    const UInt8 * pDataBytes = static_cast<const UInt8*>(pDataBits);
    UInt8 lastByte = pDataBytes[dataBitsCount / 8 - 1];
    if (lastByte * 8u > dataBitsCount)
    {   //  Invalid padding
        throw PaddedDataCorruptException();
    }
    for (UIntSize offset = 0; offset < lastByte; offset++)
    {
        if (pDataBytes[dataBitsCount / 8 - 1 - offset] != lastByte)
        {   //  Invalid padding
            throw PaddedDataCorruptException();
        }
    }
    return lastByte * 8;
}


//  End of Pkcs5PaddingAlgorithm.cpp
