//
//  Random.cpp - Implementation of the carl::Random class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:53 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Operations
Int8 Random::nextInt8()
{
    return static_cast<Int8>(_next());
}

Int8 Random::nextInt8(Int8 lowerLimit, Int8 upperLimit)
{
    UInt8 resultOffset = nextUInt8(0, upperLimit - lowerLimit);
    return lowerLimit + resultOffset;
}

Int16 Random::nextInt16()
{
    return static_cast<Int16>(nextUInt16());
}

Int16 Random::nextInt16(Int16 lowerLimit, Int16 upperLimit)
{
    UInt16 resultOffset = nextUInt16(0, upperLimit - lowerLimit);
    return lowerLimit + resultOffset;
}

Int32 Random::nextInt32()
{
    return static_cast<Int32>(nextUInt32());
}

Int32 Random::nextInt32(Int32 lowerLimit, Int32 upperLimit)
{
    UInt32 resultOffset = nextUInt32(0, upperLimit - lowerLimit);
    return lowerLimit + resultOffset;
}

Int64 Random::nextInt64()
{
    return static_cast<Int64>(nextUInt64());
}

Int64 Random::nextInt64(Int64 lowerLimit, Int64 upperLimit)
{
    UInt64 resultOffset = nextUInt64(0, upperLimit - lowerLimit);
    return lowerLimit + resultOffset;
}

UInt8 Random::nextUInt8()
{
    return static_cast<UInt8>(_next());
}

UInt8 Random::nextUInt8(UInt8 lowerLimit, UInt8 upperLimit)
{
    require(lowerLimit <= upperLimit);

    UInt8 span = upperLimit - lowerLimit + 1;
    return nextUInt8() % span + lowerLimit;
}

UInt16 Random::nextUInt16()
{
    return static_cast<UInt16>(_next() | 
                               (_next() << 15));
}

UInt16 Random::nextUInt16(UInt16 lowerLimit, UInt16 upperLimit)
{
    require(lowerLimit <= upperLimit);

    UInt16 span = upperLimit - lowerLimit + 1;
    return nextUInt16() % span + lowerLimit;
}

UInt32 Random::nextUInt32()
{
    return static_cast<UInt32>(_next() | 
                               (_next() << 15) |
                               (_next() << 30));
}

UInt32 Random::nextUInt32(UInt32 lowerLimit, UInt32 upperLimit)
{
    require(lowerLimit <= upperLimit);

    UInt32 span = upperLimit - lowerLimit + 1;
    return nextUInt32() % span + lowerLimit;
}

UInt64 Random::nextUInt64()
{
    return static_cast<UInt64>(_next()) | 
           (static_cast<UInt64>(_next()) << 15) |
           (static_cast<UInt64>(_next()) << 30) |
           (static_cast<UInt64>(_next()) << 45) |
           (static_cast<UInt64>(_next()) << 60);
}

UInt64 Random::nextUInt64(UInt64 lowerLimit, UInt64 upperLimit)
{
    require(lowerLimit <= upperLimit);

    UInt64 span = upperLimit - lowerLimit + 1;
    return nextUInt64() % span + lowerLimit;
}

BinaryInteger Random::nextBinaryInteger(UIntSize numberOfBits)
{
    require(numberOfBits > 0);

    Array<UInt8> bits;
    bits.setSize(numberOfBits / 8 + 1);
    nextBytes(bits.getData(), bits.size());
    BinaryInteger result(1, bits, System::HostByteOrder);
    result >>= (result.getRequiredBits() - 1 - numberOfBits);
    result.setBit(numberOfBits - 1);
    return result;
}

BinaryInteger Random::nextBinaryInteger(const BinaryInteger & lowerLimit, const BinaryInteger & upperLimit)
{
    require(lowerLimit <= upperLimit);

    BinaryInteger span = upperLimit - lowerLimit + 1;
    return nextBinaryInteger(span.getRequiredBits() + 1) % span + lowerLimit;
}

void Random::nextBytes(void * pBuffer, UIntSize bytesToStore)
{
    for (UIntSize i = 0; i < bytesToStore; i++)
    {
        (static_cast<UInt8*>(pBuffer))[i] = static_cast<UInt8>(_next());
    }
}

//////////
//  Implementation helpers
UInt32 Random::_next()
{
    return (((_seed = _seed * 214013 + 2531011) >> 16) & 0x7fff);
}

//  End of Random.cpp
