//
//  RealFormatProperties.cpp - Implementation of the carl::RealFormatProperties class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.11 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

RealFormatProperties::RealFormatProperties()
{
    _setStandardProperties();
}

RealFormatProperties::RealFormatProperties(const char * pCrtFormatString)
{
    _parseCrtFormatString(String(pCrtFormatString));
}

RealFormatProperties::RealFormatProperties(const Char * pCrtFormatString)
{
    _parseCrtFormatString(String(pCrtFormatString));
}

RealFormatProperties::RealFormatProperties(const String & crtFormatString)
{
    _parseCrtFormatString(crtFormatString);
}

RealFormatProperties::RealFormatProperties(const RealFormatProperties & src)
{
    _radix = src._radix;
    _minimumDigits = src._minimumDigits;
    _maximumDigits = src._maximumDigits;
    _minimumFractionalDigits = src._minimumFractionalDigits;
    _maximumFractionalDigits = src._maximumFractionalDigits;
    _minimumExponentDigits = src._minimumExponentDigits;
    _maximumExponentDigits = src._maximumExponentDigits;
    _minimumWidth = src._minimumWidth;
    _maximumWidth = src._maximumWidth;
    _fillerChar = src._fillerChar;
    _errorChar = src._errorChar;
    _alignment = src._alignment;
    _letterCase = src._letterCase;
    _useSeparator = src._useSeparator;
    _mandatorySign = src._mandatorySign;
    _format = src._format;
}

RealFormatProperties & RealFormatProperties::operator = (const RealFormatProperties & src)
{
    if (this != (&src))
    {
        _radix = src._radix;
        _minimumDigits = src._minimumDigits;
        _maximumDigits = src._maximumDigits;
        _minimumFractionalDigits = src._minimumFractionalDigits;
        _maximumFractionalDigits = src._maximumFractionalDigits;
        _minimumExponentDigits = src._minimumExponentDigits;
        _maximumExponentDigits = src._maximumExponentDigits;
        _minimumWidth = src._minimumWidth;
        _maximumWidth = src._maximumWidth;
        _fillerChar = src._fillerChar;
        _errorChar = src._errorChar;
        _alignment = src._alignment;
        _letterCase = src._letterCase;
        _useSeparator = src._useSeparator;
        _mandatorySign = src._mandatorySign;
        _format = src._format;
    }
    return *this;
}

//////////
//  Operations
RealFormatProperties & RealFormatProperties::setRadix(UIntSize radix)
{
    require(radix >= 2 && radix <= 36);
    _radix = radix;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMinimumDigits(UIntSize digits)
{
    _minimumDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMaximumDigits(UIntSize digits)
{
    _maximumDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMinimumFractionalDigits(UIntSize digits)
{
    _minimumFractionalDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMaximumFractionalDigits(UIntSize digits)
{
    _maximumFractionalDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMinimumExponentDigits(UIntSize digits)
{
    _minimumExponentDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMaximumExponentDigits(UIntSize digits)
{
    _maximumExponentDigits = digits;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMinimumWidth(UIntSize width)
{
    _minimumWidth = width;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMaximumWidth(UIntSize width)
{
    _maximumWidth = width;
    return *this;
}

RealFormatProperties & RealFormatProperties::setFillerChar(Char ch)
{
    _fillerChar = ch;
    return *this;
}

RealFormatProperties & RealFormatProperties::setErrorChar(Char ch)
{
    _errorChar = ch;
    return *this;
}

RealFormatProperties & RealFormatProperties::setAlignment(Alignment alignment)
{
    _alignment = alignment;
    return *this;
}

RealFormatProperties & RealFormatProperties::setLetterCase(LetterCase letterCase)
{
    _letterCase = letterCase;
    return *this;
}

RealFormatProperties & RealFormatProperties::setUseSeparator(bool useSeparator)
{
    _useSeparator = useSeparator;
    return *this;
}

RealFormatProperties & RealFormatProperties::setMandatorySign(bool mandatorySign)
{
    _mandatorySign = mandatorySign;
    return *this;
}

RealFormatProperties & RealFormatProperties::setFormat(Format format)
{
    _format = format;
    return *this;
}

//////////
//  Implementation helpers
void RealFormatProperties::_setStandardProperties()
{
    _radix = 10;
    _minimumDigits = 0;
    _maximumDigits = 0;
    _minimumFractionalDigits = 0;
    _maximumFractionalDigits = 0;
    _minimumExponentDigits = 0;
    _maximumExponentDigits = 0;
    _minimumWidth = 0;
    _maximumWidth = 0;
    _fillerChar = ' ';
    _errorChar = '*';
    _alignment = Alignment_Right;
    _letterCase = LetterCase_Lower;
    _useSeparator = false;
    _mandatorySign = false;
    _format = Format_AutoSelect;
}

void RealFormatProperties::_parseCrtFormatString(const carl::String & crtFormatString)
{
    _setStandardProperties();
    UIntSize scan = 0;
    //  Skip optional leading '%'
    if (scan < crtFormatString.length() && crtFormatString[scan] == '%')
    {
        scan++;
    }
    //  Process flags
    for (; scan < crtFormatString.length(); scan++)
    {
        switch (crtFormatString[scan])
        {
            case '-':
                _alignment = Alignment_Left;
                break;
            case '+':
                _mandatorySign = true;
                break;
            default:
                goto FlagsProcessed;
        }
    }
FlagsProcessed:;
    //  Process width
    bool widthSpecified = false;
    UIntSize width = 0;
    bool precisionSpecified = false;
    UIntSize precision = 0;
    if (scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]))
    {
        widthSpecified = true;
        for (; scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]); scan++)
        {
            width = 10 * width + (crtFormatString[scan] - '0');
        }
    }
    //  Process precision
    if (scan + 1 < crtFormatString.length() && crtFormatString[scan] == '.' && Iso10646::isDecimalDigit(crtFormatString[scan + 1]))
    {
        precisionSpecified = true;
        for (scan++; scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]); scan++)
        {
            precision = 10 * precision + (crtFormatString[scan] - '0');
        }
    }
    //  Process type
    require(scan < crtFormatString.length());
    switch (crtFormatString[scan])
    {
        case 'E':
            _letterCase = LetterCase_Upper;
            //  Fall through
        case 'e':
            scan++;
            _format = Format_FloatingPoint;
            if (widthSpecified)
            {
                _minimumWidth = width;
            }
            if (precisionSpecified)
            {
                _minimumDigits = _maximumDigits = precision;
            }
            break;
        case 'f':
            scan++;
            _format = Format_FixedPoint;
            if (widthSpecified)
            {
                _minimumWidth = width;
            }
            if (precisionSpecified)
            {
                _minimumFractionalDigits = _maximumFractionalDigits = precision;
            }
            break;
        case 'G':
            _letterCase = LetterCase_Upper;
            //  Fall through
        case 'g':
            scan++;
            if (widthSpecified)
            {
                _minimumWidth = width;
            }
            if (precisionSpecified)
            {
                _maximumDigits = precision;
            }
            break;
        default:
            require(false);
    }
    if (scan != crtFormatString.length())
    {
        require(false);
    }
}

//  End of RealFormatProperties.cpp
