//
//  RealParseProperties.cpp - Implementation of the carl::RealParseProperties class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
RealParseProperties::RealParseProperties(const RealParseProperties & src)
{
    _radix = src._radix;
    _minimumWidth = src._minimumWidth;
    _maximumWidth = src._maximumWidth;
    _allowSeparator = src._allowSeparator;
}

RealParseProperties & RealParseProperties::operator = (const RealParseProperties & src)
{
    if (this != (&src))
    {
        _radix = src._radix;
        _minimumWidth = src._minimumWidth;
        _maximumWidth = src._maximumWidth;
        _allowSeparator = src._allowSeparator;
    }
    return *this;
}

//////////
//  Operations
RealParseProperties & RealParseProperties::setRadix(UIntSize radix)
{
    require(radix >= 2 && radix <=36);

    _radix = radix;
    return *this;
}

RealParseProperties & RealParseProperties::setMinimumWidth(UIntSize width)
{
    _minimumWidth = width;
    return *this;
}

RealParseProperties & RealParseProperties::setMaximumWidth(UIntSize width)
{
    _maximumWidth = width;
    return *this;
}

RealParseProperties & RealParseProperties::setAllowSeparator(bool allowSeparator)
{
    _allowSeparator = allowSeparator;
    return *this;
}

//////////
//  Implementation helpers
void RealParseProperties::_setStandardProperties()
{
    _radix = 10;
    _minimumWidth = 0;
    _maximumWidth = 0;
    _allowSeparator = false;
}

void RealParseProperties::_parseCrtFormatString(const String & crtFormatString)
{
    _setStandardProperties();
    UIntSize scan = 0;
    //  Skip optional leading '%'
    if (scan < crtFormatString.length() && crtFormatString[scan] == '%')
    {
        scan++;
    }
    //  Process width
    if (scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]))
    {
        for (; scan < crtFormatString.length() && Iso10646::isDecimalDigit(crtFormatString[scan]); scan++)
        {
            _maximumWidth = 10 * _maximumWidth + (crtFormatString[scan] - '0');
        }
    }
    //  Process type
    require(scan < crtFormatString.length());
    switch (crtFormatString[scan])
    {
        case 'e':   case 'E':   case 'f':   case 'g':   case 'G':
            scan++;
            break;
        default:
            require(false);
    }
    if (scan != crtFormatString.length())
    {
        require(false);
    }
}

//  End of RealParseProperties.cpp
