//
//  RsaDecryptor.cpp - Implementation of the carl::_impl::_RsaDecryptor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.4 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Construction/destruction
_impl::_RsaDecryptor::_RsaDecryptor(const BinaryInteger & modulus, const BinaryInteger & exponent, UIntSize cleartextChunkSize)
    :   _modulus(modulus),
        _exponent(exponent),
        _cleartextChunkSize(cleartextChunkSize)
{
}

_impl::_RsaDecryptor::~_RsaDecryptor()
{
}

//////////
//  Operations
Cipher * _impl::_RsaDecryptor::getCipher() const
{
    return _RsaCipher::getInstance();
}

UIntSize _impl::_RsaDecryptor::getCleartextSizeForCiphertext(const void * pCiphertextBits, UIntSize ciphertextBitsCount)
{
    require(pCiphertextBits != NULL);
    require(System::BitsPerByte == 8);
    require(ciphertextBitsCount % 8 == 0);

    return _decrypt(pCiphertextBits, ciphertextBitsCount / 8).size() * 8;
}

UIntSize _impl::_RsaDecryptor::decryptBlock(const void * pCiphertextBits, UIntSize ciphertextBitsCount, void * pCleartextBits) throw(EncryptedDataCorruptException)
{
    require(pCiphertextBits != NULL);
    require(System::BitsPerByte == 8);
    require(ciphertextBitsCount % 8 == 0);
    require(pCleartextBits != NULL);

    Array<UInt8> cleartext = _decrypt(pCiphertextBits, ciphertextBitsCount / 8);
    Memory::copy(pCleartextBits, cleartext.getData(), cleartext.size());
    return cleartext.size() * 8;
}

//////////
//  Implementation helpers
Array<UInt8> _impl::_RsaDecryptor::_decrypt(const void * pCiphertextBytes, UIntSize ciphertextBytesCount)  throw(EncryptedDataCorruptException)
{
    require(pCiphertextBytes != NULL);

    MemoryInputStream mis(pCiphertextBytes, ciphertextBytesCount, false);
    try
    {
        Array<UInt8> result;
        while (mis.available() != 0)
        {
            UIntSize chunkSize = ULeb128::readUIntSize(mis);
            if (chunkSize == 0 || chunkSize > _cleartextChunkSize)
            {
                throw EncryptedDataCorruptException();
            }
            BinaryInteger ciphertextValue = Leb128::readBinaryInteger(mis);
            BinaryInteger cleartextValue = ciphertextValue.modPow(_exponent, _modulus);
            for (UIntSize i = 0; i < chunkSize; i++)
            {
                result.append(cleartextValue.toUInt8());
                cleartextValue >>= 8;
            }
        }
        if (result.size() * 8 != 1024)
        {
            throw EncryptedDataCorruptException();
        }
        return result;
    }
    catch (IOException)
    {
        throw EncryptedDataCorruptException();
    }
}

//  End of RsaDecryptor.cpp
