//
//  RsaEncryptor.cpp - Implementation of the carl::_impl::_RsaEncryptor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.4 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "EncryptionImpl.hpp"

//////////
//  Construction/destruction
_impl::_RsaEncryptor::_RsaEncryptor(const BinaryInteger & modulus, const BinaryInteger & exponent, UIntSize cleartextChunkSize)
    :   _modulus(modulus),
        _exponent(exponent),
        _cleartextChunkSize(cleartextChunkSize)
{
}

_impl::_RsaEncryptor::~_RsaEncryptor()
{
}

//////////
//  Operations
Cipher * _impl::_RsaEncryptor::getCipher() const
{
    return _RsaCipher::getInstance();
}

UIntSize _impl::_RsaEncryptor::getCiphertextSizeForCleartext(const void * pCleartextBits, UIntSize cleartextBitsCount)
{
    require(pCleartextBits != NULL);
    require(System::BitsPerByte == 8);

    if (cleartextBitsCount != 1024)
    {
        throw InvalidCleartextBlockSizeError();
    }
    return _encrypt(static_cast<const UInt8*>(pCleartextBits), cleartextBitsCount / 8).size() * 8;
}

UIntSize _impl::_RsaEncryptor::encryptBlock(const void * pCleartextBits, UIntSize cleartextBitsCount, void * pCiphertextBits)
{
    require(pCleartextBits != NULL);
    require(System::BitsPerByte == 8);
    require(pCiphertextBits != NULL);

    if (cleartextBitsCount != 1024)
    {
        throw InvalidCleartextBlockSizeError();
    }
    Array<UInt8> ciphertextBytes = _encrypt(static_cast<const UInt8*>(pCleartextBits), cleartextBitsCount / 8);
    Memory::copy(pCiphertextBits, ciphertextBytes.getData(), ciphertextBytes.size());
    return ciphertextBytes.size() * 8;
}

//////////
//  Implementation helpers
Array<UInt8> _impl::_RsaEncryptor::_encrypt(const UInt8 * pCleartextBytes, UIntSize cleartextBytesCount)
{
    require(pCleartextBytes != NULL);

    MemoryOutputStream mos;
    for (UIntSize scan = 0; scan < cleartextBytesCount; )
    {
        //  Determine how many bytes will be encrypted during this round
        UIntSize chunkSize = cleartextBytesCount - scan;
        if (chunkSize > _cleartextChunkSize)
        {
            chunkSize = _cleartextChunkSize;
        }
        //  Write chunk size
        ULeb128::writeUIntSize(mos, chunkSize);
        //  Convert chunk to a number
        BinaryInteger cleartext(1, pCleartextBytes + scan, chunkSize, ByteOrder_LittleEndian);
        //  Encrypt & write the number
        Leb128::writeBinaryInteger(mos, cleartext.modPow(_exponent, _modulus));
        //  Advance
        scan += chunkSize;
    }
    return mos.getContents();
}

//  End of RsaEncryptor.cpp
