//
//  RunLengthCompressor.cpp - Implementation of the carl::_impl::_RunLengthCompressor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_RunLengthCompressor::_RunLengthCompressor()
{
}

_impl::_RunLengthCompressor::~_RunLengthCompressor()
{
}

_impl::_RunLengthCompressor * _impl::_RunLengthCompressor::getInstance()
{
    static _RunLengthCompressor instance;
    return &instance;
}

//////////
//  Operations
CompressionAlgorithm * _impl::_RunLengthCompressor::getCompressionAlgorithm()
{
    return _RunLengthCompressionAlgorithm::getInstance();
}

UIntSize _impl::_RunLengthCompressor::getRequiredOutputBufferSize(const void * pUncompressedData, UIntSize uncompressedDataSize)
{
    const UInt8 * pUncompressedBytes = static_cast<const UInt8 *>(pUncompressedData);
    UIntSize scan = 0;
    UIntSize sequenceLength;
    UIntSize compressedDataSize;

    compressedDataSize = 0;
    while (scan < uncompressedDataSize)
    {
        //  Does a sequence of three or more identical bytes start here ?
        if (_repeatedSequenceStartsAt(pUncompressedBytes, uncompressedDataSize, scan))
        {   //  Yes. Determine the length of the sequence
            for (sequenceLength = 3; 
                 scan + sequenceLength < uncompressedDataSize && 
                 pUncompressedBytes[scan + sequenceLength] == pUncompressedBytes[scan] && 
                 sequenceLength < 255; 
                 sequenceLength++)
            {   //  All work is done in the loop header
            }
            //  Store the repeater & advance
            compressedDataSize += 3;
            scan += sequenceLength;
            continue;
        }
        //  The next bytes must be stored literally. Determine how many
        for (sequenceLength = 1;
             sequenceLength < 255 &&
             scan + sequenceLength < uncompressedDataSize &&
             !_repeatedSequenceStartsAt(pUncompressedBytes, uncompressedDataSize, scan + sequenceLength);
             sequenceLength++)
         {   // All work is done in the loop header
         }
         // Store the bytes & advance
         compressedDataSize += sequenceLength + 1;
         scan += sequenceLength;
    }
    //  Done
    return compressedDataSize;
}

UIntSize _impl::_RunLengthCompressor::compress(const void * pUncompressedData, UIntSize uncompressedDataSize, void * pCompressedData)
{
    const UInt8 * pUncompressedBytes = static_cast<const UInt8 *>(pUncompressedData);
    UInt8 * pCompressedBytes = static_cast<UInt8 *>(pCompressedData);
    UIntSize scan = 0;
    UIntSize sequenceLength;
    UIntSize compressedDataSize;

    compressedDataSize = 0;

    while (scan < uncompressedDataSize)
    {
        //  Does a sequence of three or more identical bytes start here ?
        if (_repeatedSequenceStartsAt(pUncompressedBytes, uncompressedDataSize, scan))
        {   //  Yes. Determine the length of the sequence
            for (sequenceLength = 3; 
                 scan + sequenceLength < uncompressedDataSize && 
                 pUncompressedBytes[scan + sequenceLength] == pUncompressedBytes[scan] && 
                 sequenceLength < 255; 
                 sequenceLength++)
            {   //  All work is done in the loop header
            }
            //  Store the repeater & advance
            pCompressedBytes[compressedDataSize++] = 0x00;
            pCompressedBytes[compressedDataSize++] = static_cast<UInt8>(sequenceLength);
            pCompressedBytes[compressedDataSize++] = pUncompressedBytes[scan];
            scan += sequenceLength;
            continue;
        }
        //  The next bytes must be stored literally. Determine how many
        for (sequenceLength = 1;
             sequenceLength < 255 &&
             scan + sequenceLength < uncompressedDataSize &&
             !_repeatedSequenceStartsAt(pUncompressedBytes, uncompressedDataSize, scan + sequenceLength);
             sequenceLength++)
         {   // All work is done in the loop header
         }
         // Store the bytes & advance
         pCompressedBytes[compressedDataSize++] = static_cast<UInt8>(sequenceLength);
         for (; sequenceLength != 0; sequenceLength--)
         {
             pCompressedBytes[compressedDataSize++] = pUncompressedBytes[scan++];
         }
    }
    //  Done
    return compressedDataSize;
}

//////////
//  Helpers
bool _impl::_RunLengthCompressor::_repeatedSequenceStartsAt(const UInt8 * pUncompressedBytes,
                                                            UIntSize uncompressedDataSize, 
                                                            UIntSize offset)
{
    return offset + 2 < uncompressedDataSize &&
           pUncompressedBytes[offset + 1] == pUncompressedBytes[offset] &&
           pUncompressedBytes[offset + 2] == pUncompressedBytes[offset];
}
//  End of RunLengthCompressor.cpp
