//
//  RunLengthDecompressor.cpp - Implementation of the carl::_impl::_RunLengthDecompressor class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CompressionImpl.hpp"

//////////
//  Singleton
_impl::_RunLengthDecompressor::_RunLengthDecompressor()
{
}

_impl::_RunLengthDecompressor::~_RunLengthDecompressor()
{
}

_impl::_RunLengthDecompressor * _impl::_RunLengthDecompressor::getInstance()
{
    static _RunLengthDecompressor instance;
    return &instance;
}

//////////
//  Operations
CompressionAlgorithm * _impl::_RunLengthDecompressor::getCompressionAlgorithm()
{
    return _RunLengthCompressionAlgorithm::getInstance();
}

UIntSize _impl::_RunLengthDecompressor::getRequiredOutputBufferSize(const void * pCompressedData, UIntSize compressedDataSize) throw(CompressedDataCorruptException)
{
    const UInt8 * pCompressedBytes = static_cast<const UInt8 *>(pCompressedData);
    UIntSize scan = 0;
    UIntSize uncompressedDataSize = 0;
    UInt8 count;

    while (scan < compressedDataSize)
    {
        if (pCompressedBytes[scan] == 0)
        {   //  Repeated sequence
            if (scan + 3 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            count = pCompressedBytes[scan + 1];
            scan += 3;
            uncompressedDataSize += count;
        }
        else
        {   //  Literal bytes
            count = pCompressedBytes[scan++];
            if (scan + count > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            scan += count;
            uncompressedDataSize += count;
        }
    }
    return uncompressedDataSize;
}

UIntSize _impl::_RunLengthDecompressor::decompress(const void * pCompressedData, UIntSize compressedDataSize, void * pUncompressedData) throw(CompressedDataCorruptException)
{
    const UInt8 * pCompressedBytes = static_cast<const UInt8 *>(pCompressedData);
    UInt8 * pUncompressedBytes = static_cast<UInt8 *>(pUncompressedData);
    UIntSize scan = 0;
    UIntSize uncompressedDataSize = 0;
    UInt8 count, value;

    while (scan < compressedDataSize)
    {
        if (pCompressedBytes[scan] == 0)
        {   //  Repeated sequence
            if (scan + 3 > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            count = pCompressedBytes[scan + 1];
            value = pCompressedBytes[scan + 2];
            scan += 3;
            for (; count != 0; count--)
            {
                pUncompressedBytes[uncompressedDataSize++] = value;
            }
        }
        else
        {   //  Literal bytes
            count = pCompressedBytes[scan++];
            if (scan + count > compressedDataSize)
            {
                throw CompressedDataCorruptException();
            }
            for (; count != 0; count--)
            {
                pUncompressedBytes[uncompressedDataSize++] = pCompressedBytes[scan++];
            }
        }
    }
    return uncompressedDataSize;
}

//  End of RunLengthDecompressor.cpp
