//
//  Semaphore.cpp - Implementation of the carl::Semaphore class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:54 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "SemaphoreImpl.hpp"

//////////
//  Construction
Semaphore::Semaphore(UIntSize initialCount)
{
    _pImpl = (_impl::_SemaphoreImpl*)malloc(sizeof(_impl::_SemaphoreImpl));
    if (_pImpl == NULL)
    {
        throw OutOfMemoryError(sizeof(_impl::_SemaphoreImpl));
    }
    _pImpl->initialize(initialCount);
}

Semaphore::~Semaphore()
{
    _pImpl->deinitialize();
    free(_pImpl);
}

//////////
//  SynchronizationObject
void Semaphore::waitFor()
{
    _pImpl->waitFor();
}

bool Semaphore::waitFor(const TimeSpan & timeout)
{
    return _pImpl->waitFor(timeout);
}

void Semaphore::release()
{
    _pImpl->release();
}

//////////
//  Semaphore
void Semaphore::down()
{
    waitFor();
}

bool Semaphore::down(const TimeSpan & timeout)
{
    return waitFor(timeout);
}

void Semaphore::up()
{
    release();
}

//  End of Semaphore.cpp
