//
//  SettingsNode.cpp - Implementation of the carl::SettingsNode class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
SettingsNode::SettingsNode()
    :   _path('/')
{
}

SettingsNode::SettingsNode(const char * pPath)
    :   _path(pPath)
{
    if (_path.endsWith('/'))
    {
        _path = _path.left(_path.length() - 1);
    }
    if (_path.length() == 0 || _path[0] != '/')
    {
        _path = String('/') + _path;
    }
}

SettingsNode::SettingsNode(const Char * pPath)
    :   _path(pPath)
{
    if (_path.endsWith('/'))
    {
        _path = _path.left(_path.length() - 1);
    }
    if (_path.length() == 0 || _path[0] != '/')
    {
        _path = String('/') + _path;
    }
}

SettingsNode::SettingsNode(const String & path)
    :   _path(path)
{
    if (_path.endsWith('/'))
    {
        _path = _path.left(_path.length() - 1);
    }
    if (_path.length() == 0 || _path[0] != '/')
    {
        _path = String('/') + _path;
    }
}

SettingsNode::SettingsNode(const SettingsNode & parent, const char * pSubnode)
    :   _path(parent._path)
{
    String subnode(pSubnode);

    if (subnode.length() != 0)
    {
        if (_path.endsWith('/'))
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode.mid(1);
            }
            else
            {
                _path += subnode;
            }
        }
        else
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode;
            }
            else
            {
                _path += '/';
                _path += subnode;
            }
        }
    }
}

SettingsNode::SettingsNode(const SettingsNode & parent, const Char * pSubnode)
    :   _path(parent._path)
{
    String subnode(pSubnode);

    if (subnode.length() != 0)
    {
        if (_path.endsWith('/'))
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode.mid(1);
            }
            else
            {
                _path += subnode;
            }
        }
        else
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode;
            }
            else
            {
                _path += '/';
                _path += subnode;
            }
        }
    }
}

SettingsNode::SettingsNode(const SettingsNode & parent, const String & subnode)
    :   _path(parent._path)
{
    if (subnode.length() != 0)
    {
        if (_path.endsWith('/'))
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode.mid(1);
            }
            else
            {
                _path += subnode;
            }
        }
        else
        {
            if (subnode.startsWith('/'))
            {
                _path += subnode;
            }
            else
            {
                _path += '/';
                _path += subnode;
            }
        }
    }
}

SettingsNode::SettingsNode(const SettingsNode & src)
    :   _path(src._path)
{
}

SettingsNode & SettingsNode::operator = (const SettingsNode & src)
{
    if (this != (&src))
    {
        _path = src._path;
    }
    return *this;
}

//////////
//  Operations
SettingsNode SettingsNode::getParent() const
{
    if (_path == '/' || _path == "//")
    {
        return *this;
    }
    else
    {
        UIntSize rightmostPathSeparatorIndex;
        bool b = _path.findLast('/', rightmostPathSeparatorIndex);
        require(b);
        if (rightmostPathSeparatorIndex == 0)
        {
            return SettingsNode();
        }
        else
        {
            return SettingsNode(_path.left(rightmostPathSeparatorIndex - 1));
        }
    }
}

//  End of SettingsNode.cpp
