//
//  SingleByteCharacterSet.cpp - Implementation of the carl::_impl::_SingleByteCharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Construction/destruction - from derived classes only
_impl::_SingleByteCharacterSet::_SingleByteCharacterSet(const Char * pCharTable256)
    :   _pCharTable256(pCharTable256),
        _mapCharsToBytes(StandardHashFunctions::hashChar)
{
    require(pCharTable256 != NULL);

    _initialized = false;
}

_impl::_SingleByteCharacterSet::~_SingleByteCharacterSet()
{
}

//////////
//  Operations
int _impl::_SingleByteCharacterSet::getMaximumEncodingLength() const
{
    return 1;
}

int _impl::_SingleByteCharacterSet::getEncodingLengthForCharacter(Char c) const
{
    _initialize();
    if (_mapCharsToBytes.containsKey(c))
    {
        return 1;
    }
    return -1;
}

int _impl::_SingleByteCharacterSet::encodeCharacter(Char c, void * buffer) const
{
    _initialize();
    if (_mapCharsToBytes.containsKey(c))
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = _mapCharsToBytes.get(c);
        return 1;
    }
    return -1;
}

int _impl::_SingleByteCharacterSet::getEncodingLengthForLeadByte(UInt8 leadByte) const
{
    if (_pCharTable256[leadByte] != 0xFFFFFFFF)
    {
        return 1;
    }
    return -1;
}

int _impl::_SingleByteCharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
    if (bytesAvailable >= 1 && _pCharTable256[pBytes[0]] != 0xFFFFFFFF)
    {
        c = _pCharTable256[pBytes[0]];
        return 1;
    }
    return -1;
}

//////////
//  Implementation helpers
void _impl::_SingleByteCharacterSet::_initialize() const
{
    if (!_initialized)
    {
        for (int i = 0; i < 256; i++)
        {
            if (_pCharTable256[i] != 0xFFFFFFFF)
            {
                if(_mapCharsToBytes.containsKey(_pCharTable256[i]))
                {
                    throw InconsistentCharacterSetException();
                }
                _mapCharsToBytes.insert(_pCharTable256[i], static_cast<UInt8>(i));
            }
        }

        _initialized = true;
    }
}

//  End of SingleByteCharacterSet.cpp
