//
//  String.cpp - Implementation of the carl::String class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/29 11:19:44 $
//  $Revision: 1.16 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdlib.h>
#include <string.h>

//////////
//  Construction/destruction/assignment
String::String(_impl::_StringImpl * pImpl)
{
    require(pImpl != NULL);
    _pImpl = pImpl;
    _pImpl->incrementReferenceCount();
}

String::String()
{
    _pImpl = new _impl::_StringImpl();
    _pImpl->incrementReferenceCount();
}

String::String(char ch)
{
    _pImpl = new _impl::_StringImpl(ch);
    _pImpl->incrementReferenceCount();
}

String::String(Char ch)
{
    _pImpl = new _impl::_StringImpl(ch);
    _pImpl->incrementReferenceCount();
}

String::String(char ch, UIntSize repeatCount)
{
    _pImpl = new _impl::_StringImpl(ch, repeatCount);
    _pImpl->incrementReferenceCount();
}

String::String(Char ch, UIntSize repeatCount)
{
    _pImpl = new _impl::_StringImpl(ch, repeatCount);
    _pImpl->incrementReferenceCount();
}

String::String(const char * pStr)
{
    _pImpl = new _impl::_StringImpl(pStr);
    _pImpl->incrementReferenceCount();
}

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
String::String(const wchar_t * pStr)
{
    _pImpl = new _impl::_StringImpl(pStr);
    _pImpl->incrementReferenceCount();
}
#endif

String::String(const Char * pStr)
{
    _pImpl = new _impl::_StringImpl(pStr);
    _pImpl->incrementReferenceCount();
}

String::String(const char * pChars, UIntSize charCount)
{
    _pImpl = new _impl::_StringImpl(pChars, charCount);
    _pImpl->incrementReferenceCount();
}

String::String(const Char * pChars, UIntSize charCount)
{
    _pImpl = new _impl::_StringImpl(pChars, charCount);
    _pImpl->incrementReferenceCount();
}

String::String(const String & str)
{
    _pImpl = str._pImpl;
    _pImpl->incrementReferenceCount();
}

String::~String()
{
    _pImpl->decrementReferenceCount();
    if (_pImpl->getReferenceCount() == 0)
    {
        delete _pImpl;
    }
}

String & String::operator = (char ch)
{
    _ensureNonsharedImplementation();
    (*_pImpl) = ch;
    return *this;
}

String & String::operator = (Char ch)
{
    _ensureNonsharedImplementation();
    (*_pImpl) = ch;
    return *this;
}

String & String::operator = (const char * pStr)
{
    _ensureNonsharedImplementation();
    (*_pImpl) = pStr;
    return *this;
}

String & String::operator = (const Char * pStr)
{
    _ensureNonsharedImplementation();
    (*_pImpl) = pStr;
    return *this;
}

String & String::operator = (const String & str)
{
    if (this != (&str))
    {
        _pImpl->decrementReferenceCount();
        if (_pImpl->getReferenceCount() == 0)
        {
            delete _pImpl;
        }
        _pImpl = str._pImpl;
        _pImpl->incrementReferenceCount();
    }
    return *this;
}

//////////
//  Operations
void String::setLength(UIntSize length)
{
    _ensureNonsharedImplementation();
    _pImpl->setLength(length);
}

String String::operator + (char ch) const
{
    String result(*this);
    result += ch;
    return result;
}

String String::operator + (Char ch) const
{
    String result(*this);
    result += ch;
    return result;
}

String String::operator + (const char * pStr) const
{
    String result(*this);
    result += pStr;
    return result;
}

String String::operator + (const Char * pStr) const
{
    String result(*this);
    result += pStr;
    return result;
}

String String::operator + (const String & str) const
{
    String result(*this);
    result += str;
    return result;
}

String & String::operator += (char ch)
{
    _ensureNonsharedImplementation();
    (*_pImpl) += ch;
    return *this;
}

String & String::operator += (Char ch)
{
    _ensureNonsharedImplementation();
    (*_pImpl) += ch;
    return *this;
}

String & String::operator += (const char * pStr)
{
    _ensureNonsharedImplementation();
    (*_pImpl) += pStr;
    return *this;
}

String & String::operator += (const Char * pStr)
{
    _ensureNonsharedImplementation();
    (*_pImpl) += pStr;
    return *this;
}

String & String::operator += (const String & str)
{
    _ensureNonsharedImplementation();
    (*_pImpl) += (*str._pImpl);
    return *this;
}

void String::insert(char ch, UIntSize atIndex)
{
    _ensureNonsharedImplementation();
    _pImpl->insert(ch, atIndex);
}

void String::insert(Char ch, UIntSize atIndex)
{
    _ensureNonsharedImplementation();
    _pImpl->insert(ch, atIndex);
}

void String::insert(const char * pStr, UIntSize atIndex)
{
    _ensureNonsharedImplementation();
    _pImpl->insert(pStr, atIndex);
}

void String::insert(const Char * pStr, UIntSize atIndex)
{
    _ensureNonsharedImplementation();
    _pImpl->insert(pStr, atIndex);
}

void String::insert(const String & str, UIntSize atIndex)
{
    _ensureNonsharedImplementation();
    _pImpl->insert(*str._pImpl, atIndex);
}

void String::remove(UIntSize startIndex, UIntSize charCount)
{
    _ensureNonsharedImplementation();
    _pImpl->remove(startIndex, charCount);
}

void String::replace(UIntSize startIndex, UIntSize charCount, char ch)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, ch);
}

void String::replace(UIntSize startIndex, UIntSize charCount, Char ch)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, ch);
}

void String::replace(UIntSize startIndex, UIntSize charCount, const char * pStr)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, pStr);
}

void String::replace(UIntSize startIndex, UIntSize charCount, const Char * pStr)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, pStr);
}

void String::replace(UIntSize startIndex, UIntSize charCount, const char * pChars, UIntSize replacementCharCount)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, pChars, replacementCharCount);
}

void String::replace(UIntSize startIndex, UIntSize charCount, const Char * pChars, UIntSize replacementCharCount)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, pChars, replacementCharCount);
}

void String::replace(UIntSize startIndex, UIntSize charCount, const String & str)
{
    _ensureNonsharedImplementation();
    _pImpl->replace(startIndex, charCount, *str._pImpl);
}

void String::trimLeft()
{
    _ensureNonsharedImplementation();
    _pImpl->trimLeft();
}

void String::trimRight()
{
    _ensureNonsharedImplementation();
    _pImpl->trimRight();
}

void String::trim()
{
    _ensureNonsharedImplementation();
    _pImpl->trim();
}

String String::trimmedLeft() const
{
    String result(*this);
    result.trimLeft();
    return result;
}

String String::trimmedRight() const
{
    String result(*this);
    result.trimRight();
    return result;
}

String String::trimmed() const
{
    String result(*this);
    result.trim();
    return result;
}

void String::upperCase()
{
    _ensureNonsharedImplementation();
    _pImpl->upperCase();
}

void String::lowerCase()
{
    _ensureNonsharedImplementation();
    _pImpl->lowerCase();
}

void String::titleCase()
{
    _ensureNonsharedImplementation();
    _pImpl->titleCase();
}

String String::upperCased() const
{
    String result(*this);
    result.upperCase();
    return result;
}

String String::lowerCased() const
{
    String result(*this);
    result.lowerCase();
    return result;
}

String String::titleCased() const
{
    String result(*this);
    result.titleCase();
    return result;
}

//////////
//  Implementation helpers
void String::_ensureNonsharedImplementation()
{
    if (_pImpl->getReferenceCount() != 1)
    {
        _impl::_StringImpl * pNewImpl = new _impl::_StringImpl(*_pImpl);
        _pImpl->decrementReferenceCount();
        pNewImpl->incrementReferenceCount();
        _pImpl = pNewImpl;
    }
}

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
bool String::_store(wchar_t * pBuffer, UIntSize bufferSize) const
{
    return _pImpl->_store(pBuffer, bufferSize);
}
#endif

bool String::_store(char * pBuffer, UIntSize bufferSize) const
{
    return _pImpl->_store(pBuffer, bufferSize);
}

//////////
//  Hashing strings
UIntSize CARL_CALLBACK StandardHashFunctions::hashString(const String & key)
{
    UIntSize hashCode = 0;
    for (UIntSize i = 0; i < key.length(); i++)
    {
        hashCode = 31 * hashCode + key[i];
    }
    return hashCode;
}

UIntSize CARL_CALLBACK StandardHashFunctions::hashStringIgnoreCase(const String & key)
{
    UIntSize hashCode = 0;
    for (UIntSize i = 0; i < key.length(); i++)
    {
        Char c = Iso10646::toLower(key[i]);
        hashCode = 31 * hashCode + c;
    }
    return hashCode;
}

//  End of String.cpp
