//
//  StringImpl.cpp - Implementation of the carl::_impl::_StringImpl class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/07/28 10:40:28 $
//  $Revision: 1.16 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdlib.h>
#include <string.h>

//////////
//  Construction/destruction/assignment
_impl::_StringImpl::_StringImpl()
{
    _pChars = NULL;
    _allocated = 0;
    _used = 0;
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(char ch)
{
    _pChars = new Char[1];
    _allocated = 1;
    _used = 1;
    _pChars[0] = static_cast<unsigned char>(ch);
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(Char ch)
{
    _pChars = new Char[1];
    _allocated = 1;
    _used = 1;
    _pChars[0] = ch;
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(char ch, UIntSize repeatCount)
{
    _pChars = new Char[repeatCount];
    _allocated = repeatCount;
    _used = repeatCount;
    for (UIntSize i = 0; i < repeatCount; i++)
    {
        _pChars[i] = ch;
    }
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(Char ch, UIntSize repeatCount)
{
    _pChars = new Char[repeatCount];
    _allocated = repeatCount;
    _used = repeatCount;
    for (UIntSize i = 0; i < repeatCount; i++)
    {
        _pChars[i] = ch;
    }
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(const char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    _pChars = new Char[strLen];
    _allocated = strLen;
    _used = strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[i] = static_cast<unsigned char>(pStr[i]);
    }
    _referenceCount = 0;
}

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
_impl::_StringImpl::_StringImpl(const wchar_t * pStr)
{
    require(pStr != NULL);

    UIntSize strLen;
    for (strLen = 0; pStr[strLen] != 0; strLen++)
    {   //  All work done in the loop header
    }
    _pChars = new Char[strLen];
    _allocated = strLen;
    _used = strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[i] = static_cast<unsigned char>(pStr[i]);
    }
    _referenceCount = 0;
}
#endif

_impl::_StringImpl::_StringImpl(const Char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    _pChars = new Char[strLen];
    _allocated = strLen;
    _used = strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[i] = pStr[i];
    }
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(const char * pChars, UIntSize charCount)
{
    if (charCount == 0)
    {
        _pChars = NULL;
        _allocated = 0;
        _used = 0;
    }
    else
    {
        require(pChars != NULL);

        _pChars = new Char[charCount];
        _allocated = charCount;
        _used = charCount;
        for (UIntSize i = 0; i < charCount; i++)
        {
            _pChars[i] = static_cast<unsigned char>(pChars[i]);
        }
    }
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(const Char * pChars, UIntSize charCount)
{
    if (charCount == 0)
    {
        _pChars = NULL;
        _allocated = 0;
        _used = 0;
    }
    else
    {
        require(pChars != NULL);

        _pChars = new Char[charCount];
        _allocated = charCount;
        _used = charCount;
        for (UIntSize i = 0; i < charCount; i++)
        {
            _pChars[i] = pChars[i];
        }
    }
    _referenceCount = 0;
}

_impl::_StringImpl::_StringImpl(const _StringImpl & str)
{
    if (str._used == 0)
    {
        _pChars = NULL;
        _allocated = 0;
        _used = 0;
    }
    else
    {
        _pChars = new Char[str._used];
        _allocated = str._used;
        _used = str._used;
        for (UIntSize i = 0; i < str._used; i++)
        {
            _pChars[i] = str._pChars[i];
        }
    }
    _referenceCount = 0;
}

_impl::_StringImpl::~_StringImpl()
{
    delete [] _pChars;
}

void _impl::_StringImpl::operator = (char ch)
{
    _ensureAllocation(1);
    _pChars[0] = static_cast<unsigned char>(ch);
    _used = 1;
}

void _impl::_StringImpl::operator = (Char ch)
{
    _ensureAllocation(1);
    _pChars[0] = ch;
    _used = 1;
}

void _impl::_StringImpl::operator = (const char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    _ensureAllocation(strLen);
    _used = strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[i] = static_cast<unsigned char>(pStr[i]);
    }
}

void _impl::_StringImpl::operator = (const Char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    _ensureAllocation(strLen);
    _used = strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[i] = pStr[i];
    }
}

//////////
//  Operations
void _impl::_StringImpl::setLength(UIntSize length)
{
    _ensureAllocation(length);
    while (_used < length)
    {
        _pChars[_used++] = ' ';
    }
    _used = length;
}

int _impl::_StringImpl::compare(char ch) const
{
    if (_used == 0)
    {   //  _StringImpl is empty
        return -1;
    }
    else if (_pChars[0] < static_cast<unsigned char>(ch))
    {   //  1st character of the string is < ch
        return -1;
    }
    else if (_pChars[0] > static_cast<unsigned char>(ch))
    {   //  1st character of the string is > ch
        return 1;
    }
    else
    {   //  1st character of the string is == ch
        if (_used == 1)
        {   //  _StringImpl is exactly 1 character long
            return 0;
        }
        else
        {
            return 1;
        }
    }
}

int _impl::_StringImpl::compare(Char ch) const
{
    if (_used == 0)
    {   //  _StringImpl is empty
        return -1;
    }
    else if (_pChars[0] < ch)
    {   //  1st character of the string is < ch
        return -1;
    }
    else if (_pChars[0] > ch)
    {   //  1st character of the string is > ch
        return 1;
    }
    else
    {   //  1st character of the string is == ch
        if (_used == 1)
        {   //  _StringImpl is exactly 1 character long
            return 0;
        }
        else
        {
            return 1;
        }
    }
}

int _impl::_StringImpl::compare(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    for (UIntSize i = 0; i < _used && i < strLen; i++)
    {
        Char ch1 = _pChars[i];
        Char ch2 = static_cast<unsigned char>(pStr[i]);
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < strLen)
    {
        return -1;
    }
    if (_used > strLen)
    {
        return 1;
    }
    return 0;
}

int _impl::_StringImpl::compare(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    for (UIntSize i = 0; i < _used && i < strLen; i++)
    {
        Char ch1 = _pChars[i];
        Char ch2 = pStr[i];
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < strLen)
    {
        return -1;
    }
    if (_used > strLen)
    {
        return 1;
    }
    return 0;
}

int _impl::_StringImpl::compare(const _StringImpl & str) const
{
    for (UIntSize i = 0; i < _used && i < str._used; i++)
    {
        Char ch1 = _pChars[i];
        Char ch2 = str._pChars[i];
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < str._used)
    {
        return -1;
    }
    if (_used > str._used)
    {
        return 1;
    }
    return 0;
}

int _impl::_StringImpl::compareIgnoreCase(char ch) const
{
    if (_used == 0)
    {   //  _StringImpl is empty
        return -1;
    }
    else
    {
        Char c1 = Iso10646::toUpper(_pChars[0]);
        Char c2 = Iso10646::toUpper(static_cast<unsigned char>(ch));
        if (c1 < c2)
        {   //  1st character of the string is < ch
            return -1;
        }
        else if (c1 > c2)
        {   //  1st character of the string is > ch
            return 1;
        }
        else
        {   //  1st character of the string is == ch
            if (_used == 1)
            {   //  _StringImpl is exactly 1 character long
                return 0;
            }
            else
            {
                return 1;
            }
        }
    }
}

int _impl::_StringImpl::compareIgnoreCase(Char ch) const
{
    if (_used == 0)
    {   //  _StringImpl is empty
        return -1;
    }
    else
    {
        Char c1 = Iso10646::toUpper(_pChars[0]);
        Char c2 = Iso10646::toUpper(ch);
        if (c1 < c2)
        {   //  1st character of the string is < ch
            return -1;
        }
        else if (c1 > c2)
        {   //  1st character of the string is > ch
            return 1;
        }
        else
        {   //  1st character of the string is == ch
            if (_used == 1)
            {   //  _StringImpl is exactly 1 character long
                return 0;
            }
            else
            {
                return 1;
            }
        }
    }
}

int _impl::_StringImpl::compareIgnoreCase(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    for (UIntSize i = 0; i < _used && i < strLen; i++)
    {
        Char ch1 = Iso10646::toUpper(_pChars[i]);
        Char ch2 = Iso10646::toUpper(static_cast<unsigned char>(pStr[i]));
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < strLen)
    {
        return -1;
    }
    if (_used > strLen)
    {
        return 1;
    }
    return 0;
}

int _impl::_StringImpl::compareIgnoreCase(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    for (UIntSize i = 0; i < _used && i < strLen; i++)
    {
        Char ch1 = Iso10646::toUpper(_pChars[i]);
        Char ch2 = Iso10646::toUpper(pStr[i]);
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < strLen)
    {
        return -1;
    }
    if (_used > strLen)
    {
        return 1;
    }
    return 0;
}

int _impl::_StringImpl::compareIgnoreCase(const _StringImpl & str) const
{
    for (UIntSize i = 0; i < _used && i < str._used; i++)
    {
        Char ch1 = Iso10646::toUpper(_pChars[i]);
        Char ch2 = Iso10646::toUpper(str._pChars[i]);
        if (ch1 < ch2)
        {
            return -1;
        }
        if (ch1 > ch2)
        {
            return 1;
        }
    }
    if (_used < str._used)
    {
        return -1;
    }
    if (_used > str._used)
    {
        return 1;
    }
    return 0;
}

bool _impl::_StringImpl::startsWith(char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return _pChars[0] == static_cast<unsigned char>(ch);
}

bool _impl::_StringImpl::startsWith(Char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return _pChars[0] == ch;
}

bool _impl::_StringImpl::startsWith(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (_pChars[i] != static_cast<unsigned char>(pStr[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::startsWith(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (_pChars[i] != pStr[i])
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::startsWith(const _StringImpl & str) const
{
    if (_used < str._used)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < str._used; i++)
    {
        if (_pChars[i] != str._pChars[i])
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::startsWithIgnoreCase(char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return Iso10646::toUpper(_pChars[0]) == Iso10646::toUpper(static_cast<unsigned char>(ch));
}

bool _impl::_StringImpl::startsWithIgnoreCase(Char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return Iso10646::toUpper(_pChars[0]) == Iso10646::toUpper(ch);
}

bool _impl::_StringImpl::startsWithIgnoreCase(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (Iso10646::toUpper(_pChars[i]) != Iso10646::toUpper(static_cast<unsigned char>(pStr[i])))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::startsWithIgnoreCase(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (Iso10646::toUpper(_pChars[i]) != Iso10646::toUpper(pStr[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::startsWithIgnoreCase(const _StringImpl & str) const
{
    if (_used < str._used)
    {   //  _StringImpl too short
        return false;
    }
    for (UIntSize i = 0; i < str._used; i++)
    {
        if (Iso10646::toUpper(_pChars[i]) != Iso10646::toUpper(str._pChars[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWith(char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return _pChars[_used - 1] == static_cast<unsigned char>(ch);
}

bool _impl::_StringImpl::endsWith(Char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return _pChars[_used - 1] == ch;
}

bool _impl::_StringImpl::endsWith(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (_pChars[startIndex + i] != static_cast<unsigned char>(pStr[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWith(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (_pChars[startIndex + i] != pStr[i])
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWith(const _StringImpl & str) const
{
    if (_used < str._used)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - str._used;
    for (UIntSize i = 0; i < str._used; i++)
    {
        if (_pChars[startIndex + i] != str._pChars[i])
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWithIgnoreCase(char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return Iso10646::toUpper(_pChars[_used - 1]) == Iso10646::toUpper(static_cast<unsigned char>(ch));
}

bool _impl::_StringImpl::endsWithIgnoreCase(Char ch) const
{
    if (_used < 1)
    {   //  _StringImpl too short
        return false;
    }
    return Iso10646::toUpper(_pChars[_used - 1]) == Iso10646::toUpper(ch);
}

bool _impl::_StringImpl::endsWithIgnoreCase(const char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (Iso10646::toUpper(_pChars[startIndex + i]) != Iso10646::toUpper(static_cast<unsigned char>(pStr[i])))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWithIgnoreCase(const Char * pStr) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (_used < strLen)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - strLen;
    for (UIntSize i = 0; i < strLen; i++)
    {
        if (Iso10646::toUpper(_pChars[startIndex + i]) != Iso10646::toUpper(pStr[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::endsWithIgnoreCase(const _StringImpl & str) const
{
    if (_used < str._used)
    {   //  _StringImpl too short
        return false;
    }
    UIntSize startIndex = _used - str._used;
    for (UIntSize i = 0; i < str._used; i++)
    {
        if (Iso10646::toUpper(_pChars[startIndex + i]) != Iso10646::toUpper(str._pChars[i]))
        {
            return false;
        }
    }
    return true;
}

bool _impl::_StringImpl::findFirst(char ch, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + 1 <= _used; i++)
    {
        if (_pChars[i] == static_cast<unsigned char>(ch))
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirst(Char ch, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + 1 <= _used; i++)
    {
        if (_pChars[i] == ch)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirst(const char * pStr, UIntSize & position, UIntSize startIndex) const
{
    require(pStr != NULL && startIndex <= _used);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    for (UIntSize i = startIndex; i + strLen <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (_pChars[i + j] != static_cast<unsigned char>(pStr[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirst(const Char * pStr, UIntSize & position, UIntSize startIndex) const
{
    require(pStr != NULL && startIndex <= _used);

    UIntSize strLen = _strlen(pStr);
    for (UIntSize i = startIndex; i + strLen <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (_pChars[i + j] != pStr[j])
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirst(const _StringImpl & str, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + str._used <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < str._used; j++)
        {
            if (_pChars[i + j] != str._pChars[j])
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirstIgnoreCase(char ch, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + 1 <= _used; i++)
    {
        if (Iso10646::toUpper(_pChars[i]) == Iso10646::toUpper(static_cast<unsigned char>(ch)))
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirstIgnoreCase(Char ch, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + 1 <= _used; i++)
    {
        if (Iso10646::toUpper(_pChars[i]) == Iso10646::toUpper(ch))
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirstIgnoreCase(const char * pStr, UIntSize & position, UIntSize startIndex) const
{
    require(pStr != NULL && startIndex <= _used);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    for (UIntSize i = startIndex; i + strLen <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j]) != Iso10646::toUpper(static_cast<unsigned char>(pStr[j])))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirstIgnoreCase(const Char * pStr, UIntSize & position, UIntSize startIndex) const
{
    require(pStr != NULL && startIndex <= _used);

    UIntSize strLen = _strlen(pStr);
    for (UIntSize i = startIndex; i + strLen <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j]) != Iso10646::toUpper(pStr[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findFirstIgnoreCase(const _StringImpl & str, UIntSize & position, UIntSize startIndex) const
{
    require(startIndex <= _used);

    for (UIntSize i = startIndex; i + str._used <= _used; i++)
    {
        bool match = true;
        for (UIntSize j = 0; j < str._used; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j]) != Iso10646::toUpper(str._pChars[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLast(char ch, UIntSize & position) const
{
    for (UIntSize i = _used; i > 0; i--)
    {
        if (_pChars[i - 1] == static_cast<unsigned char>(ch))
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLast(Char ch, UIntSize & position) const
{
    for (UIntSize i = _used; i > 0; i--)
    {
        if (_pChars[i - 1] == ch)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLast(const char * pStr, UIntSize & position) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (strLen > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - strLen; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (_pChars[i + j - 1] != static_cast<unsigned char>(pStr[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLast(const Char * pStr, UIntSize & position) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (strLen > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - strLen; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (_pChars[i + j - 1] != pStr[j])
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLast(const _StringImpl & str, UIntSize & position) const
{
    if (str._used > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - str._used; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < str._used; j++)
        {
            if (_pChars[i + j - 1] != str._pChars[j])
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLastIgnoreCase(char ch, UIntSize & position) const
{
    for (UIntSize i = _used; i > 0; i--)
    {
        if (Iso10646::toUpper(_pChars[i - 1]) == Iso10646::toUpper(static_cast<unsigned char>(ch)))
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLastIgnoreCase(Char ch, UIntSize & position) const
{
    for (UIntSize i = _used; i > 0; i--)
    {
        if (Iso10646::toUpper(_pChars[i - 1]) == Iso10646::toUpper(ch))
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLastIgnoreCase(const char * pStr, UIntSize & position) const
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (strLen > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - strLen; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j - 1]) != Iso10646::toUpper(static_cast<unsigned char>(pStr[j])))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLastIgnoreCase(const Char * pStr, UIntSize & position) const
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (strLen > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - strLen; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < strLen; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j - 1]) != Iso10646::toUpper(pStr[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

bool _impl::_StringImpl::findLastIgnoreCase(const _StringImpl & str, UIntSize & position) const
{
    if (str._used > _used)
    {   //  Substring is longer than the string itself - inclusion not possible
        return false;
    }
    for (UIntSize i = _used + 1 - str._used; i > 0; i--)
    {
        bool match = true;
        for (UIntSize j = 0; j < str._used; j++)
        {
            if (Iso10646::toUpper(_pChars[i + j - 1]) != Iso10646::toUpper(str._pChars[j]))
            {
                match = false;
                break;
            }
        }
        if (match)
        {
            position = i - 1;
            return true;
        }
    }
    return false;
}

void _impl::_StringImpl::operator += (char ch)
{
    _ensureAllocation(_used + 1);
    _pChars[_used++] = static_cast<unsigned char>(ch);
}

void _impl::_StringImpl::operator += (Char ch)
{
    _ensureAllocation(_used + 1);
    _pChars[_used++] = ch;
}

void _impl::_StringImpl::operator += (const char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    _ensureAllocation(_used + strLen);
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[_used++] = static_cast<unsigned char>(pStr[i]);
    }
}

void _impl::_StringImpl::operator += (const Char * pStr)
{
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    _ensureAllocation(_used + strLen);
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[_used++] = pStr[i];
    }
}

void _impl::_StringImpl::operator += (const _StringImpl & str)
{
    UIntSize strUsed = str._used;   //  Needed to handle s += s properly
    _ensureAllocation(_used + str._used);
    for (UIntSize i = 0; i < strUsed; i++)
    {
        _pChars[_used++] = str._pChars[i];
    }
}

void _impl::_StringImpl::insert(char ch, UIntSize atIndex)
{
    require(atIndex <= _used);

    _ensureAllocation(_used + 1);
    for (UIntSize i = _used; i > atIndex; i--)
    {
        _pChars[i] = _pChars[i - 1];
    }
    _pChars[atIndex] = static_cast<unsigned char>(ch);
    _used++;
}

void _impl::_StringImpl::insert(Char ch, UIntSize atIndex)
{
    require(atIndex <= _used);

    _ensureAllocation(_used + 1);
    for (UIntSize i = _used; i > atIndex; i--)
    {
        _pChars[i] = _pChars[i - 1];
    }
    _pChars[atIndex] = static_cast<unsigned char>(ch);
    _used++;
}

void _impl::_StringImpl::insert(const char * pStr, UIntSize atIndex)
{
    require(pStr != NULL && atIndex <= _used);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (strLen != 0)
    {
        _ensureAllocation(_used + strLen);
        for (UIntSize i = _used + strLen - 1; i > atIndex + strLen - 1; i--)
        {
            _pChars[i] = _pChars[i - strLen];
        }
        for (UIntSize i = 0; i < strLen; i++)
        {
            _pChars[atIndex + i] = static_cast<unsigned char>(pStr[i]);
        }
        _used += strLen;
    }
}

void _impl::_StringImpl::insert(const Char * pStr, UIntSize atIndex)
{
    require(pStr != NULL && atIndex <= _used);

    UIntSize strLen = _strlen(pStr);
    if (strLen != 0)
    {
        _ensureAllocation(_used + strLen);
        for (UIntSize i = _used + strLen - 1; i > atIndex + strLen - 1; i--)
        {
            _pChars[i] = _pChars[i - strLen];
        }
        for (UIntSize i = 0; i < strLen; i++)
        {
            _pChars[atIndex + i] = pStr[i];
        }
        _used += strLen;
    }
}

void _impl::_StringImpl::insert(const _StringImpl & str, UIntSize atIndex)
{
    require(atIndex <= _used);

    if (str._used != 0)
    {
        _ensureAllocation(_used + str._used);
        for (UIntSize i = _used + str._used - 1; i > atIndex + str._used - 1; i--)
        {
            _pChars[i] = _pChars[i - str._used];
        }
        for (UIntSize i = 0; i < str._used; i++)
        {
            _pChars[atIndex + i] = str._pChars[i];
        }
        _used += str._used;
    }
}

void _impl::_StringImpl::remove(UIntSize startIndex, UIntSize charCount)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);

    for (UIntSize i = startIndex + charCount; i < _used; i++)
    {
        _pChars[i - charCount] = _pChars[i];
    }
    _used -= charCount;
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, char ch)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);

    if (charCount > 1)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + 1] = _pChars[i];
        }
        _used -= charCount - 1;
    }
    else if (charCount < 1)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used + 1);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i] = _pChars[i - 1];
        }
        _used--;
    }
    //  Insert new characters
    _pChars[startIndex] = static_cast<unsigned char>(ch);
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, Char ch)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);

    if (charCount > 1)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + 1] = _pChars[i];
        }
        _used -= charCount - 1;
    }
    else if (charCount < 1)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used + 1);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i] = _pChars[i - 1];
        }
        _used++;
    }
    //  Insert new characters
    _pChars[startIndex] = ch;
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, const char * pStr)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);
    require(pStr != NULL);

    UIntSize strLen = static_cast<UIntSize>(strlen(pStr));
    if (charCount > strLen)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + strLen] = _pChars[i];
        }
        _used -= charCount - strLen;
    }
    else if (charCount < strLen)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used - charCount + strLen);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i - 2 + strLen] = _pChars[i - 1];
        }
        _used += strLen - charCount;
    }
    //  Insert new characters
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[startIndex + i] = static_cast<unsigned char>(pStr[i]);
    }
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, const Char * pStr)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);
    require(pStr != NULL);

    UIntSize strLen = _strlen(pStr);
    if (charCount > strLen)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + strLen] = _pChars[i];
        }
        _used -= charCount - strLen;
    }
    else if (charCount < strLen)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used - charCount + strLen);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i - 2 + strLen] = _pChars[i - 1];
        }
        _used += strLen - charCount;
    }
    //  Insert new characters
    for (UIntSize i = 0; i < strLen; i++)
    {
        _pChars[startIndex + i] = pStr[i];
    }
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, const char * pChars, UIntSize replacementCharCount)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);
    require(pChars != NULL);

    if (charCount > replacementCharCount)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + replacementCharCount] = _pChars[i];
        }
        _used -= charCount - replacementCharCount;
    }
    else if (charCount < replacementCharCount)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used - charCount + replacementCharCount);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i - 2 + replacementCharCount] = _pChars[i - 1];
        }
        _used += replacementCharCount - charCount;
    }
    //  Insert new characters
    for (UIntSize i = 0; i < replacementCharCount; i++)
    {
        _pChars[startIndex + i] = static_cast<unsigned char>(pChars[i]);
    }
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, const Char * pChars, UIntSize replacementCharCount)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);
    require(pChars != NULL);

    if (charCount > replacementCharCount)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + replacementCharCount] = _pChars[i];
        }
        _used -= charCount - replacementCharCount;
    }
    else if (charCount < replacementCharCount)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used - charCount + replacementCharCount);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i - 2 + replacementCharCount] = _pChars[i - 1];
        }
        _used += replacementCharCount - charCount;
    }
    //  Insert new characters
    for (UIntSize i = 0; i < replacementCharCount; i++)
    {
        _pChars[startIndex + i] = pChars[i];
    }
}

void _impl::_StringImpl::replace(UIntSize startIndex, UIntSize charCount, const _StringImpl & str)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);

    if (charCount > str._used)
    {   //  More characters are removed than inserted
        //  Change string length
        for (UIntSize i = startIndex + charCount; i < _used; i++)
        {
            _pChars[i - charCount + str._used] = _pChars[i];
        }
        _used -= charCount - str._used;
    }
    else if (charCount < str._used)
    {   //  More characters are inserted than removed
        //  Change string length
        _ensureAllocation(_used - charCount + str._used);
        for (UIntSize i = _used; i > startIndex + charCount; i--)
        {
            _pChars[i - 2 + str._used] = _pChars[i - 1];
        }
        _used += str._used - charCount;
    }
    //  Insert new characters
    for (UIntSize i = 0; i < str._used; i++)
    {
        _pChars[startIndex + i] = str._pChars[i];
    }
}

_impl::_StringImpl * _impl::_StringImpl::left(UIntSize charCount)
{
    if (charCount <= _used)
    {
        return new _StringImpl(_pChars, charCount);
    }
    else
    {
        return this;
    }
}

_impl::_StringImpl * _impl::_StringImpl::right(UIntSize charCount)
{
    if (charCount <= _used)
    {
        return new _StringImpl(_pChars + (_used - charCount), charCount);
    }
    else
    {
        return this;
    }
}

_impl::_StringImpl * _impl::_StringImpl::mid(UIntSize startIndex)
{
    require(startIndex <= _used);

    if (startIndex != 0)
    {
        return new _StringImpl(_pChars + startIndex, _used - startIndex);
    }
    else
    {
        return this;
    }
}

_impl::_StringImpl * _impl::_StringImpl::mid(UIntSize startIndex, UIntSize charCount)
{
    require(startIndex <= _used);
    require(charCount <= _used);
    require(startIndex + charCount >= startIndex &&
            startIndex + charCount >= charCount &&
            startIndex + charCount <= _used);

    if (startIndex != 0 || charCount <= _used)
    {
        return new _StringImpl(_pChars + startIndex, charCount);
    }
    else
    {
        return this;
    }
}

void _impl::_StringImpl::trimLeft()
{
    UIntSize numLeadingSpaces;
    for (numLeadingSpaces = 0; numLeadingSpaces < _used; numLeadingSpaces++)
    {
        if (!Iso10646::isSpace(_pChars[numLeadingSpaces]))
        {
            break;
        }
    }
    if (numLeadingSpaces != 0)
    {
        for (UIntSize i = numLeadingSpaces; i < _used; i++)
        {
            _pChars[i - numLeadingSpaces] = _pChars[i];
        }
        _used -= numLeadingSpaces;
    }
}

void _impl::_StringImpl::trimRight()
{
    while (_used != 0 && Iso10646::isSpace(_pChars[_used - 1]))
    {
        _used--;
    }
}

void _impl::_StringImpl::upperCase()
{
    for (UIntSize i = 0; i < _used; i++)
    {
        _pChars[i] = Iso10646::toUpper(_pChars[i]);
    }
}

void _impl::_StringImpl::lowerCase()
{
    for (UIntSize i = 0; i < _used; i++)
    {
        _pChars[i] = Iso10646::toLower(_pChars[i]);
    }
}

void _impl::_StringImpl::titleCase()
{
    for (UIntSize i = 0; i < _used; i++)
    {
        _pChars[i] = Iso10646::toTitle(_pChars[i]);
    }
}

//////////
//  Implementation helpers
UIntSize _impl::_StringImpl::_strlen(const Char * pStr)
{
    for (UIntSize i = 0; ; i++)
    {
        if (pStr[i] == 0)
        {
            return i;
        }
    }
}

void _impl::_StringImpl::_ensureAllocation(UIntSize charCount)
{
    if (charCount > _allocated)
    {   //  Must allocate a larger buffer
        UIntSize newAllocated = _allocated;
        while (newAllocated < charCount)
        {
            newAllocated = (3 * newAllocated / 2) + 1;
        }
        Char * pNewChars = new Char[newAllocated];
        if (_pChars != NULL)
        {
            for (UIntSize i = 0; i < _used; i++)
            {
                pNewChars[i] = _pChars[i];
            }
            delete [] _pChars;
        }
        _pChars = pNewChars;
        _allocated = newAllocated;
    }
}

#if defined(CARL_CPU_IA32) && defined(CARL_OS_WIN32) && defined(CARL_TOOLCHAIN_MSVC)
bool _impl::_StringImpl::_store(wchar_t * pBuffer, UIntSize bufferSize) const
{
    if (_used + 1 > bufferSize)
    {
        return false;
    }
    for (UIntSize i = 0; i < _used; i++)
    {
        pBuffer[i] = static_cast<wchar_t>(_pChars[i]);
        if (pBuffer[i] != _pChars[i])
        {
            return false;
        }
    }
    pBuffer[_used] = 0;
    return true;
}
#endif

bool _impl::_StringImpl::_store(char * pBuffer, UIntSize bufferSize) const
{
    if (_used + 1 > bufferSize)
    {
        return false;
    }
    for (UIntSize i = 0; i < _used; i++)
    {
        pBuffer[i] = static_cast<char>(_pChars[i]);
        if (static_cast<unsigned char>(pBuffer[i]) != _pChars[i])
        {
            return false;
        }
    }
    pBuffer[_used] = 0;
    return true;
}

//  End of StringImpl.cpp
