//
//  StringInputStream.cpp - Implementation of the carl::StringInputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
StringInputStream::StringInputStream(const String & str)
    :   _isOpen(true),
        _str(str),
        _currentPosition(0),
        _charsPushedBack()
{
}

//////////
//  Operations
bool StringInputStream::ready() throw(IOException)
{
    require(isOpen());

    return !_charsPushedBack.isEmpty() || _currentPosition < _str.length();
}

bool StringInputStream::read(Char & c) throw(IOException)
{
    require(isOpen());

    if (!_charsPushedBack.isEmpty())
    {
        c = _charsPushedBack.pop();
        return true;
    }
    else if (_currentPosition < _str.length())
    {
        c = _str[_currentPosition++];
        return true;
    }
    else
    {
        return false;
    }
}

UIntSize StringInputStream::read(Char * pBuffer, UIntSize charsToRead) throw(IOException)
{
    require(isOpen());
    require(pBuffer != NULL);

    for (UIntSize i = 0; i < charsToRead; i++)
    {
        if (!read(pBuffer[i]))
        {
            return i;
        }
    }
    return charsToRead;
}

bool StringInputStream::readLine(String & line, bool discardEoln /*= false*/) throw(IOException)
{
    require(isOpen());

    line.setLength(0);
    for (; ; )
    {
        Char c;

        if (!read(c))
        {
            return (line.length() != 0);
        }
        else if (c == '\n')
        {
            if (!discardEoln)
            {
                line += c;
            }
            return true;
        }
        else
        {
            line += c;
        }
    }
}

UIntSize StringInputStream::skip(UIntSize charsToSkip) throw(IOException)
{
    require(isOpen());

    Char c;
    for (UIntSize i = 0; i < charsToSkip; i++)
    {
        if (!read(c))
        {
            return i;
        }
    }
    return charsToSkip;
}

bool StringInputStream::unread(Char c) throw(IOException)
{
    require(isOpen());

    _charsPushedBack.push(c);
    return true;
}

//  End of StringInputStream.cpp
