//
//  StringOutputStream.cpp - Implementation of the carl::StringOutputStream class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
StringOutputStream::StringOutputStream()
    :   _isOpen(true),
        _str()
{
}

//////////
//  Operations
void StringOutputStream::write(Char c) throw(IOException)
{
    require(_isOpen);

    _str += c;
}

void StringOutputStream::write(const char * pBuffer) throw(IOException)
{
    require(_isOpen);

    _str += pBuffer;
}

void StringOutputStream::write(const char * pBuffer, UIntSize charsToWrite) throw(IOException)
{
    require(_isOpen);

    _str += String(pBuffer, charsToWrite);
}

void StringOutputStream::write(const Char * pBuffer) throw(IOException)
{
    require(_isOpen);

    _str += pBuffer;
}

void StringOutputStream::write(const Char * pBuffer, UIntSize charsToWrite) throw(IOException)
{
    require(_isOpen);

    _str += String(pBuffer, charsToWrite);
}

void StringOutputStream::write(const String & s) throw(IOException)
{
    require(_isOpen);

    _str += s;
}

void StringOutputStream::writeLn() throw(IOException)
{
    require(_isOpen);

    _str += '\n';
}

void StringOutputStream::writeLn(Char c) throw(IOException)
{
    _str += c;
    _str += '\n';
}

void StringOutputStream::writeLn(const char * pBuffer) throw(IOException)
{
    require(_isOpen);

    _str += pBuffer;
    _str += '\n';
}

void StringOutputStream::writeLn(const char * pBuffer, UIntSize charsToWrite) throw(IOException)
{
    require(_isOpen);

    _str += String(pBuffer, charsToWrite);
    _str += '\n';
}

void StringOutputStream::writeLn(const Char * pBuffer) throw(IOException)
{
    require(_isOpen);

    _str += pBuffer;
    _str += '\n';
}

void StringOutputStream::writeLn(const Char * pBuffer, UIntSize charsToWrite) throw(IOException)
{
    require(_isOpen);

    _str += String(pBuffer, charsToWrite);
    _str += '\n';
}

void StringOutputStream::writeLn(const String & s) throw(IOException)
{
    require(_isOpen);

    _str += s;
    _str += '\n';
}

//  End of StringOutputStream.cpp
