//
//  TimeZone.cpp - Implementation of the carl::TimeZone class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeImpl.hpp"
#include <stdlib.h>

//////////
//  Construction/destruction
_impl::_SystemTimeZone::_SystemTimeZone(Int64 bias,
                                        const String & name)
    :   _ParameterizedDstAwareTimeZone(bias)
{
    _name = name;
}

_impl::_SystemTimeZone::_SystemTimeZone(Int64 bias, 
                                        Int64 standardBias, const _DaylightSwitchpoint & standardSwitchpoint, 
                                        Int64 daylightBias, const _DaylightSwitchpoint & daylightSwitchpoint,
                                        const String & name)
    :   _ParameterizedDstAwareTimeZone(bias, 
                                       standardBias, standardSwitchpoint, 
                                       daylightBias, daylightSwitchpoint)

{
    _name = name;
}

_impl::_SystemTimeZone::~_SystemTimeZone()
{
}

//////////
//  Operations
Array<String> _impl::_SystemTimeZone::getAllNames() const
{
    Array<String> result;
    result.append(_name);
    return result;
}

String _impl::_SystemTimeZone::getCanonicalName() const
{
    return _name;
}

String _impl::_SystemTimeZone::getPreferredName() const
{
    return _name;
}

String _impl::_SystemTimeZone::getDisplayName() const
{
    return _DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_name);
}

String _impl::_SystemTimeZone::getDisplayName(const Locale & forLocale) const
{
    return _DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(_name, forLocale);
}

String _impl::_SystemTimeZone::getDisplayName(const DateTime & dateTime) const
{
    String key = _name + (_isDst(dateTime) ? ":Daylight" : ":Standard");
    return _DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(key);
}

String _impl::_SystemTimeZone::getDisplayName(const DateTime & dateTime, const Locale & forLocale) const
{
    String key = _name + (_isDst(dateTime) ? ":Daylight" : ":Standard");
    return _DateTimeManagerImpl::getInstance()->getResourceFactory()->getString(key, forLocale);
}

//  End of TimeZone.cpp
