//
//  TextFileReader.cpp - Implementation of the carl::TextFileReader class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
TextFileReader::TextFileReader(const char * pFileName) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis),
        _tis(&_bis)
{
}

TextFileReader::TextFileReader(const Char * pFileName) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis),
        _tis(&_bis)
{
}

TextFileReader::TextFileReader(const String & fileName) throw(IOException)
    :   _fis(fileName),
        _bis(&_fis),
        _tis(&_bis)
{
}

TextFileReader::TextFileReader(const PathName & pathName) throw(IOException)
    :   _fis(pathName),
        _bis(&_fis),
        _tis(&_bis)
{
}

TextFileReader::TextFileReader(const char * pFileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const Char * pFileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const String & fileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fis(fileName),
        _bis(&_fis),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const PathName & pathName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fis(pathName),
        _bis(&_fis),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const char * pFileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis, bufferSize),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const Char * pFileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fis(pFileName),
        _bis(&_fis, bufferSize),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const String & fileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fis(fileName),
        _bis(&_fis, bufferSize),
        _tis(&_bis, pCharacterSet)
{
}

TextFileReader::TextFileReader(const PathName & pathName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fis(pathName),
        _bis(&_fis, bufferSize),
        _tis(&_bis, pCharacterSet)
{
}

//////////
//  Operations
void TextFileReader::close()
{
    if (isOpen())
    {
        _tis.close();
        _bis.close();
        _fis.close();
    }
}

//  End of TextFileReader.cpp
