//
//  TextFileWriter.cpp - Implementation of the carl::TextFileWriter class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:55 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction
TextFileWriter::TextFileWriter(const char * pFileName) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos),
        _tos(&_bos)
{
}

TextFileWriter::TextFileWriter(const Char * pFileName) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos),
        _tos(&_bos)
{
}

TextFileWriter::TextFileWriter(const String & fileName) throw(IOException)
    :   _fos(fileName),
        _bos(&_fos),
        _tos(&_bos)
{
}

TextFileWriter::TextFileWriter(const PathName & pathName) throw(IOException)
    :   _fos(pathName),
        _bos(&_fos),
        _tos(&_bos)
{
}

TextFileWriter::TextFileWriter(const char * pFileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const Char * pFileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const String & fileName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fos(fileName),
        _bos(&_fos),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const PathName & pathName, CharacterSet * pCharacterSet) throw(IOException)
    :   _fos(pathName),
        _bos(&_fos),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const char * pFileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos, bufferSize),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const Char * pFileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fos(pFileName),
        _bos(&_fos, bufferSize),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const String & fileName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fos(fileName),
        _bos(&_fos, bufferSize),
        _tos(&_bos, pCharacterSet)
{
}

TextFileWriter::TextFileWriter(const PathName & pathName, CharacterSet * pCharacterSet, UIntSize bufferSize) throw(IOException)
    :   _fos(pathName),
        _bos(&_fos, bufferSize),
        _tos(&_bos, pCharacterSet)
{
}

//////////
//  Operations
void TextFileWriter::close()
{
    if (isOpen())
    {
        _tos.close();
        _bos.close();
        _fos.close();
    }
}

//  End of TextFileWriter.cpp
