//
//  Time.cpp - Implementation of the carl::Time class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:56 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;

//////////
//  Construction/destruction/assignment
Time::Time()
{
    _isValid = false;
    _hour = 0;
    _minute = 0;
    _second = 0;
    _nanosecond = 0;
}

Time::Time(int hour, int minute, int second, UInt32 nanosecond)
{
    require(hour <= 23);
    require(minute <= 59);
    require(second <= 59);
    require(nanosecond <= 999999999);

    _isValid = true;
    _hour = static_cast<UInt8>(hour);
    _minute = static_cast<UInt8>(minute);
    _second = static_cast<UInt8>(second);
    _nanosecond = nanosecond;
}

Time::Time(const Time & src)
{
    _isValid = src._isValid;
    _hour = src._hour;
    _minute = src._minute;
    _second = src._second;
    _nanosecond = src._nanosecond;
}

Time & Time::operator = (const Time & src)
{
    if (this != (&src))
    {
        _isValid = src._isValid;
        _hour = src._hour;
        _minute = src._minute;
        _second = src._second;
        _nanosecond = src._nanosecond;
    }
    return *this;
}

//////////
//  Operations
int Time::compare(const Time & time) const
{
    if (_isValid)
    {
        if (time._isValid)
        {   //  <valid time> <compare to> <valid time>
            if (_hour < time._hour)
            {
                return -1;
            }
            if (_hour > time._hour)
            {
                return 1;
            }
            if (_minute < time._minute)
            {
                return -1;
            }
            if (_minute > time._minute)
            {
                return 1;
            }
            if (_second < time._second)
            {
                return -1;
            }
            if (_second > time._second)
            {
                return 1;
            }
            if (_nanosecond < time._nanosecond)
            {
                return -1;
            }
            if (_nanosecond > time._nanosecond)
            {
                return 1;
            }
            return 0;
        }
        else
        {   //  <valid time> <compare to> <invalid time>
            return 1;
        }
    }
    else
    {
        if (time._isValid)
        {   //  <invalid time> <compare to> <valid time>
            return -1;
        }
        else
        {   //  <invalid time> <compare to> <invalid time>
            return 0;
        }
    }
}

//////////
//  Hashing
UIntSize CARL_CALLBACK StandardHashFunctions::hashTime(const Time & key)
{
    return key.getSecond() + 60 * (key.getMinute() + 24 * key.getHour());
}

//  End of Time.cpp
