//
//  TimeParse.cpp - Implementation of the carl::TimeParse class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:56 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeFormatImpl.hpp"

//////////
//  Operations
void TimeParse::registerInstance(const Locale & forLocale, TimeParse * pTimeParse)
{
    require(pTimeParse != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    _impl::_DateTimeFormatManager::getInstance()->registerTimeParse(forLocale, pTimeParse);
}

TimeParse * TimeParse::getInstance()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(Locale::getDefault());
}

TimeParse * TimeParse::getInstance(const Locale & forLocale)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeFormatManager::getInstance()->getTimeParse(forLocale);
}

//  Parsing of full strings
Time TimeParse::parse(const char * pTimeString) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const char * pTimeString, Precision precision) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan, precision);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const char * pTimeString, const String & format) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan, format);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const Char * pTimeString) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const Char * pTimeString, Precision precision) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan, precision);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const Char * pTimeString, const String & format) const throw(ParseException)
{
    require(pTimeString != NULL);

    UIntSize scan = 0;
    Time result = parse(pTimeString, scan, format);
    if (pTimeString[scan] != 0)
    { 
        throw ParseException(String(pTimeString), scan);
    }
    return result;
}

Time TimeParse::parse(const String & timeString) const throw(ParseException)
{
    UIntSize scan = 0;
    Time result = parse(timeString, scan);
    if (scan < timeString.length())
    { 
        throw ParseException(timeString, scan);
    }
    return result;
}

Time TimeParse::parse(const String & timeString, Precision precision) const throw(ParseException)
{
    UIntSize scan = 0;
    Time result = parse(timeString, scan, precision);
    if (scan < timeString.length())
    { 
        throw ParseException(timeString, scan);
    }
    return result;
}

Time TimeParse::parse(const String & timeString, const String & format) const throw(ParseException)
{
    UIntSize scan = 0;
    Time result = parse(timeString, scan, format);
    if (scan < timeString.length())
    { 
        throw ParseException(timeString, scan);
    }
    return result;
}

//////////
//  Implementation
CriticalSection * TimeParse::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of DateTimeFormat.cpp
