//
//  TimeZone.cpp - Implementation of the carl::TimeZone class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:54:56 $
//  $Revision: 1.9 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "DateTimeImpl.hpp"
#include <stdlib.h>

//////////
//  Operations
void TimeZone::registerInstance(TimeZone * pTimeZone) throw (InconsistentTimeZoneException, DuplicateTimeZoneException)
{
    require(pTimeZone != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeManagerImpl::getInstance()->registerTimeZone(pTimeZone);
}

Array<TimeZone*> TimeZone::getAllInstances()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeManagerImpl::getInstance()->getAllTimeZones();
}

TimeZone * TimeZone::getInstance(const char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

TimeZone * TimeZone::getInstance(const Char * pName)
{
    CriticalSectionLock lock(*_getStaticGuard());
    return getInstance(String(pName));
}

TimeZone * TimeZone::getInstance(const String & name)
{
    CriticalSectionLock lock(*_getStaticGuard());
    TimeZone * pTimeZone = _impl::_DateTimeManagerImpl::getInstance()->findTimeZone(name);
    if (pTimeZone != NULL)
    {
        return pTimeZone;
    }
    throw NoSuchElementError();
}

TimeZone * TimeZone::getSystem()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeManagerImpl::getInstance()->getSystem();
}

TimeZone * TimeZone::getDefault()
{
    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeManagerImpl::getInstance()->getDefault();
}

void TimeZone::setDefault(TimeZone * pTimeZone)
{
    require(pTimeZone != NULL);

    CriticalSectionLock lock(*_getStaticGuard());
    return _impl::_DateTimeManagerImpl::getInstance()->setDefault(pTimeZone);
}

//////////
//  Implementation
CriticalSection * TimeZone::_getStaticGuard()
{
    static CriticalSection guard;
    return &guard;
}

//  End of TimeZone.cpp
