//
//  Utf16leCharacterSet.cpp - Implementation of the carl::_impl::_Utf16leCharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:55:08 $
//  $Revision: 1.8 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Singleton
_impl::_Utf16leCharacterSet::_Utf16leCharacterSet()
{
    _names.append(String("UTF-16LE"));
    //
    _signature.append(0xFF);
    _signature.append(0xFE);
}

_impl::_Utf16leCharacterSet::~_Utf16leCharacterSet()
{
}

_impl::_Utf16leCharacterSet * _impl::_Utf16leCharacterSet::getInstance()
{
    static _Utf16leCharacterSet instance;
    return &instance;
}

//////////
//  Operations
CharacterSetFamily * _impl::_Utf16leCharacterSet::getFamily() const
{
    return _Iso10646CharacterSetFamily::getInstance();
}

Array<String> _impl::_Utf16leCharacterSet::getAllNames() const
{
    return _names;
}

String _impl::_Utf16leCharacterSet::getCanonicalName() const
{
    return String("UTF-16LE");
}

String _impl::_Utf16leCharacterSet::getPreferredName() const
{
    return String("UTF-16LE");
}

String _impl::_Utf16leCharacterSet::getDisplayName() const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), Locale::getDefault());
}

String _impl::_Utf16leCharacterSet::getDisplayName(const Locale & forLocale) const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), forLocale);
}

int _impl::_Utf16leCharacterSet::getMaximumEncodingLength() const
{
    return 4;
}

int _impl::_Utf16leCharacterSet::getEncodingLengthForCharacter(Char c) const
{
    if (c < 0x0000D800 || (c > 0x0000E000 && c < 0x00010000))
    {
        return 2;
    }
    else if (c >= 0x00010000 && c <= 0x0010FFFF)
    {
        return 4;
    }
    else
    {
        return -1;
    }
}

int _impl::_Utf16leCharacterSet::encodeCharacter(Char c, void * buffer) const
{
    if (c < 0x0000D800 || (c > 0x0000E000 && c < 0x00010000))
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[1] = static_cast<UInt8>(c >> 8);
        pBytes[0] = static_cast<UInt8>(c);
        return 2;
    }
    else if (c >= 0x00010000 && c <= 0x0010FFFF)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        UInt32 y = ((c - 0x00010000) / 0x00000400) + 0x0000D800;
        UInt32 z = ((c - 0x00010000) % 0x00000400) + 0x0000DC00;
        pBytes[1] = static_cast<UInt8>(y >> 8);
        pBytes[0] = static_cast<UInt8>(y);
        pBytes[3] = static_cast<UInt8>(z >> 8);
        pBytes[2] = static_cast<UInt8>(z);
        return 4;
    }
    return -1;
}

int _impl::_Utf16leCharacterSet::getEncodingLengthForLeadByte(UInt8 /*leadByte*/) const
{
    return 2;
}

int _impl::_Utf16leCharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    if (bytesAvailable < 2)
    {
        return -1;
    }
    const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
    UInt32 x = (static_cast<UInt32>(pBytes[1]) << 8) |
               (static_cast<UInt32>(pBytes[0]));
    if (x < 0x0000D800 || (x > 0x0000E000 && x < 0x00010000))
    {
        c = x;
        return 2;
    }
    else if (x >= 0x0000D800 && x <= 0x0000DBFF)
    {
        if (bytesAvailable < 4)
        {
            return -1;
        }
        UInt32 y = (static_cast<UInt32>(pBytes[3]) << 8) |
                   (static_cast<UInt32>(pBytes[2]));
        if (y >= 0x0000DC00 && y <= 0x0000DFFF)
        {
            c = ((x - 0x0000D800) * 0x00000400 + (y - 0x0000DC00)) + 0x00010000;
            return 4;
        }
    }
    return -1;
}

Char _impl::_Utf16leCharacterSet::getDefaultCharacter() const
{
    return '?';
}

Array<UInt8> _impl::_Utf16leCharacterSet::getSignature() const
{
    return _signature;
}

//  End of Utf16beCharacterSet.cpp
