//
//  Utf32beCharacterSet.cpp - Implementation of the carl::_impl::_Utf32beCharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:55:08 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Singleton
_impl::_Utf32beCharacterSet::_Utf32beCharacterSet()
{
    _names.append(String("UTF-32BE"));
    //
    _signature.append(0x00);
    _signature.append(0x00);
    _signature.append(0xFE);
    _signature.append(0xFF);
}

_impl::_Utf32beCharacterSet::~_Utf32beCharacterSet()
{
}

_impl::_Utf32beCharacterSet * _impl::_Utf32beCharacterSet::getInstance()
{
    static _Utf32beCharacterSet instance;
    return &instance;
}

//////////
//  Operations
CharacterSetFamily * _impl::_Utf32beCharacterSet::getFamily() const
{
    return _Iso10646CharacterSetFamily::getInstance();
}

Array<String> _impl::_Utf32beCharacterSet::getAllNames() const
{
    return _names;
}

String _impl::_Utf32beCharacterSet::getCanonicalName() const
{
    return String("UTF-32BE");
}

String _impl::_Utf32beCharacterSet::getPreferredName() const
{
    return String("UTF-32BE");
}

String _impl::_Utf32beCharacterSet::getDisplayName() const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), Locale::getDefault());
}

String _impl::_Utf32beCharacterSet::getDisplayName(const Locale & forLocale) const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), forLocale);
}

int _impl::_Utf32beCharacterSet::getMaximumEncodingLength() const
{
    return 4;
}

int _impl::_Utf32beCharacterSet::getEncodingLengthForCharacter(Char /*c*/) const
{
    return 4;
}

int _impl::_Utf32beCharacterSet::encodeCharacter(Char c, void * buffer) const
{
    UInt8 * pBytes = static_cast<UInt8*>(buffer);
    pBytes[0] = static_cast<UInt8>(c >> 24);
    pBytes[1] = static_cast<UInt8>(c >> 16);
    pBytes[2] = static_cast<UInt8>(c >> 8);
    pBytes[3] = static_cast<UInt8>(c);
    return 4;
}

int _impl::_Utf32beCharacterSet::getEncodingLengthForLeadByte(UInt8 /*leadByte*/) const
{
    return 4;
}

int _impl::_Utf32beCharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    if (bytesAvailable >= 4)
    {
        const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
        c = (static_cast<UInt32>(pBytes[0]) << 24) |
            (static_cast<UInt32>(pBytes[1]) << 16) |
            (static_cast<UInt32>(pBytes[2]) << 8) |
            (static_cast<UInt32>(pBytes[3]));
        return 4;
    }
    return -1;
}

Char _impl::_Utf32beCharacterSet::getDefaultCharacter() const
{
    return '?';
}

Array<UInt8> _impl::_Utf32beCharacterSet::getSignature() const
{
    return _signature;
}

//  End of Utf32beCharacterSet.cpp
