//
//  Utf8CharacterSet.cpp - Implementation of the carl::_impl::_Utf8CharacterSet class
//
//  Copyright (C) 2006-2008, Cybernetic Intelligence GmbH
//
//  $Date: 2008/02/22 11:55:08 $
//  $Revision: 1.10 $
//  $Author: akapusti $
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//////////
#include "CARL.hpp"
using namespace carl;
#include "CharsetImpl.hpp"

//////////
//  Singleton
_impl::_Utf8CharacterSet::_Utf8CharacterSet()
{
    _names.append(String("UTF-8"));
    //
    _signature.append(0xEF);
    _signature.append(0xBB);
    _signature.append(0xBF);
}

_impl::_Utf8CharacterSet::~_Utf8CharacterSet()
{
}

_impl::_Utf8CharacterSet * _impl::_Utf8CharacterSet::getInstance()
{
    static _Utf8CharacterSet instance;
    return &instance;
}

//////////
//  Operations
CharacterSetFamily * _impl::_Utf8CharacterSet::getFamily() const
{
    return _Iso10646CharacterSetFamily::getInstance();
}

Array<String> _impl::_Utf8CharacterSet::getAllNames() const
{
    return _names;
}

String _impl::_Utf8CharacterSet::getCanonicalName() const
{
    return String("UTF-8");
}

String _impl::_Utf8CharacterSet::getPreferredName() const
{
    return String("UTF-8");
}

String _impl::_Utf8CharacterSet::getDisplayName() const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), Locale::getDefault());
}

String _impl::_Utf8CharacterSet::getDisplayName(const Locale & forLocale) const
{
    return _CharsetManagerImpl::getInstance()->getResourceFactory()->getString("CS:" + getCanonicalName(), forLocale);
}

int _impl::_Utf8CharacterSet::getMaximumEncodingLength() const
{
    return 7;
}

int _impl::_Utf8CharacterSet::getEncodingLengthForCharacter(Char c) const
{
    if ((c & 0xFFFFFF80) == 0 && c != 0)
    {
        return 1;
    }
    else if ((c & 0xFFFFF800) == 0)
    {
        return 2;
    }
    else if ((c & 0xFFFF0000) == 0)
    {
        return 3;
    }
    else if ((c & 0xFFE00000) == 0)
    {
        return 4;
    }
    else if ((c & 0xFC000000) == 0)
    {
        return 5;
    }
    else if ((c & 0x80000000) == 0)
    {
        return 6;
    }
    else
    {
        return 7;
    }
}

int _impl::_Utf8CharacterSet::encodeCharacter(Char c, void * buffer) const
{
    if ((c & 0xFFFFFF80) == 0 && c != 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(c);
        return 1;
    }
    else if ((c & 0xFFFFF800) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(((c >> 6) & 0x1F) | 0xC0);
        pBytes[1] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 2;
    }
    else if ((c & 0xFFFF0000) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(((c >> 12) & 0x0F) | 0xE0);
        pBytes[1] = static_cast<UInt8>(((c >> 6) & 0x3F) | 0x80);
        pBytes[2] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 3;
    }
    else if ((c & 0xFFE00000) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(((c >> 18) & 0x07) | 0xF0);
        pBytes[1] = static_cast<UInt8>(((c >> 12) & 0x3F) | 0x80);
        pBytes[2] = static_cast<UInt8>(((c >> 6)& 0x3F) | 0x80);
        pBytes[3] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 4;
    }
    else if ((c & 0xFC000000) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(((c >> 24) & 0x03) | 0xF8);
        pBytes[1] = static_cast<UInt8>(((c >> 18) & 0x3F) | 0x80);
        pBytes[2] = static_cast<UInt8>(((c >> 12)& 0x3F) | 0x80);
        pBytes[3] = static_cast<UInt8>(((c >> 6) & 0x3F) | 0x80);
        pBytes[4] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 5;
    }
    else if ((c & 0x80000000) == 0)
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(((c >> 30) & 0x01) | 0xFC);
        pBytes[1] = static_cast<UInt8>(((c >> 24) & 0x3F) | 0x80);
        pBytes[2] = static_cast<UInt8>(((c >> 18)& 0x3F) | 0x80);
        pBytes[3] = static_cast<UInt8>(((c >> 12) & 0x3F) | 0x80);
        pBytes[4] = static_cast<UInt8>(((c >> 6) & 0x3F) | 0x80);
        pBytes[5] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 6;
    }
    else
    {
        UInt8 * pBytes = static_cast<UInt8*>(buffer);
        pBytes[0] = static_cast<UInt8>(0xFE);
        pBytes[1] = static_cast<UInt8>(((c >> 30) & 0x3F) | 0x80);
        pBytes[2] = static_cast<UInt8>(((c >> 24)& 0x3F) | 0x80);
        pBytes[3] = static_cast<UInt8>(((c >> 18) & 0x3F) | 0x80);
        pBytes[4] = static_cast<UInt8>(((c >> 12) & 0x3F) | 0x80);
        pBytes[5] = static_cast<UInt8>(((c >> 6) & 0x3F) | 0x80);
        pBytes[6] = static_cast<UInt8>((c & 0x3F) | 0x80);
        return 7;
    }
}

int _impl::_Utf8CharacterSet::getEncodingLengthForLeadByte(UInt8 leadByte) const
{
    if (leadByte == 0xFE)
    {
        return 7;
    }
    else if ((leadByte & 0xFE) == 0xFC)
    {
        return 6;
    }
    else if ((leadByte & 0xFC) == 0xF8)
    {
        return 5;
    }
    else if ((leadByte & 0xF8) == 0xF0)
    {
        return 4;
    }
    else if ((leadByte & 0xF0) == 0xE0)
    {
        return 3;
    }
    else if ((leadByte & 0xE0) == 0xC0)
    {
        return 2;
    }
    else if ((leadByte & 0x80) == 0x00)
    {
        return 1;
    }
    return -1;
}

int _impl::_Utf8CharacterSet::decodeCharacter(const void * buffer, UIntSize bytesAvailable, Char & c) const
{
    if (bytesAvailable < 1)
    {
        return -1;
    }
    const UInt8 * pBytes = static_cast<const UInt8*>(buffer);
    if (pBytes[0] == 0xFE)
    {
        if ((pBytes[1] & 0xC0) != 0x80 ||
            (pBytes[2] & 0xC0) != 0x80 ||
            (pBytes[3] & 0xC0) != 0x80 ||
            (pBytes[4] & 0xC0) != 0x80 ||
            (pBytes[5] & 0xC0) != 0x80 ||
            (pBytes[6] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        UInt32 b2 = (pBytes[2] & 0x0000003F);
        UInt32 b3 = (pBytes[3] & 0x0000003F);
        UInt32 b4 = (pBytes[4] & 0x0000003F);
        UInt32 b5 = (pBytes[5] & 0x0000003F);
        UInt32 b6 = (pBytes[6] & 0x0000003F);
        c = (b1 << 30) | (b2 << 24) | (b3 << 18) | (b4 << 12) | (b5 << 6) | b6;
        return 7;
    }
    else if ((pBytes[0] & 0xFE) == 0xFC)
    {
        if ((pBytes[1] & 0xC0) != 0x80 ||
            (pBytes[2] & 0xC0) != 0x80 ||
            (pBytes[3] & 0xC0) != 0x80 ||
            (pBytes[4] & 0xC0) != 0x80 ||
            (pBytes[5] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b0 = (pBytes[0] & 0x00000001);
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        UInt32 b2 = (pBytes[2] & 0x0000003F);
        UInt32 b3 = (pBytes[3] & 0x0000003F);
        UInt32 b4 = (pBytes[4] & 0x0000003F);
        UInt32 b5 = (pBytes[5] & 0x0000003F);
        c = (b0 << 30) | (b1 << 24) | (b2 << 18) | (b3 << 12) | (b4 << 6) | b5;
        return 6;
    }
    else if ((pBytes[0] & 0xFC) == 0xF8)
    {
        if ((pBytes[1] & 0xC0) != 0x80 ||
            (pBytes[2] & 0xC0) != 0x80 ||
            (pBytes[3] & 0xC0) != 0x80 ||
            (pBytes[4] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b0 = (pBytes[0] & 0x00000003);
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        UInt32 b2 = (pBytes[2] & 0x0000003F);
        UInt32 b3 = (pBytes[3] & 0x0000003F);
        UInt32 b4 = (pBytes[4] & 0x0000003F);
        c = (b0 << 24) | (b1 << 18) | (b2 << 12) | (b3 << 6) | b4;
        return 5;
    }
    else if ((pBytes[0] & 0xF8) == 0xF0)
    {
        if ((pBytes[1] & 0xC0) != 0x80 ||
            (pBytes[2] & 0xC0) != 0x80 ||
            (pBytes[3] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b0 = (pBytes[0] & 0x00000007);
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        UInt32 b2 = (pBytes[2] & 0x0000003F);
        UInt32 b3 = (pBytes[3] & 0x0000003F);
        c = (b0 << 18) | (b1 << 12) | (b2 << 6) | b3;
        return 4;
    }
    else if ((pBytes[0] & 0xF0) == 0xE0)
    {
        if ((pBytes[1] & 0xC0) != 0x80 ||
            (pBytes[2] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b0 = (pBytes[0] & 0x0000000F);
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        UInt32 b2 = (pBytes[2] & 0x0000003F);
        c = (b0 << 12) | (b1 << 6) | b2;
        return 3;
    }
    else if ((pBytes[0] & 0xE0) == 0xC0)
    {
        if ((pBytes[1] & 0xC0) != 0x80)
        {
            return -1;
        }
        UInt32 b0 = (pBytes[0] & 0x0000001F);
        UInt32 b1 = (pBytes[1] & 0x0000003F);
        c = (b0 << 6) | b1;
        return 2;
    }
    else if ((pBytes[0] & 0x80) == 0x00)
    {
        c = pBytes[0];
        return 1;
    }
    return -1;
}

Char _impl::_Utf8CharacterSet::getDefaultCharacter() const
{
    return '?';
}

Array<UInt8> _impl::_Utf8CharacterSet::getSignature() const
{
    return _signature;
}

//  End of Utf8CharacterSet.cpp
