//
//  Main.cpp
//
//  The CARL character set - aware text streams test.
//
//  $Date: 2008/02/22 12:09:20 $
//  $Revision: 1.6 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 
    Array<Char> charactersToTest;

    //  Exercise all character sets
    Array<CharacterSet*> allCharacterSets = CharacterSet::getAllInstances();
    for (UIntSize i = 0; i < allCharacterSets.size(); i++)
    {
        bool dataMismatch = false;
        CharacterSet * pCharacterSet = allCharacterSets[i];
        StandardStreams::out.write("Testing " + pCharacterSet->getDisplayName() + "...");
        //  Determine a set of code points that are supported by this character set
        charactersToTest.setSize(0);
        for (Char c = 0; c < 0x00200000; c++)
        {
            if (pCharacterSet->getEncodingLengthForCharacter(c) != -1)
            {
                charactersToTest.append(c);
            }
        }
        //  Prepare output streams
        MemoryOutputStream mos;
        TextOutputStream tos(&mos, pCharacterSet);
        //  Write!
        TimeSpan beginWriteTime = TimeSpan::getTimeSpanSinceReferencePoint();
        for (UIntSize i = 0; i < charactersToTest.size(); i++)
        {
            tos.write(charactersToTest[i]);
        }
        TimeSpan endWriteTime = TimeSpan::getTimeSpanSinceReferencePoint();
        tos.close();
        Array<UInt8> binaryData = mos.getContents();
        //  Prepare to read
        MemoryInputStream mis(binaryData.getData(), binaryData.size(), false);
        TextInputStream tis(&mis, pCharacterSet);
        //  Read & compare
        Char c;
        TimeSpan beginReadTime = TimeSpan::getTimeSpanSinceReferencePoint();
        try
        {
            for (UIntSize i = 0; i < charactersToTest.size(); i++)
            {
                if (!tis.read(c) || c != charactersToTest[i])
                {
                    dataMismatch = true;
                    break;
                }
            }
            if (tis.read(c))
            {
                dataMismatch = true;
            }
        }
        catch (InvalidDataException)
        {
            dataMismatch = true;
        }
        TimeSpan endReadTime = TimeSpan::getTimeSpanSinceReferencePoint();

        if (dataMismatch)
        {
            StandardStreams::out.writeLn("Failed");
            exitCode = Process::ExitCode_Failure;
        }
        else
        {
            TimeSpan readTime = endReadTime - beginReadTime;
            if (readTime.getNanoseconds() == 0)
            {
                readTime = TimeSpan(1);
            }
            TimeSpan writeTime = endWriteTime - beginWriteTime;
            if (writeTime.getNanoseconds() == 0)
            {
                writeTime = TimeSpan(1);
            }
            StandardStreams::out.writeLn("OK, read speed " +
                                         NumberFormat::getInstance()->format((charactersToTest.size() * CARL_UINT64_CONSTANT(1000000000)) / (readTime.getNanoseconds())) +
                                         " chars/sec, write speed " +
                                         NumberFormat::getInstance()->format((charactersToTest.size() * CARL_UINT64_CONSTANT(1000000000)) / (writeTime.getNanoseconds())) +
                                         " chars/sec");
        }
    }

    return exitCode;
}

//  End of Main.cpp
