//
//  Main.cpp
//
//  The CARL Array test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.3 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

#define MATCH_ARRAYS(a1,a2)                                 \
    if ((a1).size() != (a2).size())                         \
    {                                                       \
        exitCode = Process::ExitCode_Failure;               \
    }                                                       \
    else                                                    \
    {                                                       \
        for (UIntSize i = 0; i < (a1).size(); i++)          \
        {                                                   \
            if ((a1)[i] != (a2)[i])                         \
            {                                               \
                exitCode = Process::ExitCode_Failure;       \
            }                                               \
        }                                                   \
    }

#define MATCH_ARRAY_ELEMENTS(a,e)                           \
    if ((a).size() != sizeof(e) / sizeof((e)[0]))           \
    {                                                       \
        exitCode = Process::ExitCode_Failure;               \
    }                                                       \
    else                                                    \
    {                                                       \
        for (UIntSize i = 0; i < (a).size(); i++)           \
        {                                                   \
            if ((a)[i] != (e)[i])                           \
            {                                               \
                exitCode = Process::ExitCode_Failure;       \
            }                                               \
        }                                                   \
    }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    //  Construction
    {
        Array<int> a;
        CHECK(a.size() == 0)
    }
    {
        Array<int> a(10);
        CHECK(a.size() == 0)
    }
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        MATCH_ARRAY_ELEMENTS(a, e)
        Array<int> b(a);
        MATCH_ARRAYS(a, b)
    }

    //  Assignment
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        Array<int> b;
        b = a;
        MATCH_ARRAYS(a, b)
    }

    //  isEmpty
    {
        Array<int> a;
        CHECK(a.isEmpty())
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> b(e, sizeof(e) / sizeof(e[0]));
        CHECK(!b.isEmpty())
    }

    //  setSize()
    {
        Array<int> a;
        a.setSize(5);
        CHECK(a.size() == 5)
    }

    //  Operator []
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        a[2] = 6;
        static int f[] = { 1, 2, 6, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, f)
    }

    //  append
    {
        Array<int> a;
        a.append(1);
        a.append(2);
        a.append(3);
        a.append(4);
        a.append(5);
        static int e[] = { 1, 2, 3, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, e)
        static int f[] = { 6, 7, 8, 9 };
        a.append(f, sizeof(f) / sizeof(f[0]));
        static int g[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9 };
        MATCH_ARRAY_ELEMENTS(a, g)
        static int h[] = { 10, 11 };
        Array<int> b(h, sizeof(h) / sizeof(h[0]));
        a.append(b);
        static int k[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
        MATCH_ARRAY_ELEMENTS(a, k)
        b.append(b);
        static int m[] = { 10, 11, 10, 11 };
        MATCH_ARRAY_ELEMENTS(b, m)
    }

    //  insertAt
    {
        Array<int> a;
        a.insertAt(0, 5);
        a.insertAt(0, 4);
        a.insertAt(0, 3);
        a.insertAt(0, 2);
        a.insertAt(0, 1);
        static int e[] = { 1, 2, 3, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, e)
        static int f[] = { 6, 7, 8, 9 };
        a.insertAt(2, f, sizeof(f) / sizeof(f[0]));
        static int g[] = { 1, 2, 6, 7, 8, 9, 3, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, g)
        static int h[] = { 10, 11 };
        Array<int> b(h, sizeof(h) / sizeof(h[0]));
        a.insertAt(1, b);
        static int k[] = { 1, 10, 11, 2, 6, 7, 8, 9, 3, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, k)
        b.insertAt(1, b);
        static int m[] = { 10, 10, 11, 11 };
        MATCH_ARRAY_ELEMENTS(b, m)
    }

    //  removeFirst
    {
        static int e[] = { 1, 2, 3, 2, 4, 2, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        a.removeFirst(2);
        static int f[] = { 1, 3, 2, 4, 2, 5 };
        MATCH_ARRAY_ELEMENTS(a, f)
        a.removeFirst(6);
        MATCH_ARRAY_ELEMENTS(a, f)
    }

    //  removeLast
    {
        static int e[] = { 1, 2, 3, 2, 4, 2, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        a.removeLast(2);
        static int f[] = { 1, 2, 3, 2, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, f)
        a.removeLast(6);
        MATCH_ARRAY_ELEMENTS(a, f)
    }

    //  removeAll
    {
        static int e[] = { 1, 2, 3, 2, 4, 2, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        a.removeAll(2);
        static int f[] = { 1, 3, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, f)
        a.removeAll();
        CHECK(a.size() == 0)
    }

    //  removeAt
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        a.removeAt(2);
        static int f[] = { 1, 2, 4, 5 };
        MATCH_ARRAY_ELEMENTS(a, f)
    }

    //  contains
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        CHECK(a.contains(1))
        CHECK(a.contains(2))
        CHECK(a.contains(3))
        CHECK(a.contains(4))
        CHECK(a.contains(5))
        CHECK(!a.contains(6))
    }

    //  findFirst
    {
        static int e[] = { 1, 2, 3, 2, 4, 2, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        UIntSize index;
        CHECK(a.findFirst(1, index) && index == 0)
        CHECK(a.findFirst(2, index) && index == 1)
        CHECK(a.findFirst(2, index, 2) && index == 3)
        CHECK(!a.findFirst(6, index))
        CHECK(!a.findFirst(2, index, 10))
    }

    //  findLast
    {
        static int e[] = { 1, 2, 3, 2, 4, 2, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        UIntSize index;
        CHECK(a.findLast(1, index) && index == 0)
        CHECK(a.findLast(2, index) && index == 5)
        CHECK(!a.findLast(6, index))
    }

    //  enumerateElementsStartToEnd
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        Iterator<int> it = a.enumerateElementsStartToEnd();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[i])
            it.advance();
        }
        CHECK(it.finished())
        it = a.enumerateElementsStartToEnd();
        a.removeAll();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[i])
            it.advance();
        }
        CHECK(it.finished())
    }

    //  enumerateElementsEndToStart
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Array<int> a(e, sizeof(e) / sizeof(e[0]));
        Iterator<int> it = a.enumerateElementsEndToStart();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
            it.advance();
        }
        CHECK(it.finished())
        it = a.enumerateElementsEndToStart();
        a.removeAll();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
            it.advance();
        }
        CHECK(it.finished())
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
