//
//  Main.cpp
//
//  The CARL HashMap test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

struct Mapping
{
    int key;
    int value;
};

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

#define MATCH_MAP_MEMBERS(s,e)                              \
    if ((s).size() != sizeof(e) / sizeof((e)[0]))           \
    {                                                       \
        exitCode = Process::ExitCode_Failure;               \
    }                                                       \
    else                                                    \
    {                                                       \
        for (UIntSize i = 0; i < (s).size(); i++)           \
        {                                                   \
            if (!s.containsKey((e)[i].key) ||               \
                 s.get((e)[i].key) != (e)[i].value)         \
            {                                               \
                exitCode = Process::ExitCode_Failure;       \
            }                                               \
        }                                                   \
    }

#define INIT_MAP(s,e)                                           \
    for (UIntSize i = 0; i < sizeof(e) / sizeof((e)[0]); i++)   \
    {                                                           \
        s.insert((e)[i].key, (e)[i].value);                     \
    }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        CHECK(s.size() == 0);
    }

    {
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt, 10);
        CHECK(s.size() == 0);
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        MATCH_MAP_MEMBERS(s, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {3,4}, {5,6}, {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e1)
        MATCH_MAP_MEMBERS(s, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {3,4}, {5,6}, {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e1)
        HashMap<int,int> s1(s);
        MATCH_MAP_MEMBERS(s1, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {3,4}, {5,6}, {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e1)
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s;
        MATCH_MAP_MEMBERS(s1, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {3,4}, {5,6}, {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e1)
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s;
        MATCH_MAP_MEMBERS(s, e)
        MATCH_MAP_MEMBERS(s1, e)
        s.removeKey(3);
        MATCH_MAP_MEMBERS(s1, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {3,4}, {5,6}, {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e1)
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        CHECK(s1.isEmpty())
        s1 = s;
        CHECK(!s1.isEmpty())
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,1}, {7,8}, {9,10} };
        static Mapping e2[] = { {1,1}, {3,4}, {5,6}, {7,8}, {9,10} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s1, e1)
        s.insert(s1);
        MATCH_MAP_MEMBERS(s, e2)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        static Mapping e1[] = { {1,2}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        s.removeKey(3);
        MATCH_MAP_MEMBERS(s, e1)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        s.removeKey(2);
        MATCH_MAP_MEMBERS(s, e)
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        s.removeAll();
        CHECK(s.isEmpty())
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        UIntSize elementCount = 0;
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s1, e)
        for (Iterator<int> it = s.enumerateKeys(); !it.finished(); it.advance())
        {
            CHECK(s1.containsKey(it.item()))
            elementCount++;
            s1.removeKey(it.item());
        }
        CHECK(elementCount == s.size())
    }

    {
        static Mapping e[] = { {1,2}, {3,4}, {5,6} };
        HashMap<int,int> s(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s, e)
        UIntSize elementCount = 0;
        HashMap<int,int> s1(StandardHashFunctions::hashSignedInt);
        INIT_MAP(s1, e)
        for (Iterator<Pair<int,int> > it = s.enumerateElements(); !it.finished(); it.advance())
        {
            CHECK(s1.containsKey(it.item().first()))
            CHECK(s1.get(it.item().first()) == it.item().second())
            elementCount++;
            s1.removeKey(it.item().first());
        }
        CHECK(elementCount == s.size())
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
