//
//  Main.cpp
//
//  The CARL HashSet test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

#define MATCH_SET_MEMBERS(s,e)                              \
    if ((s).size() != sizeof(e) / sizeof((e)[0]))           \
    {                                                       \
        exitCode = Process::ExitCode_Failure;               \
    }                                                       \
    else                                                    \
    {                                                       \
        for (UIntSize i = 0; i < (s).size(); i++)           \
        {                                                   \
            if (!s.contains((e)[i]))                        \
            {                                               \
                exitCode = Process::ExitCode_Failure;       \
            }                                               \
        }                                                   \
    }

#define INIT_SET(s,e)                                           \
    for (UIntSize i = 0; i < sizeof(e) / sizeof((e)[0]); i++)   \
    {                                                           \
        s.insert((e)[i]);                                       \
    }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        CHECK(s.size() == 0);
    }

    {
        HashSet<int> s(StandardHashFunctions::hashSignedInt, 10);
        CHECK(s.size() == 0);
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        MATCH_SET_MEMBERS(s, e)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e1)
        MATCH_SET_MEMBERS(s, e)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e1)
        HashSet<int> s1(s);
        MATCH_SET_MEMBERS(s1, e)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e1)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s;
        MATCH_SET_MEMBERS(s1, e)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e1)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s;
        MATCH_SET_MEMBERS(s, e)
        MATCH_SET_MEMBERS(s1, e)
        s.remove(2);
        MATCH_SET_MEMBERS(s1, e)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e1)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        CHECK(s1.isEmpty())
        s1 = s;
        CHECK(!s1.isEmpty())
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 6, 7, 8 };
        static int e2[] = { 1, 2, 3, 4, 5, 6, 7, 8 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        s.insert(s1);
        MATCH_SET_MEMBERS(s, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s.remove(3);
        MATCH_SET_MEMBERS(s, e1)
    }

    {

        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 3, 4 };
        static int e2[] = { 2, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        s.remove(s1);
        MATCH_SET_MEMBERS(s, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s.removeAll();
        CHECK(s.isEmpty())
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e)
        UIntSize elementCount = 0;
        for (Iterator<int> it = s.enumerateElements(); !it.finished(); it.advance())
        {
            CHECK(s1.contains(it.item()))
            elementCount++;
            s1.remove(it.item());
        }
        CHECK(elementCount == s.size())
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e)
        CHECK(s == s1)
        s1.remove(1);
        CHECK(!(s == s1))
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e)
        CHECK(!(s != s1))
        s1.remove(1);
        CHECK(s != s1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4 };
        static int e2[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        INIT_SET(s2, e2)
        CHECK(!(s < s))
        CHECK(!(s < s1))
        CHECK(s < s2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4 };
        static int e2[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        INIT_SET(s2, e2)
        CHECK(s <= s)
        CHECK(!(s <= s1))
        CHECK(s <= s2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4 };
        static int e2[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        INIT_SET(s2, e2)
        CHECK(!(s > s))
        CHECK(s > s1)
        CHECK(!(s > s2))
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4 };
        static int e2[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        INIT_SET(s2, e2)
        CHECK(s >= s)
        CHECK(s >= s1)
        CHECK(!(s >= s2))
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        CHECK(!(s >= s1) && !(s <= s1))
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s += 6;
        MATCH_SET_MEMBERS(s, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 5, 6, 7 };
        static int e2[] = { 1, 2, 3, 4, 5, 6, 7 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        s += s1;
        MATCH_SET_MEMBERS(s, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s -= 3;
        MATCH_SET_MEMBERS(s, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 2, 5, 6, 7 };
        static int e2[] = { 1, 3, 4 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        s -= s1;
        MATCH_SET_MEMBERS(s, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 3 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s *= 3;
        MATCH_SET_MEMBERS(s, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        s *= 6;
        CHECK(s.isEmpty())
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 2, 5, 6, 7 };
        static int e2[] = { 2, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        s *= s1;
        MATCH_SET_MEMBERS(s, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 3, 4, 5, 6 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s + 6;
        MATCH_SET_MEMBERS(s1, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 5, 6, 7 };
        static int e2[] = { 1, 2, 3, 4, 5, 6, 7 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        s2 = s + s1;
        MATCH_SET_MEMBERS(s2, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 1, 2, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s - 3;
        MATCH_SET_MEMBERS(s1, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 2, 5, 6, 7 };
        static int e2[] = { 1, 3, 4 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        s2 = s - s1;
        MATCH_SET_MEMBERS(s2, e2)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 3 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s * 3;
        MATCH_SET_MEMBERS(s1, e1)
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        s1 = s * 6;
        CHECK(s1.isEmpty())
    }

    {
        static int e[] = { 1, 2, 3, 4, 5 };
        static int e1[] = { 2, 5, 6, 7 };
        static int e2[] = { 2, 5 };
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        INIT_SET(s, e)
        HashSet<int> s1(StandardHashFunctions::hashSignedInt);
        INIT_SET(s1, e1)
        HashSet<int> s2(StandardHashFunctions::hashSignedInt);
        s2 = s * s1;
        MATCH_SET_MEMBERS(s2, e2)
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
