//
//  Main.cpp
//
//  The CARL Iterator test.
//
//  $Date: 2008/02/25 14:47:12 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    //  Array forward iterator
    {
        Array<int> a;
        a.append(1);
        a.append(2);
        a.append(3);
        a.append(4);
        a.append(5);
        Iterator<int> it = a.enumerateElementsStartToEnd();
        a[0] = 6;
        a[1] = 7;
        a[2] = 8;
        a[3] = 9;
        a[4] = 0;
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(it.finished())
    }

    //  Array backward iterator
    {
        Array<int> a;
        a.append(1);
        a.append(2);
        a.append(3);
        a.append(4);
        a.append(5);
        Iterator<int> it = a.enumerateElementsEndToStart();
        a[0] = 6;
        a[1] = 7;
        a[2] = 8;
        a[3] = 9;
        a[4] = 0;
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(it.finished())
    }

    //  Queue forward iterator
    {
        Queue<int> q;
        q.enqueue(1);
        q.enqueue(2);
        q.enqueue(3);
        q.enqueue(4);
        q.enqueue(5);
        Iterator<int> it = q.enumerateElementsHeadToTail();
        q.removeAll();
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(it.finished())
    }

    //  Queue backward iterator
    {
        Queue<int> q;
        q.enqueue(1);
        q.enqueue(2);
        q.enqueue(3);
        q.enqueue(4);
        q.enqueue(5);
        Iterator<int> it = q.enumerateElementsTailToHead();
        q.removeAll();
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(it.finished())
    }

    //  Stack top-down iterator
    {
        Stack<int> s;
        s.push(1);
        s.push(2);
        s.push(3);
        s.push(4);
        s.push(5);
        Iterator<int> it = s.enumerateElementsTopToBottom();
        s.removeAll();
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(it.finished())
    }

    //  Stack bottom-up iterator
    {
        Stack<int> s;
        s.push(1);
        s.push(2);
        s.push(3);
        s.push(4);
        s.push(5);
        Iterator<int> it = s.enumerateElementsBottomToTop();
        s.removeAll();
        CHECK(!it.finished())
        CHECK(it.item() == 1)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 2)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 3)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 4)
        it.advance();
        CHECK(!it.finished())
        CHECK(it.item() == 5)
        it.advance();
        CHECK(it.finished())
    }

    //  Hash set iterator
    {
        HashSet<int> s(StandardHashFunctions::hashSignedInt);
        s.insert(1);
        s.insert(2);
        s.insert(3);
        s.insert(4);
        s.insert(5);
        Iterator<int> it = s.enumerateElements();
        s.removeAll();
        bool found[5] = { false, false, false, false, false };
        int valueCount = 0;
        for (; !it.finished(); it.advance())
        {
            int value = it.item();
            CHECK(value >= 1 && value <= 5)
            if (value >= 1 && value <= 5)
            {
                found[value - 1] = true;
            }
            valueCount++;
        }
        CHECK(valueCount == 5)
        CHECK(found[0])
        CHECK(found[1])
        CHECK(found[2])
        CHECK(found[3])
        CHECK(found[4])
    }

    //  Hash map iterator
    {
        HashMap<int, int> m(StandardHashFunctions::hashSignedInt);
        m.insert(1, 21);
        m.insert(2, 22);
        m.insert(3, 23);
        m.insert(4, 24);
        m.insert(5, 25);
        Iterator<Pair<int, int> > it = m.enumerateElements();
        m.removeAll();
        int value[5] = { 0, 0, 0, 0, 0 };
        int valueCount = 0;
        for (; !it.finished(); it.advance())
        {
            Pair<int, int> mapping = it.item();
            CHECK(mapping.first() >= 1 && mapping.first() <= 5)
            if (mapping.first() >= 1 && mapping.first() <= 5)
            {
                value[mapping.first() - 1] = mapping.second();
            }
            valueCount++;
        }
        CHECK(valueCount == 5)
        CHECK(value[0] == 21)
        CHECK(value[1] == 22)
        CHECK(value[2] == 23)
        CHECK(value[3] == 24)
        CHECK(value[4] == 25)
    }

    return exitCode;
}

//  End of Main.cpp
