//
//  Main.cpp
//
//  The CARL Stack test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

#define INIT_STACK(q,e)                                         \
    for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)     \
    {                                                           \
        q.push(e[i]);                                        \
    }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    //  Construction
    {
        Stack<int> a;
        CHECK(a.size() == 0)
        static int e[] = { 1, 2, 3, 4, 5 };
        INIT_STACK(a, e)
        Stack<int> b(a);
        CHECK(b.size() == sizeof(e) / sizeof(e[0]))
    }

    //  Assignment
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        CHECK(a.size() == sizeof(e) / sizeof(e[0]))
        Stack<int> b;
        b = a;
        CHECK(b.size() == sizeof(e) / sizeof(e[0]))
    }

    //  isEmpty
    {
        Stack<int> a;
        CHECK(a.isEmpty())
        a.push(1);
        CHECK(!a.isEmpty())
    }

    //  push/pop
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(a.pop() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
        }
        CHECK(a.isEmpty())
    }

    //  top
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(a.top() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
            a.pop();
        }
    }

    //  removeAll
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        CHECK(!a.isEmpty())
        a.removeAll();
        CHECK(a.isEmpty())
    }

    //  enumerateElementsTopToBottom
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        Iterator<int> it = a.enumerateElementsTopToBottom();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
            it.advance();
        }
        CHECK(it.finished())
        it = a.enumerateElementsTopToBottom();
        a.removeAll();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[sizeof(e) / sizeof(e[0]) - 1 - i])
            it.advance();
        }
        CHECK(it.finished())
    }

    //  enumerateElementsBottomToTop
    {
        static int e[] = { 1, 2, 3, 4, 5 };
        Stack<int> a;
        INIT_STACK(a, e)
        Iterator<int> it = a.enumerateElementsBottomToTop();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[i])
            it.advance();
        }
        CHECK(it.finished())
        it = a.enumerateElementsBottomToTop();
        a.removeAll();
        for (UIntSize i = 0; i < sizeof(e) / sizeof(e[0]); i++)
        {
            CHECK(!it.finished())
            CHECK(it.item() == e[i])
            it.advance();
        }
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
