//
//  Main.cpp
//
//  The CARL Date test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        Date d;
        CHECK(!d.isValid())
    }

    {
        Date d(2008, 1, 1);
        CHECK(d.isValid())
    }

    {
        Date d;
        CHECK(d.getYear() == 0)
        CHECK(d.getMonth() == 0)
        CHECK(d.getDay() == 0)
    }

    {
        Date d(2008, 6, 22);
        CHECK(d.getYear() == 2008)
        CHECK(d.getMonth() == 6)
        CHECK(d.getDay() == 22)
    }

    {
        Date d1, d2;
        CHECK(d1 == d2)
        CHECK(!(d1 != d2))
        CHECK(!(d1 < d2))
        CHECK(d1 <= d2)
        CHECK(!(d1 > d2))
        CHECK(d1 >= d2)
    }

    {
        Date d1(2008, 2, 2), d2(2008, 2, 1);
        CHECK(!(d1 == d2))
        CHECK(d1 != d2)
        CHECK(!(d1 < d2))
        CHECK(!(d1 <= d2))
        CHECK(d1 > d2)
        CHECK(d1 >= d2)
    }

    {
        Date d1, d2(2008, 2, 1);
        CHECK(!(d1 == d2))
        CHECK(d1 != d2)
        CHECK(d1 < d2)
        CHECK(d1 <= d2)
        CHECK(!(d1 > d2))
        CHECK(!(d1 >= d2))
    }

    return exitCode;
}

//  End of Main.cpp
