//
//  Main.cpp
//
//  The CARL Time test.
//
//  $Date: 2008/08/06 13:43:22 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        DateTime dt;
        CHECK(!dt.isValid())
    }

    {
        DateTime dt = DateTime(Date(), Time());
        CHECK(!dt.isValid())
    }

    {
        DateTime dt1(2008, 6, 7, 10, 20);
        CHECK(dt1.isValid())
        DateTime dt2(2008, 6, 7, 10, 20, 40);
        CHECK(dt2.isValid())
        DateTime dt3(2008, 6, 7, 10, 20, 40, 50);
        CHECK(dt3.isValid())
    }

    {
        DateTime dt1(Date(2008, 6, 7), Time(10, 20));
        CHECK(dt1.isValid())
        DateTime dt2(Date(2008, 6, 7), Time(10, 20, 40));
        CHECK(dt2.isValid())
        DateTime dt3(Date(2008, 6, 7), Time(10, 20, 40, 50));
        CHECK(dt3.isValid())
    }

    {
        Date d(2008, 6, 7);
        Time t(10, 20);
        DateTime dt(d, t);
        CHECK(dt.getDate() == d)
        CHECK(dt.getTime() == t)
    }

    {
        DateTime dt;
        CHECK(dt.getYear() == 0)
        CHECK(dt.getMonth() == 0)
        CHECK(dt.getDay() == 0)
        CHECK(dt.getHour() == 0)
        CHECK(dt.getMinute() == 0)
        CHECK(dt.getSecond() == 0)
        CHECK(dt.getNanosecond() == 0)
    }

    {
        DateTime dt(2008, 6, 7, 10, 20, 30, 40);
        CHECK(dt.getYear() == 2008)
        CHECK(dt.getMonth() == 6)
        CHECK(dt.getDay() == 7)
        CHECK(dt.getHour() == 10)
        CHECK(dt.getMinute() == 20)
        CHECK(dt.getSecond() == 30)
        CHECK(dt.getNanosecond() == 40)
    }

    {
        DateTime dt(Date(2008, 6, 7), Time(10, 20, 30, 40));
        CHECK(dt.getYear() == 2008)
        CHECK(dt.getMonth() == 6)
        CHECK(dt.getDay() == 7)
        CHECK(dt.getHour() == 10)
        CHECK(dt.getMinute() == 20)
        CHECK(dt.getSecond() == 30)
        CHECK(dt.getNanosecond() == 40)
    }

    {
        DateTime dt1, dt2;
        CHECK(dt1 == dt2)
        CHECK(!(dt1 != dt2))
        CHECK(!(dt1 < dt2))
        CHECK(dt1 <= dt2)
        CHECK(!(dt1 > dt2))
        CHECK(dt1 >= dt2)
    }

    {
        DateTime dt1(2008, 6, 7, 10, 20, 30, 41), dt2(2008, 6, 7, 10, 20, 30, 40);
        CHECK(!(dt1 == dt2))
        CHECK(dt1 != dt2)
        CHECK(!(dt1 < dt2))
        CHECK(!(dt1 <= dt2))
        CHECK(dt1 > dt2)
        CHECK(dt1 >= dt2)
    }

    {
        DateTime dt1, dt2(2008, 6, 7, 10, 20, 30, 40);
        CHECK(!(dt1 == dt2))
        CHECK(dt1 != dt2)
        CHECK(dt1 < dt2)
        CHECK(dt1 <= dt2)
        CHECK(!(dt1 > dt2))
        CHECK(!(dt1 >= dt2))
    }

    return exitCode;
}

//  End of Main.cpp
