//
//  Main.cpp
//
//  The CARL Time test.
//
//  $Date: 2008/08/06 13:25:41 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        Time t;
        CHECK(!t.isValid())
    }

    {
        Time t1(10, 20);
        CHECK(t1.isValid())
        Time t2(10, 20, 40);
        CHECK(t2.isValid())
        Time t3(10, 20, 40, 50);
        CHECK(t3.isValid())
    }

    {
        Time t;
        CHECK(t.getHour() == 0)
        CHECK(t.getMinute() == 0)
        CHECK(t.getSecond() == 0)
        CHECK(t.getNanosecond() == 0)
    }

    {
        Time t(10, 20, 30, 40);
        CHECK(t.getHour() == 10)
        CHECK(t.getMinute() == 20)
        CHECK(t.getSecond() == 30)
        CHECK(t.getNanosecond() == 40)
    }

    {
        Time t1, t2;
        CHECK(t1 == t2)
        CHECK(!(t1 != t2))
        CHECK(!(t1 < t2))
        CHECK(t1 <= t2)
        CHECK(!(t1 > t2))
        CHECK(t1 >= t2)
    }

    {
        Time t1(10, 20, 30, 41), t2(10, 20, 30, 40);
        CHECK(!(t1 == t2))
        CHECK(t1 != t2)
        CHECK(!(t1 < t2))
        CHECK(!(t1 <= t2))
        CHECK(t1 > t2)
        CHECK(t1 >= t2)
    }

    {
        Time t1, t2(10, 20, 30, 40);
        CHECK(!(t1 == t2))
        CHECK(t1 != t2)
        CHECK(t1 < t2)
        CHECK(t1 <= t2)
        CHECK(!(t1 > t2))
        CHECK(!(t1 >= t2))
    }

    return exitCode;
}

//  End of Main.cpp
