//
//  Main.cpp
//
//  The CARL time zone test.
//
//  $Date: 2008/02/22 12:09:20 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 

    //  Exercise all time zones
    Array<TimeZone*> allTimeZones = TimeZone::getAllInstances();
    for (UIntSize i = 0; i < allTimeZones.size(); i++)
    {
        TimeZone * pTimeZone = allTimeZones[i];
        StandardStreams::out.write("Testing " + pTimeZone->getDisplayName() + "...");
        //  Get current time & convert to local time and back
        TimeStamp originalTimeStamp = TimeStamp::now();
        DateTime originalDateTime = pTimeZone->timeToLocal(originalTimeStamp);
        TimeStamp recoveredTimeStamp = pTimeZone->timeFromLocal(originalDateTime);
        //  Compare
        if (originalTimeStamp != recoveredTimeStamp)
        {
            StandardStreams::out.writeLn("Failed");
            exitCode = Process::ExitCode_Failure;
        }
        else
        {
            StandardStreams::out.writeLn("OK");
        }
    }

    return exitCode;
}

//  End of Main.cpp
