//
//  Main.cpp
//
//  The DateFormat/DateParse test.
//
//  $Date: 2008/08/09 13:18:18 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 

    Random r;
    DateFormat * pDateFormat = DateFormat::getInstance(Locale());
    DateParse * pDateParse = DateParse::getInstance(Locale());

    //  Standard formats
    for (int year = 1980; year < 2080; year += r.nextInt32(1, 10))
    {
        for (int month = 1; month <= 12; month += r.nextInt32(1, 3))
        {
            StandardStreams::out.writeLn("Testing standard formatting for " + 
                                         NumberFormat::getInstance()->format(month) + "/" +
                                         NumberFormat::getInstance()->format(year));
            for (int day = 1; day <= Calendar::daysInMonth(year, month); day += r.nextInt32(1, 10))
            {
                Date d(year, month, day);
                CHECK(pDateParse->parse(pDateFormat->format(d, DateFormat::Style_Short)) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, DateFormat::Style_Medium)) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, DateFormat::Style_Long)) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, DateFormat::Style_Full)) == d)
            }
        }
    }

    //  Custom formats
    for (int year = 1980; year < 2080; year += r.nextInt32(1, 10))
    {
        for (int month = 1; month <= 12; month += r.nextInt32(1, 3))
        {
            StandardStreams::out.writeLn("Testing custom formatting for " + 
                                         NumberFormat::getInstance()->format(month) + "/" +
                                         NumberFormat::getInstance()->format(year));
            for (int day = 1; day <= Calendar::daysInMonth(year, month); day += r.nextInt32(1, 10))
            {
                Date d(year, month, day);
                String formatString1("{year:full}{month:full}{day:full}");
                String formatString2("{year:full} {month:full} {day:full}");
                String formatString3("{year:full}/{month:full}/{day:full}");
                String formatString4("[{year:full},{month:full},{day:full}]");
                CHECK(pDateParse->parse(pDateFormat->format(d, formatString1), formatString1) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, formatString2), formatString2) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, formatString3), formatString3) == d)
                CHECK(pDateParse->parse(pDateFormat->format(d, formatString4), formatString4) == d)
            }
        }
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
