//
//  Main.cpp
//
//  The DateTimeFormat/DateTimeParse test.
//
//  $Date: 2008/08/09 20:13:45 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

namespace
{
    bool match(DateTime dt1, DateTime dt2, DateTimeFormat::Precision precision)
    {
        switch (precision)
        {
            case DateTimeFormat::Precision_Minute:
                return dt1.getYear() == dt2.getYear() &&
                       dt1.getMonth() == dt2.getMonth() &&
                       dt1.getDay() == dt2.getDay() &&
                       dt1.getHour() == dt2.getHour() &&
                       dt1.getMinute() == dt2.getMinute();
            case DateTimeFormat::Precision_Second:
                return dt1.getYear() == dt2.getYear() &&
                       dt1.getMonth() == dt2.getMonth() &&
                       dt1.getDay() == dt2.getDay() &&
                       dt1.getHour() == dt2.getHour() &&
                       dt1.getMinute() == dt2.getMinute() &&
                       dt1.getSecond() == dt2.getSecond();
            case DateTimeFormat::Precision_Millisecond:
                return dt1.getYear() == dt2.getYear() &&
                       dt1.getMonth() == dt2.getMonth() &&
                       dt1.getDay() == dt2.getDay() &&
                       dt1.getHour() == dt2.getHour() &&
                       dt1.getMinute() == dt2.getMinute() &&
                       dt1.getSecond() == dt2.getSecond() &&
                       (dt1.getNanosecond() / 1000000) == (dt2.getNanosecond() / 1000000);
            case DateTimeFormat::Precision_Microsecond:
                return dt1.getYear() == dt2.getYear() &&
                       dt1.getMonth() == dt2.getMonth() &&
                       dt1.getDay() == dt2.getDay() &&
                       dt1.getHour() == dt2.getHour() &&
                       dt1.getMinute() == dt2.getMinute() &&
                       dt1.getSecond() == dt2.getSecond() &&
                       (dt1.getNanosecond() / 1000) == (dt2.getNanosecond() / 1000);
            case TimeFormat::Precision_Nanosecond:
                return dt1.getYear() == dt2.getYear() &&
                       dt1.getMonth() == dt2.getMonth() &&
                       dt1.getDay() == dt2.getDay() &&
                       dt1.getHour() == dt2.getHour() &&
                       dt1.getMinute() == dt2.getMinute() &&
                       dt1.getSecond() == dt2.getSecond() &&
                       dt1.getNanosecond() == dt2.getNanosecond();
            default:
                require(false);
                return false;
        }
    }
}

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 

    Random r;
    DateTimeFormat * pDateTimeFormat = DateTimeFormat::getInstance(Locale());
    DateTimeParse * pDateTimeParse = DateTimeParse::getInstance(Locale());

    //  Standard formats
    for (int year = 1980; year < 2080; year += r.nextInt32(1, 20))
    {
        for (int month = 1; month <= 12; month += r.nextInt32(1, 6))
        {
            StandardStreams::out.writeLn("Testing standard formatting for " + 
                                         NumberFormat::getInstance()->format(month) + "/" +
                                         NumberFormat::getInstance()->format(year));
            for (int day = 1; day <= Calendar::daysInMonth(year, month); day += r.nextInt32(1, 20))
            {
                DateTime dt(year, month, day, r.nextInt32(0,23), r.nextInt32(0,59), r.nextInt32(0,59), r.nextInt32(0, 999999999));
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Short, DateTimeFormat::Precision_Minute)), dt, DateTimeFormat::Precision_Minute))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Medium, DateTimeFormat::Precision_Minute)), dt, DateTimeFormat::Precision_Minute))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Long, DateTimeFormat::Precision_Minute)), dt, DateTimeFormat::Precision_Minute))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Full, DateTimeFormat::Precision_Minute)), dt, DateTimeFormat::Precision_Minute))

                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Short, DateTimeFormat::Precision_Second)), dt, DateTimeFormat::Precision_Second))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Medium, DateTimeFormat::Precision_Second)), dt, DateTimeFormat::Precision_Second))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Long, DateTimeFormat::Precision_Second)), dt, DateTimeFormat::Precision_Second))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Full, DateTimeFormat::Precision_Second)), dt, DateTimeFormat::Precision_Second))

                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Short, DateTimeFormat::Precision_Millisecond)), dt, DateTimeFormat::Precision_Millisecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Medium, DateTimeFormat::Precision_Millisecond)), dt, DateTimeFormat::Precision_Millisecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Long, DateTimeFormat::Precision_Millisecond)), dt, DateTimeFormat::Precision_Millisecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Full, DateTimeFormat::Precision_Millisecond)), dt, DateTimeFormat::Precision_Millisecond))

                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Short, DateTimeFormat::Precision_Microsecond)), dt, DateTimeFormat::Precision_Microsecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Medium, DateTimeFormat::Precision_Microsecond)), dt, DateTimeFormat::Precision_Microsecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Long, DateTimeFormat::Precision_Microsecond)), dt, DateTimeFormat::Precision_Microsecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Full, DateTimeFormat::Precision_Microsecond)), dt, DateTimeFormat::Precision_Microsecond))

                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Short, DateTimeFormat::Precision_Nanosecond)), dt, DateTimeFormat::Precision_Nanosecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Medium, DateTimeFormat::Precision_Nanosecond)), dt, DateTimeFormat::Precision_Nanosecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Long, DateTimeFormat::Precision_Nanosecond)), dt, DateTimeFormat::Precision_Nanosecond))
                CHECK(match(pDateTimeParse->parse(pDateTimeFormat->format(dt, DateTimeFormat::Style_Full, DateTimeFormat::Precision_Nanosecond)), dt, DateTimeFormat::Precision_Nanosecond))
            }
        }
    }

    //  Custom formats
    for (int year = 1980; year < 2080; year += r.nextInt32(1, 10))
    {
        for (int month = 1; month <= 12; month += r.nextInt32(1, 3))
        {
            StandardStreams::out.writeLn("Testing custom formatting for " + 
                                         NumberFormat::getInstance()->format(month) + "/" +
                                         NumberFormat::getInstance()->format(year));
            for (int day = 1; day <= Calendar::daysInMonth(year, month); day += r.nextInt32(1, 10))
            {
                DateTime dt(year, month, day, r.nextInt32(0,23), r.nextInt32(0,59), r.nextInt32(0,59), r.nextInt32(0, 999999999));
                String formatString1("{year:full}{month:full}{day:full}{hour24:full}{minute:full}{second:full}{nanosecond:full}");
                String formatString2("{year:full} {month:full} {day:full} {hour24:full} {minute:full} {second:full} {nanosecond:full}");
                String formatString3("{year:full}/{month:full}/{day:full} {hour24:full}:{minute:full}:{second:full}:{nanosecond:full}");
                String formatString4("[{year:full},{month:full},{day:full},{hour24:full},{minute:full},{second:full},{nanosecond:full}]");
                String formatString5("{year:full}{month:full}{day:full}{hour12:full}{minute:full}{second:full}{nanosecond:full}{ampm}");
                String formatString6("{year:full} {month:full} {day:full} {hour12:full} {minute:full} {second:full} {nanosecond:full} {ampm}");
                String formatString7("{year:full}/{month:full}/{day:full} {hour12:full}:{minute:full}:{second:full}:{nanosecond:full} {ampm}");
                String formatString8("[{year:full},{month:full},{day:full},{hour12:full},{minute:full},{second:full},{nanosecond:full},{ampm}]");
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString1), formatString1) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString2), formatString2) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString3), formatString3) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString4), formatString4) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString5), formatString5) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString6), formatString6) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString7), formatString7) == dt)
                CHECK(pDateTimeParse->parse(pDateTimeFormat->format(dt, formatString8), formatString8) == dt)
            }
        }
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
