//
//  Main.cpp
//
//  The integer NumberFormat/NumberParse test.
//
//  $Date: 2008/08/13 13:28:54 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 

    Random r;
    NumberFormat * pNumberFormat = NumberFormat::getInstance(Locale());
    NumberParse * pNumberParse = NumberParse::getInstance(Locale());

    //  Default formatting (decimal)
    {
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(123)) == "123")
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(-123)) == "-123")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0)) == "0")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(255)) == "255")

        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(12345)) == "12345")
        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(-12345)) == "-12345")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0)) == "0")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(65535)) == "65535")

        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(1234567)) == "1234567")
        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(-1234567)) == "-1234567")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0)) == "0")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(4294967295)) == "4294967295")

        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(12345678901)) == "12345678901")
        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(-12345678901)) == "-12345678901")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0)) == "0")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(18446744073709551615)) == "18446744073709551615")

        CHECK(pNumberFormat->format(BinaryInteger("1234567890123456789012345678901234567890")) == "1234567890123456789012345678901234567890")
        CHECK(pNumberFormat->format(BinaryInteger("-1234567890123456789012345678901234567890")) == "-1234567890123456789012345678901234567890")
        CHECK(pNumberFormat->format(DecimalInteger("1234567890123456789012345678901234567890")) == "1234567890123456789012345678901234567890")
        CHECK(pNumberFormat->format(DecimalInteger("-1234567890123456789012345678901234567890")) == "-1234567890123456789012345678901234567890")
    }

    //  Default formatting (hexadecimal)
    {
        carl::IntegerFormatProperties fp("%X");

        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(0x12), fp) == "12")
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(-0x12), fp) == "-12")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0xFF), fp) == "FF")

        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(0x1234), fp) == "1234")
        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(-0x1234), fp) == "-1234")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0xFFFF), fp) == "FFFF")

        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(0x1234567), fp) == "1234567")
        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(-0x1234567), fp) == "-1234567")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0xFFFFFFFF), fp) == "FFFFFFFF")

        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(0x12345678901), fp) == "12345678901")
        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(-0x12345678901), fp) == "-12345678901")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0xFFFFFFFFFFFFFFFF), fp) == "FFFFFFFFFFFFFFFF")

        CHECK(pNumberFormat->format(BinaryInteger("1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF")
        CHECK(pNumberFormat->format(BinaryInteger("-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF")
        CHECK(pNumberFormat->format(DecimalInteger("1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF")
        CHECK(pNumberFormat->format(DecimalInteger("-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF")
    }

    //  Prefixed formatting (hexadecimal)
    {
        carl::IntegerFormatProperties fp("%#x");

        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(0x12), fp) == "0x12")
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(-0x12), fp) == "-0x12")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0), fp) == "0x0")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0xFF), fp) == "0xff")

        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(0x1234), fp) == "0x1234")
        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(-0x1234), fp) == "-0x1234")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0), fp) == "0x0")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0xFFFF), fp) == "0xffff")

        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(0x1234567), fp) == "0x1234567")
        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(-0x1234567), fp) == "-0x1234567")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0), fp) == "0x0")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0xFFFFFFFF), fp) == "0xffffffff")

        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(0x12345678901), fp) == "0x12345678901")
        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(-0x12345678901), fp) == "-0x12345678901")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0), fp) == "0x0")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0xFFFFFFFFFFFFFFFF), fp) == "0xffffffffffffffff")

        CHECK(pNumberFormat->format(BinaryInteger("1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef")
        CHECK(pNumberFormat->format(BinaryInteger("-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "-0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef")
        CHECK(pNumberFormat->format(DecimalInteger("1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef")
        CHECK(pNumberFormat->format(DecimalInteger("-1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF1234567890ABCDEF", 16), fp) == "-0x1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef")
    }

    //  Zero-padded formatting
    {
        carl::IntegerFormatProperties fp("%015d");

        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(123), fp) == "000000000000123")
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(-123), fp) == "-000000000000123")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0), fp) == "000000000000000")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(255), fp) == "000000000000255")

        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(12345), fp) == "000000000012345")
        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(-12345), fp) == "-000000000012345")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0), fp) == "000000000000000")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(65535), fp) == "000000000065535")

        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(1234567), fp) == "000000001234567")
        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(-1234567), fp) == "-000000001234567")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0), fp) == "000000000000000")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(4294967295), fp) == "000004294967295")

        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(12345678901), fp) == "000012345678901")
        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(-12345678901), fp) == "-000012345678901")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0), fp) == "000000000000000")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(18446744073709551615), fp) == "18446744073709551615")

        CHECK(pNumberFormat->format(BinaryInteger("01234567890"), fp) == "000001234567890")
        CHECK(pNumberFormat->format(BinaryInteger("-01234567890"), fp) == "-000001234567890")
        CHECK(pNumberFormat->format(DecimalInteger("01234567890"), fp) == "000001234567890")
        CHECK(pNumberFormat->format(DecimalInteger("-01234567890"), fp) == "-000001234567890")
    }

    //  Formatting with groups
    {
        carl::IntegerFormatProperties fp;
        fp.setUseSeparator(true);

        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(123), fp) == "123")
        CHECK(pNumberFormat->format(CARL_INT8_CONSTANT(-123), fp) == "-123")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT8_CONSTANT(255), fp) == "255")

        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(12345), fp) == "12,345")
        CHECK(pNumberFormat->format(CARL_INT16_CONSTANT(-12345), fp) == "-12,345")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT16_CONSTANT(65535), fp) == "65,535")

        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(1234567), fp) == "1,234,567")
        CHECK(pNumberFormat->format(CARL_INT32_CONSTANT(-1234567), fp) == "-1,234,567")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT32_CONSTANT(4294967295), fp) == "4,294,967,295")

        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(12345678901), fp) == "12,345,678,901")
        CHECK(pNumberFormat->format(CARL_INT64_CONSTANT(-12345678901), fp) == "-12,345,678,901")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(0), fp) == "0")
        CHECK(pNumberFormat->format(CARL_UINT64_CONSTANT(18446744073709551615), fp) == "18,446,744,073,709,551,615")

        CHECK(pNumberFormat->format(BinaryInteger("123456789012345678901234567890"), fp) == "123,456,789,012,345,678,901,234,567,890")
        CHECK(pNumberFormat->format(BinaryInteger("-123456789012345678901234567890"), fp) == "-123,456,789,012,345,678,901,234,567,890")
        CHECK(pNumberFormat->format(DecimalInteger("123456789012345678901234567890"), fp) == "123,456,789,012,345,678,901,234,567,890")
        CHECK(pNumberFormat->format(DecimalInteger("-123456789012345678901234567890"), fp) == "-123,456,789,012,345,678,901,234,567,890")
    }

    //  Parsing (decimal)
    const int Iterations = 100;
    for (int i = 0; i < Iterations; i++)
    {
        StandardStreams::out.writeLn("Parsing test vector " + NumberFormat::getInstance()->format(i));
        Int8 i8 = r.nextInt8();
        CHECK(pNumberParse->parseInt8(pNumberFormat->format(i8)) == i8)
        UInt8 ui8 = r.nextUInt8();
        CHECK(pNumberParse->parseUInt8(pNumberFormat->format(ui8)) == ui8)
        Int16 i16 = r.nextInt16();
        CHECK(pNumberParse->parseInt16(pNumberFormat->format(i16)) == i16)
        UInt16 ui16 = r.nextUInt16();
        CHECK(pNumberParse->parseUInt16(pNumberFormat->format(ui16)) == ui16)
        Int32 i32 = r.nextInt32();
        CHECK(pNumberParse->parseInt32(pNumberFormat->format(i32)) == i32)
        UInt32 ui32 = r.nextUInt32();
        CHECK(pNumberParse->parseUInt32(pNumberFormat->format(ui32)) == ui32)
        Int64 i64 = r.nextInt64();
        CHECK(pNumberParse->parseInt64(pNumberFormat->format(i64)) == i64)
        UInt64 ui64 = r.nextUInt64();
        CHECK(pNumberParse->parseUInt64(pNumberFormat->format(ui64)) == ui64)
        BinaryInteger bi = r.nextBinaryInteger(128);
        CHECK(pNumberParse->parseBinaryInteger(pNumberFormat->format(bi)) == bi)
        DecimalInteger di(bi.toString());
        CHECK(pNumberParse->parseDecimalInteger(pNumberFormat->format(di)) == di)
    }

    //  Parsing (hexadecimal)
    for (int i = 0; i < Iterations; i++)
    {
        StandardStreams::out.writeLn("Parsing test vector " + NumberFormat::getInstance()->format(i));
        IntegerFormatProperties fp("%X");
        IntegerParseProperties pp("%x");
        Int8 i8 = r.nextInt8();
        CHECK(pNumberParse->parseInt8(pNumberFormat->format(i8, fp), pp) == i8)
        UInt8 ui8 = r.nextUInt8();
        CHECK(pNumberParse->parseUInt8(pNumberFormat->format(ui8, fp), pp) == ui8)
        Int16 i16 = r.nextInt16();
        CHECK(pNumberParse->parseInt16(pNumberFormat->format(i16, fp), pp) == i16)
        UInt16 ui16 = r.nextUInt16();
        CHECK(pNumberParse->parseUInt16(pNumberFormat->format(ui16, fp), pp) == ui16)
        Int32 i32 = r.nextInt32();
        CHECK(pNumberParse->parseInt32(pNumberFormat->format(i32, fp), pp) == i32)
        UInt32 ui32 = r.nextUInt32();
        CHECK(pNumberParse->parseUInt32(pNumberFormat->format(ui32, fp), pp) == ui32)
        Int64 i64 = r.nextInt64();
        CHECK(pNumberParse->parseInt64(pNumberFormat->format(i64, fp), pp) == i64)
        UInt64 ui64 = r.nextUInt64();
        CHECK(pNumberParse->parseUInt64(pNumberFormat->format(ui64, fp), pp) == ui64)
        BinaryInteger bi = r.nextBinaryInteger(128);
        CHECK(pNumberParse->parseBinaryInteger(pNumberFormat->format(bi, fp), pp) == bi)
        DecimalInteger di(bi.toString());
        CHECK(pNumberParse->parseDecimalInteger(pNumberFormat->format(di, fp), pp) == di)
    }

    //  Parsing with groups
    for (int i = 0; i < Iterations; i++)
    {
        StandardStreams::out.writeLn("Parsing test vector " + NumberFormat::getInstance()->format(i));
        IntegerFormatProperties fp;
        fp.setUseSeparator(true);
        IntegerParseProperties pp;
        pp.setAllowSeparator(true);
        Int8 i8 = r.nextInt8();
        CHECK(pNumberParse->parseInt8(pNumberFormat->format(i8, fp), pp) == i8)
        UInt8 ui8 = r.nextUInt8();
        CHECK(pNumberParse->parseUInt8(pNumberFormat->format(ui8, fp), pp) == ui8)
        Int16 i16 = r.nextInt16();
        CHECK(pNumberParse->parseInt16(pNumberFormat->format(i16, fp), pp) == i16)
        UInt16 ui16 = r.nextUInt16();
        CHECK(pNumberParse->parseUInt16(pNumberFormat->format(ui16, fp), pp) == ui16)
        Int32 i32 = r.nextInt32();
        CHECK(pNumberParse->parseInt32(pNumberFormat->format(i32, fp), pp) == i32)
        UInt32 ui32 = r.nextUInt32();
        CHECK(pNumberParse->parseUInt32(pNumberFormat->format(ui32, fp), pp) == ui32)
        Int64 i64 = r.nextInt64();
        CHECK(pNumberParse->parseInt64(pNumberFormat->format(i64, fp), pp) == i64)
        UInt64 ui64 = r.nextUInt64();
        CHECK(pNumberParse->parseUInt64(pNumberFormat->format(ui64, fp), pp) == ui64)
        BinaryInteger bi = r.nextBinaryInteger(128);
        CHECK(pNumberParse->parseBinaryInteger(pNumberFormat->format(bi, fp), pp) == bi)
        DecimalInteger di(bi.toString());
        CHECK(pNumberParse->parseDecimalInteger(pNumberFormat->format(di, fp), pp) == di)
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
