//
//  Main.cpp
//
//  The CARL collation test.
//
//  $Date: 2008/09/12 12:05:50 $
//  $Revision: 1.3 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    Collator * pCollator = Collator::getInstance(Locale("en"));

    //  Test primary collation
    {
        CollatorBasedStringComparator comparator(pCollator, Collator::CollationStrength_Primary, Collator::Decomposition_Full);

        CHECK(comparator.equal(String("abc"), String("abc")))
        CHECK(!comparator.less(String("abc"), String("abc")))
        CHECK(!comparator.greater(String("abc"), String("abc")))

        CHECK(comparator.equal(String("abc"), String("aBc")))
        CHECK(!comparator.less(String("abc"), String("aBc")))
        CHECK(!comparator.greater(String("abc"), String("aBc")))

        CHECK(!comparator.equal(String("abc"), String("aBd")))
        CHECK(comparator.less(String("abc"), String("aBd")))
        CHECK(!comparator.greater(String("abc"), String("aBd")))

        CHECK(comparator.equal(String("axy"), String("\xE4xy")))
        CHECK(!comparator.less(String("axy"), String("\xE4xy")))
        CHECK(!comparator.greater(String("axy"), String("\xE4xy")))

        CHECK(comparator.equal(String("axy"), String("\xC4xy")))
        CHECK(!comparator.less(String("axy"), String("\xC4xy")))
        CHECK(!comparator.greater(String("axy"), String("\xC4xy")))
    }

    //  Test secondary collation
    {
        CollatorBasedStringComparator comparator(pCollator, Collator::CollationStrength_Secondary, Collator::Decomposition_Full);

        CHECK(comparator.equal(String("abc"), String("abc")))
        CHECK(!comparator.less(String("abc"), String("abc")))
        CHECK(!comparator.greater(String("abc"), String("abc")))

        CHECK(!comparator.equal(String("abc"), String("\xE4xy")))
        CHECK(comparator.less(String("axy"), String("\xE4xy")))
        CHECK(!comparator.greater(String("axy"), String("\xE4xy")))

        CHECK(comparator.equal(String("\xC4xy"), String("\xE4xy")))
        CHECK(!comparator.less(String("\xC4xy"), String("\xE4xy")))
        CHECK(!comparator.greater(String("\xC4xy"), String("\xE4xy")))
    }

    //  Test tertiary collation
    {
        CollatorBasedStringComparator comparator(pCollator, Collator::CollationStrength_Tertiary, Collator::Decomposition_Full);

        CHECK(comparator.equal(String("abc"), String("abc")))
        CHECK(!comparator.less(String("abc"), String("abc")))
        CHECK(!comparator.greater(String("abc"), String("abc")))

        CHECK(!comparator.equal(String("axy"), String("\xE4xy")))
        CHECK(comparator.less(String("axy"), String("\xE4xy")))
        CHECK(!comparator.greater(String("axy"), String("\xE4xy")))

        CHECK(!comparator.equal(String("\xC4xy"), String("\xE4xy")))
        CHECK(comparator.less(String("\xC4xy"), String("\xE4xy")))
        CHECK(!comparator.greater(String("\xC4xy"), String("\xE4xy")))
    }

    //  Test identical collation
    {
        CollatorBasedStringComparator comparator(pCollator, Collator::CollationStrength_Identical, Collator::Decomposition_Full);

        CHECK(comparator.equal(String("abc"), String("abc")))
        CHECK(!comparator.less(String("abc"), String("abc")))
        CHECK(!comparator.greater(String("abc"), String("abc")))

        CHECK(!comparator.equal(String("abc"), String("aBc")))
        CHECK(!comparator.less(String("abc"), String("aBc")))
        CHECK(comparator.greater(String("abc"), String("aBc")))
    }

    return exitCode;
}

//  End of Main.cpp
