//
//  Main.cpp
//
//  The CARL ISO-10646 test.
//
//  $Date: 2008/08/02 00:46:20 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <ctype.h>
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    //  ASCII-7 is locale-independent, so CRT character classification
    //  shall match ISO-10646 character classification.
    for (int i = 0; i <= 127; i++)
    {
        CHECK((isalpha(i)  != 0) == (Iso10646::isAlpha(i) != 0))
        CHECK((isupper(i)  != 0) == (Iso10646::isUpper(i) != 0))
        CHECK((islower(i)  != 0) == (Iso10646::isLower(i) != 0))
        CHECK((isdigit(i)  != 0) == (Iso10646::isDecimalDigit(i) != 0))
        CHECK((isdigit(i)  != 0) == (Iso10646::isDigit(i) != 0))
        CHECK((isxdigit(i) != 0) == (Iso10646::isHexadecimalDigit(i) != 0))
        CHECK((isspace(i)  != 0) == (Iso10646::isSpace(i) != 0))
        CHECK((ispunct(i)  != 0) == (Iso10646::isPunct(i) != 0))
        CHECK((isalnum(i)  != 0) == (Iso10646::isAlNum(i) != 0))
        CHECK((isprint(i)  != 0) == (Iso10646::isPrint(i) != 0))
        CHECK((isgraph(i)  != 0) == (Iso10646::isGraph(i) != 0))
        CHECK((iscntrl(i)  != 0) == (Iso10646::isControl(i) != 0))
        CHECK(toupper(i) == static_cast<int>(Iso10646::toUpper(i)))
        CHECK(tolower(i) == static_cast<int>(Iso10646::toLower(i)))
    }

    return exitCode;
}

//  End of Main.cpp
