//
//  Main.cpp
//
//  The CARL ISO-3166 test.
//
//  $Date: 2008/02/29 11:24:50 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    Array<const Iso3166::Country*> allCountries = Iso3166::getAllCountries();
    for (UIntSize i = 0; i < allCountries.size(); i++)
    {
        const Iso3166::Country * pCountry = allCountries[i];
        CHECK(pCountry->getCode2().length() == 2)
        CHECK(pCountry->getCode2().upperCased() == pCountry->getCode2())
        CHECK(pCountry->getCode3().length() == 3)
        CHECK(pCountry->getCode3().upperCased() == pCountry->getCode3())
        CHECK(pCountry->getCodeN() > 0 && pCountry->getCodeN() <= 999)
        CHECK(Iso3166::isValidCode2(pCountry->getCode2()))
        CHECK(Iso3166::isValidCode2(pCountry->getCode2().lowerCased()))
        CHECK(Iso3166::findCountryByCode2(pCountry->getCode2()) == pCountry)
        CHECK(Iso3166::findCountryByCode2(pCountry->getCode2().lowerCased()) == pCountry)
        CHECK(Iso3166::isValidCode3(pCountry->getCode3()))
        CHECK(Iso3166::isValidCode3(pCountry->getCode3().lowerCased()))
        CHECK(Iso3166::findCountryByCode3(pCountry->getCode3()) == pCountry)
        CHECK(Iso3166::findCountryByCode3(pCountry->getCode3().lowerCased()) == pCountry)
        CHECK(Iso3166::isValidCodeN(pCountry->getCodeN()))
        CHECK(Iso3166::findCountryByCodeN(pCountry->getCodeN()) == pCountry)
    }

    return exitCode;
}

//  End of Main.cpp
