//
//  Main.cpp
//
//  The CARL locale test.
//
//  $Date: 2008/02/26 11:08:27 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    CHECK(Locale().getLanguage() == "")
    CHECK(Locale().getCountry() == "")
    CHECK(Locale().getVariant() == "")

    CHECK(Locale("en").getLanguage() == "en")
    CHECK(Locale("en").getCountry() == "")
    CHECK(Locale("en").getVariant() == "")

    CHECK(Locale("en", "US").getLanguage() == "en")
    CHECK(Locale("en", "US").getCountry() == "US")
    CHECK(Locale("en", "US").getVariant() == "")

    CHECK(Locale("en", "US", "foo").getLanguage() == "en")
    CHECK(Locale("en", "US", "foo").getCountry() == "US")
    CHECK(Locale("en", "US", "foo").getVariant() == "foo")

    CHECK(Locale().toString() == "")
    CHECK(Locale("en").toString() == "en")
    CHECK(Locale("en", "us").toString() == "en_US")
    CHECK(Locale("EN", "us", "foo").toString() == "en_US_foo")

    CHECK(Locale::fromString("").getLanguage() == "")
    CHECK(Locale::fromString("").getCountry() == "")
    CHECK(Locale::fromString("").getVariant() == "")

    CHECK(Locale::fromString("en").getLanguage() == "en")
    CHECK(Locale::fromString("en").getCountry() == "")
    CHECK(Locale::fromString("en").getVariant() == "")

    CHECK(Locale::fromString("EN_us").getLanguage() == "en")
    CHECK(Locale::fromString("EN_us").getCountry() == "US")
    CHECK(Locale::fromString("EN_us").getVariant() == "")

    CHECK(Locale::fromString("EN_us_foo").getLanguage() == "en")
    CHECK(Locale::fromString("EN_us_foo").getCountry() == "US")
    CHECK(Locale::fromString("EN_us_foo").getVariant() == "foo")

    CHECK(Locale().isInvariant())
    CHECK(!Locale("en").isInvariant())
    CHECK(!Locale("en", "us").isInvariant())
    CHECK(!Locale("EN", "us", "foo").isInvariant())

    CHECK(!Locale().isNeutral())
    CHECK(Locale("en").isNeutral())
    CHECK(!Locale("en", "us").isNeutral())
    CHECK(!Locale("EN", "us", "foo").isNeutral())

    CHECK(!Locale().isCulture())
    CHECK(!Locale("en").isCulture())
    CHECK(Locale("en", "us").isCulture())
    CHECK(!Locale("EN", "us", "foo").isCulture())

    CHECK(!Locale().isFull())
    CHECK(!Locale("en").isFull())
    CHECK(!Locale("en", "us").isFull())
    CHECK(Locale("EN", "us", "foo").isFull())

    CHECK(Locale("EN", "us", "foo").getParent() == Locale("en", "us"))
    CHECK(Locale("EN", "us").getParent() == Locale("en"))
    CHECK(Locale("EN").getParent() == Locale())

    Locale::setDefault(Locale());
    CHECK(Locale::getDefault() == Locale())

    Locale::setDefault(Locale("EN"));
    CHECK(Locale::getDefault() == Locale("en"))

    Locale::setDefault(Locale("EN", "us"));
    CHECK(Locale::getDefault() == Locale("en", "US"))

    Locale::setDefault(Locale("EN", "us", "foo"));
    CHECK(Locale::getDefault() == Locale("en", "US", "foo"))

    return exitCode;
}

//  End of Main.cpp
