//
//  Main.cpp
//
//  The CARL LEB128 I/O test.
//
//  $Date: 2008/02/22 12:09:20 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) failed = true

int MyApp::run(const Array<String> & /*args*/)
{
    Random r;
    bool failed = false;
    for (int i = 0; i < 1000; i++)
    {
        MemoryOutputStream mos;
        Int8 i8 = r.nextInt8();
        Int16 i16 = r.nextInt16();
        Int32 i32 = r.nextInt32();
        Int64 i64 = r.nextInt64();
        UInt8 u8 = r.nextUInt8();
        UInt16 u16 = r.nextUInt16();
        UInt32 u32 = r.nextUInt32();
        UInt64 u64 = r.nextUInt64();
        BinaryInteger bi = r.nextInt32();
        Leb128::writeInt8(mos, i8);
        Leb128::writeInt16(mos, i16);
        Leb128::writeInt32(mos, i32);
        Leb128::writeInt64(mos, i64);
        ULeb128::writeUInt8(mos, u8);
        ULeb128::writeUInt16(mos, u16);
        ULeb128::writeUInt32(mos, u32);
        ULeb128::writeUInt64(mos, u64);
        Leb128::writeBinaryInteger(mos, bi);
        MemoryInputStream mis(mos.getData(), mos.getDataSize(), true);
        CHECK(i8 == Leb128::readInt8(mis));
        CHECK(i16 == Leb128::readInt16(mis));
        CHECK(i32 == Leb128::readInt32(mis));
        CHECK(i64 == Leb128::readInt64(mis));
        CHECK(u8 == ULeb128::readUInt8(mis));
        CHECK(u16 == ULeb128::readUInt16(mis));
        CHECK(u32 == ULeb128::readUInt32(mis));
        CHECK(u64 == ULeb128::readUInt64(mis));
        CHECK(bi == Leb128::readBinaryInteger(mis));
    }
    return failed ? 1 : 0;
}

//  End of Main.cpp
