//
//  Main.cpp
//
//  The CARL temporary file test #1 (manual creation).
//
//  $Date: 2008/02/22 12:09:20 $
//  $Revision: 1.3 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

int MyApp::run(const Array<String> & /*args*/)
{
    const int NumFiles = 100;

    Array<String> tempFileNames;
    PathName tempDirectory = FileSystem::getTempDirectory();
    if (tempDirectory.isEmpty())
    {
        StandardStreams::out.writeLn("Cannot determine the temp directory");
        return Process::ExitCode_Failure;
    }
    StandardStreams::out.writeLn("Temp directory is '" + tempDirectory.toString() + "'");
    for (int i = 0; i < NumFiles; i++)
    {
        PathName tempFileName = FileSystem::getTempFileName(tempDirectory);
        if (tempFileName.isEmpty())
        {
            StandardStreams::out.writeLn("Cannot generate the temp file name");
            return Process::ExitCode_Failure;
        }
        StandardStreams::out.write("Creating temp file '" + tempFileName.toString() + "' ...");
        tempFileNames.append(tempFileName.toString());
        try
        {
            RandomAccessFile raf(tempFileName, RandomAccessFile::OpenMode_CreateNew, RandomAccessFile::AccessMode_ReadWrite);
            StandardStreams::out.writeLn("OK");
        }
        catch (IOException)
        {
            StandardStreams::out.writeLn("Failed");
            return Process::ExitCode_Failure;
        }
    }
    for (UIntSize i = 0; i < tempFileNames.size(); i++)
    {
        if (!FileSystem::deleteFile(tempFileNames[i]))
        {
            StandardStreams::out.writeLn("Error deleting temp file '" + tempFileNames[i] + "'");
            return Process::ExitCode_Failure;
        }
    }
    return Process::ExitCode_Success;
}

//  End of Main.cpp
