//
//  Main.cpp
//
//  The CARL Random test.
//
//  $Date: 2008/02/25 11:32:03 $
//  $Revision: 1.2 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

int MyApp::run(const Array<String> & /*args*/)
{
    const carl::UIntSize NumSamples = 1000000;
    const carl::UIntSize NumSlots = 1000;

    Random r;

    carl::UInt32 slots[NumSlots];
    for (carl::UIntSize i = 0; i < NumSlots; i++)
    {
        slots[i] = 0;
    }

    for (carl::UIntSize i = 0; i < NumSamples; i++)
    {
        carl::UInt32 slot = r.nextUInt32(0, NumSlots - 1);
        slots[slot]++;
    }

    carl::Real64 average = 0;
    for (carl::UIntSize i = 0; i < NumSlots; i++)
    {
        average += slots[i];
    }
    average /= NumSlots;

    carl::Real64 meanSquareDeviation = 0;
    for (carl::UIntSize i = 0; i < NumSlots; i++)
    {
        carl::Real64 variance = slots[i] - average;
        meanSquareDeviation += variance * variance;
    }
    meanSquareDeviation /= NumSlots;
    meanSquareDeviation = Math::sqrt(meanSquareDeviation);

    carl::UInt32 minimum = NumSamples, maximum = 0;
    for (carl::UIntSize i = 0; i < NumSlots; i++)
    {
        minimum = Math::min(minimum, slots[i]);
        maximum = Math::max(minimum, slots[i]);
    }
    carl::Real64 median = (static_cast<carl::Real64>(minimum) + static_cast<carl::Real64>(maximum)) / 2;
    carl::Real64 maximumDeviation = maximum - median;

    NumberFormat * pNumberFormat = NumberFormat::getInstance();
    StandardStreams::out.writeLn("Average               : " + pNumberFormat->format(average));
    StandardStreams::out.writeLn("Median                : " + pNumberFormat->format(median));
    StandardStreams::out.writeLn("Maximum deviation     : " + pNumberFormat->format(maximumDeviation));
    StandardStreams::out.writeLn("Mean square deviation : " + pNumberFormat->format(meanSquareDeviation));

    //  Done
    return Process::ExitCode_Success;
}

//  End of Main.cpp
