//
//  Main.cpp
//
//  The CARL basic types test.
//
//  $Date: 2008/02/22 12:09:22 $
//  $Revision: 1.3 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    //  Check basic integer types for consistency
    CHECK(sizeof(Int8) == 1)
    CHECK(sizeof(Int16) == 2)
    CHECK(sizeof(Int32) == 4)
    CHECK(sizeof(Int64) == 8)
    CHECK(sizeof(UInt8) == 1)
    CHECK(sizeof(UInt16) == 2)
    CHECK(sizeof(UInt32) == 4)
    CHECK(sizeof(UInt64) == 8)
    CHECK(sizeof(UIntSize) >= sizeof(UInt8) && sizeof(UIntSize) <= sizeof(UInt64))
    CHECK(sizeof(UIntPtr) >= sizeof(UInt8) && sizeof(UIntPtr) <= sizeof(UInt64))
    CHECK(sizeof(IntPtrDiff) >= sizeof(UInt8) && sizeof(IntPtrDiff) <= sizeof(UInt64))
    CHECK(sizeof(UIntPtr) == sizeof(void*))
    CHECK(sizeof(IntPtrDiff) == sizeof(void*))
    CHECK(sizeof(UIntSize) <= sizeof(void*))

    //  Check basic real types for consistency
    CHECK(sizeof(Real32) == 4);
    CHECK(sizeof(Real64) == 8);

    //  Check ISO 10646 character type for consistency
    CHECK(sizeof(Char) == 4);

    return exitCode;
}

//  End of Main.cpp
