//
//  Main.cpp
//
//  The CARL basic type limits test.
//
//  $Date: 2008/02/22 12:09:22 $
//  $Revision: 1.4 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define TEST_MIN(Type)                                  \
    {                                                   \
        Type x = Limits::Type ##_Min;                   \
        Type y = x - 1;                                 \
        if (y <= x)                                     \
        {                                               \
            exitCode = Process::ExitCode_Failure;       \
        }                                               \
    }

#define TEST_MAX(Type)                                  \
    {                                                   \
        Type x = Limits::Type ##_Max;                   \
        Type y = x + 1;                                 \
        if (y >= x)                                     \
        {                                               \
            exitCode = Process::ExitCode_Failure;       \
        }                                               \
    }

#define TEST(Type)  \
    TEST_MIN(Type)  \
    TEST_MAX(Type)

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success; 

    TEST(Int8)
    TEST(Int16)
    TEST(Int32)
    TEST(Int64)
    TEST(UInt8)
    TEST(UInt16)
    TEST(UInt32)
    TEST(UInt64)
    TEST(UIntSize)
    TEST(UIntPtr)
    TEST(IntPtrDiff)
    TEST(Char)

    return exitCode;
}

//  End of Main.cpp
