//
//  Main.cpp
//
//  The CARL Memory test.
//
//  $Date: 2008/02/25 11:32:03 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    const int Iterations = 1000;
    Random r;
    for (int iteration = 0; iteration < Iterations; iteration++)
    {
        //  int16be (aligned)
        {
            carl::Int16 buffer;
            carl::Int16 originalValue = r.nextInt16();
            Memory::storeInt16(originalValue, &buffer, ByteOrder_BigEndian);
            carl::Int16 transformedValue = Memory::loadInt16(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int16le (aligned)
        {
            carl::Int16 buffer;
            carl::Int16 originalValue = r.nextInt16();
            Memory::storeInt16(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::Int16 transformedValue = Memory::loadInt16(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int32be (aligned)
        {
            carl::Int32 buffer;
            carl::Int32 originalValue = r.nextInt32();
            Memory::storeInt32(originalValue, &buffer, ByteOrder_BigEndian);
            carl::Int32 transformedValue = Memory::loadInt32(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int32le (aligned)
        {
            carl::Int32 buffer;
            carl::Int32 originalValue = r.nextInt32();
            Memory::storeInt32(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::Int32 transformedValue = Memory::loadInt32(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int64be (aligned)
        {
            carl::Int64 buffer;
            carl::Int64 originalValue = r.nextInt64();
            Memory::storeInt64(originalValue, &buffer, ByteOrder_BigEndian);
            carl::Int64 transformedValue = Memory::loadInt64(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int64le (aligned)
        {
            carl::Int64 buffer;
            carl::Int64 originalValue = r.nextInt64();
            Memory::storeInt64(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::Int64 transformedValue = Memory::loadInt64(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint16be (aligned)
        {
            carl::UInt16 buffer;
            carl::UInt16 originalValue = r.nextUInt16();
            Memory::storeUInt16(originalValue, &buffer, ByteOrder_BigEndian);
            carl::UInt16 transformedValue = Memory::loadUInt16(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint16le (aligned)
        {
            carl::UInt16 buffer;
            carl::UInt16 originalValue = r.nextUInt16();
            Memory::storeUInt16(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::UInt16 transformedValue = Memory::loadUInt16(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint32be (aligned)
        {
            carl::UInt32 buffer;
            carl::UInt32 originalValue = r.nextUInt32();
            Memory::storeUInt32(originalValue, &buffer, ByteOrder_BigEndian);
            carl::UInt32 transformedValue = Memory::loadUInt32(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint32le (aligned)
        {
            carl::UInt32 buffer;
            carl::UInt32 originalValue = r.nextUInt32();
            Memory::storeUInt32(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::UInt32 transformedValue = Memory::loadUInt32(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint64be (aligned)
        {
            carl::UInt64 buffer;
            carl::UInt64 originalValue = r.nextUInt64();
            Memory::storeUInt64(originalValue, &buffer, ByteOrder_BigEndian);
            carl::UInt64 transformedValue = Memory::loadUInt64(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint64le (aligned)
        {
            carl::UInt64 buffer;
            carl::UInt64 originalValue = r.nextUInt64();
            Memory::storeUInt64(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::UInt64 transformedValue = Memory::loadUInt64(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real32be (aligned)
        {
            carl::Real32 buffer;
            carl::Real32 originalValue = static_cast<carl::Real32>(r.nextUInt64()) / static_cast<carl::Real32>(Limits::UInt64_Max);
            Memory::storeReal32(originalValue, &buffer, ByteOrder_BigEndian);
            carl::Real32 transformedValue = Memory::loadReal32(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real32le (aligned)
        {
            carl::Real32 buffer;
            carl::Real32 originalValue = static_cast<carl::Real32>(r.nextUInt64()) / static_cast<carl::Real32>(Limits::UInt64_Max);
            Memory::storeReal32(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::Real32 transformedValue = Memory::loadReal32(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real64be (aligned)
        {
            carl::Real64 buffer;
            carl::Real64 originalValue = static_cast<carl::Real64>(r.nextUInt64()) / static_cast<carl::Real64>(Limits::UInt64_Max);
            Memory::storeReal64(originalValue, &buffer, ByteOrder_BigEndian);
            carl::Real64 transformedValue = Memory::loadReal64(&buffer, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real64le (aligned)
        {
            carl::Real64 buffer;
            carl::Real64 originalValue = static_cast<carl::Real64>(r.nextUInt64()) / static_cast<carl::Real64>(Limits::UInt64_Max);
            Memory::storeReal64(originalValue, &buffer, ByteOrder_LittleEndian);
            carl::Real64 transformedValue = Memory::loadReal64(&buffer, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int16be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int16) + 1];
            carl::Int16 originalValue = r.nextInt16();
            Memory::storeInt16(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::Int16 transformedValue = Memory::loadInt16(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int16le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int16) + 1];
            carl::Int16 originalValue = r.nextInt16();
            Memory::storeInt16(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::Int16 transformedValue = Memory::loadInt16(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int32be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int32) + 1];
            carl::Int32 originalValue = r.nextInt32();
            Memory::storeInt32(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::Int32 transformedValue = Memory::loadInt32(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int32le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int32) + 1];
            carl::Int32 originalValue = r.nextInt32();
            Memory::storeInt32(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::Int32 transformedValue = Memory::loadInt32(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int64be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int64) + 1];
            carl::Int64 originalValue = r.nextInt64();
            Memory::storeInt64(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::Int64 transformedValue = Memory::loadInt64(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  int64le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Int64) + 1];
            carl::Int64 originalValue = r.nextInt64();
            Memory::storeInt64(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::Int64 transformedValue = Memory::loadInt64(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint16be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt16) + 1];
            carl::UInt16 originalValue = r.nextUInt16();
            Memory::storeUInt16(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::UInt16 transformedValue = Memory::loadUInt16(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint16le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt16) + 1];
            carl::UInt16 originalValue = r.nextUInt16();
            Memory::storeUInt16(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::UInt16 transformedValue = Memory::loadUInt16(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint32be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt32) + 1];
            carl::UInt32 originalValue = r.nextUInt32();
            Memory::storeUInt32(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::UInt32 transformedValue = Memory::loadUInt32(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint32le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt32) + 1];
            carl::UInt32 originalValue = r.nextUInt32();
            Memory::storeUInt32(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::UInt32 transformedValue = Memory::loadUInt32(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint64be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt64) + 1];
            carl::UInt64 originalValue = r.nextUInt64();
            Memory::storeUInt64(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::UInt64 transformedValue = Memory::loadUInt64(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  uint64le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::UInt64) + 1];
            carl::UInt64 originalValue = r.nextUInt64();
            Memory::storeUInt64(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::UInt64 transformedValue = Memory::loadUInt64(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real32be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Real32) + 1];
            carl::Real32 originalValue = static_cast<carl::Real32>(r.nextUInt64()) / static_cast<carl::Real32>(Limits::UInt64_Max);
            Memory::storeReal32(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::Real32 transformedValue = Memory::loadReal32(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real32le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Real32) + 1];
            carl::Real32 originalValue = static_cast<carl::Real32>(r.nextUInt64()) / static_cast<carl::Real32>(Limits::UInt64_Max);
            Memory::storeReal32(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::Real32 transformedValue = Memory::loadReal32(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real64be (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Real64) + 1];
            carl::Real64 originalValue = static_cast<carl::Real64>(r.nextUInt64()) / static_cast<carl::Real64>(Limits::UInt64_Max);
            Memory::storeReal64(originalValue, buffer + 1, ByteOrder_BigEndian);
            carl::Real64 transformedValue = Memory::loadReal64(buffer + 1, ByteOrder_BigEndian);
            CHECK(transformedValue == originalValue)
        }

        //  real64le (unaligned)
        {
            carl::UInt8 buffer[sizeof(carl::Real64) + 1];
            carl::Real64 originalValue = static_cast<carl::Real64>(r.nextUInt64()) / static_cast<carl::Real64>(Limits::UInt64_Max);
            Memory::storeReal64(originalValue, buffer + 1, ByteOrder_LittleEndian);
            carl::Real64 transformedValue = Memory::loadReal64(buffer + 1, ByteOrder_LittleEndian);
            CHECK(transformedValue == originalValue)
        }
    }

    {
        int x;
        CHECK(Memory::addressOf(&x) == reinterpret_cast<carl::UIntPtr>(&x))
        CHECK(Memory::pointerTo(reinterpret_cast<carl::UIntPtr>(&x)) == &x)
    }

    {
        carl::UInt8 src[] = { 1, 2, 3, 4, 5 };
        carl::UInt8 dst[] = { 0, 0, 0, 0, 0, };
        Memory::copy(dst + 1, src + 1, 3);
        CHECK(src[0] == 1)
        CHECK(src[1] == 2)
        CHECK(src[2] == 3)
        CHECK(src[3] == 4)
        CHECK(src[4] == 5)
        CHECK(dst[0] == 0)
        CHECK(dst[1] == 2)
        CHECK(dst[2] == 3)
        CHECK(dst[3] == 4)
        CHECK(dst[4] == 0)
    }

    {
        carl::UInt8 dst[] = { 1, 2, 3, 4, 5 };
        Memory::set(dst + 1, 6, 3);
        CHECK(dst[0] == 1)
        CHECK(dst[1] == 6)
        CHECK(dst[2] == 6)
        CHECK(dst[3] == 6)
        CHECK(dst[4] == 5)
    }

    {
        carl::UInt8 src1[] = { 1, 2, 3, 4, 5 };
        carl::UInt8 src2[] = { 1, 2, 3, 4, 6 };
        carl::UInt8 src3[] = { 1, 2, 3, 4, 4 };

        CHECK(Memory::compare(src1, src1, 5) == 0)
        CHECK(Memory::compare(src1, src2, 5) < 0)
        CHECK(Memory::compare(src1, src3, 5) > 0)
    }

    return exitCode;
}

//  End of Main.cpp
