//
//  Main.cpp
//
//  The CARL search framework test.
//
//  $Date: 2008/02/25 12:11:27 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(condition) if (!(condition)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        int data[] = { 1, 2, 3, 4, 5, 2, 3 };
        CHECK(LinearSearch<int>::findFirst(data, sizeof(data) / sizeof(data[0]), 0) == NULL)
        CHECK(LinearSearch<int>::findFirst(data, sizeof(data) / sizeof(data[0]), 2) == data + 1)
        CHECK(LinearSearch<int>::findLast(data, sizeof(data) / sizeof(data[0]), 0) == NULL)
        CHECK(LinearSearch<int>::findLast(data, sizeof(data) / sizeof(data[0]), 2) == data + 5)
        Array<const int*> all = LinearSearch<int>::findAll(data, sizeof(data) / sizeof(data[0]), 0);
        CHECK(all.size() == 0)
        all = LinearSearch<int>::findAll(data, sizeof(data) / sizeof(data[0]), 2);
        CHECK(all.size() == 2 && all[0] == data + 1 && all[1] == data + 5)
    }

    {
        int data[] = { 1, 2, 2, 3, 4, 5 };
        CHECK(BinarySearch<int>::findFirst(data, sizeof(data) / sizeof(data[0]), 0) == NULL)
        CHECK(BinarySearch<int>::findFirst(data, sizeof(data) / sizeof(data[0]), 2) == data + 1)
        CHECK(BinarySearch<int>::findLast(data, sizeof(data) / sizeof(data[0]), 0) == NULL)
        CHECK(BinarySearch<int>::findLast(data, sizeof(data) / sizeof(data[0]), 2) == data + 2)
        Array<const int*> all = BinarySearch<int>::findAll(data, sizeof(data) / sizeof(data[0]), 0);
        CHECK(all.size() == 0)
        all = BinarySearch<int>::findAll(data, sizeof(data) / sizeof(data[0]), 2);
        CHECK(all.size() == 2 && all[0] == data + 1 && all[1] == data + 2)
    }

    return exitCode;
}

//  End of Main.cpp
