//
//  Main.cpp
//
//  The CARL sorting test.
//
//  $Date: 2008/02/22 12:09:22 $
//  $Revision: 1.7 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) failed = true

int MyApp::run(const Array<String> & /*args*/)
{
    Random r;
    bool failed = false;
    for (UIntSize size = 1; size <= 1024 * 1024; size *= 2)
    {
        StandardStreams::out.writeLn("Sorting " + NumberFormat::getInstance()->format(size) + " elements");
        Array<Int32> data;
        for (UIntSize i = 0; i < size; i++)
        {
            data.append(r.nextInt32());
        }
        NaturalIncreasingSortOrder<Int32> order;
        //
        Array<Int32> quicksortData = data;
        Array<Int32> heapsortData = data;
        Array<Int32> mergesortData = data;
        //
        TimeSpan quicksortStart = TimeSpan::getTimeSpanSinceReferencePoint();
        (QuickSort<Int32>()).sort(quicksortData.getData(), quicksortData.size(), order);
        TimeSpan quicksortEnd = TimeSpan::getTimeSpanSinceReferencePoint();
        for (UIntSize i = 0; i + 1 < quicksortData.size(); i++)
        {
            CHECK(quicksortData[i] <= quicksortData[i + 1]);
        }
        //
        TimeSpan heapsortStart = TimeSpan::getTimeSpanSinceReferencePoint();
        (HeapSort<Int32>()).sort(heapsortData.getData(), heapsortData.size(), order);
        TimeSpan heapsortEnd = TimeSpan::getTimeSpanSinceReferencePoint();
        for (UIntSize i = 0; i + 1 < heapsortData.size(); i++)
        {
            CHECK(heapsortData[i] <= heapsortData[i + 1]);
        }
        //
        TimeSpan mergesortStart = TimeSpan::getTimeSpanSinceReferencePoint();
        (MergeSort<Int32>()).sort(mergesortData.getData(), mergesortData.size(), order);
        TimeSpan mergesortEnd = TimeSpan::getTimeSpanSinceReferencePoint();
        for (UIntSize i = 0; i + 1 < mergesortData.size(); i++)
        {
            CHECK(mergesortData[i] <= mergesortData[i + 1]);
        }
        //
        StandardStreams::out.writeLn("    Quicksort: " + NumberFormat::getInstance()->format(quicksortEnd.getNanoseconds() - quicksortStart.getNanoseconds()) + "ns");
        StandardStreams::out.writeLn("    Heapsort : " + NumberFormat::getInstance()->format(heapsortEnd.getNanoseconds() - heapsortStart.getNanoseconds()) + "ns");
        StandardStreams::out.writeLn("    Mergesort: " + NumberFormat::getInstance()->format(mergesortEnd.getNanoseconds() - mergesortStart.getNanoseconds()) + "ns");
    }
    return failed ? 1 : 0;
}

//  End of Main.cpp
