//
//  Main.cpp
//
//  The CARL String test.
//
//  $Date: 2007/07/19 12:07:56 $
//  $Revision: 1.1 $
//  $Author: akapusti $
//
//  Copyright (C) 2006 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;
#include <stdio.h>

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

#define CHECK(x) if (!(x)) { exitCode = Process::ExitCode_Failure; }

int MyApp::run(const Array<String> & /*args*/)
{
    Process::ExitCode exitCode = Process::ExitCode_Success;

    {
        String s;
        CHECK(s.length() == 0)
    }

    {
        String s('a');
        CHECK(s.length() == 1)
        CHECK(s[0] == 'a')
    }

    {
        String s(Char('a'));
        CHECK(s.length() == 1)
        CHECK(s[0] == Char('a'))
    }

    {
        String s('a', 3);
        CHECK(s.length() == 3)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'a')
        CHECK(s[2] == 'a')
    }

    {
        String s(Char('a'), 3);
        CHECK(s.length() == 3)
        CHECK(s[0] == Char('a'))
        CHECK(s[1] == Char('a'))
        CHECK(s[2] == Char('a'))
    }

    {
        String s("abc");
        CHECK(s.length() == 3)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'b')
        CHECK(s[2] == 'c')
    }

    {
        Char abc[] = { 'a', 'b', 'c', 0 };
        String s(abc);
        CHECK(s.length() == 3)
        CHECK(s[0] == Char('a'))
        CHECK(s[1] == Char('b'))
        CHECK(s[2] == Char('c'))
    }

    {
        String s("abcd", 3);
        CHECK(s.length() == 3)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'b')
        CHECK(s[2] == 'c')
    }

    {
        Char abcd[] = { 'a', 'b', 'c', 'd', 0 };
        String s(abcd, 3);
        CHECK(s.length() == 3)
        CHECK(s[0] == Char('a'))
        CHECK(s[1] == Char('b'))
        CHECK(s[2] == Char('c'))
    }

    {
        String s1("abc");
        String s2(s1);
        CHECK(s1.length() == 3)
        CHECK(s2.length() == 3)
        CHECK(s2[0] == 'a')
        CHECK(s2[1] == 'b')
        CHECK(s2[2] == 'c')
        s1 = "def";
        CHECK(s2[0] == 'a')
        CHECK(s2[1] == 'b')
        CHECK(s2[2] == 'c')
    }

    {
        String s('a');
        s = 'b';
        CHECK(s.length() == 1)
        CHECK(s[0] == 'b')
    }

    {
        String s('a');
        s = Char('b');
        CHECK(s.length() == 1)
        CHECK(s[0] == Char('b'))
    }

    {
        String s('a');
        s = "abc";
        CHECK(s.length() == 3)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'b')
        CHECK(s[2] == 'c')
    }

    {
        Char abc[] = { 'a', 'b', 'c', 0 };
        String s('a');
        s = abc;
        CHECK(s.length() == 3)
        CHECK(s[0] == Char('a'))
        CHECK(s[1] == Char('b'))
        CHECK(s[2] == Char('c'))
    }

    {
        String s1('a');
        String s2;
        s2 = s1;
        CHECK(s2.length() == 1)
        CHECK(s2[0] == 'a')
    }

    {
        String s1;
        CHECK(s1.isEmpty())
        String s2('a');
        CHECK(!s2.isEmpty())
    }

    {
        String s("abcde");
        s.setLength(3);
        CHECK(s.length() == 3)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'b')
        CHECK(s[2] == 'c')
        s.setLength(5);
        CHECK(s.length() == 5)
        CHECK(s[0] == 'a')
        CHECK(s[1] == 'b')
        CHECK(s[2] == 'c')
        CHECK(s[3] == ' ')
        CHECK(s[4] == ' ')
    }

    {
        String s('b');
        CHECK(s.compare('a') > 0)
        CHECK(s.compare('b') == 0)
        CHECK(s.compare('c') < 0)
        CHECK(s.compare('B') != 0)
    }

    {
        String s('b');
        CHECK(s.compare(Char('a')) > 0)
        CHECK(s.compare(Char('b')) == 0)
        CHECK(s.compare(Char('c')) < 0)
        CHECK(s.compare(Char('B')) != 0)
    }

    {
        String s("abc");
        CHECK(s.compare("abb") > 0)
        CHECK(s.compare("abc") == 0)
        CHECK(s.compare("abcd") < 0)
        CHECK(s.compare("abd") < 0)
        CHECK(s.compare("ABC") != 0)
    }

    {
        Char abb[] = { 'a', 'b', 'b', 0 };
        Char abc[] = { 'a', 'b', 'c', 0 };
        Char abcd[] = { 'a', 'b', 'c', 'd', 0 };
        Char abd[] = { 'a', 'b', 'd', 0 };
        Char ABC[] = { 'A', 'B', 'C', 0 };
        String s("abc");
        CHECK(s.compare(abb) > 0)
        CHECK(s.compare(abc) == 0)
        CHECK(s.compare(abcd) < 0)
        CHECK(s.compare(abd) < 0)
        CHECK(s.compare(ABC) != 0)
    }

    {
        String s("abc");
        CHECK(s.compare(String("abb")) > 0)
        CHECK(s.compare(String("abc")) == 0)
        CHECK(s.compare(String("abcd")) < 0)
        CHECK(s.compare(String("abd")) < 0)
        CHECK(s.compare(String("ABC")) != 0)
    }

    {
        String s('b');
        CHECK(s.compareIgnoreCase('a') > 0)
        CHECK(s.compareIgnoreCase('b') == 0)
        CHECK(s.compareIgnoreCase('c') < 0)
        CHECK(s.compareIgnoreCase('A') > 0)
        CHECK(s.compareIgnoreCase('B') == 0)
        CHECK(s.compareIgnoreCase('C') < 0)
    }

    {
        String s('b');
        CHECK(s.compareIgnoreCase(Char('a')) > 0)
        CHECK(s.compareIgnoreCase(Char('b')) == 0)
        CHECK(s.compareIgnoreCase(Char('c')) < 0)
        CHECK(s.compareIgnoreCase(Char('A')) > 0)
        CHECK(s.compareIgnoreCase(Char('B')) == 0)
        CHECK(s.compareIgnoreCase(Char('C')) < 0)
    }

    {
        String s("abc");
        CHECK(s.compareIgnoreCase("abb") > 0)
        CHECK(s.compareIgnoreCase("abc") == 0)
        CHECK(s.compareIgnoreCase("abcd") < 0)
        CHECK(s.compareIgnoreCase("abd") < 0)
        CHECK(s.compareIgnoreCase("ABB") > 0)
        CHECK(s.compareIgnoreCase("ABC") == 0)
        CHECK(s.compareIgnoreCase("ABCD") < 0)
        CHECK(s.compareIgnoreCase("ABD") < 0)
    }

    {
        Char abb[] = { 'a', 'b', 'b', 0 };
        Char abc[] = { 'a', 'b', 'c', 0 };
        Char abcd[] = { 'a', 'b', 'c', 'd', 0 };
        Char abd[] = { 'a', 'b', 'd', 0 };
        Char ABB[] = { 'A', 'B', 'B', 0 };
        Char ABC[] = { 'A', 'B', 'C', 0 };
        Char ABCD[] = { 'A', 'B', 'C', 'D', 0 };
        Char ABD[] = { 'A', 'B', 'D', 0 };
        String s("abc");
        CHECK(s.compareIgnoreCase(abb) > 0)
        CHECK(s.compareIgnoreCase(abc) == 0)
        CHECK(s.compareIgnoreCase(abcd) < 0)
        CHECK(s.compareIgnoreCase(abd) < 0)
        CHECK(s.compareIgnoreCase(ABB) > 0)
        CHECK(s.compareIgnoreCase(ABC) == 0)
        CHECK(s.compareIgnoreCase(ABCD) < 0)
        CHECK(s.compareIgnoreCase(ABD) < 0)
    }

    {
        String s("abc");
        CHECK(s.compareIgnoreCase(String("abb")) > 0)
        CHECK(s.compareIgnoreCase(String("abc")) == 0)
        CHECK(s.compareIgnoreCase(String("abcd")) < 0)
        CHECK(s.compareIgnoreCase(String("abd")) < 0)
        CHECK(s.compareIgnoreCase(String("ABB")) > 0)
        CHECK(s.compareIgnoreCase(String("ABC")) == 0)
        CHECK(s.compareIgnoreCase(String("ABCD")) < 0)
        CHECK(s.compareIgnoreCase(String("ABD")) < 0)
    }

    {
        String s("abc");
        CHECK(s.startsWith('a'))
        CHECK(!s.startsWith('b'))
        CHECK(!s.startsWith('A'))
    }

    {
        String s("abc");
        CHECK(s.startsWith(Char('a')))
        CHECK(!s.startsWith(Char('b')))
        CHECK(!s.startsWith(Char('A')))
    }

    {
        String s("abc");
        CHECK(s.startsWith("ab"))
        CHECK(!s.startsWith("ac"))
        CHECK(!s.startsWith("AB"))
    }

    {
        Char ab[] = { 'a', 'b', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char AB[] = { 'A', 'B', 0 };
        String s("abc");
        CHECK(s.startsWith(ab))
        CHECK(!s.startsWith(ac))
        CHECK(!s.startsWith(AB))
    }

    {
        String s("abc");
        CHECK(s.startsWith(String("ab")))
        CHECK(!s.startsWith(String("ac")))
        CHECK(!s.startsWith(String("AB")))
    }

    {
        String s("abc");
        CHECK(s.startsWithIgnoreCase('a'))
        CHECK(!s.startsWithIgnoreCase('b'))
        CHECK(s.startsWithIgnoreCase('A'))
    }

    {
        String s("abc");
        CHECK(s.startsWithIgnoreCase(Char('a')))
        CHECK(!s.startsWithIgnoreCase(Char('b')))
        CHECK(s.startsWithIgnoreCase(Char('A')))
    }

    {
        String s("abc");
        CHECK(s.startsWithIgnoreCase("ab"))
        CHECK(!s.startsWithIgnoreCase("ac"))
        CHECK(s.startsWithIgnoreCase("AB"))
    }

    {
        Char ab[] = { 'a', 'b', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char AB[] = { 'A', 'B', 0 };
        String s("abc");
        CHECK(s.startsWithIgnoreCase(ab))
        CHECK(!s.startsWithIgnoreCase(ac))
        CHECK(s.startsWithIgnoreCase(AB))
    }

    {
        String s("abc");
        CHECK(s.startsWithIgnoreCase(String("ab")))
        CHECK(!s.startsWithIgnoreCase(String("ac")))
        CHECK(s.startsWithIgnoreCase(String("AB")))
    }

    {
        String s("abc");
        CHECK(s.endsWith('c'))
        CHECK(!s.endsWith('b'))
        CHECK(!s.endsWith('C'))
    }

    {
        String s("abc");
        CHECK(s.endsWith(Char('c')))
        CHECK(!s.endsWith(Char('b')))
        CHECK(!s.endsWith(Char('C')))
    }

    {
        String s("abc");
        CHECK(s.endsWith("bc"))
        CHECK(!s.endsWith("ac"))
        CHECK(!s.endsWith("BC"))
    }

    {
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        String s("abc");
        CHECK(s.endsWith(bc))
        CHECK(!s.endsWith(ac))
        CHECK(!s.endsWith(BC))
    }

    {
        String s("abc");
        CHECK(s.endsWith(String("bc")))
        CHECK(!s.endsWith(String("ac")))
        CHECK(!s.endsWith(String("BC")))
    }

    {
        String s("abc");
        CHECK(s.endsWithIgnoreCase('c'))
        CHECK(!s.endsWithIgnoreCase('b'))
        CHECK(s.endsWithIgnoreCase('C'))
    }

    {
        String s("abc");
        CHECK(s.endsWithIgnoreCase(Char('c')))
        CHECK(!s.endsWithIgnoreCase(Char('b')))
        CHECK(s.endsWithIgnoreCase(Char('C')))
    }

    {
        String s("abc");
        CHECK(s.endsWithIgnoreCase("bc"))
        CHECK(!s.endsWithIgnoreCase("ac"))
        CHECK(s.endsWithIgnoreCase("BC"))
    }

    {
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        String s("abc");
        CHECK(s.endsWithIgnoreCase(bc))
        CHECK(!s.endsWithIgnoreCase(ac))
        CHECK(s.endsWithIgnoreCase(BC))
    }

    {
        String s("abc");
        CHECK(s.endsWithIgnoreCase(String("bc")))
        CHECK(!s.endsWithIgnoreCase(String("ac")))
        CHECK(s.endsWithIgnoreCase(String("BC")))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirst('b', position) && position == 1)
        CHECK(s.findFirst('b', position, 3) && position == 4)
        CHECK(!s.findFirst('b', position, 5))
        CHECK(!s.findFirst('d', position))
        CHECK(!s.findFirst('B', position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirst(Char('b'), position) && position == 1)
        CHECK(s.findFirst(Char('b'), position, 3) && position == 4)
        CHECK(!s.findFirst(Char('b'), position, 5))
        CHECK(!s.findFirst(Char('d'), position))
        CHECK(!s.findFirst(Char('B'), position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirst("bc", position) && position == 1)
        CHECK(s.findFirst("bc", position, 3) && position == 4)
        CHECK(!s.findFirst("bc", position, 5))
        CHECK(!s.findFirst("ac", position))
        CHECK(!s.findFirst("BC", position))
    }

    {
        String s("abcabc");
        UIntSize position;
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        CHECK(s.findFirst(bc, position) && position == 1)
        CHECK(s.findFirst(bc, position, 3) && position == 4)
        CHECK(!s.findFirst(bc, position, 5))
        CHECK(!s.findFirst(ac, position))
        CHECK(!s.findFirst(BC, position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirst(String("bc"), position) && position == 1)
        CHECK(s.findFirst(String("bc"), position, 3) && position == 4)
        CHECK(!s.findFirst(String("bc"), position, 5))
        CHECK(!s.findFirst(String("ac"), position))
        CHECK(!s.findFirst(String("BC"), position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirstIgnoreCase('b', position) && position == 1)
        CHECK(s.findFirstIgnoreCase('b', position, 3) && position == 4)
        CHECK(!s.findFirstIgnoreCase('b', position, 5))
        CHECK(!s.findFirstIgnoreCase('d', position))
        CHECK(s.findFirstIgnoreCase('B', position) && position == 1)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirstIgnoreCase(Char('b'), position) && position == 1)
        CHECK(s.findFirstIgnoreCase(Char('b'), position, 3) && position == 4)
        CHECK(!s.findFirstIgnoreCase(Char('b'), position, 5))
        CHECK(!s.findFirstIgnoreCase(Char('d'), position))
        CHECK(s.findFirstIgnoreCase(Char('B'), position) && position == 1)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirstIgnoreCase("bc", position) && position == 1)
        CHECK(s.findFirstIgnoreCase("bc", position, 3) && position == 4)
        CHECK(!s.findFirstIgnoreCase("bc", position, 5))
        CHECK(!s.findFirstIgnoreCase("ac", position))
        CHECK(s.findFirstIgnoreCase("BC", position) && position == 1)
    }

    {
        String s("abcabc");
        UIntSize position;
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        CHECK(s.findFirstIgnoreCase(bc, position) && position == 1)
        CHECK(s.findFirstIgnoreCase(bc, position, 3) && position == 4)
        CHECK(!s.findFirstIgnoreCase(bc, position, 5))
        CHECK(!s.findFirstIgnoreCase(ac, position))
        CHECK(s.findFirstIgnoreCase(BC, position) && position == 1)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findFirstIgnoreCase(String("bc"), position) && position == 1)
        CHECK(s.findFirstIgnoreCase(String("bc"), position, 3) && position == 4)
        CHECK(!s.findFirstIgnoreCase(String("bc"), position, 5))
        CHECK(!s.findFirstIgnoreCase(String("ac"), position))
        CHECK(s.findFirstIgnoreCase(String("BC"), position) && position == 1)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLast('b', position) && position == 4)
        CHECK(!s.findLast('d', position))
        CHECK(!s.findLast('B', position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLast(Char('b'), position) && position == 4)
        CHECK(!s.findLast(Char('d'), position))
        CHECK(!s.findLast(Char('B'), position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLast("bc", position) && position == 4)
        CHECK(!s.findLast("ac", position))
        CHECK(!s.findLast("BC", position))
    }

    {
        String s("abcabc");
        UIntSize position;
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        CHECK(s.findLast(bc, position) && position == 4)
        CHECK(!s.findLast(ac, position))
        CHECK(!s.findLast(BC, position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLast(String("bc"), position) && position == 4)
        CHECK(!s.findLast(String("ac"), position))
        CHECK(!s.findLast(String("BC"), position))
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLastIgnoreCase('b', position) && position == 4)
        CHECK(!s.findLastIgnoreCase('d', position))
        CHECK(s.findLastIgnoreCase('B', position) && position == 4)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLastIgnoreCase(Char('b'), position) && position == 4)
        CHECK(!s.findLastIgnoreCase(Char('d'), position))
        CHECK(s.findLastIgnoreCase(Char('B'), position) && position == 4)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLastIgnoreCase("bc", position) && position == 4)
        CHECK(!s.findLastIgnoreCase("ac", position))
        CHECK(s.findLastIgnoreCase("BC", position) && position == 4)
    }

    {
        String s("abcabc");
        UIntSize position;
        Char bc[] = { 'b', 'c', 0 };
        Char ac[] = { 'a', 'c', 0 };
        Char BC[] = { 'B', 'C', 0 };
        CHECK(s.findLastIgnoreCase(bc, position) && position == 4)
        CHECK(!s.findLastIgnoreCase(ac, position))
        CHECK(s.findLastIgnoreCase(BC, position) && position == 4)
    }

    {
        String s("abcabc");
        UIntSize position;
        CHECK(s.findLastIgnoreCase(String("bc"), position) && position == 4)
        CHECK(!s.findLastIgnoreCase(String("ac"), position))
        CHECK(s.findLastIgnoreCase(String("BC"), position) && position == 4)
    }

    {
        String s('a');
        CHECK(s == 'a')
        CHECK(!(s == 'b'))
        CHECK(!(s == 'A'))
    }

    {
        String s('a');
        CHECK(s == Char('a'))
        CHECK(!(s == Char('b')))
        CHECK(!(s == Char('A')))
    }

    {
        String s("abc");
        CHECK(s == "abc")
        CHECK(!(s == "abd"))
        CHECK(!(s == "ABC"))
    }

    {
        Char abc[] = { 'a', 'b', 'c', 0 };
        Char abd[] = { 'a', 'b', 'd', 0 };
        Char ABC[] = { 'A', 'B', 'C', 0 };
        String s("abc");
        CHECK(s == abc)
        CHECK(!(s == abd))
        CHECK(!(s == ABC))
    }

    {
        String s("abc");
        CHECK(s == String("abc"))
        CHECK(!(s == String("abd")))
        CHECK(!(s == String("ABC")))
    }

    {
        String s('a');
        CHECK(s != 'b')
        CHECK(!(s != 'a'))
        CHECK(s != 'A')
    }

    {
        String s('a');
        CHECK(s != Char('b'))
        CHECK(!(s != Char('a')))
        CHECK(s != Char('A'))
    }

    {
        String s("abc");
        CHECK(s != "abd")
        CHECK(!(s != "abc"))
        CHECK(s != "ABC")
    }

    {
        Char abc[] = { 'a', 'b', 'c', 0 };
        Char abd[] = { 'a', 'b', 'd', 0 };
        Char ABC[] = { 'A', 'B', 'C', 0 };
        String s("abc");
        CHECK(s != abd)
        CHECK(!(s != abc))
        CHECK(s != ABC)
    }

    {
        String s("abc");
        CHECK(s != String("abd"))
        CHECK(!(s != String("abc")))
        CHECK(s != String("ABC"))
    }

    {
        String s('b');
        CHECK(!(s < 'a'))
        CHECK(!(s < 'b'))
        CHECK(s < 'c')
    }

    {
        String s('b');
        CHECK(!(s < Char('a')))
        CHECK(!(s < Char('b')))
        CHECK(s < Char('c'))
    }

    {
        String s("abc");
        CHECK(!(s < "abc"))
        CHECK(s < "abd")
        CHECK(s < "abcd")
        CHECK(!(s < "ab"))
    }

    {
        String s("abc");
        Char abc[] = {'a', 'b', 'c', 0 };
        Char abd[] = {'a', 'b', 'd', 0 };
        Char ab[] = {'a', 'b', 0 };
        Char abcd[] = {'a', 'b', 'c', 'd', 0 };
        CHECK(!(s < abc))
        CHECK(s < abd)
        CHECK(s < abcd)
        CHECK(!(s < ab))
    }

    {
        String s("abc");
        CHECK(!(s < String("abc")))
        CHECK(s < String("abd"))
        CHECK(s < String("abcd"))
        CHECK(!(s < String("ab")))
    }

    {
        String s('b');
        CHECK(!(s <= 'a'))
        CHECK(s <= 'b')
        CHECK(s <= 'c')
    }

    {
        String s('b');
        CHECK(!(s <= Char('a')))
        CHECK(s <= Char('b'))
        CHECK(s <= Char('c'))
    }

    {
        String s("abc");
        CHECK(s <= "abc")
        CHECK(s <= "abd")
        CHECK(s <= "abcd")
        CHECK(!(s <= "ab"))
    }

    {
        String s("abc");
        Char abc[] = {'a', 'b', 'c', 0 };
        Char abd[] = {'a', 'b', 'd', 0 };
        Char ab[] = {'a', 'b', 0 };
        Char abcd[] = {'a', 'b', 'c', 'd', 0 };
        CHECK(s <= abc)
        CHECK(s <= abd)
        CHECK(s <= abcd)
        CHECK(!(s <= ab))
    }

    {
        String s("abc");
        CHECK(s <= String("abc"))
        CHECK(s <= String("abd"))
        CHECK(s <= String("abcd"))
        CHECK(!(s <= String("ab")))
    }

    {
        String s('b');
        CHECK(!(s > 'c'))
        CHECK(!(s > 'b'))
        CHECK(s > 'a')
    }

    {
        String s('b');
        CHECK(!(s > Char('c')))
        CHECK(!(s > Char('b')))
        CHECK(s > Char('a'))
    }

    {
        String s("abc");
        CHECK(!(s > "abc"))
        CHECK(!(s > "abd"))
        CHECK(!(s > "abcd"))
        CHECK(s > "ab")
    }

    {
        String s("abc");
        Char abc[] = {'a', 'b', 'c', 0 };
        Char abd[] = {'a', 'b', 'd', 0 };
        Char ab[] = {'a', 'b', 0 };
        Char abcd[] = {'a', 'b', 'c', 'd', 0 };
        CHECK(!(s > abc))
        CHECK(!(s > abd))
        CHECK(!(s > abcd))
        CHECK(s > ab)
    }

    {
        String s("abc");
        CHECK(!(s > String("abc")))
        CHECK(!(s > String("abd")))
        CHECK(!(s > String("abcd")))
        CHECK(s > String("ab"))
    }

    {
        String s('b');
        CHECK(!(s >= 'c'))
        CHECK(s >= 'b')
        CHECK(s >= 'a')
    }

    {
        String s('b');
        CHECK(!(s >= Char('c')))
        CHECK(s >= Char('b'))
        CHECK(s >= Char('a'))
    }

    {
        String s("abc");
        CHECK(s >= "abc")
        CHECK(!(s >= "abd"))
        CHECK(!(s >= "abcd"))
        CHECK(s >= "ab")
    }

    {
        String s("abc");
        Char abc[] = {'a', 'b', 'c', 0 };
        Char abd[] = {'a', 'b', 'd', 0 };
        Char ab[] = {'a', 'b', 0 };
        Char abcd[] = {'a', 'b', 'c', 'd', 0 };
        CHECK(s >= abc)
        CHECK(!(s >= abd))
        CHECK(!(s >= abcd))
        CHECK(s >= ab)
    }

    {
        String s("abc");
        CHECK(s >= String("abc"))
        CHECK(!(s >= String("abd")))
        CHECK(!(s >= String("abcd")))
        CHECK(s >= String("ab"))
    }

    {
        String s("abc");
        s[1] = 'd';
        CHECK(s == "adc")
    }

    {
        String s("abc");
        CHECK(s + 'd' == "abcd")
    }

    {
        String s("abc");
        CHECK(s + Char('d') == "abcd")
    }

    {
        String s("abc");
        CHECK(s + "def" == "abcdef")
    }

    {
        String s("abc");
        Char def[] = { 'd', 'e', 'f', 0 };
        CHECK(s + def == "abcdef")
    }

    {
        String s("abc");
        CHECK(s + String("def") == "abcdef")
    }

    {
        String s("abc");
        s += 'd';
        CHECK(s == "abcd")
    }

    {
        String s("abc");
        s += Char('d');
        CHECK(s == "abcd")
    }

    {
        String s("abc");
        s += "def";
        CHECK(s == "abcdef")
    }

    {
        String s("abc");
        Char def[] = { 'd', 'e', 'f', 0 };
        s += def;
        CHECK(s == "abcdef")
    }

    {
        String s("abc");
        s += String("def");
        CHECK(s == "abcdef")
    }

    {
        String s("abc");
        s.insert('d', 1);
        CHECK(s == "adbc")
    }

    {
        String s("abc");
        s.insert(Char('d'), 1);
        CHECK(s == "adbc")
    }

    {
        String s("abc");
        s.insert("def", 1);
        CHECK(s == "adefbc")
    }

    {
        String s("abc");
        Char def[] = { 'd', 'e', 'f', 0 };
        s.insert(def, 1);
        CHECK(s == "adefbc")
    }

    {
        String s("abc");
        s.insert(String("def"), 1);
        CHECK(s == "adefbc")
    }

    {
        String s("abc");
        s.remove(1, 0);
        CHECK(s == "abc");
        s.remove(1, 1);
        CHECK(s == "ac");
        s.remove(0, 2);
        CHECK(s == "");
    }

    {
        String s("abcd");
        s.replace(1, 2, 'e');
        CHECK(s == "aed")
    }

    {
        String s("abcd");
        s.replace(1, 2, Char('e'));
        CHECK(s == "aed")
    }

    {
        String s("abc");
        s.replace(1, 1, "def");
        CHECK(s == "adefc")
    }

    {
        String s("abc");
        Char def[] = { 'd', 'e', 'f', 0 };
        s.replace(1, 1, def);
        CHECK(s == "adefc")
    }

    {
        String s("abc");
        s.replace(1, 1, String("def"));
        CHECK(s == "adefc")
    }

    {
        String s("abc");
        CHECK(s.left(0) == "")
        CHECK(s.left(1) == "a")
        CHECK(s.left(2) == "ab")
        CHECK(s.left(3) == "abc")
        CHECK(s.left(4) == "abc")
    }

    {
        String s("abc");
        CHECK(s.right(0) == "")
        CHECK(s.right(1) == "c")
        CHECK(s.right(2) == "bc")
        CHECK(s.right(3) == "abc")
        CHECK(s.right(4) == "abc")
    }

    {
        String s("abc");
        CHECK(s.mid(0) == "abc")
        CHECK(s.mid(1) == "bc")
        CHECK(s.mid(2) == "c")
        CHECK(s.mid(3) == "")
    }

    {
        String s("abc");
        CHECK(s.mid(0, 0) == "")
        CHECK(s.mid(0, 1) == "a")
        CHECK(s.mid(0, 2) == "ab")
        CHECK(s.mid(0, 3) == "abc")
        CHECK(s.mid(1, 0) == "")
        CHECK(s.mid(1, 1) == "b")
        CHECK(s.mid(1, 2) == "bc")
        CHECK(s.mid(2, 0) == "")
        CHECK(s.mid(2, 1) == "c")
        CHECK(s.mid(3, 0) == "")
    }

    {
        String s(" \t\na \t\n");
        s.trimLeft();
        CHECK(s == "a \t\n")
    }

    {
        String s(" \t\na \t\n");
        s.trimRight();
        CHECK(s == " \t\na")
    }

    {
        String s(" \t\na \t\n");
        s.trim();
        CHECK(s == "a")
    }

    {
        String s(" \t\na \t\n");
        CHECK(s.trimmedLeft() == "a \t\n")
    }

    {
        String s(" \t\na \t\n");
        CHECK(s.trimmedRight() == " \t\na")
    }

    {
        String s(" \t\na \t\n");
        CHECK(s.trimmed() == "a")
    }

    {
        String s("abc");
        s.upperCase();
        CHECK(s == "ABC")
    }

    {
        String s("ABC");
        s.lowerCase();
        CHECK(s == "abc")
    }

    {
        String s("abc");
        s.titleCase();
        CHECK(s == "ABC")
    }

    {
        String s("abc");
        CHECK(s.upperCased() == "ABC")
    }

    {
        String s("ABC");
        CHECK(s.lowerCased() == "abc")
    }

    {
        String s("abc");
        CHECK(s.titleCased() == "ABC")
    }

    //  Done
    return exitCode;
}

//  End of Main.cpp
