//
//  Main.cpp
//
//  The CARL sorting test.
//
//  $Date: 2008/02/22 12:09:22 $
//  $Revision: 1.5 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

static void _pingChildProcess(Process & process, const String & command)
{
    StandardStreams::out.writeLn("sending command to child '" + command + "'");
    process.getStdinStream()->writeLn(command);

    String stdoutReply;
    if (!process.getStdoutStream()->readLine(stdoutReply, true))
    {
        throw IOException();
    }
    StandardStreams::out.writeLn("stdout reply from child '" + stdoutReply + "'");
    if (stdoutReply != command)
    {
        throw IOException();
    }

    String stderrReply;
    if (!process.getStderrStream()->readLine(stderrReply, true))
    {
        throw IOException();
    }
    StandardStreams::out.writeLn("stderr reply from child '" + stderrReply + "'");
    if (stderrReply != command)
    {
        throw IOException();
    }
}

int MyApp::run(const Array<String> & args)
{
    if (args.size() == 1)
    {   //  This is a parent process
        StandardStreams::out.writeLn("parent starts");
        //  Start child process...
        Process childProcess;
        Array<String> childProcessNameAndArguments;
        childProcessNameAndArguments.append(args[0]);
        childProcessNameAndArguments.append(String("child"));
        if (childProcess.start(childProcessNameAndArguments, Process::CaptureStandardInputStream | Process::CaptureStandardOutputStream | Process::CaptureStandardErrorStream) != Process::StartOutcome_Success)
        {
            return 1;
        }
        //  Send several commands to child process, receive & verify echo
        try
        {
            _pingChildProcess(childProcess, String("command #1"));
            _pingChildProcess(childProcess, String("command #2"));
            _pingChildProcess(childProcess, String("exit"));
        }
        catch (IOException)
        {
            return 1;
        }
        StandardStreams::out.writeLn("parent ends");
    }
    else
    {   //  This is a child process
        //  Read from standard input & echo until "exit"
        String command;
        while (StandardStreams::in.readLine(command, true))
        {
            StandardStreams::out.writeLn(command);
            StandardStreams::err.writeLn(command);
            if (command == "exit")
            {
                break;
            }
        }
    }
    return 0;
}

//  End of Main.cpp
