//
//  Main.cpp
//
//  The CARL mutex test.
//
//  $Date: 2008/02/22 12:09:22 $
//  $Revision: 1.4 $
//  $Author: akapusti $
//
//  Copyright (C) 2006-2008 Cybernetic Intelligence GmbH
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////
#include "CARL.hpp"
using namespace carl;

class MyApp : public Application
{
    //////////
    //  Can't assign or copy construct
private:
    MyApp(const MyApp &);
    void                        operator = (const MyApp &);

    //////////
    //  Construction/destruction
public:
    MyApp() {}
    virtual ~MyApp() {}

    //////////
    //  Operations
public:
    virtual int run(const Array<String> & args);
};

MyApp theApp;

class MyThread : public Thread
{
public:
    static const int Iterations = 10;

    MyThread(int & counter, Mutex & guard)
        :   _counter(counter),
            _guard(guard)
    {
    }

    virtual void        run()
    {
        for (int i = 0; i < Iterations; i++)
        {
            _guard.acquire();
            int n = _counter;
            sleep(TimeSpan(10000000));
            _counter = n + 1;
            _guard.release();
            sleep(TimeSpan(10000000));
        }
    }

private:
    int &               _counter;
    Mutex &             _guard;
};

int MyApp::run(const Array<String> & /*args*/)
{
    int counter = 0;
    Mutex guard;

    //  Create threads
    const int NumThreads = 10;
    Thread * threads[NumThreads];
    for (int i = 0; i < NumThreads; i++)
    {
        threads[i] = new MyThread(counter, guard);
    }
    //  Start!
    for (int i = 0; i < NumThreads; i++)
    {
        threads[i]->start();
    }
    //  Wait until they all finish
    for (; ; )
    {
        bool finished = true;
        for (int i = 0; i < NumThreads; i++)
        {
            if (threads[i]->getState() != Thread::State_Finished)
            {
                finished = false;
                break;
            }
        }
        if (finished)
        {
            break;
        }
        Thread::sleep(TimeSpan(10000000));
    }
    //  Check counter & return
    return (counter == MyThread::Iterations * NumThreads) ? Process::ExitCode_Success : Process::ExitCode_Failure;
}

//  End of Main.cpp
