#
#   build.py
#
#   CARL builder.
#
#   $Date: 2008/07/28 10:39:59 $
#   $Revision: 1.4 $
#   $Author: akapusti $
#
#   Copyright (C) 2007-2008 Cybernetic Intelligence GmbH
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##########
import sys, getopt, subprocess, time

#   Supported platforms
class Platform(object):
    X86Win32Msvc2003 = "x86-win32-msvc2003"
    X86Win32Msvc2005 = "x86-win32-msvc2005"
    X86Win32Cygwin   = "x86-win32-cygwin"
    X86LinuxGcc      = "x86-linux-gcc"

#   Supported configurations
class Configuration(object):
    Debug   = "debug"
    Release = "release"
    All     = "all"

#   Supported linkages                          
class Linkage(object):
    Static  = "static"
    Dynamic = "dynamic"
    All     = "all"

#   Command line help
def printUsage():
    print("Usage:")
    print("    python build.py --platform=<platform> --configuration=<configuration>")
    print("                    --linkage=<linkage>")
    print("")
    print("Where:")
    print("    --platform=x86-win32-msvc2003- Build CARL for MSVC2003 on x86/Win32")
    print("    --platform=x86-win32-msvc2005- Build CARL for MSVC2005 on x86/Win32")
    print("    --platform=x86-win32-cygwin  - Build CARL for cygwin on x86/Win32")
    print("    --platform=x86-linux-gcc     - Build CARL for gcc on x86/Linux")
    print("")
    print("    --configuration=debug        - Build Debug version of CARL")
    print("    --configuration=release      - Build Release version of CARL")
    print("    --configuration=all          - Build both Debug and Release versions")
    print("")
    print("    --linkage=static             - Build CARL static library")
    print("    --linkage=dynamic            - Build CARL dynamic library")
    print("    --linkage=all                - Build both static and dynamic libraries")

#   The builder function
def build(platform, configuration, linkage):
    if configuration == Configuration.All:
        build(platform, Configuration.Debug, linkage)
        build(platform, Configuration.Release, linkage)
    elif linkage == Linkage.All:
        build(platform, configuration, Linkage.Static)
        build(platform, configuration, Linkage.Dynamic)
    else:
        projectFileName = "CARL-1_0-" + platform + "-" + configuration + "-" + linkage + ".spp"
        #   Invoke the builder
        try:
            #   Start the process
            commandLineArguments = ["python", "Builder/splbuild.py", projectFileName]
            childProcess = subprocess.Popen(commandLineArguments, universal_newlines = True)
            #   Now wait for the process to terminate
            exitCode = None
            while True:
                exitCode = childProcess.poll()
                if exitCode is not None:
                    break
                time.sleep(1)
            if exitCode != 0:
                sys.exit(exitCode)
        except OSError:
            commandLine = " ".join(commandLineArguments)
            print "Can't execute command '" + commandLine + "'"
            sys.exit(exitCode)

#   Program's entry point
if __name__ == "__main__":
    #   Fetch command line options
    opts, args = getopt.getopt(sys.argv[1:], "", ["platform=", "configuration=", "linkage="])
    platform = None
    configuration = None
    linkage = None
    for name, value in opts:
        if name == "--platform":
            platform = value
        elif name == "--configuration":
            configuration = value
        elif name == "--linkage":
            linkage = value
    if (platform is None) or (configuration is None) or (linkage is None):
        printUsage()
        sys.exit(0)
    #   Validate command line options
    if (platform != Platform.X86Win32Msvc2003) and (platform != Platform.X86Win32Msvc2005) and (platform != Platform.X86Win32Cygwin) and (platform != Platform.X86LinuxGcc):
        print("Invalid platform " + platform)
        printUsage()
        sys.exit(1)
    if (configuration != Configuration.Debug) and (configuration != Configuration.Release) and (configuration != Configuration.All):
        print("Invalid configuration " + configuration)
        printUsage();
        sys.exit(1)
    if (linkage != Linkage.Static) and (linkage != Linkage.Dynamic) and (linkage != Linkage.All):
        print("Invalid linkage " + linkage)
        printUsage();
        sys.exit(1)
    #   Build!
    build(platform, configuration, linkage)

#   End of build.py
